package com.chinaitop.depot.act.controller;

import java.io.IOException;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import org.activiti.engine.FormService;
import org.activiti.engine.HistoryService;
import org.activiti.engine.IdentityService;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.form.FormProperty;
import org.activiti.engine.impl.form.StartFormDataImpl;
import org.activiti.engine.repository.ProcessDefinition;
import org.activiti.engine.repository.ProcessDefinitionQuery;
import org.activiti.engine.runtime.ProcessInstance;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.act.model.DeploymentList;
import com.chinaitop.depot.act.service.ActTaskService;
import com.chinaitop.depot.business.model.UserInfo;
import com.chinaitop.depot.utils.activiti.PageUtils;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

/**
 * 动态表单Controller
 *
 * @author xujh
 */
@RestController
@RequestMapping(value = "/act/form/dynamic")
public class ActDynamicFormController {
	private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private RepositoryService repositoryService;
    @Autowired
    private FormService formService;
    @Autowired
    private TaskService taskService;
    @Autowired
    private IdentityService identityService;
    @Autowired
    private HistoryService historyService;
    @Autowired
    private RuntimeService runtimeService;
    /*@Resource
    private ActTaskService actTaskService;*/
    
    /**
     * 动态form流程列表
     *
     * @param model
     * @return
     */
    @RequestMapping(value = {"process-list", ""})
    public PageInfo<DeploymentList> processDefinitionList(Integer pageNum, Integer pageSize,Model model, @RequestParam(value = "processType", required = false) String processType, HttpServletRequest request) {
        PageHelper.startPage(pageNum, pageSize);

        // 读取所有流程
        ProcessDefinitionQuery query = repositoryService.createProcessDefinitionQuery().active().orderByDeploymentId().desc();
        List<ProcessDefinition> list = query.list();
        
        List<DeploymentList> deploymentlist=new ArrayList<DeploymentList>();
		if(list!=null&&list.size()>0){
			for(ProcessDefinition m: list){
				DeploymentList deploymentLists=new DeploymentList();
				deploymentLists.setId(m.getId());
				deploymentLists.setName(m.getName());
				deploymentLists.setKey(m.getKey());
				deploymentLists.setVersion(m.getVersion());
				deploymentLists.setResourceName(m.getResourceName());
				deploymentLists.setDiagramResourceName(m.getDiagramResourceName());
				deploymentLists.setDeploymentId(m.getDeploymentId());
				deploymentlist.add(deploymentLists);
			}
			
		}
		PageInfo<DeploymentList> pageInfo = PageUtils.page2(deploymentlist, pageNum, pageSize);
        return pageInfo;
    }
    
    
    /**
     * 初始化启动流程，读取启动流程的表单字段来渲染start form
     */
    @RequestMapping(value = "get-form/start")
    @ResponseBody
    @SuppressWarnings("unchecked")
    public Map<String, Object> findStartForm(String processDefinitionId,String key) throws Exception {
        Map<String, Object> result = new HashMap<String, Object>();
        StartFormDataImpl startFormData = (StartFormDataImpl) formService.getStartFormData(processDefinitionId);
        startFormData.setProcessDefinition(null);

	    /*
	     * 读取enum类型数据，用于下拉框
	     */
        List<FormProperty> formProperties = startFormData.getFormProperties();
        for (FormProperty formProperty : formProperties) {
            Map<String, String> values = (Map<String, String>) formProperty.getType().getInformation("values");
            if (values != null) {
                for (Entry<String, String> enumEntry : values.entrySet()) {
                    logger.debug("enum, key: {}, value: {}", enumEntry.getKey(), enumEntry.getValue());
                }
                result.put("enum_" + formProperty.getId(), values);
            }
        }

        result.put("form", startFormData);
        result.put("key", key);
        result.put("processDefinitionId", processDefinitionId);

        return result;
    }
    
    public static Map<String, Object> objectToMap(Object obj) throws IllegalAccessException {
    	         Map<String, Object> map = new HashMap<>();
    	         Class<?> clazz = obj.getClass();
    	          for (Field field : clazz.getDeclaredFields()) {
    	              field.setAccessible(true);
    	              String fieldName = field.getName();
    	             Object value = field.get(obj);
    	             map.put(fieldName, value);
    	         }
    	          return map;
    	      }
    
    /**
     * 办理任务，提交task的并保存form
     * @throws IOException 
     * @throws JsonMappingException 
     * @throws JsonParseException 
     */
    @RequestMapping(value = "task/complete")
    public String completeTask(RedirectAttributes redirectAttributes, HttpServletRequest request, String formJson,String keys,String processDefinitionId) throws JsonParseException, JsonMappingException, IOException {
    	
    	UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
		// 把字符串转换为JSONArray对象
//		Object parse = JSON.parse(formJson);
		Map<String, String> formProperties = new HashMap<String, String>();
		List<JSONObject> array = JSON.parseArray(formJson, JSONObject.class);
		for(JSONObject obj : array) {
			String id = obj.getString("id");
			String value = obj.getString("value");
			formProperties.put(id, value);
		}
		

        logger.debug("start form parameters: {}", formProperties);
        ProcessInstance processInstance = null;
        try {
            //设置提交人的名字
			identityService.setAuthenticatedUserId(userInfo.getUserId().toString());
//            formService.saveFormData(processDefinitionId, formProperties);
            processInstance = formService.submitStartFormData(processDefinitionId, formProperties);
//            formService.submitStartFormData(processDefinition.getId(), formProperties);
        } finally {
            identityService.setAuthenticatedUserId(userInfo.getUserId().toString());
        }

        String processType = null;
		//        redirectAttributes.addFlashAttribute("message", "任务完成：taskId=" + taskId);
        return processType;
    }
}
