package com.chinaitop.depot.act.controller;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.stream.XMLStreamException;

import org.activiti.engine.repository.ProcessDefinition;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.chinaitop.depot.act.model.DeploymentList;
import com.chinaitop.depot.act.service.ActProcessService;
import com.chinaitop.depot.act.service.DeleteService;
import com.chinaitop.depot.utils.activiti.PageUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

/**
 * 流程定义相关Controller
 * @author xujh
 * @version 2017-10-10
 */
@RestController
@RequestMapping(value = "/act/process")
public class ActProcessController{

	@Resource
	private ActProcessService actProcessService;
	@Resource
	private DeleteService deleteService;

	/**
	 * 流程定义列表
	 * @param pageNum 页数
	 * @param pageSize 每页多少条数
	 * @return
	 */
	@RequestMapping(value="/processList")
	public PageInfo<DeploymentList> processList(Integer pageNum, Integer pageSize, HttpServletRequest request) {
		PageHelper.startPage(pageNum, pageSize);
		List<ProcessDefinition> processList = actProcessService.processList(request);
		List<DeploymentList> deploymentlist=new ArrayList<DeploymentList>();
		if(processList!=null&&processList.size()>0){
			for(ProcessDefinition m: processList){
				DeploymentList deploymentLists=new DeploymentList();
				deploymentLists.setId(m.getId());
				deploymentLists.setName(m.getName());
				deploymentLists.setKey(m.getKey());
				deploymentLists.setVersion(m.getVersion());
				deploymentLists.setResourceName(m.getResourceName());
				deploymentLists.setDiagramResourceName(m.getDiagramResourceName());
				deploymentLists.setDeploymentId(m.getDeploymentId());
				deploymentlist.add(deploymentLists);
			}
			
		}
		PageInfo<DeploymentList> pageInfo = PageUtils.page2(deploymentlist, pageNum, pageSize);
//		PageInfo<DeploymentList> pageInfo = new PageInfo<DeploymentList>(deploymentlist);
		return pageInfo;
	}
	
	/**
	 * 运行中的实例列表
	 */
	@RequestMapping(value = "running")
	public String runningList(String procInsId, String procDefKey, HttpServletRequest request, HttpServletResponse response, Model model) {
	    /*Page<ProcessInstance> page = actProcessService.runningList(new Page<ProcessInstance>(request, response), procInsId, procDefKey);
		model.addAttribute("page", page);
		model.addAttribute("procInsId", procInsId);
		model.addAttribute("procDefKey", procDefKey);*/
		return "modules/act/actProcessRunningList";
	}

	/**
	 * 部署流程
	 */
	@RequestMapping(value = "/deploy", method=RequestMethod.GET)
	public String deploy(Model model) {
		return "modules/act/actProcessDeploy";
	}
	
	/**
	 * 部署流程 - 保存
	 */
	@RequestMapping(value = "/deploy", method=RequestMethod.POST)
	public String deploy(@Value("#{APP_PROP['activiti.export.diagram.path']}") String exportDir, 
			String category, MultipartFile file, RedirectAttributes redirectAttributes) {

		String fileName = file.getOriginalFilename();
		
		if (fileName==null){
			redirectAttributes.addFlashAttribute("message", "请选择要部署的流程文件");
		}else{
			String message = actProcessService.deploy(exportDir, category, file);
			redirectAttributes.addFlashAttribute("message", message);
		}

//		return "redirect:" + adminPath + "/act/process";
		return null;
	}
	
	/**
	 * 设置流程分类
	 */
	@RequestMapping(value = "updateCategory")
	public String updateCategory(String procDefId, String category, RedirectAttributes redirectAttributes) {
		actProcessService.updateCategory(procDefId, category);
//		return "redirect:" + adminPath + "/act/process";
		return null;
	}

	/**
	 * 挂起、激活流程实例
	 */
	@RequestMapping(value = "update/{state}")
	public String updateState(@PathVariable("state") String state, String procDefId, RedirectAttributes redirectAttributes) {
		String message = actProcessService.updateState(state, procDefId);
		redirectAttributes.addFlashAttribute("message", message);
//		return "redirect:" + adminPath + "/act/process";
		return null;
	}
	
	/**
	 * 将部署的流程转换为模型
	 */
	@RequestMapping(value = "convert/toModel")
	public String convertToModel(String procDefId, RedirectAttributes redirectAttributes) throws UnsupportedEncodingException, XMLStreamException {
		org.activiti.engine.repository.Model modelData = actProcessService.convertToModel(procDefId);
		redirectAttributes.addFlashAttribute("message", "转换模型成功，模型ID="+modelData.getId());
//		return "redirect:" + adminPath + "/act/model";
		return null;
	}
	
	/**
	 * 导出图片文件到硬盘
	 */
	@RequestMapping(value = "export/diagrams")
	@ResponseBody
	public List<String> exportDiagrams(@Value("#{APP_PROP['activiti.export.diagram.path']}") String exportDir) throws IOException {
		List<String> files = actProcessService.exportDiagrams(exportDir);
		return files;
	}

	/**
	 * 删除部署的流程，级联删除流程实例
	 * @param deploymentId 流程部署ID
	 * @param redirectAttributes 返回信息
	 */
	@RequestMapping(value = "delete")
	public String delete(String deploymentId,RedirectAttributes redirectAttributes) {
		deleteService.deleteDeployment(deploymentId);
		redirectAttributes.addFlashAttribute("message", "删除成功，模型ID=" + deploymentId);
		return null;
	}
	
	/**
	 * 删除流程实例
	 * @param procInsId 流程实例ID
	 * @param reason 删除原因
	 * @param redirectAttributes 返回信息
	 */
	@RequestMapping(value = "deleteProcIns")
	public String deleteProcIns(String procInsId, String reason, RedirectAttributes redirectAttributes) {
		if (reason==null){
			addMessage(redirectAttributes, "请填写删除原因");
		}else{
			actProcessService.deleteProcIns(procInsId, reason);
			addMessage(redirectAttributes, "删除流程实例成功，实例ID=" + procInsId);
		}
//		return "redirect:" + adminPath + "/act/process/running/";
		return null;
	}
	
	/**
	 * 添加Flash消息
	 */
	protected void addMessage(RedirectAttributes redirectAttributes, String... messages) {
		StringBuilder sb = new StringBuilder();
		for (String message : messages){
			sb.append(message).append(messages.length>1?"<br/>":"");
		}
		redirectAttributes.addFlashAttribute("message", sb.toString());
	}
	
	/**
	 * 查看流程图
	 * @throws IOException 
	 * @throws Exception 
	 * @param deploymentId 流程定义的id
	 * @param imageName 定义流程图的名字
	 */
	@RequestMapping(value = "viewImage")
	public String viewImage(String deploymentId,String imageName,HttpServletResponse response) throws IOException{
		InputStream in  = deleteService.findImageInputStream(deploymentId, imageName);
		//3：从response对象获取输出流
		OutputStream out = response.getOutputStream();
//		OutputStream out = ServletActionContext.getResponse().getOutputStream();
		//4：将输入流中的数据读取出来，写到输出流中
		for(int b=-1;(b=in.read())!=-1;){
			out.write(b);
		}
		out.close();
		in.close();
		return null;
	}
	
	@RequestMapping(value = "resource/read")
	public void resourceRead(String procDefId, String proInsId, String resType, HttpServletResponse response) throws Exception {
		resType="xml";
		InputStream resourceAsStream = actProcessService.resourceRead(procDefId, proInsId, resType);
		byte[] b = new byte[1024];
		int len = -1;
		while ((len = resourceAsStream.read(b, 0, 1024)) != -1) {
			response.getOutputStream().write(b, 0, len);
		}
	}
	
}
