package com.chinaitop.depot.act.controller;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.springframework.http.MediaType;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import com.chinaitop.depot.act.model.Act;
import com.chinaitop.depot.act.model.DeploymentList;
import com.chinaitop.depot.act.model.HistoryList;
import com.chinaitop.depot.act.service.ActTaskService;
import com.chinaitop.depot.business.model.BusinessPlan;
import com.chinaitop.depot.business.model.enums.TaskType;
import com.chinaitop.depot.business.service.BusinessPlanService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;


/**
 * 流程个人任务相关Controller
 * @author xujh
 * @version 2017-10-10
 */
@RestController
@RequestMapping(value = "/act/task")
public class ActTaskController {

	@Resource
	private ActTaskService actTaskService;
	@Resource
	private BusinessPlanService planService;
	
	/**
	 * 获取待办列表
	 * @param name 名字
	 * @param act 实体类
	 * @param pageNum 页数
	 * @param pageSize 每页显示多少条
	 * @param taskType  任务类型
	 * @return
	 */
	@RequestMapping(value="/todoList", produces = MediaType.APPLICATION_JSON_VALUE)
	public PageInfo<DeploymentList> todoList(String name,Act act,HttpServletResponse response, Integer pageNum, Integer pageSize,TaskType taskType) {
		PageHelper.startPage(pageNum, pageSize);
		name="1";
		List<DeploymentList> todoList = actTaskService.todoList(name,taskType);
		
		PageInfo<DeploymentList> pageInfo = new PageInfo<DeploymentList>(todoList);
		return pageInfo;
	}
	
	
	/**
	 * 获取已办任务
	 * @param name名字
	 * @param taskType 任务类型
	 * @param pageNum 页数
	 * @param pageSize 每页显示多少条
	 * @return
	 */
	@RequestMapping(value = "historic")
	public PageInfo<DeploymentList> historicList(String name,String taskType, Integer pageNum, Integer pageSize) throws Exception {
		PageHelper.startPage(pageNum, pageSize);
		List<DeploymentList> historicList = actTaskService.historicList(name, taskType);
		
		PageInfo<DeploymentList> pageInfo = new PageInfo<DeploymentList>(historicList);
		return pageInfo;
	}

	
	/**
	 * 获取流程表单
	 * @param act	流程实体类
	 * @param request	
	 * @param model 
	 * @param startAct 开始
	 * @param endAct 结束
	 */
	@RequestMapping(value = "form")
	public Map<String, Object> form(Act act, HttpServletRequest request, Model model, String startAct, String endAct){
		
		// 获取流程XML上的表单KEY
		String formKey = actTaskService.getFormKey(act.getProcDefId(), act.getTaskDefKey());

		// 获取流程实例对象
		if (act.getProcInsId() != null){
			act.setProcIns(actTaskService.getProcIns(act.getProcInsId()));
		}
		Map<String, Object> modelMap = new HashMap<String, Object>();
		modelMap.put("url", formKey);
		modelMap.put("taskId", act.getTaskId());
		modelMap.put("taskName", act.getTaskName());
		modelMap.put("taskDefKey", act.getTaskDefKey());
		modelMap.put("procInsId", act.getProcInsId());
		modelMap.put("procDefId", act.getProcDefId());
		modelMap.put("status", act.getStatus());
		modelMap.put("id", act.getBusinessId());
		//得到流程对应的业务id
		String businessId = act.getBusinessId();
		Integer id = Integer.valueOf(businessId);
		BusinessPlan plan = planService.findById(id);
		//根据procInsId查找历史审批意见
		String procInsId = act.getProcInsId();
		
		List<HistoryList> histoicFlowList = actTaskService.histoicFlowList(procInsId, startAct, endAct);
		
		List<HistoryList> historyList=new ArrayList<HistoryList>();
		
		if(histoicFlowList!=null&&histoicFlowList.size()>0){
			for(HistoryList m: histoicFlowList){
				HistoryList historyLists=new HistoryList();
				historyLists.setAssigneeName(m.getAssigneeName());
				historyLists.setComment(m.getComment());
				historyList.add(historyLists);
			}
		}
		modelMap.put("plan", plan);
		modelMap.put("histoicFlowList", historyList);
		return modelMap;
//		return "redirect:" + ActUtils.getFormUrl(formKey, act);
	}
	

	/**
	 * 完成任务
	 * @param act 流程定义的实体类
	 * @param procInsId 流程实例ID
	 * @param comment 任务提交意见的内容
	 */
	@RequestMapping(value = "complete")
	@ResponseBody
	public String complete(Act act,String taskId,String procInsId,String comment) {
//		actTaskService.complete(act.getTaskId(), act.getProcInsId(), act.getComment(), act.getVars().getVariableMap());
		return "true";//adminPath + "/act/task";
	}
	

	/**
	 * 获取历史审批意见
	 * @param procInsId	流程实例ID
	 * @param startAct	开始
	 * @param endAct 结束
	 * @param pageNum 页数
	 * @param pageSize 每页显示多少条
	 */
	@RequestMapping(value = "/historyList")
	public Map<String, Object> historyList(String procInsId,String startAct, String endAct,Integer pageNum, Integer pageSize){
		List<HistoryList> histoicFlowList = actTaskService.histoicFlowList(procInsId, startAct, endAct);
		Map<String, Object> modelMap = new HashMap<String, Object>();
		List<HistoryList> historyList=new ArrayList<HistoryList>();
		if(histoicFlowList!=null&&histoicFlowList.size()>0){
			for(HistoryList m: histoicFlowList){
				HistoryList historyLists=new HistoryList();
				historyLists.setAssigneeName(m.getAssigneeName());
				historyLists.setComment(m.getComment());
				historyList.add(historyLists);
			}
		}
		modelMap.put("histoicFlowList", historyList);
		return modelMap;
	}
	
	@RequestMapping(value = "/submintAgent")
	public Map<String, Object> SubmintAgent(String processInstanceId,String userId){
		Map<String, Object> modelMap = new HashMap<String, Object>();
		String mgs = actTaskService.setAssigneeTask(processInstanceId, userId);
		modelMap.put("mgs", mgs);
		return modelMap;
	}
}
