package com.chinaitop.depot.act.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import org.activiti.engine.FormService;
import org.activiti.engine.HistoryService;
import org.activiti.engine.IdentityService;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.history.HistoricActivityInstance;
import org.activiti.engine.history.HistoricProcessInstance;
import org.activiti.engine.history.HistoricTaskInstance;
import org.activiti.engine.history.HistoricTaskInstanceQuery;
import org.activiti.engine.impl.RepositoryServiceImpl;
import org.activiti.engine.impl.bpmn.behavior.UserTaskActivityBehavior;
import org.activiti.engine.impl.persistence.entity.CommentEntity;
import org.activiti.engine.impl.persistence.entity.ExecutionEntity;
import org.activiti.engine.impl.persistence.entity.ProcessDefinitionEntity;
import org.activiti.engine.impl.pvm.PvmActivity;
import org.activiti.engine.impl.pvm.PvmTransition;
import org.activiti.engine.impl.pvm.process.ActivityImpl;
import org.activiti.engine.impl.task.TaskDefinition;
import org.activiti.engine.repository.ProcessDefinitionQuery;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.task.Comment;
import org.activiti.engine.task.Task;
import org.activiti.engine.task.TaskQuery;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.act.model.Act;
import com.chinaitop.depot.act.model.DeploymentList;
import com.chinaitop.depot.act.model.HistoryList;
import com.chinaitop.depot.business.model.enums.TaskType;
import com.chinaitop.depot.business.mapper.UserInfoMapper;
import com.chinaitop.depot.business.mapper.UserRoleMapper;
import com.chinaitop.depot.business.model.OrgInfo;
import com.chinaitop.depot.business.model.UserInfo;
import com.chinaitop.depot.business.model.UserInfoExample;
import com.chinaitop.depot.business.model.UserRole;
import com.chinaitop.depot.business.model.UserRoleExample;
import com.chinaitop.depot.business.model.BusinessPlanExample.Criteria;
import com.chinaitop.depot.business.service.OrgInfoService;
import com.chinaitop.depot.business.service.UserInfoService;
import com.chinaitop.depot.utils.activiti.StringUtils;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

/**
 * 流程定义相关Service
 * @author xujh
 * @version 2017-10-11
 */
@Service
public class ActTaskService {
	
	/**
	 * 日志对象
	 */
	protected Logger logger = LoggerFactory.getLogger(getClass());

	@Autowired
	private RuntimeService runtimeService;
	@Autowired
	private TaskService taskService;
	@Autowired
	private FormService formService;
	@Autowired
	private HistoryService historyService;
	@Autowired
	private RepositoryService repositoryService;
	@Autowired
	private IdentityService identityService;
	@Resource
	private UserRoleMapper userRoleMapper;
	@Resource
	private UserInfoMapper userInfoMapper;
	@Resource
	private UserInfoService userInfoService;
	@Resource
	private OrgInfoService orgInfoService;
	
	/**
	 * 获取待办列表
	 * @param procDefKey 流程定义标识
	 * @return
	 */
	public List<DeploymentList> todoList(String name,TaskType taskType){
		List<Act> result = new ArrayList<Act>();
		
		// =============== 已经签收的任务  ===============
		TaskQuery todoTaskQuery = taskService.createTaskQuery().taskAssignee(name).active()
				.includeProcessVariables().orderByTaskCreateTime().desc();
		
		// 设置查询条件
		if(taskType != null) {
			todoTaskQuery.processDefinitionKey(taskType.getTaskType());
		}
		
		// 查询列表
		List<Task> todoList = todoTaskQuery.list();
		for (Task task : todoList) {
			Act e = new Act();
			e.setTask(task);
			e.setVars(task.getProcessVariables());
			e.setStatus("todo");
			result.add(e);
		}
		
		// =============== 等待签收的任务  ===============
		TaskQuery toClaimQuery = taskService.createTaskQuery().taskCandidateUser(name)
				.includeProcessVariables().active().orderByTaskCreateTime().desc();
		
		// 查询列表
		List<Task> toClaimList = toClaimQuery.list();
		for (Task task : toClaimList) {
			Act e = new Act();
			e.setTask(task);
			e.setVars(task.getProcessVariables());
			e.setStatus("claim");
			result.add(e);
		}
		
		List<DeploymentList> deploymentlist=new ArrayList<DeploymentList>();
		if(result!=null&&result.size()>0){
			for(Act m: result){
				DeploymentList deploymentLists=new DeploymentList();
				//第一个参数
				deploymentLists.setTaskId(m.getTaskId());
				//第二个参数
				deploymentLists.setTaskName(m.getTaskName());
				//第三个参数
				deploymentLists.setTaskDefKey(m.getTaskDefKey());
				//第四个参数
				deploymentLists.setProcInsId(m.getProcInsId());
				//第五个参数
				deploymentLists.setProcDefId(m.getProcDefId());
				//第六个参数
				deploymentLists.setStatus(m.getStatus());
				deploymentLists.setName(m.getProcDefId());
				deploymentLists.setResourceName(m.getAssignee());
				deploymentlist.add(deploymentLists);
			}
			
		}
		
		
		
		return deploymentlist;
	}

	
	/**
	 * 获取已办任务
	 * @param page
	 * @param procDefKey 流程定义标识
	 * @return
	 */
	public List<DeploymentList> historicList(String name,String taskType){
//		String userId = "";//ObjectUtils.toString(UserUtils.getUser().getId());

		HistoricTaskInstanceQuery histTaskQuery = historyService.createHistoricTaskInstanceQuery().taskAssignee(name).finished()
				.includeProcessVariables().orderByHistoricTaskInstanceEndTime().desc();
		
		// 设置查询条件
		if (StringUtils.isNotBlank(taskType)){
			histTaskQuery.processDefinitionKey(taskType);
		}
		/*if (act.getBeginDate() != null){
			histTaskQuery.taskCompletedAfter(act.getBeginDate());
		}
		if (act.getEndDate() != null){
			histTaskQuery.taskCompletedBefore(act.getEndDate());
		}*/
		
		// 查询列表
		List<HistoricTaskInstance> list = histTaskQuery.list();
		//处理分页问题
		List<Act> actList=Lists.newArrayList();
		for (HistoricTaskInstance histTask : list) {
			Act e = new Act();
			e.setHistTask(histTask);
			e.setVars(histTask.getProcessVariables());
			e.setStatus("finish");
			actList.add(e);
		}
		
		List<DeploymentList> deploymentlist=new ArrayList<DeploymentList>();
		if(actList!=null&&actList.size()>0){
			for(Act m: actList){
				DeploymentList deploymentLists=new DeploymentList();
				//第一个参数
				deploymentLists.setTaskId(m.getTaskId());
				//第二个参数
				deploymentLists.setTaskName(m.getTaskName());
				//第三个参数
				deploymentLists.setTaskDefKey(m.getTaskDefKey());
				//第四个参数
				deploymentLists.setProcInsId(m.getProcInsId());
				//第五个参数
				deploymentLists.setProcDefId(m.getProcDefId());
				//第六个参数
				deploymentLists.setStatus(m.getStatus());
				deploymentLists.setName(m.getProcDefId());
				deploymentLists.setResourceName(m.getAssignee());
				deploymentlist.add(deploymentLists);
			}
			
		}
		
		return deploymentlist;
	}
	
	/**
	 * 获取流转历史列表
	 * @param procInsId 流程实例
	 * @param startAct 开始活动节点名称
	 * @param endAct 结束活动节点名称
	 */
	@SuppressWarnings("unused")
	public List<HistoryList> histoicFlowList(String procInsId, String startAct, String endAct){
		List<Act> actList = Lists.newArrayList();
		List<HistoricActivityInstance> list = historyService.createHistoricActivityInstanceQuery().processInstanceId(procInsId)
				.orderByHistoricActivityInstanceStartTime().asc().orderByHistoricActivityInstanceEndTime().asc().list();
		
		boolean start = false;
		Map<String, Integer> actMap = Maps.newHashMap();
		
		for (int i=0; i<list.size(); i++){
			
			HistoricActivityInstance histIns = list.get(i);
			
			// 过滤开始节点前的节点
			if (StringUtils.isNotBlank(startAct) && startAct.equals(histIns.getActivityId())){
				start = true;
			}
			if (StringUtils.isNotBlank(startAct) && !start){
				continue;
			}
			
			// 只显示开始节点和结束节点，并且执行人不为空的任务
			if (StringUtils.isNotBlank(histIns.getAssignee())
					 || "startEvent".equals(histIns.getActivityType())
					 || "endEvent".equals(histIns.getActivityType())){
				
				// 给节点增加一个序号
				Integer actNum = actMap.get(histIns.getActivityId());
				if (actNum == null){
					actMap.put(histIns.getActivityId(), actMap.size());
				}
				
				Act e = new Act();
				e.setHistIns(histIns);
				// 获取流程发起人名称
				if ("startEvent".equals(histIns.getActivityType())){
					List<HistoricProcessInstance> il = historyService.createHistoricProcessInstanceQuery().processInstanceId(procInsId).orderByProcessInstanceStartTime().asc().list();
//					List<HistoricIdentityLink> il = historyService.getHistoricIdentityLinksForProcessInstance(procInsId);
					if (il.size() > 0){
						if (StringUtils.isNotBlank(il.get(0).getStartUserId())){
//							User user = UserUtils.getByLoginName(il.get(0).getStartUserId());
							UserInfo user = null;
							if (user != null){
								e.setAssignee(histIns.getAssignee());
								e.setAssigneeName(user.getUsername());
							}
						}
					}
				}
				// 获取任务执行人名称
				if (StringUtils.isNotEmpty(histIns.getAssignee())){
//					User user = UserUtils.getByLoginName(histIns.getAssignee());
					UserInfo user = null;
					if (user != null){
						e.setAssignee(histIns.getAssignee());
						e.setAssigneeName(user.getUsername());
					}
				}
				// 获取意见评论内容
				if (StringUtils.isNotBlank(histIns.getTaskId())){
					List<Comment> commentList = taskService.getTaskComments(histIns.getTaskId());
					/*if (commentList.size()>0){
						e.setComment(commentList.get(0).getFullMessage());
					}*/
					if (commentList.size()>0){
						//e.setComment(commentList.get(0).getFullMessage());
						CommentEntity commentEntity = (CommentEntity)commentList.get(0);
						e.setComment(commentEntity.getMessage());
						//System.out.println(commentEntity.getFullMessage()+"--------"+commentEntity.getMessage());
						 
					}
				}
				actList.add(e);
			}
			
			// 过滤结束节点后的节点
			if (StringUtils.isNotBlank(endAct) && endAct.equals(histIns.getActivityId())){
				boolean bl = false;
				Integer actNum = actMap.get(histIns.getActivityId());
				// 该活动节点，后续节点是否在结束节点之前，在后续节点中是否存在
				for (int j=i+1; j<list.size(); j++){
					HistoricActivityInstance hi = list.get(j);
					Integer actNumA = actMap.get(hi.getActivityId());
					if ((actNumA != null && actNumA < actNum) || StringUtils.equals(hi.getActivityId(), histIns.getActivityId())){
						bl = true;
					}
				}
				if (!bl){
					break;
				}
			}
		}
		
		List<HistoryList> historyList1=new ArrayList<HistoryList>();
		for (int j = 0; j < actList.size(); j++) {
			if(j != 0&&j!=actList.size()-1){
				HistoryList historyList=new HistoryList();
				historyList.setAssigneeName(actList.get(j).getHistIns().getAssignee());//审批人名字
				
				Integer userid = Integer.valueOf(actList.get(j).getHistIns().getAssignee());
				if (null != userid) {
					UserInfo userInfo = userInfoService.findById(userid);
					if (null != userInfo) {
						Integer orgId = userInfo.getOrgId();
						OrgInfo orgInfo=orgInfoService.findById(orgId);
						String departmentName=orgInfo.getOrgName();
						historyList.setDepartment(departmentName);
					}
				}

				historyList.setComment(actList.get(j).getComment());//审批意见
				historyList.setStartTime(actList.get(j).getHistIns().getStartTime());//开始时间
				historyList.setEndDate(actList.get(j).getHistIns().getEndTime());//结束时间
				historyList.setActivityName(actList.get(j).getHistIns().getActivityName());//审批人岗位
				historyList1.add(historyList);
			}
		}
		return historyList1;
	}

	/**
	 * 获取流程列表
	 * @param category 流程分类
	 */
	public Object[] processList(String category) {
		/*
		 * 保存两个对象，一个是ProcessDefinition（流程定义），一个是Deployment（流程部署）
		 */
	    ProcessDefinitionQuery processDefinitionQuery = repositoryService.createProcessDefinitionQuery()
	    		.latestVersion().active().orderByProcessDefinitionKey().asc();
	    
	    if (StringUtils.isNotEmpty(category)){
	    	processDefinitionQuery.processDefinitionCategory(category);
		}
	    
		return null;
	}
	
	/**
	 * 获取流程表单（首先获取任务节点表单KEY，如果没有则取流程开始节点表单KEY）
	 * @return
	 */
	public String getFormKey(String procDefId, String taskDefKey){
		String formKey = "";
		if (StringUtils.isNotBlank(procDefId)){
			if (StringUtils.isNotBlank(taskDefKey)){
				try{
					formKey = formService.getTaskFormKey(procDefId, taskDefKey);
				}catch (Exception e) {
					formKey = "";
				}
			}
			if (StringUtils.isBlank(formKey)){
				formKey = formService.getStartFormKey(procDefId);
			}
			if (StringUtils.isBlank(formKey)){
				formKey = "/404";
			}
		}
		logger.debug("getFormKey: {}", formKey);
		return formKey;
	}
	
	/**
	 * 获取流程实例对象
	 * @param procInsId
	 * @return
	 */
	@Transactional(readOnly = false)
	public ProcessInstance getProcIns(String procInsId) {
		return runtimeService.createProcessInstanceQuery().processInstanceId(procInsId).singleResult();
	}

	

	/**
	 * 获取任务
	 * @param taskId 任务ID
	 */
	public Task getTask(String taskId){
		return taskService.createTaskQuery().taskId(taskId).singleResult();
	}
	
	/**
	 * 删除任务
	 * @param taskId 任务ID
	 * @param deleteReason 删除原因
	 */
	@Transactional(readOnly = false)
	public void deleteTask(String taskId, String deleteReason){
		taskService.deleteTask(taskId, deleteReason);
	}
	
	
	/**
	 * 提交任务, 并保存意见
	 * @param taskId 任务ID
	 * @param procInsId 流程实例ID，如果为空，则不保存任务提交意见
	 * @param comment 任务提交意见的内容
	 * @param title			流程标题，显示在待办任务标题
	 * @param vars 任务变量
	 */
	@SuppressWarnings("unused")
	@Transactional(readOnly = false)
	public void complete(String taskId, String procInsId, String comment, String title, Map<String, Object> vars){
		String pass = (String) vars.get("pass");
		if(pass.equals("0")&&StringUtils.isNotBlank(procInsId)){
			// 添加意见
			String advice="退回";
			advice+=comment;
		
			taskService.addComment(taskId, procInsId, advice);
			
		}else if (pass.equals("1")&&StringUtils.isNotBlank(procInsId)) {
			// 添加意见
			String advice="同意";
			advice+=comment;
			taskService.addComment(taskId, procInsId, advice);
			
		}else if (pass.equals("2")&&StringUtils.isNotBlank(procInsId)) {
			// 添加意见
			String advice="驳回";
			advice+=comment;
			taskService.addComment(taskId, procInsId, advice);
		}else if (pass.equals("3")&&StringUtils.isNotBlank(procInsId)) {
			// 添加意见
			String advice="拒绝";
			advice+=comment;
			taskService.addComment(taskId, procInsId, advice);
		}
		
		// 设置流程变量
		if (vars == null){
			vars = Maps.newHashMap();
		}
		
		// 设置流程标题
		if (StringUtils.isNotBlank(title)){
			vars.put("title", title);
		}
		
		// 提交任务
		taskService.complete(taskId, vars);
	}
	
	/**
	 * 添加任务意见
	 */
	public void addTaskComment(String taskId, String procInsId, String comment){
		taskService.addComment(taskId, procInsId, comment);
	}
	
	
	/**
	 * 设置当前处理人信息
	 * @param vars
	 * @param currentTask
	 */
	@SuppressWarnings("unused")
	private void setCurrentTaskAssignee(Map<String, Object> vars, Task currentTask) {
		String assignee = currentTask.getAssignee();
		if (assignee != null) {
			org.activiti.engine.identity.User assigneeUser = identityService.createUserQuery().userId(assignee).singleResult();
			String userInfo = assigneeUser.getFirstName() + " " + assigneeUser.getLastName();
			vars.put("当前处理人", userInfo);
		}
	}
	
	/**
	 * 设置当前处理人信息
	 * @param type 启动流程的类型
	 * @param id 业务id
	 * @param name启动人的名字
	 */
	public ProcessInstance startProcess(String type,String id,String name,String userId,String deportId) {
		// 启动流程
        Map<String, Object> variables = new HashMap<String, Object>();
        // TODO 用来设置启动流程的人员ID
        identityService.setAuthenticatedUserId(name);
        String businessKey = id.toString();
        variables.put("type", type);
        variables.put("busId", businessKey);
		variables.put("inputUser", userId);
        //新增拼接businessKey作为业务数据与流程绑定
        String objId =type + ":" + businessKey;
        // 启动流程
        ProcessInstance processInstance = runtimeService.startProcessInstanceByKey(type, objId, variables);
        
//        ProcessInstance processInstance = runtimeService.startProcessInstanceByKey(type+deportId, objId, variables);
        
		return processInstance;
		
	}
	
	public String startProcessNew(String type,String id,String name,String userId,String deportId){
		ProcessInstance startProcess = this.startProcess(type, id, name, userId, deportId);
		String processInstanceId = startProcess.getProcessInstanceId();
		// 设置流程定义id.
        String processDefinitionId = startProcess.getProcessDefinitionId();
        JSONObject json=new JSONObject();
        json.put("processInstanceId", processInstanceId);
        json.put("processDefinitionId",processDefinitionId);
		return json.toString();
	}
	
	/** 
     * 根据实例编号查找下一个任务节点 
     * @param String procInstId ：实例编号 
     * @return 
     */  
    public TaskDefinition nextTaskDefinition(String procInstId){  
        //流程标示  
        String processDefinitionId = historyService.createHistoricProcessInstanceQuery().processInstanceId(procInstId).singleResult().getProcessDefinitionId();  
          
        ProcessDefinitionEntity def = (ProcessDefinitionEntity) ((RepositoryServiceImpl)repositoryService).getDeployedProcessDefinition(processDefinitionId);  
        //执行实例  
        ExecutionEntity execution = (ExecutionEntity) runtimeService.createProcessInstanceQuery().processInstanceId(procInstId).singleResult();  
        //当前实例的执行到哪个节点  
        String activitiId = execution.getActivityId();  
        //获得当前任务的所有节点  
        List<ActivityImpl> activitiList = def.getActivities();  
        String id = null;  
        for(ActivityImpl activityImpl:activitiList){    
            id = activityImpl.getId();   
            if(activitiId.equals(id)){  
                System.out.println("当前任务："+activityImpl.getProperty("name"));  
                return nextTaskDefinition(activityImpl, activityImpl.getId(),"${pass==1}");  
//              System.out.println(taskDefinition.getCandidateGroupIdExpressions().toArray()[0]);  
//              return taskDefinition;  
            }  
        }  
        return null;  
    }  
      
    /** 
     * 下一个任务节点 
     * @param activityImpl 
     * @param activityId 
     * @param elString 
     * @return 
     */  
    private TaskDefinition nextTaskDefinition(ActivityImpl activityImpl, String activityId, String elString){  
            if("userTask".equals(activityImpl.getProperty("type")) && !activityId.equals(activityImpl.getId())){  
                TaskDefinition taskDefinition = ((UserTaskActivityBehavior)activityImpl.getActivityBehavior()).getTaskDefinition();  
//              taskDefinition.getCandidateGroupIdExpressions().toArray();  
                return taskDefinition;  
            }else{  
                List<PvmTransition> outTransitions = activityImpl.getOutgoingTransitions();  
                List<PvmTransition> outTransitionsTemp = null;  
                for(PvmTransition tr:outTransitions){    
                    PvmActivity ac = tr.getDestination(); //获取线路的终点节点    
                    if("exclusiveGateway".equals(ac.getProperty("type"))){  
                        outTransitionsTemp = ac.getOutgoingTransitions();  
                        if(outTransitionsTemp.size() == 1){  
                            return nextTaskDefinition((ActivityImpl)outTransitionsTemp.get(0).getDestination(), activityId, elString);  
                        }else if(outTransitionsTemp.size() > 1){  
                            for(PvmTransition tr1 : outTransitionsTemp){  
                                Object s = tr1.getProperty("conditionText");  
                                String string = s.toString();
                                if(elString.equals(string)){  
                                    return nextTaskDefinition((ActivityImpl)tr1.getDestination(), activityId, elString);  
                                }  
                            }  
                        }  
                    }else if("userTask".equals(ac.getProperty("type"))){  
                        return ((UserTaskActivityBehavior)((ActivityImpl)ac).getActivityBehavior()).getTaskDefinition();  
                    }else{  
//                        logger.debug(ac.getProperty("type"));  
                    }  
                }   
            return null;  
        }  
    }
    
    public List<UserInfo> getUserByRoleId(Integer roleId) {
    	UserRoleExample userRoleExample = new UserRoleExample();
    	userRoleExample.createCriteria().andRoleIdEqualTo(roleId);
    	List<UserRole> urlist = userRoleMapper.selectByExample(userRoleExample);
    	List<UserInfo> userlist = new ArrayList<UserInfo>();
    	if (urlist.size() > 0) {
        	List<Integer> userIds = new ArrayList<Integer>();
        	for (UserRole userRole : urlist) {
    			userIds.add(userRole.getUserId());
    		}
        	UserInfoExample userExample = new UserInfoExample();
        	userExample.createCriteria().andUserIdIn(userIds);
        	userlist = userInfoMapper.selectByExample(userExample);
    	}
    	return userlist;
    };
    
    
    public List<UserInfo> getUserByRoleId(Integer roleId, Integer orgId) {
    	UserRoleExample userRoleExample = new UserRoleExample();
    	userRoleExample.createCriteria().andRoleIdEqualTo(roleId);
    	List<UserRole> urlist = userRoleMapper.selectByExample(userRoleExample);
    	List<UserInfo> userlist = new ArrayList<UserInfo>();
    	if (urlist.size() > 0) {
    		List<Integer> userIds = new ArrayList<Integer>();
    		for (UserRole userRole : urlist) {
    			userIds.add(userRole.getUserId());
    		}
    		UserInfoExample userExample = new UserInfoExample();
    		UserInfoExample.Criteria criteria = userExample.createCriteria();
    		criteria.andUserIdIn(userIds);
    		criteria.andOrgIdEqualTo(orgId);
    		userlist = userInfoMapper.selectByExample(userExample);
    	}
    	return userlist;
    };
    
    
    //可以分配个人任务从一个人到另一个人（认领任务:）
    //任务ID
    //指定的办理人
	public String setAssigneeTask(String processInstanceId,String userId){
		Task singleResult = selectTask(processInstanceId);
		String taskId = singleResult.getId();
		taskService.setAssignee(taskId, userId);
		String mgs="设置成功";
		return mgs;
	}
	
	//使用流程实例id 查询当前任务信息
	public Task selectTask(String processInstanceId){
		//使用流程实例id 查询当前任务信息
		Task singleResult = taskService.createTaskQuery().processInstanceId(processInstanceId).singleResult();
		return singleResult;
	}
	//使用流程实例id 查询当前任务id
	public String selectTaskId(String processInstanceId){
		Task selectTask = this.selectTask(processInstanceId);
		return selectTask.getId();
	}
	
	/**查询历史任务(获取上一个人的名字)*/
	public String findHistoryTaskName(String processInstanceId){
		String assignee = null;
		@SuppressWarnings("deprecation")
		List<HistoricTaskInstance> list = historyService//与历史数据（历史表）相关的Service
						.createHistoricTaskInstanceQuery()//创建历史任务实例查询
						.processInstanceId(processInstanceId)//
						.orderByHistoricTaskInstanceStartTime().asc()
						.list();
		if(list!=null && list.size()>0){
			int length = list.size()-1;
			for (int i = 0; i < list.size(); i++) {
				if(i==length){
					assignee = list.get(length).getAssignee();
				}
			}
		}
		return assignee;
	}
	/**查询历史任务(获取第一个人的名字)*/
	public String findHistoryFirstTaskName(String processInstanceId){
		String assignee = null;
		@SuppressWarnings("deprecation")
		List<HistoricTaskInstance> list = historyService//与历史数据（历史表）相关的Service
						.createHistoricTaskInstanceQuery()//创建历史任务实例查询
						.processInstanceId(processInstanceId)//
						.orderByHistoricTaskInstanceStartTime().asc()
						.list();
		if(list!=null && list.size()>0){
			assignee=list.get(0).getAssignee();
		}
		return assignee;
	}
	/**查询流程状态（判断流程正在执行，还是结束）*/
	public ProcessInstance isProcessEnd(String processInstanceId){
		ProcessInstance pi = runtimeService//表示正在执行的流程实例和执行对象
						.createProcessInstanceQuery()//创建流程实例查询
						.processInstanceId(processInstanceId)//使用流程实例ID查询
						.singleResult();
		return pi;
	}
	
	/**查询流程状态（判断流程正在执行，还是结束）*/
	public boolean isProcessEndNew(String processInstanceId){
		ProcessInstance processEnd = this.isProcessEnd(processInstanceId);
		if(processEnd!=null){
			
			return false;
		}
		return true;
	}
	
	/**删除正在执行中的任务*/
	public void delRunTask(String processInstanceId,String deleteReason){
		runtimeService.deleteProcessInstance(processInstanceId, deleteReason);
	}
	
	/**统计正在执行中的任务*/
	public Map<String, Object> todoType(String userId){
		//计划总条数
		int countPlan = 0;
		//合同总条数
		int countContract = 0;
		//合同变更总条数
		Map<String, Object> modelMap = new HashMap<String, Object>();
		TaskType taskType = null;
		List<DeploymentList> todoList = todoList(userId, taskType);
		for (DeploymentList deploymentList : todoList) {
			String procDefId = deploymentList.getProcDefId();
			String[] split = procDefId.split(":");
			String string = split[0];
			if(string.equals("plan")){
				countPlan++;
			}else if (string.equals("contract")) {
				countContract++;
			}
		}
		modelMap.put("plan", countPlan);
		modelMap.put("contract", countContract);
		return modelMap;
	}
	
	// 获取到当前进行到的任务节点.
	public String taskNode(String processInstanceId){
        Task singleResult = taskService.createTaskQuery().processInstanceId(processInstanceId).singleResult();
        JSONObject json=new JSONObject();
        
        if(singleResult!=null){
        	String id = singleResult.getId();
        	String name = singleResult.getName();
        	json.put("taskId", id);
        	json.put("taskName",name);
        }
		return json.toString();
	} 
}