package com.chinaitop.depot.business.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.business.model.BusinessContractReceive;
import com.chinaitop.depot.business.service.BusinessContractReceiveService;
import com.chinaitop.depot.business.service.SystemOutsideDataService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping(value="/businessContractReceiveController")
@Api(value= "BusinessContractController", description = "储备粮合同接收管理")
public class BusinessContractReceiveController {

	@Resource
	private BusinessContractReceiveService businessContractReceiveService;

	@Resource
	private SystemOutsideDataService systemOutsideDataService;

	public static final Logger logger = LoggerFactory.getLogger(BusinessContractController.class);
	
	@RequestMapping(value="/getList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="合同接收列表", notes = "支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "htbh", value = "合同编号", paramType = "query"),
		@ApiImplicitParam(name = "orgId", value = "组织机构ID", paramType = "query"),
		@ApiImplicitParam(name = "houseId", value = "仓房ID", paramType = "query"),
		@ApiImplicitParam(name = "warehouseId", value = "货位ID", paramType = "query"),
		@ApiImplicitParam(name = "htlx", value = "合同类型(0:采购,1:销售)", paramType = "query")
	})
    public PageInfo<BusinessContractReceive> getList(Integer pageNum, Integer pageSize, 
    		String htbh ,Integer orgId, Integer houseId, Integer warehouseId, String htlx, String surplus) {
       
		PageInfo<BusinessContractReceive> pageInfo = null;

        List<BusinessContractReceive> list = null;
        try {
        	if (null != pageNum && null != pageSize) {
        		PageHelper.startPage(pageNum, pageSize);
        	}
			list = businessContractReceiveService.queryByExample(htbh, orgId, houseId, warehouseId, htlx, surplus);
			pageInfo = new PageInfo<BusinessContractReceive>(list);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
        return pageInfo;
    }
	
	@RequestMapping(value="/findbyparmaryKey",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="查询一条从市平台推送下来的合同数据", notes = "")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "数据串", paramType = "query")
	})
	public Map<String, Object> findbyparmaryKey(Integer id) {
		Map<String, Object> result = new HashMap<>();
		try {
			BusinessContractReceive businessContractReceive = businessContractReceiveService.findbyParmaryKey(id);
			result.put("status", "200");
			result.put("data", businessContractReceive);
			result.put("msg", "请求成功");
		} catch (Exception e) {
			result.put("status", "500");
			result.put("msg", "接口异常，请联系管理员");
			logger.error(e.getMessage(), e);
		}
		return result;
	}

	@RequestMapping(value="/saveContract",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="合同接收保存", notes = "市平台推送来的合同")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "notice", value = "数据串", paramType = "form")
	})
	public String saveContract(String notice) {
		logger.info("市平台推送合同数据如下：");
		logger.info(notice);
		Map<String, String> result = new HashMap<>();
		try {
			result = businessContractReceiveService.receiveContractData(notice);
			systemOutsideDataService.addlogger("htjs", notice, result.toString());
		} catch (Exception e) {
			result.put("status", "500");
			result.put("msg", "接口异常，请联系管理员");
			logger.error(e.getMessage(), e);
			systemOutsideDataService.addlogger("htjs", notice, result.toString());
		}
		return result.toString();
	}

	@RequestMapping(value="/updateYjssl",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="修改已结算数量", notes = "")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "htid", value = "合同ID", dataType="int", paramType = "form"),
		@ApiImplicitParam(name = "bcjssl", value = "本次结算数量(吨)", dataType="String", paramType = "form")
	})
	public Map<String, String> updateYjssl(Integer htid, String bcjssl) {
		Map<String, String> map = null;
		try {
			map = businessContractReceiveService.updateYjssl(htid, bcjssl);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return map;
	}

}
