package com.chinaitop.depot.business.controller;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.business.model.BusinessLading;
import com.chinaitop.depot.business.model.BusinessLadingExample;
import com.chinaitop.depot.business.service.BusinessLadingService;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping(value="/depot/business/lading")
@Api(value= "BusinessLadingController", description = "提货单")

public class BusinessLadingController {

	@Resource
    private BusinessLadingService ladingService;
	
	/**
     * 列表.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
	@RequestMapping(value="/getList", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="获取提货单列表信息", notes = "查询数据列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query")
	})
    public PageInfo<BusinessLading> getList(Integer pageNum, Integer pageSize) {
    	BusinessLadingExample example = new BusinessLadingExample();
        example.setOrderByClause("id desc");
        if (pageNum != null && pageSize != null) {
            PageHelper.startPage(pageNum, pageSize);
        }
        List<BusinessLading> list = ladingService.queryByExample(example);    
        PageInfo<BusinessLading> pageInfo = new PageInfo<BusinessLading>(list);
        return pageInfo;
    }
    
    
	/**
	 * 保存
	 * @param ladingJson
	 * @param request
	 * @return
	 * @throws JsonParseException
	 * @throws JsonMappingException
	 * @throws IOException
	 */
	@RequestMapping(value="/save", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="保存数据", notes = "保存")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "ladingJson", value = "新增数据", paramType = "query")
	})
    public Map<String, Object> save(String ladingJson, HttpServletRequest request)
            throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        try {
           /* // 用户信息.
            UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
            Integer userId = userInfo.getUserId();
            // 组织机构信息.
            OrgInfo depotInfo = (OrgInfo)request.getSession().getAttribute("depotInfo");*/
            
            // JSON字符串转对象
            ObjectMapper mapper = new ObjectMapper();
            BusinessLading lading = (BusinessLading)mapper.readValue(ladingJson, BusinessLading.class);
            if (lading.getId() == null) {
                Map<String, Object> returnMap = ladingService.add(lading);
                if (returnMap != null && returnMap.get("error") != null) {
                    modelMap.put("status", "error");
                    modelMap.put("msg", returnMap.get("error"));
                    return modelMap;
                }
            } else {
                modelMap = ladingService.update(lading);
            }
            modelMap.put("status", "success");
        } catch (Exception e) {
            e.printStackTrace();
            modelMap.put("status", "error");
            modelMap.put("msg", "保存失败！");
        }
        return modelMap;
    }
    
	/**
     * 根据id查询.
     * @param id 主键
     * @return
     */
	@RequestMapping(value="/edit", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="根据id查找", notes = "根据id查找")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "主键id", paramType = "query")
	})
    public BusinessLading edit(Integer id) {
    	BusinessLading lading = new BusinessLading();
        if (id != null) {
        	lading = ladingService.findById(id);
        }
        return lading;
    }

	
	
}
