package com.chinaitop.depot.business.controller;


import com.chinaitop.depot.business.model.BusinessFile;
import com.chinaitop.depot.business.model.BusinessPlan;
import com.chinaitop.depot.business.model.BusinessStoreWareDetail;
import com.chinaitop.depot.business.model.enums.AuditType;
import com.chinaitop.depot.business.model.enums.StoreWareType;
import com.chinaitop.depot.business.model.vo.BusinessPlanAuditVO;
import com.chinaitop.depot.business.service.BusinessFileService;
import com.chinaitop.depot.business.service.BusinessPlanService;
import com.chinaitop.depot.business.service.BusinessStoreWareDetailService;
import com.chinaitop.depot.utils.HTTPUtils;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 粮库业务管理-计划管理
 * <p>Title: BusinessPlanController</p>
 * <p>Description: </p>
 * 
 */
@RestController
@RequestMapping(value="/depot/business/plan")
@Api(value= "BusinessPlanController", description = "计划管理")

public class BusinessPlanController {

    @Resource
    private BusinessPlanService businessPlanService;
    @Autowired
    private BusinessFileService businessFileService;
    @Resource
    private BusinessStoreWareDetailService businessStoreWareDetailService;

    // 获取ip地址
    @Value("${reportUrl}")
    private String reportPath;

    /**
     * 查找申请列表
     * @param pageNum  页码
     * @param pageSize 每页显示条数
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/getList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="计划申请列表", notes = "查询数据列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "userId", value = "当前登录的用户编号", paramType = "query")
    })
    public PageInfo<BusinessPlan> getList(Integer pageNum, Integer pageSize, Integer userId ,HttpServletRequest request, BusinessPlan businessPlan) throws Exception {

        // Integer userId = userInfo.getUserId();
        List<BusinessPlan> list = businessPlanService.queryByExample(pageNum, pageSize, userId, businessPlan, "FIELD(audit_state,0,3,1,2,4 ), id desc", null,null,null);
        PageInfo<BusinessPlan> pageInfo = new PageInfo<BusinessPlan>(list);
        return pageInfo;
    }

    /**
     * 审批通过列表
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
    @RequestMapping(value="/getAuditPassList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="计划审批通过列表", notes = "查询数据列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query")
    })
    public PageInfo<BusinessPlan> getAuditPassList(Integer pageNum, Integer pageSize, BusinessPlan businessPlan, Integer billType,String type,Integer orgId) {
        // 可以查询所有人通过的计划，必须是应用的.
        businessPlan.setAuditState(AuditType.AGREE.getAuditCode());
        List<BusinessPlan> list = businessPlanService.queryByExample(pageNum, pageSize, null, businessPlan, " agree_time desc", billType,type,orgId);
        PageInfo<BusinessPlan> pageInfo = new PageInfo<BusinessPlan>(list);
        return pageInfo;
    }

    /**
     * 获取进度列表.
     * @param pageNum
     * @param pageSize
     * @param businessPlan
     * @return
     */
    @RequestMapping(value="/getScheduleList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="获取计划进度列表", notes = "查询数据列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "库ID", paramType = "query")
    })
    public PageInfo<BusinessPlanAuditVO> getScheduleList(Integer pageNum, Integer pageSize, BusinessPlan businessPlan, Integer orgId) {
        PageInfo<BusinessPlanAuditVO> pageInfo = businessPlanService.getScheduleList(pageNum, pageSize, businessPlan, orgId);
        return pageInfo;
    }

    /**
     * 根据id查询
     * @param id 主键
     * @return
     */
    @RequestMapping(value="/edit", produces = MediaType.APPLICATION_JSON_VALUE,method=RequestMethod.GET)
    @ApiOperation(value="根据计划id查找信息", notes = "根据id查找")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "计划id", paramType = "query")
    })
    public Map<String, Object> edit(Integer id) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        BusinessPlan businessPlan = new BusinessPlan();
        if (id != null) {
            businessPlan = businessPlanService.findById(id);
            // 获取对应的文件.
            List<BusinessFile> fileList= businessFileService.getByBid(id, "plan", null, null);
            modelMap.put("fileList", fileList);
        }else{
            businessPlan=new BusinessPlan();
        }
        modelMap.put("plan", businessPlan);

        return modelMap;
    }

    /**
     * 保存
     * @param businessPlanJson 计划json
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @RequestMapping(value="/save",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    @ApiOperation(value="保存数据", notes = "保存")
    @SuppressWarnings("unchecked")
    @ApiImplicitParams({
            @ApiImplicitParam(name="businessPlanJson", value="主表数据", paramType="form"),
            @ApiImplicitParam(name="businessStoreWareDetailJson", value="子表数据", paramType="form"),
            @ApiImplicitParam(name="fileIds", value="文件数据", paramType="form"),
            @ApiImplicitParam(name="userId", value="当前登录人的id", paramType="form"),
            @ApiImplicitParam(name="orgId", value="当前登录人的组织id", paramType="form"),
            @ApiImplicitParam(name="name", value="当前登录人的name", paramType="form"),
            @ApiImplicitParam(name="depotInfoOrgId", value="当前登录人的组织id", paramType="form")
    })
    public Map<String, Object> save(String businessPlanJson, String businessStoreWareDetailJson,
                                    String fileIds, String deleteFileIds, Integer userId , Integer orgId ,String name ,Integer depotInfoOrgId,
                                    HttpServletRequest request) throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();
       /* UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
        OrgInfo depotInfo = (OrgInfo)request.getSession().getAttribute("depotInfo");*/
        try {
            // JSON字符串转对象
            ObjectMapper mapper = new ObjectMapper();
            BusinessPlan businessPlan = (BusinessPlan)mapper.readValue(businessPlanJson, BusinessPlan.class);

            List<BusinessStoreWareDetail> storeWareDetailList = null;
            if (businessStoreWareDetailJson != null) {
                storeWareDetailList = (List<BusinessStoreWareDetail>)mapper.readValue(businessStoreWareDetailJson, new TypeReference<List<BusinessStoreWareDetail>>(){});
            }

            if (businessPlan.getId() == null) {
                Map<String, Object> returnMap = businessPlanService.add(businessPlan, storeWareDetailList, fileIds , userId ,  orgId , name , depotInfoOrgId);
                if (returnMap.get("error") != null) {
                    modelMap.put("status", "error");
                    modelMap.put("msg", returnMap.get("error"));
                    return modelMap;
                }
                modelMap.put("id", returnMap.get("id"));
            } else {
                businessPlanService.update(businessPlan, storeWareDetailList, fileIds, deleteFileIds);
            }
            modelMap.put("status", "success");
        } catch (Exception e) {
            e.printStackTrace();
            modelMap.put("status", "error");
            modelMap.put("msg", "保存失败！");
            return modelMap;
        }
        return modelMap;
    }

    /**
     * 删除(同时删除流程)
     * @param id 主键
     * @param processInstanceId 流程实例id
     * @param deleteReason 删除原因
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @RequestMapping(value="/remove", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    @ApiOperation(value="删除(同时删除流程)", notes = "根据id删除")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "计划id", paramType = "query"),
            @ApiImplicitParam(name = "processInstanceId", value = "流程实例id", paramType = "query")
    })
    public Map<String, Object> remove(Integer id, String processInstanceId, String deleteReason, HttpServletRequest request)
            throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        // 服务器路径
        String rootPath = request.getSession().getServletContext().getRealPath("");
        if (id != null) {
            businessPlanService.remove(id, processInstanceId, deleteReason, rootPath);
        }
        modelMap.put("status", "success");
        return modelMap;
    }

    /**
     * 提交
     * @param businessDeliveryStorageNotice 业务主体
     * @param assignee 下一个审批人
     * @return
     */
    @RequestMapping(value="/submit",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    @ApiOperation(value="计划提交", notes = "计划提交")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "计划id", paramType = "query"),
            @ApiImplicitParam(name = "assignee", value = "下一个审批人", paramType = "query"),
            @ApiImplicitParam(name = "userId", value = "当前登录人的id", paramType = "query"),
            @ApiImplicitParam(name = "realName", value = "当前登录人姓名", paramType = "query")
    })
    public Map<String, Object> submit(Integer id, String assignee,Integer userId , String realName ,HttpServletRequest request) {
        // UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
        Map<String, Object> returnMap = businessPlanService.submit(id, assignee,userId ,realName);
        return returnMap;
    }

    /**
     * 启用
     * @param id 主键.
     * @return
     */
    @RequestMapping(value="/application", produces = MediaType.APPLICATION_JSON_VALUE,method=RequestMethod.POST)
    @ApiOperation(value="计划启用", notes = "计划启用")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "计划id", paramType = "query")
    })
    public Map<String, Object> application(Integer id) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        businessPlanService.application(id);
        modelMap.put("status", "success");
        return modelMap;
    }

    /**
     * 暂停
     * @param id 主键.
     * @return
     */
    @RequestMapping(value="/discontinue", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    @ApiOperation(value="计划暂停", notes = "计划暂停")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "计划id", paramType = "query")
    })
    public Map<String, Object> discontinue(Integer id,String refuseReason) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        businessPlanService.discontinue(id,refuseReason);
        modelMap.put("status", "success");
        return modelMap;
    }

    /**
     * 终止
     * @param id 主键.
     * @return
     */
    @RequestMapping(value="/finish", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    @ApiOperation(value="计划终止", notes = "计划终止")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "计划id", paramType = "query")
    })
    public Map<String, Object> finish(Integer id,String refuseReason) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        businessPlanService.finish(id,refuseReason);
        modelMap.put("status", "success");
        return modelMap;
    }

    /**
     * 校验计划编号，是否可用.
     * @param id 主键.
     * @param planNumber 计划编号.
     * @return
     */
    @RequestMapping(value="/validPlanNumber", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    @ApiOperation(value="校验计划编号，是否可用", notes = "校验计划编号，是否可用")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "计划id", paramType = "query"),
            @ApiImplicitParam(name = "planNumber", value = "计划编号", paramType = "query")
    })
    public Boolean validPlanNumber(Integer id, String planNumber) {
        return businessPlanService.validPlanNumber(id, planNumber);
    }


    /**
     * 根据id查询追溯数据.
     * @param id 主键
     * @return
     */
    @RequestMapping(value="/getSynthInfo", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="根据id查询计划追溯数据", notes = "根据id查找")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "计划id", paramType = "query")
    })
    public Map<String, Object> getSynthInfo(Integer id) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        if (id != null) {
            modelMap = businessPlanService.getSynthInfo(id);
        }
        return modelMap;
    }

    /**
     * 计划已上报的列表展示(省)
     * @param pageNum
     * @param pageSize
     * @param userId
     * @param request
     * @param businessPlan
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/getReportList", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="计划已上报列表", notes = "查询数据列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "userId", value = "当前登录人的id", paramType = "query")
    })
    public PageInfo<BusinessPlan> getReportList(Integer pageNum, Integer pageSize, Integer userId ,HttpServletRequest request, BusinessPlan businessPlan) throws Exception {

        // 可以查询所有人通过的计划，必须是应用的.
        businessPlan.setAuditState(AuditType.AGREE.getAuditCode());
        List<BusinessPlan> list = businessPlanService.queryReportByExample(pageNum, pageSize, userId, businessPlan);
        for (BusinessPlan plan : list) {
            int id=plan.getId();
            List<BusinessStoreWareDetail> storeWareDetailList= businessStoreWareDetailService.getByZidAndType(id, StoreWareType.plan.getValue());
            int grainAttribute = storeWareDetailList.get(0).getGrainAttribute();
            plan.setGrainAttributeByfirst(String.valueOf(grainAttribute));
        }
        PageInfo<BusinessPlan> pageInfo = new PageInfo<BusinessPlan>(list);
        return pageInfo;
    }


    /**
     * 计划已上报的列表的查看(省)
     * @param pageNum
     * @param pageSize
     * @param userId
     * @param request
     * @param businessPlan
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/getReportEduit", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="计划已上报列表查看", notes = "计划已上报列表查看")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "计划id", paramType = "query"),
            @ApiImplicitParam(name = "userName", value = "当前登录人的姓名", paramType = "query")
    })
    public Map<String, Object> getReportEduit(Integer id,String userName) throws Exception {

        Map<String, Object> modelMap = new HashMap<String, Object>();
        // 详情.
        BusinessPlan plan = businessPlanService.findById(id);
        modelMap.put("plan", plan);

        // 获取对应的文件.
        List<BusinessFile> fileList= businessFileService.getByBid(id, "plan", null, null);

        modelMap.put("fileList", fileList);

        // 仓号货位号数量质量标准.
        List<BusinessStoreWareDetail> storeWareDetailList= businessStoreWareDetailService.getByZidAndType(id, StoreWareType.plan.getValue());
        modelMap.put("storeWareDetailList", storeWareDetailList);
        return modelMap;
    }


    /**
     * 上报 修改状态
     * @param id
     * @param request
     * @return
     */
    @RequestMapping(value="/updatePlanStatus", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    @ApiOperation(value="修改计划上报状态", notes = "修改计划上报状态")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "计划id", paramType = "query"),
            @ApiImplicitParam(name = "status", value = "计划状态", paramType = "query")
    })
    public Map<String, Object> updatePlanStatus(Integer id,Integer status,String totalQuantity,String enterpriseRemark,String provinceRemark) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        BusinessPlan plan = new BusinessPlan();
        try {
            if(id!=null && status!=null){
            	Date date = new Date();
                plan.setId(id);
                if(status==10){ //上报的状态
                    plan.setReportDate(date);
                }
                if(status==40){ //下发的状态
                    plan.setSendDate(date);
                }
                plan.setCustomerPlanState(String.valueOf(status));
                if(totalQuantity!=null){
                	plan.setTotalQuantity(totalQuantity);
                }
                if(enterpriseRemark!=null){
                	plan.setEnterpriseRemark(enterpriseRemark);
                }
                if(provinceRemark!=null){
                	plan.setProvinceRemark(provinceRemark);
                }
                businessPlanService.updatePlanStatus(plan);
                modelMap.put("plan", plan);
                modelMap.put("status", "success");
            }else{
                throw new RuntimeException("id或status无值");
            }
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            modelMap.put("status", "error");
            modelMap.put("msg", "保存失败！");
        }

        return modelMap;

    }


    /**
     *
     * 验收申请
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/acceptanceSave",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="计划申请列表", notes = "查询数据列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "planJson", value = "计划", paramType = "query"),
            @ApiImplicitParam(name = "houseJson", value = "明细信息", paramType = "query"),
            @ApiImplicitParam(name = "fileJson", value = "文件", paramType = "query")
    })
    public Map<String, Object> acceptanceSave(String planJson, String houseJson, String fileJson) throws Exception {
        // Integer userId = userInfo.getUserId();
        Map<String, Object> modelMap = new HashMap<String, Object>();
        StringBuffer req = new StringBuffer();
        try {
            String plan = URLEncoder.encode(planJson, "UTF-8");
            String house = URLEncoder.encode(houseJson, "UTF-8");
            String file = URLEncoder.encode(fileJson, "UTF-8");


            String url = reportPath+"/business-management/businessPlan/houseSave";

            String count= HTTPUtils.doGet(url+"?planJson="+plan+"&houseJson="+house+"&fileJson="+file);
            System.out.println(count+"********验收***"+reportPath+"地址");
            if(count.contains("Success") || count.contains("200")){
                modelMap.put("status", "success");
                modelMap.put("msg", "成功！");
            }

        } catch (Exception e){
            // req.append("?data=");
            e.printStackTrace();
            modelMap.put("status", "error");
            modelMap.put("msg", "保存失败！");
        }

        return modelMap;
    }


    /**
     *
     * 上报申请
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/reportSave",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="计划申请列表", notes = "查询数据列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "businessPlan", value = "计划", paramType = "query")

    })
    public Map<String, Object> reportSave(String businessPlan) throws Exception {
        // Integer userId = userInfo.getUserId();
        Map<String, Object> modelMap = new HashMap<String, Object>();
        StringBuffer req = new StringBuffer();
        try {
            String plan = URLEncoder.encode(businessPlan, "UTF-8");

            String url = reportPath+"/business-management/businessPlan/save";

            String count= HTTPUtils.doGet(url+"?businessPlan="+plan);
            System.out.println(count+"********上报*****"+reportPath+"地址");
            if(count.contains("Success") || count.contains("200")){
                modelMap.put("status", "success");
                modelMap.put("msg", "成功！");
            }
        } catch (Exception e){
            // req.append("?data=");
            e.printStackTrace();
            modelMap.put("status", "error");
            modelMap.put("msg", "保存失败！");
        }

        return modelMap;
    }

    /**
     * 验收列表
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
    @RequestMapping(value="/getAcceptanceList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="计划审批通过列表", notes = "查询数据列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query")
    })
    public PageInfo<Map<String, Object>> getAcceptanceList(Integer pageNum, Integer pageSize, BusinessPlan businessPlan,Integer userId,Integer orgId) {
       
    	Map<String, Object> map = new HashMap<String, Object>();
    	map.put("orgId", orgId);
    	map.put("userId", userId);
    	if(null != businessPlan.getPlanNumber()){
    		map.put("planNumber", businessPlan.getPlanNumber());
    	}
    	if(null != businessPlan.getPlanTitle()){
    		map.put("planTitle", businessPlan.getPlanTitle());
    	}
    	if(null != businessPlan.getCustomerPlanState()){
    		map.put("customerPlanState", businessPlan.getCustomerPlanState());
    	}
    	if(null != pageNum && null != pageSize){
			PageHelper.startPage(pageNum, pageSize);
		}
    	
        List<Map<String, Object>> list = businessPlanService.queryAcceptanceList(map);
        
        PageInfo<Map<String, Object>> pageInfo = new PageInfo<Map<String, Object>>(list);
        return pageInfo;
    }
    
    /**
     * 架空期查询数据
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
    @RequestMapping(value="/getJkqPlanNumberList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="架空期数据查询", notes = "架空期数据查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "executeType", value = "轮换计划", paramType = "query")
    })
    public PageInfo<Map<String, Object>> getJkqPlanNumberList(HttpServletRequest request, Integer pageNum, Integer pageSize,
    		String applyTimeA, String applyTimeB, String lhEndA, String lhEndB) {
       
    	Integer orgId = (Integer) request.getSession().getAttribute("orgId");
    	List<Map<String, Object>> countList = new ArrayList<Map<String, Object>>();
    	List<Map<String, Object>> lastList = new ArrayList<Map<String, Object>>();
    	Map<String, Object> map = new HashMap<String, Object>();
    	map.put("orgId", orgId);
    	map.put("executeType", 3156);
    	map.put("cjsjA", applyTimeA);
    	map.put("cjsjB", applyTimeB);
    	map.put("rqA", lhEndA);
    	map.put("rqB", lhEndB);
        List<Map<String, Object>> list = businessPlanService.getjkqPlanList(map);
        for (Map<String, Object> countMap : list) {
        	countMap.put("orgId", orgId);
        	countMap.put("executeType", 3156);
        	countMap.put("cjsjA", applyTimeA);
        	countMap.put("cjsjB", applyTimeB);
        	countMap.put("rqA", lhEndA);
        	countMap.put("rqB", lhEndB);
			countList = businessPlanService.getjkqPlanDayList(countMap);
			for (Map<String, Object> lastMap : countList) {
				if(lastMap != null && !lastMap.isEmpty() && lastMap.get("jkqEnd") !=null) {
				lastList.add(lastMap);
				}
        	}
		}
        PageInfo<Map<String, Object>> pageInfo = new PageInfo<Map<String, Object>>(lastList);
        return pageInfo;
    }
    
    /**
     * 架空期查询轮入数据
     * @return
     */
    @RequestMapping(value="/getJkqPlanDayList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="架空期查询轮入数据", notes = "架空期查询轮入数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "executeType", value = "轮换计划", paramType = "query")
    })
    public PageInfo<Map<String, Object>> getJkqPlanDayList(HttpServletRequest request, Integer pageNum, Integer pageSize, Integer houseId, Integer warHouseId) {
       
    	Integer orgId = (Integer) request.getSession().getAttribute("orgId");
    	Map<String, Object> map = new HashMap<String, Object>();
    	map.put("orgId", orgId);
    	map.put("executeType", 3156);
    	map.put("billType", 3);
    	map.put("ch", houseId);
    	map.put("hwh", warHouseId);
    	if(null != pageNum && null != pageSize){
			PageHelper.startPage(pageNum, pageSize);
		}
        List<Map<String, Object>> list = businessPlanService.getjkqPlanDayList(map);
        PageInfo<Map<String, Object>> pageInfo = new PageInfo<Map<String, Object>>(list);
        return pageInfo;
    }

    @RequestMapping(value="/getjybjl",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="查找本单位下经营部经理的人员", notes = "查找本单位下经营部经理的人员")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public Integer getjybjl(Integer orgId) throws Exception {
        return businessPlanService.getjybjl(orgId);
    }

    @RequestMapping(value="/getzzbjl",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="查找本单位下中转部经理的人员", notes = "查找本单位下中转部经理的人员")
    @ApiImplicitParams({
    	@ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public Integer getzzbjl(Integer orgId) throws Exception {
    	return businessPlanService.getzzbjl(orgId);
    }
    
}