package com.chinaitop.depot.business.controller;


import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.business.model.BusinessNoticeReceive;
import com.chinaitop.depot.business.model.BusinessStoreWareDetail;
import com.chinaitop.depot.business.service.BusinessContractService;
import com.chinaitop.depot.business.service.BusinessStoreWareDetailService;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 粮库业务管理-计划合同通知单-子表
* <p>Title: BusinessStoreWareDetailController</p>
* @author My   Sunshine
* 
 */
@RestController
@RequestMapping(value="/depot/business/storeWareDetail")
@Api(value= "BusinessStoreWareDetailController", description = "计划合同通知单-子表")

public class BusinessStoreWareDetailController {
    
    @Resource
    private BusinessStoreWareDetailService detailService;
    
    @Resource
    private BusinessContractService contractService;

    @RequestMapping(value="/getStoreWareDetailList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="获取", notes = "根据id删除")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "zid", value = "主表ID", paramType = "query"),
		@ApiImplicitParam(name = "dataType", value = "数据类型", paramType = "query")
	})
    public List<BusinessStoreWareDetail> getStoreWareDetailList(Integer zid, String dataType) {
    	List<BusinessStoreWareDetail> list = detailService.getByZidAndType(zid, dataType);
    	return list;
    }

    @RequestMapping(value="/remove",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="删除", notes = "根据id删除")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "计划合同通知单子表id", paramType = "query")
	})
    public Map<String, Object> remove(Integer id) throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        if (id != null) {
        	detailService.remove(id);
        }
        modelMap.put("status", "success");
        return modelMap;
    }
    
    
    @RequestMapping(value="/getByZidAndType", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="根据条件查找", notes = "根据条件查找")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "zid", value = "关联的主表id", paramType = "query"),
		@ApiImplicitParam(name = "type", value = "类型", paramType = "query")
	})
    public Map<String, Object> getByZidAndType(Integer zid, String type) throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        try {
            if (zid != null && StringUtils.isNotBlank(type)) {
                List<BusinessStoreWareDetail> list = new ArrayList<BusinessStoreWareDetail>();
                Map<String, Object> param_map = new HashMap<String, Object>();
                List<Map<String, Object>> map = null;
                if("cblContract".equals(type)){
                	list = detailService.getByZidAndType(zid, "cblContract");
        			param_map.put("contract_id", zid);
        			map = detailService.selectContractUseCount(param_map);
                }else{
                	if (!"plan".equals(type)) {
                    	list = detailService.getByZidAndType(zid, "contract");
            			param_map.put("contract_id", zid);
            			map = detailService.selectContractUseCount(param_map);
                    } else {
                    	list = detailService.getByZidAndType(zid, "plan");
                    	param_map.put("plan_id", zid);
                    	map = detailService.selectPlanUseCount(param_map);
                    }
                }
                BusinessStoreWareDetail obj = null;
                double sysl = 0d;
                double sum_use_num = 0d; //已使用总量
                double plan_sum_num = 0d; //原计划数量
                for (int i = 0; i < list.size(); i++) {
                	obj = list.get(i);
                	if (map.size() > 0) {
                		for (Map<String, Object> m : map) {
                			if(null == obj.getWarehouseId() || null == obj.getWareHouseId()){//油
                				if (m.get("house_id").equals(obj.getHouseId())) {
                    				if (!"".equals(m.get("in_count")) && null != m.get("in_count")) {
                    					if (!"".equals(obj.getInCount()) && null != obj.getInCount()) {
                    						plan_sum_num = Double.parseDouble(obj.getInCount());
                        					sum_use_num = Double.parseDouble(m.get("in_count")+"");
                        					if (sum_use_num <= plan_sum_num) {
                        						sysl = plan_sum_num - sum_use_num;
                        						obj.setRemainingNumber("0"); //页面上明细列表的剩余数量
                        						obj.setInCount(sysl+"");
                        					} else if (sum_use_num > plan_sum_num){
                        						sysl = sum_use_num - plan_sum_num;
                        						obj.setRemainingNumber("超出:"+sysl);
                        					}
                    					}
                    					//轮换计划的引用了入的，出的为空  "".equals(m.get("out_count")) && null == m.get("out_count")
                    					if(!m.containsKey("out_count")){
                    						if(!"".equals(obj.getOutCount()) && null != obj.getOutCount()){
                    							plan_sum_num = Double.parseDouble(obj.getOutCount());
                        						sum_use_num = 0d;
                        						if (sum_use_num <= plan_sum_num) {
                        							sysl = plan_sum_num - sum_use_num;
                        							obj.setOutRemainingNumber("0");
                        							obj.setOutCount(sysl+"");
                        						} 
                    						}
                    					}
                    				}
                    				if (!"".equals(m.get("out_count")) && null != m.get("out_count")) {
                    					if (!"".equals(obj.getOutCount()) && null != obj.getOutCount()) {
                    						plan_sum_num = Double.parseDouble(obj.getOutCount());
                    						sum_use_num = Double.parseDouble(m.get("out_count")+"");
                    						if (sum_use_num <= plan_sum_num) {
                    							sysl = plan_sum_num - sum_use_num;
                    							obj.setOutRemainingNumber("0");
                    							obj.setOutCount(sysl+"");
                    						} else if (sum_use_num > plan_sum_num){
                    							sysl = sum_use_num - plan_sum_num;
                    							obj.setOutRemainingNumber("超出:"+sysl);
                    						}
                    					}
                    					
                    					//轮换计划的引用了出的，入的为空  "".equals(m.get("in_count")) && null == m.get("in_count")
                    					if (!m.containsKey("in_count")) {
                        					if (!"".equals(obj.getInCount()) && null != obj.getInCount()) {
                        						plan_sum_num = Double.parseDouble(obj.getInCount());
                            					sum_use_num = 0d;
                            					if (sum_use_num <= plan_sum_num) {
                            						sysl = plan_sum_num - sum_use_num;
                            						obj.setRemainingNumber("0"); //页面上明细列表的剩余数量
                            						obj.setInCount(sysl+"");
                            					} 
                        					}
                    					}
                    					
                    				}
                    			}
                			}else{//粮食
                				if (m.get("house_id").equals(obj.getHouseId()) && m.get("warehouse_id").equals(obj.getWarehouseId())) {
                    				if (!"".equals(m.get("in_count")) && null != m.get("in_count")) {
                    					if (!"".equals(obj.getInCount()) && null != obj.getInCount()) {
                    						plan_sum_num = Double.parseDouble(obj.getInCount());
                        					sum_use_num = Double.parseDouble(m.get("in_count")+"");
                        					if (sum_use_num <= plan_sum_num) {
                        						sysl = plan_sum_num - sum_use_num;
                        						obj.setRemainingNumber("0"); //页面上明细列表的剩余数量
                        						obj.setInCount(sysl+"");
                        					} else if (sum_use_num > plan_sum_num){
                        						sysl = sum_use_num - plan_sum_num;
                        						obj.setRemainingNumber("超出:"+sysl);
                        					}
                    					}
                    					//轮换计划的引用了入的，出的为空  "".equals(m.get("out_count")) && null == m.get("out_count")
                    					if(!m.containsKey("out_count")){
                    						if(!"".equals(obj.getOutCount()) && null != obj.getOutCount()){
                    							plan_sum_num = Double.parseDouble(obj.getOutCount());
                        						sum_use_num = 0d;
                        						if (sum_use_num <= plan_sum_num) {
                        							sysl = plan_sum_num - sum_use_num;
                        							obj.setOutRemainingNumber("0");
                        							obj.setOutCount(sysl+"");
                        						} 
                    						}
                    					}
                    				}
                    				if (!"".equals(m.get("out_count")) && null != m.get("out_count")) {
                    					if (!"".equals(obj.getOutCount()) && null != obj.getOutCount()) {
                    						plan_sum_num = Double.parseDouble(obj.getOutCount());
                    						sum_use_num = Double.parseDouble(m.get("out_count")+"");
                    						if (sum_use_num <= plan_sum_num) {
                    							sysl = plan_sum_num - sum_use_num;
                    							obj.setOutRemainingNumber("0");
                    							obj.setOutCount(sysl+"");
                    						} else if (sum_use_num > plan_sum_num){
                    							sysl = sum_use_num - plan_sum_num;
                    							obj.setOutRemainingNumber("超出:"+sysl);
                    						}
                    					}
                    					
                    					//轮换计划的引用了出的，入的为空  "".equals(m.get("in_count")) && null == m.get("in_count")
                    					if (!m.containsKey("in_count")) {
                        					if (!"".equals(obj.getInCount()) && null != obj.getInCount()) {
                        						plan_sum_num = Double.parseDouble(obj.getInCount());
                            					sum_use_num = 0d;
                            					if (sum_use_num <= plan_sum_num) {
                            						sysl = plan_sum_num - sum_use_num;
                            						obj.setRemainingNumber("0"); //页面上明细列表的剩余数量
                            						obj.setInCount(sysl+"");
                            					} 
                        					}
                    					}
                    					
                    				}
                    			}
                			}
                			
                		}
                	} else {
                		/* 如果没有查询到数据，则表示还没有被任何通知单和合同引用过 */
                		obj.setRemainingNumber("0");
                		obj.setOutRemainingNumber("0");
                	}
                }
                modelMap.put("status", "success");
                modelMap.put("list", list);
            } else {
                throw new RuntimeException("zid或者type为空！");
            }
        } catch (Exception e) {
            e.printStackTrace();
            modelMap.put("status", "error");
            modelMap.put("msg", "查询失败！");
        }
        return modelMap;
    }
    
    
    /**
     * 根据id 修改数据
     * @param id 主键
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @RequestMapping(value="/updateById",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="修改", notes = "根据id修改")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "BusinessStoreWareDetail", value = "子表数据", paramType = "query")
	})
    public int updateById(BusinessStoreWareDetail detail) {
        
        	int i= detailService.updateById(detail);
        
        return i;
    }
    
    
    /**
     * 根据id 查询数据
     * @param id 主键
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @RequestMapping(value="/queryStoreWareDetailInfo",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="修改", notes = "根据id修改")
    @ApiImplicitParams({
    	@ApiImplicitParam(name = "BusinessStoreWareDetail", value = "子表数据", paramType = "query")
    })
    public BusinessStoreWareDetail queryStoreWareDetailInfo(String id) {
    	BusinessStoreWareDetail businessStoreWareDetail = null;
    	if(id!=null && !"".equals(id)){
    		 businessStoreWareDetail= detailService.queryInfo(id);
    	}
    	
    	return businessStoreWareDetail;
    }
    
    
    /**
     * 储备粮管理 剩余数量(合同的)
     * @param id 主键
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @SuppressWarnings("unchecked")
	@RequestMapping(value="/getRemainNumber",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="剩余数量", notes = "剩余数量")
    @ApiImplicitParams({
    	@ApiImplicitParam(name = "receiveNoticeList", value = "选中的数据", paramType = "query")
    })
    public Map<String, Object> getRemainNumber(String receiveNoticeListJson) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        
        ObjectMapper mapper = new ObjectMapper();
        List<BusinessNoticeReceive> noticeReceiveList = null;
        BusinessNoticeReceive obj = null;
        if (receiveNoticeListJson != null) {
            try {
            	noticeReceiveList = (List<BusinessNoticeReceive>)mapper.readValue(receiveNoticeListJson, new TypeReference<List<BusinessNoticeReceive>>(){});
            	
            	for (int i = 0; i < noticeReceiveList.size(); i++) {
            		 Map<String, Object> map = new HashMap<String, Object>();
            		 obj = noticeReceiveList.get(i);
            		 map.put("houseId", obj.getHouseId());
            		 map.put("warehouseId", obj.getWareHouseId());
            		 map.put("grainKind", obj.getGrainKind());
            		 map.put("grainAttribute", obj.getGrainAttribute());
            		 map.put("grainGrade", obj.getGrainGrade());
            		 map.put("orgId", obj.getOrgId());
            		 map.put("jstzdId", obj.getId());
            		String useSumCount = detailService.getByTypeAndGrain(map);//使用数量(吨)
            		double sysl = 0d;
                   
                    /*BigDecimal s = new BigDecimal(1000);
        			BigDecimal sumsl = new BigDecimal(obj.getCount());
        			BigDecimal sumCkslss = sumsl.multiply(s);//吨转化为公斤
*/        			double sum_num = Double.parseDouble(obj.getCount()); //总的数量(吨)
					if(useSumCount!=null && useSumCount!=""){
						/*BigDecimal s = new BigDecimal(1000);
	        			BigDecimal sumsl = new BigDecimal(useSumCount);
	        			BigDecimal sumCkslss = sumsl.divide(s);*/
	                    double use_num = Double.parseDouble(useSumCount); //使用数量(吨)divide
						if (use_num <= sum_num) {
							sysl = sum_num - use_num;
							obj.setOutRemainingNumber("0");//页面展示
							obj.setOutCount(sysl+"");
						} else if (use_num > sum_num){
							sysl = use_num - sum_num;
							obj.setOutRemainingNumber("超出:"+sysl);
							obj.setOutCount("0");//页面展示
						}
					}else{
						//没有使用过
						obj.setOutRemainingNumber("0"); //页面展示剩余数量
						obj.setOutCount(sum_num+""); //总数量
					}
				}
            	modelMap.put("status", "success");
            	modelMap.put("list", noticeReceiveList);
            } catch (IOException e) {
				 e.printStackTrace();
				 modelMap.put("status", "error");
		         modelMap.put("msg", "查询失败！");
			}
        }
    	
    	return modelMap;
    }
    
    
    /**
     * 储备粮管理 剩余数量(入库通知单)
     * @param id 主键
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @SuppressWarnings("unchecked")
    @RequestMapping(value="/getRkRemainNumber",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="储备粮管理下查询通知单剩余数量(入库通知单)", notes = "剩余数量(入库通知单)")
    @ApiImplicitParams({
    	@ApiImplicitParam(name = "receiveNoticeList", value = "选中的数据", paramType = "query")
    })
    public Map<String, Object> getRkRemainNumber(String receiveNoticeListJson) {
    	Map<String, Object> modelMap = new HashMap<String, Object>();
    	
    	ObjectMapper mapper = new ObjectMapper();
    	List<BusinessNoticeReceive> noticeReceiveList = null;
    	BusinessNoticeReceive obj = null;
    	if (receiveNoticeListJson != null) {
    		try {
    			noticeReceiveList = (List<BusinessNoticeReceive>)mapper.readValue(receiveNoticeListJson, new TypeReference<List<BusinessNoticeReceive>>(){});
    			
    			for (int i = 0; i < noticeReceiveList.size(); i++) {
    				Map<String, Object> map = new HashMap<String, Object>();
    				obj = noticeReceiveList.get(i);
    				map.put("houseId", obj.getHouseId());
    				map.put("warehouseId", obj.getWareHouseId());
    				map.put("grainKind", obj.getGrainKind());
    				map.put("grainAttribute", obj.getGrainAttribute());
    				map.put("grainGrade", obj.getGrainGrade());
    				map.put("orgId", obj.getOrgId());
    				map.put("jstzdId", obj.getId());
    				String useSumCount = detailService.getRkUseCount(map);//使用数量(吨)
    				double sysl = 0d;
    				double sum_num = Double.parseDouble(obj.getCount()); //总的数量(吨)
    				 if(useSumCount!=null && useSumCount!=""){
    					 /*BigDecimal s = new BigDecimal(1000);
	        			BigDecimal sumsl = new BigDecimal(useSumCount);
	        			BigDecimal sumCkslss = sumsl.divide(s);*/
    					 double use_num = Double.parseDouble(useSumCount); //使用数量(吨)divide
    					 if (use_num <= sum_num) {
    						 sysl = sum_num - use_num;
    						 obj.setRemainingNumber("0");//页面展示
    						 obj.setInCount(sysl+"");
    					 } else if (use_num > sum_num){
    						 sysl = use_num - sum_num;
    						 obj.setRemainingNumber("超出:"+sysl);
    						 obj.setInCount("0");//页面展示
    					 }
    				 }else{
    					 //没有使用过
    					 obj.setRemainingNumber("0"); //页面展示剩余数量
    					 obj.setInCount(sum_num+""); //总数量
    				 }
    			}
    			modelMap.put("status", "success");
    			modelMap.put("list", noticeReceiveList);
    		} catch (IOException e) {
    			e.printStackTrace();
    			modelMap.put("status", "error");
    			modelMap.put("msg", "查询失败！");
    		}
    	}
    	
    	return modelMap;
    }
    
    
    /**
     * 储备粮管理 剩余数量(出库通知单)
     * @param id 主键
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @SuppressWarnings("unchecked")
    @RequestMapping(value="/getCkRemainNumber",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="储备粮管理下查询剩余数量(出库通知单)", notes = "剩余数量(出库通知单)")
    @ApiImplicitParams({
    	@ApiImplicitParam(name = "receiveNoticeListJson", value = "选中的数据", paramType = "query")
    })
    public Map<String, Object> getCkRemainNumber(String receiveNoticeListJson) {
    	Map<String, Object> modelMap = new HashMap<String, Object>();
    	
    	ObjectMapper mapper = new ObjectMapper();
    	List<BusinessNoticeReceive> noticeReceiveList = null;
    	BusinessNoticeReceive obj = null;
    	if (receiveNoticeListJson != null) {
    		try {
    			noticeReceiveList = (List<BusinessNoticeReceive>)mapper.readValue(receiveNoticeListJson, new TypeReference<List<BusinessNoticeReceive>>(){});
    			
    			for (int i = 0; i < noticeReceiveList.size(); i++) {
    				Map<String, Object> map = new HashMap<String, Object>();
    				obj = noticeReceiveList.get(i);
    				map.put("houseId", obj.getHouseId());
    				map.put("warehouseId", obj.getWareHouseId());
    				map.put("grainKind", obj.getGrainKind());
    				map.put("grainAttribute", obj.getGrainAttribute());
    				map.put("grainGrade", obj.getGrainGrade());
    				map.put("orgId", obj.getOrgId());
    				map.put("jstzdId", obj.getId());
    				String useSumCount = detailService.getCkUseCount(map);//使用数量(吨)
    				double sysl = 0d;
    				double sum_num = Double.parseDouble(obj.getCount()); //总的数量(吨)
    				if(useSumCount!=null && useSumCount!=""){
    					double use_num = Double.parseDouble(useSumCount); //使用数量(吨)divide
    					if (use_num <= sum_num) {
    						sysl = sum_num - use_num;
    						obj.setOutRemainingNumber("0");//页面展示
    						obj.setOutCount(sysl+"");
    					} else if (use_num > sum_num){
    						sysl = use_num - sum_num;
    						obj.setOutRemainingNumber("超出:"+sysl);
    						obj.setOutCount("0");//页面展示
    					}
    				}else{
    					//没有使用过
    					obj.setOutRemainingNumber("0"); //页面展示剩余数量
    					obj.setOutCount(sum_num+""); //总数量
    				}
    			}
    			modelMap.put("status", "success");
    			modelMap.put("list", noticeReceiveList);
    		} catch (IOException e) {
    			e.printStackTrace();
    			modelMap.put("status", "error");
    			modelMap.put("msg", "查询失败！");
    		}
    	}
    	
    	return modelMap;
    }
    
}
