package com.chinaitop.depot.business.controller;


import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.act.service.ActTaskService;
import com.chinaitop.depot.business.model.BusinessContract;
import com.chinaitop.depot.business.model.BusinessFile;
import com.chinaitop.depot.business.model.BusinessStoreWareDetail;
import com.chinaitop.depot.business.model.enums.StoreWareType;
import com.chinaitop.depot.business.model.vo.BusinessContractAuditVO;
import com.chinaitop.depot.business.service.BusinessFileService;
import com.chinaitop.depot.business.service.BusinessStoreWareDetailService;
import com.chinaitop.depot.business.service.GrainContractService;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 储备粮管理-合同管理
 * @author My   Sunshine
 *
 */
@RestController
@RequestMapping(value="/depot/business/grainContract")
@Api(value= "GrainContractController", description = "储备粮管理-合同管理")

public class GrainContractController {
    
    @Resource
    private GrainContractService grainContractService;
    @Resource
    private BusinessStoreWareDetailService businessStoreWareDetailService;
    @Resource
    private ActTaskService actTaskService;
    @Autowired
    private BusinessFileService businessFileService;
    
    /**
     * 合同申请列表.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     * @throws Exception 
     */
    @RequestMapping(value="/getList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="合同申请列表", notes = "查询数据列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "query")
	})
    public PageInfo<BusinessContract> getList(Integer pageNum, Integer pageSize,HttpServletRequest request, BusinessContract contract ,Integer userId) throws Exception {
       
        List<BusinessContract> list = grainContractService.queryByExample(pageNum, pageSize, userId, contract, "FIELD(audit_state,0,3,1,2,4 ), id desc", "");
        PageInfo<BusinessContract> pageInfo = new PageInfo<BusinessContract>(list);
        return pageInfo;
    }

    /**
     * 根据id查询合同.
     * @param id 主键
     * @return
     */
    @RequestMapping(value="/edit", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="根据id查找详情", notes = "根据id查找")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "合同id", paramType = "query")
	})
    public Map<String, Object> edit(Integer id) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        if (id != null) {
            BusinessContract contract = grainContractService.findById(id);
            modelMap.put("contract", contract);
            List<BusinessFile> fileList= businessFileService.getByBid(id, "cblContract", null, null);
            modelMap.put("fileList", fileList);
        }
        return modelMap;
    }
    
    /**
     * 保存
     * @param contractJson 合同数据json
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @RequestMapping(value="/save", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="保存数据", notes = "保存")
    @SuppressWarnings("unchecked")
    @ApiImplicitParams({
    	@ApiImplicitParam(name="contractJson", value="主表数据", paramType="form"),
    	@ApiImplicitParam(name="businessStoreWareDetailJson", value="子表数据", paramType="form"),
    	@ApiImplicitParam(name="userId", value="当前登录人的id", paramType="form"),
    	@ApiImplicitParam(name="realName", value="当前登录人的name", paramType="form"),
    	@ApiImplicitParam(name="depotInfoOrgId", value="当前登录人的组织id", paramType="form")
    })
    public Map<String, Object> save(String contractJson, String businessStoreWareDetailJson,
            HttpServletRequest request,Integer userId,String realName,Integer depotInfoOrgId,String fileIds, String deleteFileIds) 
            throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        try {
            // JSON字符串转对象
            ObjectMapper mapper = new ObjectMapper();
            BusinessContract contract = (BusinessContract)mapper.readValue(contractJson, BusinessContract.class);
            
            // 明细.
            List<BusinessStoreWareDetail> storeWareDetailList = null;
            if (businessStoreWareDetailJson != null) {
                storeWareDetailList = (List<BusinessStoreWareDetail>)mapper.readValue(businessStoreWareDetailJson, new TypeReference<List<BusinessStoreWareDetail>>(){});
            }
            if (contract.getId() == null) {
                Map<String, Object> returnMap = grainContractService.add(contract, storeWareDetailList,  userId, realName, depotInfoOrgId,fileIds);
                if (returnMap != null && returnMap.get("error") != null) {
                    modelMap.put("status", "error");
                    modelMap.put("msg", returnMap.get("error"));
                    return modelMap;
                }
                modelMap.put("id", returnMap.get("id"));
            } else {
            	grainContractService.update(contract, storeWareDetailList,fileIds,deleteFileIds);
            }
            modelMap.put("status", "success");
        } catch (Exception e) {
            e.printStackTrace();
            modelMap.put("status", "error");
            modelMap.put("msg", "保存失败！");
            return modelMap;
        }
        return modelMap;
    }
    
    /**
     * 审批通过列表.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @param contract 合同体
     * @param request
     * @return
     */
    @RequestMapping(value="/getAuditPassList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="合同审批通过列表", notes = "查询数据列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query")
	})
    public PageInfo<BusinessContract> getAuditPassList(Integer pageNum, Integer pageSize, BusinessContract contract,
            Integer billType, String lyflag ,HttpServletRequest request) {
        List<BusinessContract> list = grainContractService.getAuditPassList(pageNum, pageSize, contract, billType,lyflag);
        PageInfo<BusinessContract> pageInfo = new PageInfo<BusinessContract>(list);
        return pageInfo;
    }
    
    /**
     * 获取可以合同变更的申请列表.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
    @RequestMapping(value="/getCanChangeList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="获取合同可以变更的申请列表", notes = "查询数据列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "query")
	})
    public PageInfo<BusinessContract> getCanChangeList(Integer pageNum, Integer pageSize,Integer userId,HttpServletRequest request) {

    	List<BusinessContract> new_list = grainContractService.queryContractChangeList(userId, pageNum, pageSize);

        PageInfo<BusinessContract> pageInfo = new PageInfo<BusinessContract>(new_list);

        return pageInfo;
    }
    
    
    /**
     * 合同变更列表.
     * 注：如果一条合同变更多次，只会显示最新一次的变更.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
    @RequestMapping(value="/getChangeList", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="合同变更列表", notes = "查询数据列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "query")
	})
    public PageInfo<BusinessContract> getChangeList(Integer pageNum, Integer pageSize,
            BusinessContract contract, Integer userId,HttpServletRequest request) {
    	
        List<BusinessContract> list = grainContractService.queryByExample(pageNum, pageSize, userId, contract, "FIELD(audit_state,0,3,1,2,4 ), id desc", "change");
        PageInfo<BusinessContract> pageInfo = new PageInfo<BusinessContract>(list);
        return pageInfo;
    }
    
    
    
    /**
     * 根据id查询变更的合同.
     * @param id 主键
     * @return
     */
    @RequestMapping(value="/editForChange",produces = MediaType.APPLICATION_JSON_VALUE,  method=RequestMethod.GET)
	@ApiOperation(value="根据id查找变更的合同", notes = "根据id查找")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "id", paramType = "query")
	})
    public Map<String, Object> editForChange(Integer id) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        if (id != null) {
        	// 获取当前合同主表数据
            BusinessContract contract = grainContractService.findById(id);
            modelMap.put("contract", contract);
            
            //附件
            List<BusinessFile> fileList= businessFileService.getByBid(id, "cblContract", null, null);
            modelMap.put("fileList", fileList);
            
            // 获取当前合同的历次修改原因，内容，时间.
            List<BusinessContract> changeDetailList = grainContractService.getChangeDetailList(id, "edit");
            modelMap.put("changeDetailList", changeDetailList);
            
            // 仓号货位号数量质量标准.
            List<BusinessStoreWareDetail> storeWareDetailList= businessStoreWareDetailService.getByZidAndType(id, StoreWareType.cblContract.getValue());
            if(contract.getPlanBid()!=null){
            	List<BusinessStoreWareDetail> planList= businessStoreWareDetailService.getByZidAndType(contract.getPlanBid(), StoreWareType.plan.getValue());

            	/* 根据引用内容获取被引用合同明细的剩余数量 */
            	Map<String, Object> param_map = new HashMap<String, Object>();
            	param_map.put("plan_id", contract.getPlanBid());
                List<Map<String, Object>> map = businessStoreWareDetailService.selectPlanUseCount(param_map);
                BusinessStoreWareDetail obj = null;
                double sysl = 0d;
                double sum_use_num = 0d; //已使用总量
                double plan_sum_num = 0d; //原计划数量
                for (int i = 0; i < planList.size(); i++) {
                	obj = planList.get(i);
                	for (Map<String, Object> m : map) {
                		if(null == m.get("warehouse_id")){//油
                    		if (m.get("house_id").equals(obj.getHouseId())) {
                    			if (!"".equals(m.get("in_count")) && null != m.get("in_count")) {
                    				plan_sum_num = Double.parseDouble(obj.getInCount());
                    				sum_use_num = Double.parseDouble(m.get("in_count")+"");
                    				if (sum_use_num <= plan_sum_num) {
                    					sysl = plan_sum_num - sum_use_num;
                    					obj.setRemainingNumber(sysl+""); //页面上明细列表的数量
                    					if (sysl == 0) {
                    						obj.setRemainingNumber("0");
                    					}
                    				} else if (sum_use_num > plan_sum_num){
                    					sysl = sum_use_num - plan_sum_num;
                    					obj.setRemainingNumber("超出:" + sysl);
                    				}
                    			}
                    			if (!"".equals(m.get("out_count")) && null != m.get("out_count")) {
                    				plan_sum_num = Double.parseDouble(obj.getOutCount());
                    				sum_use_num = Double.parseDouble(m.get("out_count")+"");
                    				if (sum_use_num <= plan_sum_num) {
                    					sysl = plan_sum_num - sum_use_num;
                    					obj.setOutRemainingNumber(""+sysl);
                    				} else if (sum_use_num > plan_sum_num){
                    					sysl = sum_use_num - plan_sum_num;
                    					obj.setOutRemainingNumber("超出:" + sysl);
                    				}
                    			}
                    		}
                    	
                		}else{
                    		if (m.get("house_id").equals(obj.getHouseId()) && m.get("warehouse_id").equals(obj.getWarehouseId())) {
                    			if (!"".equals(m.get("in_count")) && null != m.get("in_count")) {
                    				plan_sum_num = Double.parseDouble(obj.getInCount());
                    				sum_use_num = Double.parseDouble(m.get("in_count")+"");
                    				if (sum_use_num <= plan_sum_num) {
                    					sysl = plan_sum_num - sum_use_num;
                    					obj.setRemainingNumber(sysl+""); //页面上明细列表的数量
                    					if (sysl == 0) {
                    						obj.setRemainingNumber("0");
                    					}
                    				} else if (sum_use_num > plan_sum_num){
                    					sysl = sum_use_num - plan_sum_num;
                    					obj.setRemainingNumber("超出:" + sysl);
                    				}
                    			}
                    			if (!"".equals(m.get("out_count")) && null != m.get("out_count")) {
                    				plan_sum_num = Double.parseDouble(obj.getOutCount());
                    				sum_use_num = Double.parseDouble(m.get("out_count")+"");
                    				if (sum_use_num <= plan_sum_num) {
                    					sysl = plan_sum_num - sum_use_num;
                    					obj.setOutRemainingNumber(""+sysl);
                    				} else if (sum_use_num > plan_sum_num){
                    					sysl = sum_use_num - plan_sum_num;
                    					obj.setOutRemainingNumber("超出:" + sysl);
                    				}
                    			}
                    		}
                    	
                		}
                	}
                }

                for (BusinessStoreWareDetail contractDetail : storeWareDetailList) {
    				for (BusinessStoreWareDetail planDetail : planList) {
    					if(null == contractDetail.getWarehouseId()){
    						if(contractDetail.getHouseId().equals(planDetail.getHouseId())){   
                				contractDetail.setRemainingNumber(planDetail.getRemainingNumber());
                				contractDetail.setOutRemainingNumber(planDetail.getOutRemainingNumber());
                			}
    					}else{
    						if(contractDetail.getHouseId().equals(planDetail.getHouseId())  && 
                					contractDetail.getWarehouseId().equals(planDetail.getWarehouseId())){
       
                				contractDetail.setRemainingNumber(planDetail.getRemainingNumber());
                				contractDetail.setOutRemainingNumber(planDetail.getOutRemainingNumber());
                			}
    					}
    				}
    			}
			}
            modelMap.put("storeWareDetailList", storeWareDetailList);
        }
        return modelMap;
    }
    
    /**
     * 根据id查询.
     * @param id 主键
     * @return
     */
    @RequestMapping(value="/addForChange",produces = MediaType.APPLICATION_JSON_VALUE,  method=RequestMethod.GET)
   	@ApiOperation(value="根据id查找，获取历次变更详情(新增时，不显示当前id对应的历史记录)", notes = "根据id查找")
   	@ApiImplicitParams({
   		@ApiImplicitParam(name = "id", value = "id", paramType = "query")
   	})
    public Map<String, Object> addForChange(Integer id) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        if (id != null) {
        	// 获取当前合同主数据
            BusinessContract contract = grainContractService.findById(id);
            // 获取当前合同的历次修改原因，内容，时间.
            List<BusinessContract> changeDetailList = grainContractService.getChangeDetailList(id, "add");
            // 获取当前合同明细数据
            List<BusinessStoreWareDetail> storeWareDetailList= businessStoreWareDetailService.getByZidAndType(id, StoreWareType.cblContract.getValue());
            if(contract.getPlanBid()!=null){
            	// 查询当前合同所在的计划的明细信息
            	List<BusinessStoreWareDetail> planList= businessStoreWareDetailService.getByZidAndType(contract.getPlanBid(), StoreWareType.plan.getValue());

            	/* 根据引用内容获取被引用合同明细的剩余数量 */
            	Map<String, Object> param_map = new HashMap<String, Object>();
            	param_map.put("plan_id", contract.getPlanBid());
                List<Map<String, Object>> map = businessStoreWareDetailService.selectPlanUseCount(param_map);
                BusinessStoreWareDetail obj = null;
                double sysl = 0d;
                double sum_use_num = 0d; //已使用总量
                double plan_sum_num = 0d; //原计划数量
                for (int i = 0; i < planList.size(); i++) {
                	obj = planList.get(i);
                	for (Map<String, Object> m : map) {
                		if(null == m.get("warehouse_id")){//油
                    		if (m.get("house_id").equals(obj.getHouseId())) {
                    			if (!"".equals(m.get("in_count")) && null != m.get("in_count")) {
                    				plan_sum_num = Double.parseDouble(obj.getInCount());
                    				sum_use_num = Double.parseDouble(m.get("in_count")+"");
                    				if (sum_use_num <= plan_sum_num) {
                    					sysl = plan_sum_num - sum_use_num;
                    					obj.setRemainingNumber(sysl+""); //页面上明细列表的数量
                    					if (sysl == 0) {
                    						obj.setRemainingNumber("0");
                    					}
                    				} else if (sum_use_num > plan_sum_num){
                    					sysl = sum_use_num - plan_sum_num;
                    					obj.setRemainingNumber("超出:" + sysl);
                    				}
                    			}
                    			if (!"".equals(m.get("out_count")) && null != m.get("out_count")) {
                    				plan_sum_num = Double.parseDouble(obj.getOutCount());
                    				sum_use_num = Double.parseDouble(m.get("out_count")+"");
                    				if (sum_use_num <= plan_sum_num) {
                    					sysl = plan_sum_num - sum_use_num;
                    					obj.setOutRemainingNumber(""+sysl);
                    				} else if (sum_use_num > plan_sum_num){
                    					sysl = sum_use_num - plan_sum_num;
                    					obj.setOutRemainingNumber("超出:" + sysl);
                    				}
                    			}
                    		}
                    	
                		}else{//粮食
                    		if (m.get("house_id").equals(obj.getHouseId()) && m.get("warehouse_id").equals(obj.getWarehouseId())) {
                    			if (!"".equals(m.get("in_count")) && null != m.get("in_count")) {
                    				plan_sum_num = Double.parseDouble(obj.getInCount());
                    				sum_use_num = Double.parseDouble(m.get("in_count")+"");
                    				if (sum_use_num <= plan_sum_num) {
                    					sysl = plan_sum_num - sum_use_num;
                    					obj.setRemainingNumber(sysl+""); //页面上明细列表的数量
                    					if (sysl == 0) {
                    						obj.setRemainingNumber("0");
                    					}
                    				} else if (sum_use_num > plan_sum_num){
                    					sysl = sum_use_num - plan_sum_num;
                    					obj.setRemainingNumber("超出:" + sysl);
                    				}
                    			}
                    			if (!"".equals(m.get("out_count")) && null != m.get("out_count")) {
                    				plan_sum_num = Double.parseDouble(obj.getOutCount());
                    				sum_use_num = Double.parseDouble(m.get("out_count")+"");
                    				if (sum_use_num <= plan_sum_num) {
                    					sysl = plan_sum_num - sum_use_num;
                    					obj.setOutRemainingNumber(""+sysl);
                    				} else if (sum_use_num > plan_sum_num){
                    					sysl = sum_use_num - plan_sum_num;
                    					obj.setOutRemainingNumber("超出:" + sysl);
                    				}
                    			}
                    		}
                    	
                		}
                	}
                }
            	
                for (BusinessStoreWareDetail contractDetail : storeWareDetailList) {
    				for (BusinessStoreWareDetail planDetail : planList) {
    					if(null == contractDetail.getWarehouseId()){//油
    						if(contractDetail.getHouseId().equals(planDetail.getHouseId()) ){  
                				contractDetail.setRemainingNumber(planDetail.getRemainingNumber());
                				contractDetail.setOutRemainingNumber(planDetail.getOutRemainingNumber());
                			}
    					}else{
    						if(contractDetail.getHouseId().equals(planDetail.getHouseId())  && 
                					contractDetail.getWarehouseId().equals(planDetail.getWarehouseId())){
       
                				contractDetail.setRemainingNumber(planDetail.getRemainingNumber());
                				contractDetail.setOutRemainingNumber(planDetail.getOutRemainingNumber());
                			}
    					}
    				}
    			}
    		}
            List<BusinessFile> fileList= businessFileService.getByBid(id, "cblContract", null, null);
            modelMap.put("fileList", fileList);
            
            modelMap.put("contract", contract);
            modelMap.put("changeDetailList", changeDetailList);
            modelMap.put("storeWareDetailList", storeWareDetailList);
        }
        return modelMap;
    }

    

    /**
     * 删除
     * @param id 主键
     * @param processInstanceId 流程实例id
     * @param deleteReason 删除原因
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @RequestMapping(value="/remove",produces = MediaType.APPLICATION_JSON_VALUE,  method=RequestMethod.POST)
    @ApiOperation(value="删除合同", notes = "根据id删除")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "合同id", paramType = "query"),
            @ApiImplicitParam(name = "processInstanceId", value = "流程实例id", paramType = "query"),
            @ApiImplicitParam(name = "deleteReason", value = "删除原因", paramType = "query"),
            @ApiImplicitParam(name = "planBid", value = "计划id", paramType = "query")
    })
    public Map<String, Object> remove(Integer id, String processInstanceId, String deleteReason,Integer planBid)
            throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        if (id != null) {
        	grainContractService.remove(id, processInstanceId, deleteReason,planBid);
        }
        modelMap.put("status", "success");
        return modelMap;
    }

    /**
     * 保存合同变更
     * @param contractJson 合同数据json
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @RequestMapping(value="/saveChange", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="保存合同变更", notes = "保存")
    @SuppressWarnings("unchecked")
    @ApiImplicitParams({
    	@ApiImplicitParam(name="contractJson", value="主表数据", paramType="form"),
    	@ApiImplicitParam(name="businessStoreWareDetailJson", value="子表数据", paramType="form"),
    	@ApiImplicitParam(name="userId", value="当前登录人的id", paramType="form"),
    	@ApiImplicitParam(name="userName", value="当前登录人的name", paramType="form"),
    	@ApiImplicitParam(name="depotInfoOrgId", value="当前登录人的组织id", paramType="form"),
            @ApiImplicitParam(name="id", value="原id", paramType="form")
    })
    public Map<String, Object> saveChange(String contractJson, String businessStoreWareDetailJson,
    		Integer userId,String userName, String realName,Integer depotInfoOrgId,Integer id,String fileIds, String deleteFileIds) throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();

        try {
            // JSON字符串转对象
            ObjectMapper mapper = new ObjectMapper();
            BusinessContract contract = (BusinessContract)mapper.readValue(contractJson, BusinessContract.class);

            // 明细.
            List<BusinessStoreWareDetail> storeWareDetailList = null;
            if (businessStoreWareDetailJson != null) {
                storeWareDetailList = (List<BusinessStoreWareDetail>)mapper.readValue(businessStoreWareDetailJson, new TypeReference<List<BusinessStoreWareDetail>>(){});
            }

            if (id == null) {
                throw new RuntimeException("原合同id为空，保存失败！");
            } else {
            	contract.setCreateUnit("1"); //储备粮合同
                Map<String, Object> returnMap =  grainContractService.saveChange(contract, storeWareDetailList,  userId, userName,  realName, depotInfoOrgId,id,fileIds);
                modelMap.put("id",returnMap.get("id"));
            }
            modelMap.put("status", "success");
        } catch (Exception e) {
            e.printStackTrace();
            modelMap.put("status", "error");
            modelMap.put("msg", "保存失败！");
        }
        return modelMap;
    }

    /**
     * 修改合同变更.
     * @param contractJson 合同数据json
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @RequestMapping(value="/updateChange", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="修改合同变更", notes = "保存")
    @SuppressWarnings("unchecked")
    @ApiImplicitParams({
    	@ApiImplicitParam(name="contractJson", value="主表数据", paramType="form"),
    	@ApiImplicitParam(name="businessStoreWareDetailJson", value="子表数据", paramType="form")
    })
    public Map<String, Object> updateChange(String contractJson, String businessStoreWareDetailJson,String fileIds, String deleteFileIds)
            throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        try {
            // JSON字符串转对象
            ObjectMapper mapper = new ObjectMapper();
            BusinessContract contract = (BusinessContract)mapper.readValue(contractJson, BusinessContract.class);

            // 明细.
            List<BusinessStoreWareDetail> storeWareDetailList = null;
            if (businessStoreWareDetailJson != null) {
                storeWareDetailList = (List<BusinessStoreWareDetail>)mapper.readValue(businessStoreWareDetailJson, new TypeReference<List<BusinessStoreWareDetail>>(){});
            }

            if (contract.getId() == null) {
                throw new RuntimeException("合同id为空，保存失败！");
            } else {
            	grainContractService.update(contract, storeWareDetailList,fileIds,deleteFileIds);
            }
            modelMap.put("status", "success");
        } catch (Exception e) {
            e.printStackTrace();
            modelMap.put("status", "error");
            modelMap.put("msg", "保存失败！");
        }
        return modelMap;
    }

    /**
     * 删除合同变更.
     * @param id 主键
     * @param originalContractBid 原合同id
     * @param processInstanceId 流程实例id
     * @param deleteReason 删除原因
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @RequestMapping(value="/changeContractRemove",produces = MediaType.APPLICATION_JSON_VALUE,  method=RequestMethod.POST)
	@ApiOperation(value="删除合同变更", notes = "根据id删除")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "合同id", paramType = "query"),
		@ApiImplicitParam(name = "originalContractBid", value = "原合同id", paramType = "query"),
		@ApiImplicitParam(name = "processInstanceId", value = "流程实例id", paramType = "query"),
		@ApiImplicitParam(name = "deleteReason", value = "删除原因", paramType = "query"),
	})
    public Map<String, Object> changeContractRemove(Integer id, Integer originalContractBid,
    		String processInstanceId, String deleteReason,Integer planBid)
    		throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        if (id != null) {
            modelMap = grainContractService.changeContractRemove(id, originalContractBid, processInstanceId, deleteReason,planBid);
            if ("success".equals(modelMap.get("status"))) {
                try {
                    if (deleteReason!=null) {
                        actTaskService.delRunTask(processInstanceId, deleteReason);
                    } else {
                        deleteReason="删除原因无";
                        actTaskService.delRunTask(processInstanceId, deleteReason);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        } else {
            modelMap.put("status", "error");
            modelMap.put("msg", "id为空，删除失败！");
        }
        return modelMap;
    }
    
    /**
     * 提交
     * @param contract 合同
     * @param assignee 下一个审批人
     * @return
     */
    @RequestMapping(value="/submit", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="提交", notes = "提交")
    @ApiImplicitParams({
		@ApiImplicitParam(name = "contract", value = "合同", paramType = "query"),
		@ApiImplicitParam(name = "assignee", value = "下一个审批人", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "query"),
		@ApiImplicitParam(name = "realName", value = "当前登录人name", paramType = "query")
	})
    public Map<String, Object> submit(BusinessContract contract,String assignee,Integer userId,String realName,HttpServletRequest request) {
    	//UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
        Map<String, Object> returnMap = grainContractService.submit(contract, assignee, userId, realName);
        return returnMap;
    }
    
    /**
     * 启用
     * @param id 主键.
     * @return
     */
    @RequestMapping(value="/application", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="合同启用", notes = "合同启用")
    @ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "合同id", paramType = "query")
	})
    public Map<String, Object> application(Integer id) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        Map<String, String> returnMap = grainContractService.application(id);
        if (StringUtils.isNoneBlank(returnMap.get("error"))) {
            modelMap.put("status", "error");
            modelMap.put("msg", returnMap.get("error"));
        } else {
            modelMap.put("status", "success");
        }
        return modelMap;
    }
    
    /**
     * 暂停
     * @param id 主键.
     * @return
     */
    @RequestMapping(value="/discontinue", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="合同暂停", notes = "合同暂停")
    @ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "合同id", paramType = "query")
	})
    public Map<String, Object> discontinue(Integer id,String refuseReason) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        grainContractService.discontinue(id,refuseReason);
        modelMap.put("status", "success");
        return modelMap;
    }
    
    /**
     * 终止
     * @param id 主键.
     * @return
     */
    @RequestMapping(value="/finish", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
   	@ApiOperation(value="合同终止", notes = "合同终止")
   	@ApiImplicitParams({
   		@ApiImplicitParam(name = "id", value = "合同id", paramType = "query")
   	})
    public Map<String, Object> finish(Integer id,String refuseReason) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        grainContractService.finish(id,refuseReason);
        modelMap.put("status", "success");
        return modelMap;
    }
    
    /**
     * 校验合同编号，是否可用.
     * @param id 主键.
     * @param contractNumber 合同编号.
     * @return true 为可用，false为不可用.
     */
    @RequestMapping(value="/validContractNumber", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="校验合同编号，是否可用", notes = "校验合同编号，是否可用")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "合同id", paramType = "query"),
		@ApiImplicitParam(name = "contractNumber", value = "合同编号", paramType = "query")
	})
    public Boolean validContractNumber(Integer id, String contractNumber) {
        return grainContractService.validContractNumber(id, contractNumber);
    }
    
    
    
    /**
     * 审批通过列表(所有类型的合同，用于移动端).
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @param contract 合同体
     * @param request
     * @return
     */
    @RequestMapping(value="/getAuditPassListApp",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="合同审批通过列表", notes = "查询数据列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "billType", value = "合同类型(收购/销售)", paramType = "query")
	})
    public PageInfo<BusinessContract> getAuditPassListApp(Integer pageNum, Integer pageSize, BusinessContract contract,
            Integer billType, HttpServletRequest request) {
        List<BusinessContract> list = grainContractService.getAuditPassListApp(pageNum, pageSize, contract, billType);
        PageInfo<BusinessContract> pageInfo = new PageInfo<BusinessContract>(list);
        return pageInfo;
    }
    
    
}
