package com.chinaitop.depot.business.controller;


import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.business.model.BusinessDeliveryStorageNotice;
import com.chinaitop.depot.business.model.BusinessStoreWareDetail;
import com.chinaitop.depot.business.model.enums.AuditType;
import com.chinaitop.depot.business.model.enums.StoreWareType;
import com.chinaitop.depot.business.model.vo.BusinessDeliveryStorageNoticeAuditVO;
import com.chinaitop.depot.business.service.BusinessStoreWareDetailService;
import com.chinaitop.depot.business.service.GrainNoticeService;
import com.chinaitop.depot.business.service.SystemOutsideDataService;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 储备粮管理-通知单
 * @author My   Sunshine
 *
 */
@RestController
@RequestMapping(value="/depot/business/grainNotice")
@Api(value= "GrainNoticeController", description = "储备粮管理-出入库通知单管理")

public class GrainNoticeController {
	public static final Logger logger = LoggerFactory.getLogger(GrainNoticeController.class);
    @Resource
    private GrainNoticeService deliveryStorageNoticeService;
    @Resource
    private BusinessStoreWareDetailService businessStoreWareDetailService;
    @Resource
	private SystemOutsideDataService systemOutsideDataService;
    
    /**
     * 申请列表.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
    @RequestMapping(value="/getList", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="申请列表", notes = "查询数据列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "query")
	})
    public PageInfo<BusinessDeliveryStorageNotice> getList(Integer pageNum, Integer pageSize,
            HttpServletRequest request, BusinessDeliveryStorageNotice notice,Integer userId) {
        /*UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
        Integer userId = userInfo.getUserId();*/
        List<BusinessDeliveryStorageNotice> list = deliveryStorageNoticeService.queryByExample(pageNum, pageSize, userId, notice, "FIELD(audit_state,0,3,1,2,4 ), id desc","");  
        PageInfo<BusinessDeliveryStorageNotice> pageInfo = new PageInfo<BusinessDeliveryStorageNotice>(list);
        return pageInfo;
    }
    

    /**
     * 审批通过列表.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
    @RequestMapping(value="/getAuditPassList", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="通知单审批通过列表", notes = "查询数据列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query")
	})
    public PageInfo<BusinessDeliveryStorageNotice> getAuditPassList(Integer pageNum, Integer pageSize, BusinessDeliveryStorageNotice notice) {
        notice.setAuditState(AuditType.AGREE.getAuditCode());
        List<BusinessDeliveryStorageNotice> list = deliveryStorageNoticeService.getAuditPassList(pageNum, pageSize, notice);
        PageInfo<BusinessDeliveryStorageNotice> pageInfo = new PageInfo<BusinessDeliveryStorageNotice>(list);
        return pageInfo;
    }
   
    /**
     * 获取进度列表.
     * @param pageNum
     * @param pageSize
     * @param deliveryStorageNotice
     * @return
     */
    @RequestMapping(value="/getScheduleList",produces = MediaType.APPLICATION_JSON_VALUE,  method=RequestMethod.GET)
	@ApiOperation(value="获取通知单进度列表", notes = "查询数据列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query")
	})
    public PageInfo<BusinessDeliveryStorageNoticeAuditVO> getScheduleList(Integer pageNum, Integer pageSize, BusinessDeliveryStorageNotice deliveryStorageNotice) {
        PageInfo<BusinessDeliveryStorageNoticeAuditVO> pageInfo = deliveryStorageNoticeService.getScheduleList(pageNum, pageSize, deliveryStorageNotice);
        return pageInfo;
    }
    /**
     * 根据id查询.
     * @param id 主键
     * @return
     */
    @RequestMapping(value="/edit", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="根据id查找", notes = "根据id查找")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "通知单id", paramType = "query")
	})
    public Map<String, Object> edit(Integer id) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        if (id != null) {
            BusinessDeliveryStorageNotice deliveryStorageNotice = deliveryStorageNoticeService.findById(id);
            modelMap.put("deliveryStorageNotice", deliveryStorageNotice);
            
            List<BusinessStoreWareDetail> list = new ArrayList<BusinessStoreWareDetail>();
            Integer plan_id = deliveryStorageNotice.getPlanBid(); //计划ID
            Integer contract_id = deliveryStorageNotice.getContractBid(); //合同ID
            Map<String, Object> param_map = new HashMap<String, Object>();
            List<Map<String, Object>> map = null;
            if (null != contract_id) {
            	list = businessStoreWareDetailService.getByZidAndType(contract_id, "contract");
            	param_map.put("contract_id", contract_id);
            	map = businessStoreWareDetailService.selectContractUseCount(param_map);
            } else if (null != plan_id) {
            	list = businessStoreWareDetailService.getByZidAndType(plan_id, "plan");
            	param_map.put("plan_id", plan_id);
            	map = businessStoreWareDetailService.selectPlanUseCount(param_map);
            }
            
            BusinessStoreWareDetail obj = null;
            double sysl = 0d;
            double sum_use_num = 0d; //已使用总量
            double plan_sum_num = 0d; //原计划数量
            for (int i = 0; i < list.size(); i++) {
            	obj = list.get(i);
            	for (Map<String, Object> m : map) {
            		if(null == m.get("warehouse_id")){//油
                		if (obj.getHouseId().equals(m.get("house_id"))) {
                			if ("1".equals(deliveryStorageNotice.getBillType())) {
                				plan_sum_num = Double.parseDouble(obj.getInCount());
                				sum_use_num = Double.parseDouble(m.get("in_count")+"");
                				if (sum_use_num <= plan_sum_num) {
            						sysl = plan_sum_num - sum_use_num;
            						obj.setRemainingNumber(sysl+""); //页面上明细列表的剩余数量
            					} else if (sum_use_num > plan_sum_num){
            						sysl = sum_use_num - plan_sum_num;
            						obj.setRemainingNumber("超出:"+sysl);
            					}
                			}
                			if ("3".equals(deliveryStorageNotice.getBillType())) {
                				plan_sum_num = Double.parseDouble(obj.getOutCount());
                				sum_use_num = Double.parseDouble(m.get("out_count")+"");
                				if (sum_use_num <= plan_sum_num) {
            						sysl = plan_sum_num - sum_use_num;
            						obj.setOutRemainingNumber(sysl+""); //页面上明细列表的剩余数量
            					} else if (sum_use_num > plan_sum_num){
            						sysl = sum_use_num - plan_sum_num;
            						obj.setOutRemainingNumber("超出:"+sysl);
            					}
                			}
                		}
                	
            		}else{//粮食
                		if (obj.getHouseId().equals(m.get("house_id")) && obj.getWarehouseId().equals(m.get("warehouse_id"))) {
                			if ("1".equals(deliveryStorageNotice.getBillType())) {
                				plan_sum_num = Double.parseDouble(obj.getInCount());
                				sum_use_num = Double.parseDouble(m.get("in_count")+"");
                				if (sum_use_num <= plan_sum_num) {
            						sysl = plan_sum_num - sum_use_num;
            						obj.setRemainingNumber(sysl+""); //页面上明细列表的剩余数量
            					} else if (sum_use_num > plan_sum_num){
            						sysl = sum_use_num - plan_sum_num;
            						obj.setRemainingNumber("超出:"+sysl);
            					}
                			}
                			if ("3".equals(deliveryStorageNotice.getBillType())) {
                				plan_sum_num = Double.parseDouble(obj.getOutCount());
                				sum_use_num = Double.parseDouble(m.get("out_count")+"");
                				if (sum_use_num <= plan_sum_num) {
            						sysl = plan_sum_num - sum_use_num;
            						obj.setOutRemainingNumber(sysl+""); //页面上明细列表的剩余数量
            					} else if (sum_use_num > plan_sum_num){
            						sysl = sum_use_num - plan_sum_num;
            						obj.setOutRemainingNumber("超出:"+sysl);
            					}
                			}
                		}
                	
            		}
            	}
            }
            
            //明细
            List<BusinessStoreWareDetail> transferNoticeList= businessStoreWareDetailService.getByZidAndType(id, StoreWareType.transferNotice.getValue());
            /*for (int i = 0; i < list.size(); i++) {
            	obj = list.get(i);
            	for (BusinessStoreWareDetail m : transferNoticeList) {
            		if (obj.getHouseId().equals(m.getHouseId()) && obj.getWarehouseId().equals(m.getWarehouseId())) {
            			if ("1".equals(deliveryStorageNotice.getBillType())) {
            				m.setRemainingNumber(obj.getRemainingNumber());
            			}
            			if ("3".equals(deliveryStorageNotice.getBillType())) {
            				m.setOutRemainingNumber(obj.getOutRemainingNumber());
            			}
            		}
            	}
            }*/
            List<BusinessStoreWareDetail> noticelList= businessStoreWareDetailService.getByZidAndType(id, StoreWareType.notice.getValue());

            for (int i = 0; i < list.size(); i++) {
            	obj = list.get(i);
            	for (BusinessStoreWareDetail m : noticelList) {
            		if(null == m.getWarehouseId()){
            			if (obj.getHouseId().equals(m.getHouseId())) {
                			if ("1".equals(deliveryStorageNotice.getBillType())) {
                				m.setRemainingNumber(obj.getRemainingNumber());
                			}
                			if ("3".equals(deliveryStorageNotice.getBillType())) {
                				m.setOutRemainingNumber(obj.getOutRemainingNumber());
                			}
                		}
            		}else{
            			if (obj.getHouseId().equals(m.getHouseId()) && obj.getWarehouseId().equals(m.getWarehouseId())) {
                			if ("1".equals(deliveryStorageNotice.getBillType())) {
                				m.setRemainingNumber(obj.getRemainingNumber());
                			}
                			if ("3".equals(deliveryStorageNotice.getBillType())) {
                				m.setOutRemainingNumber(obj.getOutRemainingNumber());
                			}
                		}
            		}
            	}
            }
            if(transferNoticeList.size()>0 && transferNoticeList!=null){
            	modelMap.put("storeWareDetailList", transferNoticeList);
            }
            if(noticelList.size()>0 && noticelList!=null){
            	modelMap.put("storeWareDetailList", noticelList);
            }
            
        }
        return modelMap;
    }

    /**
     * 保存
     * @param deliveryStorageNoticeJson 主表信息
     * @param businessStoreWareDetailJson 明细
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @RequestMapping(value="/save", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="保存数据", notes = "保存")
    @SuppressWarnings("unchecked")
    @ApiImplicitParams({
		@ApiImplicitParam(name = "deliveryStorageNoticeJson", value = "主表数据", paramType = "query"),
		@ApiImplicitParam(name = "businessStoreWareDetailJson", value = "子表数据", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "query"),
		@ApiImplicitParam(name = "name", value = "当前登录人name", paramType = "query"),
		@ApiImplicitParam(name = "orgId", value = "当前登录人所在组织", paramType = "query"),
		@ApiImplicitParam(name = "depotInfoOrgId", value = "当前登录人所在组织id", paramType = "query")
	})
    public Map<String, Object> save(String deliveryStorageNoticeJson, String businessStoreWareDetailJson, 
    		Integer userId, String name ,Integer orgId,Integer depotInfoOrgId,HttpServletRequest request) 
            throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        /*UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
        OrgInfo depotInfo = (OrgInfo)request.getSession().getAttribute("depotInfo");*/
        try {
            // JSON字符串转对象
            ObjectMapper mapper = new ObjectMapper();
            BusinessDeliveryStorageNotice businessDeliveryStorageNotice = (BusinessDeliveryStorageNotice)mapper.readValue(deliveryStorageNoticeJson, BusinessDeliveryStorageNotice.class);
            
            List<BusinessStoreWareDetail> storeWareDetailList = null;
            if (businessStoreWareDetailJson != null) {
                storeWareDetailList = (List<BusinessStoreWareDetail>)mapper.readValue(businessStoreWareDetailJson, new TypeReference<List<BusinessStoreWareDetail>>(){});
                
                 //通知单的明细 库点名称都是一样的，所以取第一个
					if(storeWareDetailList.get(0).getState() != storeWareDetailList.get(0).getOrgId()){ //代储库的
						businessDeliveryStorageNotice.setHouseId(storeWareDetailList.get(0).getState());
					}else{
						businessDeliveryStorageNotice.setHouseId(orgId);
					}
            
            }
            
            if (businessDeliveryStorageNotice.getId() == null) {
                Map<String, Object> returnMap = deliveryStorageNoticeService.add(businessDeliveryStorageNotice, storeWareDetailList,  userId,  name , orgId, depotInfoOrgId);
                if (returnMap != null && returnMap.get("error") != null) {
                    modelMap.put("status", "error");
                    modelMap.put("msg", returnMap.get("error"));
                    return modelMap;
                }
                modelMap.put("id", returnMap.get("id"));
            } else {
                deliveryStorageNoticeService.update(businessDeliveryStorageNotice, storeWareDetailList);
            }
            modelMap.put("status", "success");
        } catch (Exception e) {
            e.printStackTrace();
            modelMap.put("status", "error");
            modelMap.put("msg", "保存失败！");
            return modelMap;
        }
        return modelMap;
    }
    
    /**
     * 删除(同时删除流程)
     * @param id 主键
     * @param processInstanceId 流程实例id
     * @param deleteReason 删除原因
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @RequestMapping(value="/remove", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    @ApiOperation(value="通知单删除(同时删除流程)", notes = "删除.")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "主键id", paramType = "query"),
		@ApiImplicitParam(name = "processInstanceId", value = "流程实例id", paramType = "query"),
		@ApiImplicitParam(name = "deleteReason", value = "删除原因", paramType = "query"),
        @ApiImplicitParam(name = "planBid", value = "计划id", paramType = "query"),
        @ApiImplicitParam(name = "contractBid", value = "合同id", paramType = "query")
	})
    public Map<String, Object> remove(Integer id, String processInstanceId, String deleteReason,Integer planBid,Integer contractBid) 
            throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        if (id != null) {
            deliveryStorageNoticeService.remove(id, processInstanceId, deleteReason,planBid,contractBid);
        }
        modelMap.put("status", "success");
        return modelMap;
    }
    
    /**
     * 提交
     * @param businessDeliveryStorageNotice 业务主体
     * @param assignee 下一个审批人
     * @return
     */
    @RequestMapping(value="/submit", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="提交", notes = "提交")
    @ApiImplicitParams({
		@ApiImplicitParam(name = "businessDeliveryStorageNotice", value = "业务主体", paramType = "query"),
		@ApiImplicitParam(name = "assignee", value = "下一个审批人", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "query"),
		@ApiImplicitParam(name = "realName", value = "当前登录人name", paramType = "query")
	})
    public Map<String, Object> startProcess(BusinessDeliveryStorageNotice businessDeliveryStorageNotice, String assignee,Integer userId,String realName,HttpServletRequest request) {
        //UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
        Map<String, Object> returnMap = deliveryStorageNoticeService.submit(businessDeliveryStorageNotice, assignee, userId, realName);
        return returnMap;
    }
    
    /**
     * 启用
     * @param id 主键.
     * @return
     */
    @RequestMapping(value="/application", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    @ApiOperation(value="通知单启用", notes = "启用.")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "主键id", paramType = "query")
	})
    public Map<String, Object> application(Integer id) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        Map<String, String> returnMap = deliveryStorageNoticeService.application(id);
        if (StringUtils.isNoneBlank(returnMap.get("error"))) {
            modelMap.put("status", "error");
            modelMap.put("msg", returnMap.get("error"));
        } else {
            modelMap.put("status", "success");
        }
        return modelMap;
    }
    
    /**
     * 暂停
     * @param id 主键.
     * @return
     */
    @RequestMapping(value="/discontinue",produces = MediaType.APPLICATION_JSON_VALUE,  method=RequestMethod.POST)
    @ApiOperation(value="通知单暂停", notes = "暂停.")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "主键id", paramType = "query")
	})
    public Map<String, Object> discontinue(Integer id ,String refuseReason) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        deliveryStorageNoticeService.discontinue(id,refuseReason);
        modelMap.put("status", "success");
        return modelMap;
    }
    
    /**
     * 终止.
     * @param id 主键.
     * @return
     */
    @RequestMapping(value="/finish",produces = MediaType.APPLICATION_JSON_VALUE,  method=RequestMethod.POST)
    @ApiOperation(value="通知单终止", notes = "终止.")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "主键id", paramType = "query")
	})
    public Map<String, Object> finish(Integer id,String refuseReason) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        deliveryStorageNoticeService.finish(id,refuseReason);
        modelMap.put("status", "success");
        return modelMap;
    }
    
    
    /**
     * 获取可以通知单变更的申请列表.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
    @RequestMapping(value="/getGrainCanChangeList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="获取通知单可以变更的申请列表", notes = "查询数据列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "query")
	})
    public PageInfo<BusinessDeliveryStorageNotice> getGrainCanChangeList(Integer pageNum, Integer pageSize,Integer userId,HttpServletRequest request) {

    	List<BusinessDeliveryStorageNotice> new_list = deliveryStorageNoticeService.queryGrainNoticeChangeList(userId, pageNum, pageSize);

        PageInfo<BusinessDeliveryStorageNotice> pageInfo = new PageInfo<BusinessDeliveryStorageNotice>(new_list);

        return pageInfo;
    }
    
    /**
     * 通知单变更列表.
     * 注：如果一条通知单变更多次，只会显示最新一次的变更.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
    @RequestMapping(value="/getChangeList", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="通知单变更列表", notes = "查询数据列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "query")
	})
    public PageInfo<BusinessDeliveryStorageNotice> getChangeList(Integer pageNum, Integer pageSize,
    		BusinessDeliveryStorageNotice notice, Integer userId,HttpServletRequest request) {
    	
        List<BusinessDeliveryStorageNotice> list = deliveryStorageNoticeService.queryByExample(pageNum, pageSize, userId, notice, "FIELD(audit_state,0,3,1,2,4 ), id desc", "change");
        PageInfo<BusinessDeliveryStorageNotice> pageInfo = new PageInfo<BusinessDeliveryStorageNotice>(list);
        return pageInfo;
    }
    
    /**
     * 保存通知单变更
     * @param contractJson 通知单数据json
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @RequestMapping(value="/saveChange", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    @ApiOperation(value="保存合同变更", notes = "保存")
    @SuppressWarnings("unchecked")
    @ApiImplicitParams({
        @ApiImplicitParam(name="deliveryStorageNoticeJson", value="主表数据", paramType="form"),
        @ApiImplicitParam(name="businessStoreWareDetailJson", value="子表数据", paramType="form"),
        @ApiImplicitParam(name="userId", value="当前登录人的id", paramType="form"),
        @ApiImplicitParam(name="userName", value="当前登录人的name", paramType="form"),
        @ApiImplicitParam(name="depotInfoOrgId", value="当前登录人的组织id", paramType="form"),
            @ApiImplicitParam(name="id", value="原id", paramType="form")
    })
    public Map<String, Object> saveChange(String deliveryStorageNoticeJson, String businessStoreWareDetailJson, 
            Integer userId, String name ,Integer orgId,Integer depotInfoOrgId,Integer id) throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        try {
            // JSON字符串转对象
            ObjectMapper mapper = new ObjectMapper();
            BusinessDeliveryStorageNotice businessDeliveryStorageNotice = (BusinessDeliveryStorageNotice)mapper.readValue(deliveryStorageNoticeJson, BusinessDeliveryStorageNotice.class);
            // 明细.
            List<BusinessStoreWareDetail> storeWareDetailList = null;
            if (businessStoreWareDetailJson != null) {
                storeWareDetailList = (List<BusinessStoreWareDetail>)mapper.readValue(businessStoreWareDetailJson, new TypeReference<List<BusinessStoreWareDetail>>(){});
                
                 //通知单的明细 库点名称都是一样的，所以取第一个
                    if(storeWareDetailList.get(0).getState() != storeWareDetailList.get(0).getOrgId()){ //代储库的
                        businessDeliveryStorageNotice.setHouseId(storeWareDetailList.get(0).getState());
                    }else{
                        businessDeliveryStorageNotice.setHouseId(storeWareDetailList.get(0).getOrgId());
                    }
            
            }
            if (id == null) {
                throw new RuntimeException("原通知单id为空，保存失败！");
            } else {
                Map<String, Object> returnMap =  deliveryStorageNoticeService.saveChange(businessDeliveryStorageNotice, storeWareDetailList,  userId, name, depotInfoOrgId,id);
                modelMap.put("id",returnMap.get("id"));
            }
            modelMap.put("status", "success");
            
        } catch (Exception e) {
            e.printStackTrace();
            modelMap.put("status", "error");
            modelMap.put("msg", "保存失败！");
        }
        return modelMap;
    }
    /**
     * 修改通知单变更.
     * @param deliveryStorageNoticeJson 通知单数据json
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @RequestMapping(value="/updateChange", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    @ApiOperation(value="修改合同变更", notes = "保存")
    @SuppressWarnings("unchecked")
    @ApiImplicitParams({
        @ApiImplicitParam(name="deliveryStorageNoticeJson", value="主表数据", paramType="form"),
        @ApiImplicitParam(name="businessStoreWareDetailJson", value="子表数据", paramType="form")
    })
    public Map<String, Object> updateChange(String deliveryStorageNoticeJson, String businessStoreWareDetailJson)
            throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        try {
            // JSON字符串转对象
            ObjectMapper mapper = new ObjectMapper();
            BusinessDeliveryStorageNotice businessDeliveryStorageNotice = (BusinessDeliveryStorageNotice)mapper.readValue(deliveryStorageNoticeJson, BusinessDeliveryStorageNotice.class);
            // 明细.
            List<BusinessStoreWareDetail> storeWareDetailList = null;
            if (businessStoreWareDetailJson != null) {
                storeWareDetailList = (List<BusinessStoreWareDetail>)mapper.readValue(businessStoreWareDetailJson, new TypeReference<List<BusinessStoreWareDetail>>(){});
            }
            if (businessDeliveryStorageNotice.getId() == null) {
                throw new RuntimeException("通知单id为空，保存失败！");
            } else {
                deliveryStorageNoticeService.update(businessDeliveryStorageNotice, storeWareDetailList);
            }
            modelMap.put("status", "success");
        } catch (Exception e) {
            e.printStackTrace();
            modelMap.put("status", "error");
            modelMap.put("msg", "保存失败！");
        }
        return modelMap;
    }
    /**
     * 删除通知单变更.
     * @param id 主键
     * @param originalNoticeBid 原通知单id
     * @param processInstanceId 流程实例id
     * @param deleteReason 删除原因
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @RequestMapping(value="/changeNoticeRemove",produces = MediaType.APPLICATION_JSON_VALUE,  method=RequestMethod.POST)
    @ApiOperation(value="删除合同变更", notes = "根据id删除")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "id", value = "合同id", paramType = "query"),
        @ApiImplicitParam(name = "originalNoticeBid", value = "原通知单id", paramType = "query"),
        @ApiImplicitParam(name = "processInstanceId", value = "流程实例id", paramType = "query"),
    })
    public Map<String, Object> changeContractRemove(Integer id, Integer originalNoticeBid,String processInstanceId, String deleteReason)
    		
            throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        if (id != null) {
            deliveryStorageNoticeService.changeNoticeRemove(id, originalNoticeBid, processInstanceId,deleteReason);
        } 
        modelMap.put("status", "success");
        return modelMap;
    }
    
    
    /**
     * 根据id查询.
     * @param id 主键
     * @return
     */
    @RequestMapping(value="/addForChange",produces = MediaType.APPLICATION_JSON_VALUE,  method=RequestMethod.GET)
   	@ApiOperation(value="根据id查找，获取历次变更详情(新增时，不显示当前id对应的历史记录)", notes = "根据id查找")
   	@ApiImplicitParams({
   		@ApiImplicitParam(name = "id", value = "id", paramType = "query")
   	})
    public Map<String, Object> addForChange(Integer id) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        if (id != null) {
        	// 获取当前通知单主数据
        	BusinessDeliveryStorageNotice deliveryStorageNotice = deliveryStorageNoticeService.findById(id);
            // 获取当前通知单的历次修改原因，内容，时间.
            List<BusinessDeliveryStorageNotice> changeDetailList = deliveryStorageNoticeService.getChangeDetailList(id, "add");
            // 获取当前合同明细数据
            List<BusinessStoreWareDetail> storeWareDetailList= businessStoreWareDetailService.getByZidAndType(id, StoreWareType.notice.getValue());
            
            modelMap.put("deliveryStorageNotice", deliveryStorageNotice);
            modelMap.put("changeDetailList", changeDetailList);
            modelMap.put("storeWareDetailList", storeWareDetailList);
        }
        return modelMap;
    }
    
    @RequestMapping(value="/pushCktzdts",produces = MediaType.APPLICATION_JSON_VALUE,  method=RequestMethod.GET)
   	@ApiOperation(value="给市平台推送出库通知单提示消息", notes = "")
   	@ApiImplicitParams({
   		@ApiImplicitParam(name = "fsr", value = "发送人", paramType = "query"),
   		@ApiImplicitParam(name = "fsdw", value = "发送单位", paramType = "query")
   	})
    public void pushCktzdts(String fsr, String fsdw) {
    	try {
			String resultStr = deliveryStorageNoticeService.pushCktzdts(fsr, fsdw);
			systemOutsideDataService.addlogger("cktzdts", fsr+"_"+fsdw, resultStr);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			systemOutsideDataService.addlogger("cktzdts", fsr+"_"+fsdw, e.getMessage());
		}
    }
}
