package com.chinaitop.depot.business.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.business.model.BusinessMovelibrary;
import com.chinaitop.depot.business.model.BusinessMovelibrarySon;
import com.chinaitop.depot.business.service.MoveLibraryNoticeService;
import com.chinaitop.depot.business.service.MoveLibraryNoticeSonService;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 接收通知单
 * @author My   Sunshine
 *
 */
@RestController
@RequestMapping(value="/depot/business/moveLibraryNotice")
@Api(description = "移库通知单")
public class MoveLibraryNoticeController {
	
	@Resource
	private MoveLibraryNoticeService moveLibraryNoticeService;
	@Resource
	private MoveLibraryNoticeSonService moveLibraryNoticeSonService;


	/**
	 * 接收的移库通知单新增
	 */
	@RequestMapping(value="/moveLibraryAdd", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="保存数据", notes = "保存")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "notice", value = "新增数据", paramType = "query")
	})
    public Map<String, Object> save(String notice){
		Map<String, Object> modelMap = new HashMap<String, Object>();
		modelMap = moveLibraryNoticeService.add(notice);  
        return modelMap;
    }
	
	/**
	 * 接收的通知单列表
	 */
	@RequestMapping(value="/getList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="通知单接收列表", notes = "查询数据列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query")
            
    })
    public PageInfo<BusinessMovelibrary> getList(Integer pageNum, Integer pageSize , BusinessMovelibrary businessReceiveMovelibrary) throws Exception {

        List<BusinessMovelibrary> list = moveLibraryNoticeService.queryByExample(pageNum, pageSize, businessReceiveMovelibrary);
        PageInfo<BusinessMovelibrary> pageInfo = new PageInfo<BusinessMovelibrary>(list);
        return pageInfo;
    }
   
	
	
	
	/**
	 * 根据id查询
	 */
	@RequestMapping(value="/edit", produces = MediaType.APPLICATION_JSON_VALUE,method=RequestMethod.GET)
    @ApiOperation(value="根据接收的通知单id查找信息", notes = "根据id查找")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "接收的通知单id", paramType = "query")
    })
    public Object edit(Integer id) {
		Object movelibrary = new BusinessMovelibrary();
		Object movelibrarySon = new BusinessMovelibrarySon();
        if (id != null) {
        	movelibrary = moveLibraryNoticeService.findById(id);
        	if(movelibrary!=null){
        		return movelibrary;
        	}else{
        		movelibrarySon = moveLibraryNoticeSonService.findById(id);
        		return movelibrarySon;
        	}
        }
        return movelibrarySon;
    }
	
	
	
	/**
	 * 根据子表的id通过zid查询主表的总数量
	 */
	@RequestMapping(value="/getSumPlanCount", produces = MediaType.APPLICATION_JSON_VALUE,method=RequestMethod.GET)
	@ApiOperation(value="根据子表的id通过zid查询主表的总数量", notes = "根据id查找")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "子表的id", paramType = "query")
	})
	public BusinessMovelibrary getSumPlanCount(Integer id) {
		BusinessMovelibrary businessReceiveMovelibrary = new BusinessMovelibrary();
		if (id != null) {
			businessReceiveMovelibrary = moveLibraryNoticeService.getSumPlanCount(id);
		}
		
		return businessReceiveMovelibrary;
	}
	
	
	/**
	 * 根据子表的id查询使用的总数量
	 */
	@RequestMapping(value="/useCount", produces = MediaType.APPLICATION_JSON_VALUE,method=RequestMethod.GET)
	@ApiOperation(value="根据子表的id查询使用的总数量", notes = "根据id查找")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "子表的id", paramType = "query")
	})
	public String useCount(Integer id) {
		
		String useSumCount = "0";
		if (id != null) {
			useSumCount = moveLibraryNoticeService.useCount(id);
		}
		return useSumCount;
	}
	
	
	
	/**
	 * 根据子表的id查询使用的总数量
	 */
	@RequestMapping(value="/updatePlanCount", produces = MediaType.APPLICATION_JSON_VALUE,method=RequestMethod.GET)
	@ApiOperation(value="修改子表的数量", notes = "修改子表的数量")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "子表的id", paramType = "query"),
		@ApiImplicitParam(name = "count", value = "数量", paramType = "query")
	})
	public Map<String, Object> updatePlanCount(Integer id,String count) {
		Map<String, Object> map = new HashMap<String, Object>();

		if (id != null && count !=null) {
			map.put("id", id);
			map.put("count", count);
			moveLibraryNoticeService.updatePlanCount(map);
		}
		map.put("state", "success");
		return map;
	}
	
	
	/**
	 * 根据子表的id 查询 有几条 数量是空的
	 */
	@RequestMapping(value="/getIsnullCount", produces = MediaType.APPLICATION_JSON_VALUE,method=RequestMethod.GET)
	@ApiOperation(value="修改子表的数量", notes = "修改子表的数量")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "子表的id", paramType = "query"),
	})
	public Integer getIsnullCount(Integer id) {
		Integer count = 0;
		if (id != null ) {
			 count = moveLibraryNoticeService.getIsnullCount(id);
		}
		return count;
	}
	
    
	/**
	 * 修改最后的数量
	 */
	@RequestMapping(value="/updateLastCount", produces = MediaType.APPLICATION_JSON_VALUE,method=RequestMethod.GET)
	@ApiOperation(value="修改子表的数量", notes = "修改子表的数量")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "子表的id", paramType = "query"),
		@ApiImplicitParam(name = "count", value = "数量", paramType = "query")
	})
	public Map<String, Object> updateLastCount(Integer id,String count) {
		Map<String, Object> map = new HashMap<String, Object>();

		if (id != null && count !=null) {
			map.put("id", id);
			map.put("count", count);
			moveLibraryNoticeService.updateLastCount(map);
		}
		map.put("state", "success");
		return map;
	}
	
	
	
	/**
	 * 修改使用的状态
	 */
	@RequestMapping(value="/updateUseState", produces = MediaType.APPLICATION_JSON_VALUE,method=RequestMethod.GET)
	@ApiOperation(value="修改使用的状态", notes = "修改使用的状态")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "子表的id", paramType = "query"),
	})
	public void updateUseState(Integer moveLibraryId,Integer billType) {
		
        if(billType == 4){//入库  修改子表
        	moveLibraryNoticeSonService.updateUseState(moveLibraryId);
        }
        if(billType == 5){//出库 修改主表
        	moveLibraryNoticeService.updateUseState(moveLibraryId);
        }
   
	
}
}
