package com.chinaitop.depot.business.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.business.model.BusinessNoticeReceive;
import com.chinaitop.depot.business.service.ReceiveNoticeService;
import com.chinaitop.depot.business.service.SystemOutsideDataService;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 接收通知单
 * @author My   Sunshine
 *
 */
@RestController
@RequestMapping(value="/depot/business/receiveNotice")
@Api(description = "接收通知单")
public class ReceiveNoticeController {

	public static final Logger logger = LoggerFactory.getLogger(ReceiveNoticeController.class);
	
	@Resource
	private ReceiveNoticeService receiveNoticeService;

	@Resource
	private SystemOutsideDataService systemOutsideDataService;


	/**
	 * 接收通知单列表
	 */
	@RequestMapping(value="/getList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="通知单接收列表", notes = "查询数据列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "userId", value = "当前登录的用户编号", paramType = "query")
    })
    public PageInfo<BusinessNoticeReceive> getList(Integer pageNum, Integer pageSize , BusinessNoticeReceive businessNoticeReceive,String cktzd,String lyflag) throws Exception {

        List<BusinessNoticeReceive> list = receiveNoticeService.queryByExample(pageNum, pageSize, businessNoticeReceive,cktzd,lyflag);
        PageInfo<BusinessNoticeReceive> pageInfo = new PageInfo<BusinessNoticeReceive>(list);
        return pageInfo;
    }
   
	
	
	
	/**
	 * 根据id查询
	 */
	@RequestMapping(value="/edit", produces = MediaType.APPLICATION_JSON_VALUE,method=RequestMethod.GET)
    @ApiOperation(value="根据接收的通知单id查找信息", notes = "根据id查找")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "接收的通知单id", paramType = "query")
    })
    public BusinessNoticeReceive edit(Integer id) {
		BusinessNoticeReceive businessNoticeReceive = new BusinessNoticeReceive();
        if (id != null) {
        	businessNoticeReceive = receiveNoticeService.findById(id);
        }else{
        	businessNoticeReceive=new BusinessNoticeReceive();
        }

        return businessNoticeReceive;
    }
	
	
	
	/**
	 * 修改
	 */
	@RequestMapping(value="/update", produces = MediaType.APPLICATION_JSON_VALUE,method=RequestMethod.POST)
    @ApiOperation(value="修改内容", notes = "修改内容")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "BusinessNoticeReceive", value = "接收到的通知单对象", paramType = "form")
    })
    public Map<String, Object> update(BusinessNoticeReceive businessNoticeReceive) {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		if(businessNoticeReceive.getInApplication()!=null){
			receiveNoticeService.update(businessNoticeReceive);
			modelMap.put("status", "success");
		}else{
			modelMap.put("status", "error");
		}
        return modelMap;
    }
	
	/**
	 * 新增
	 */
	@RequestMapping(value="/save", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="保存数据", notes = "保存")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "notice", value = "JSON对象数据", paramType = "form")
	})
    public Map<String, Object> save(String notice){

		logger.info("市平台传推送过来的轮换通知单如下：");
		logger.info(notice);
		Map<String, Object> modelMap = new HashMap<String, Object>();
		try {
			modelMap = receiveNoticeService.receiveNotice(notice);
			systemOutsideDataService.addlogger("tzdjs", notice, modelMap.toString());
		} catch (Exception e) {
			modelMap.put("status", "error");
			modelMap.put("msg", "接口异常，请联系管理员！");
			logger.error(e.getMessage(), e);
			systemOutsideDataService.addlogger("tzdjs", notice, modelMap.toString());
		}
		return modelMap;
    }

	@RequestMapping(value="/updateHtbh", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="修改当前数据的合同编号", notes = "")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "notice", value = "JSON对象数据", paramType = "form")
	})
	public Map<String, Object> update(String notice) {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		try {
			modelMap = receiveNoticeService.updateReceiveNotice(notice);
			//给市平台推送合同被引用的消息
			receiveNoticeService.pushHtStatus(notice);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			modelMap.put("status", 500);
			modelMap.put("msg", "操作失败");
		}
		return modelMap;
	}
	
	
	/**
	 * 未进行审核完成的【性质转变单】，该仓再下一次申请入库验收的时候，
	 * 会提示，需要将上一次的粮食性质转变单审核完成，才能进行提交
	 */
	@RequestMapping(value="/getNoticeNoAuditList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="入库验收申请时查询上一次性质转变单是否审批完成", notes = "入库验收申请时查询上一次性质转变单是否审批完成")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "xybh", value = "协议编号", paramType = "query"),
            @ApiImplicitParam(name = "sptDataId", value = "和市平台对接的货位编码", paramType = "query"),
            @ApiImplicitParam(name = "flag", value = "是油罐还是仓房", paramType = "query")
    })
    public int getNoticeNoAuditList(String xybh,String sptDataId,Integer flag) throws Exception {
        return receiveNoticeService.queryIsOrAudit(xybh,sptDataId,flag);
    }

	@RequestMapping(value="/scRkxzzbd", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="增加性质转变单数据(入库)", notes = "")
	@ApiImplicitParams({
        @ApiImplicitParam(name = "sptId", value = "市库对接的货位ID", paramType = "query"),
        @ApiImplicitParam(name = "crklx", value = "储备类型(1市储，2区储)", paramType = "query"),
        @ApiImplicitParam(name = "ysid", value = "市平台验收数据ID", paramType = "query")
    })
	public Map<String, Object> scRkxzzbd(String sptId, String crklx, String ysid) {
		Map<String, Object> map = new HashMap<>();
		String jssj = "sptId="+sptId+"&crklx="+crklx+"&ysid="+ysid;
		try {
			map = receiveNoticeService.scRkxzzbd(sptId, crklx, ysid);
			systemOutsideDataService.addlogger("rkxzzbd", jssj, map.toString());
		} catch (Exception e) {
			logger.error("性质转变单生成异常");
			logger.error(e.getMessage());
			map.put("status", "error");
			map.put("msg", "接口异常，请联系管理员！");
			systemOutsideDataService.addlogger("rkxzzbd", jssj, map.toString());
		}
		return map;
	}

	@RequestMapping(value="/updateNoticeReceiveAuditState", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="接收市平台调用修改入库性质转变单状态和划转时间", notes = "")
	@ApiImplicitParams({
        @ApiImplicitParam(name = "cblx", value = "储备类型(1市储，2区储)", paramType = "query"),
        @ApiImplicitParam(name = "ysid", value = "市平台验收数据ID", paramType = "query"),
        @ApiImplicitParam(name = "hzsj", value = "划转时间", paramType = "query")
    })
	public void updateNoticeReceiveAuditState(String cblx, String ysid, String hzsj) {
		try {
			String param = "cblx="+cblx+"&ysid="+ysid+"&hzsj="+hzsj;
			logger.error("市平台通知性质转变单审批完该结束了，推送信息是："+param);
			systemOutsideDataService.addlogger("update_rkxzzbd_zt_and_hzsj", param, null);
			receiveNoticeService.updateNoticeReceiveAuditState(cblx, ysid, hzsj);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@RequestMapping(value="/retSptMeagess", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="给给市平台发送性质转变单5部审批完的通知", notes = "")
	@ApiImplicitParams({
        @ApiImplicitParam(name = "datas", value = "数据对象", paramType = "query")
    })
	public Map<String, Object> retSptMeagess(String datas) {
		Map<String, Object> map = new HashMap<>();
		try {
			ObjectMapper mapper = new ObjectMapper();
			BusinessNoticeReceive receiveNotice = (BusinessNoticeReceive)mapper.readValue(datas, BusinessNoticeReceive.class);
			receiveNoticeService.retSptMeagess(receiveNotice);
		} catch (Exception e) {
			logger.error("推送性质转变单库级审批结束信息给市平台时代码异常");
			logger.error(e.getMessage(), e);
			map.put("status", "error");
			map.put("msg", "接口异常，请联系管理员！");
		}
		return map;
	}
	
	@RequestMapping(value="/testT", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="测试接口", notes = "")
	public void testT(BusinessNoticeReceive receiveNotice) {
		receiveNoticeService.updateQualitycheckHwxz(receiveNotice);
	}
}
