package com.chinaitop.depot.business.controller;

import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.business.model.OrgInfo;
import com.chinaitop.depot.business.model.UserInfo;
import com.chinaitop.depot.business.model.UserInfoExample;
import com.chinaitop.depot.business.model.UserInfoExample.Criteria;
import com.chinaitop.depot.business.model.UserRole;
import com.chinaitop.depot.business.service.OrgInfoService;
import com.chinaitop.depot.business.service.RoleInfoService;
import com.chinaitop.depot.business.service.UserInfoService;
import com.chinaitop.depot.business.service.UserRoleService;
import com.chinaitop.depot.utils.SecurityCode;
import com.chinaitop.depot.utils.SecurityCode.SecurityCodeLevel;
import com.chinaitop.depot.utils.SecurityImage;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 用户信息控制类
 * @author xueenxi
 *
 */
@RestController
@RequestMapping(value="/userInfo")
@Api(description = "用户信息控制类")
public class UserInfoController {
	
	@Resource
	private UserInfoService userInfoService;

	@Resource
	private OrgInfoService orgInfoService;

	@Resource
	private UserRoleService userRoleService;

	@Resource
	private RoleInfoService roleInfoService;

	@Value("${web.upload-path}")
	private String uploadPath;

	/**
	 * 查找用户信息
	 * @param pageNum	页码
	 * @param pageSize	每页条数
	 * @return
	 */
	@RequestMapping(value="/getUser", method = RequestMethod.GET)
	@ApiOperation(value="查询用户信息", notes = "查询用户信息列表，支持分页")
	public PageInfo<UserInfo> getUserInfo(HttpServletRequest request) {
                                          
		HashMap<Object, Object> map = new HashMap<Object, Object>();
		Integer orgId = (Integer)request.getSession().getAttribute("orgId");
		map.put("orgIds", orgId);
		List<UserInfo> listAll=userInfoService.queryAll(map);
		PageInfo<UserInfo> pageInfo = new PageInfo<UserInfo>(listAll);
		return pageInfo;
	}

	
	/**
	 * 获取用户登录信息,包括当前用户、当前组织、当前库、当前库编号
	 * @param request
	 * @return
	 */
	@RequestMapping(value="/getLoginInfo", method = RequestMethod.GET)
	@ApiOperation(value="获取用户登录信息", notes = "获取用户登录信息")
	public Map<String, Object> getLoginInfo(HttpServletRequest request) {
		Map<String, Object> modelMap = new HashMap<>();
		try {
			// JSON字符串转对象
			ObjectMapper mapper = new ObjectMapper();
			String userInfoObject = ObjectUtils.toString(request.getSession().getAttribute("userInfo"),"");
			if(!userInfoObject.isEmpty()) {
				UserInfo userInfo = mapper.readValue(userInfoObject, UserInfo.class);
				//UserInfo userInfo = (UserInfo)request.getSession().getAttribute("userInfo");
				modelMap.put("userInfo", userInfo);
			}
			String orgInfoObject = ObjectUtils.toString(request.getSession().getAttribute("orgInfo"),"");
			if(!orgInfoObject.isEmpty()) {
				OrgInfo orgInfo = mapper.readValue(orgInfoObject, OrgInfo.class);
				//OrgInfo orgInfo = (OrgInfo)request.getSession().getAttribute("orgInfo");
				modelMap.put("orgInfo", orgInfo);
			}
			String depotInfoObject = ObjectUtils.toString(request.getSession().getAttribute("depotInfo"),"");
			if(!depotInfoObject.isEmpty()) {
				OrgInfo depotInfo = mapper.readValue(depotInfoObject, OrgInfo.class);
				//OrgInfo depotInfo = (OrgInfo)request.getSession().getAttribute("depotInfo");
				modelMap.put("depotInfo", depotInfo);
			}
			Integer depotId = (Integer)request.getSession().getAttribute("depotId");
			modelMap.put("depotId", depotId);
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return modelMap;
	}
	
	/**
	 * 退出登录
	 * @param request
	 * @return
	 */
	@RequestMapping(value="/exitLogin", method = RequestMethod.GET)
	@ApiOperation(value="退出登录", notes = "退出登录，删除session")
	public Map<String, Object> exitLogin(HttpServletRequest request) {
		Map<String, Object> modelMap = new HashMap<>();
		request.getSession().removeAttribute("userInfo");
		modelMap.put("status", "success");
		return modelMap;
	}

	/**
	 * 验证用户名是否可用
	 * @param userId	用户编号
	 * @param username	用户名
	 * @return
	 */
	@RequestMapping(value="/validUsername", method = RequestMethod.GET)
	@ApiOperation(value="验证用户名", notes = "验证用户名是否可用")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "userId", value = "用户编号", paramType = "query"),
			@ApiImplicitParam(name = "username", value = "用户名", paramType = "query")
	})
	public Boolean validUsername(Integer userId, String username) {
		return userInfoService.validUsername(userId, username);
	}
	
	/**
	 * 验证用户输入的原密码是否正确
	 * @param oldPassword	用户输入的原密码
	 * @return
	 */
	@RequestMapping(value="/validPassword", method = RequestMethod.GET)
	@ApiOperation(value="验证用户输入的原密码是否正确", notes = "验证用户输入的原密码是否正确")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "oldPassword", value = "用户密码", paramType = "query")
	})
	public boolean validPassword(HttpServletRequest request, String oldPassword) throws Exception{
		ObjectMapper mapper = new ObjectMapper();
		String userInfoObject = ObjectUtils.toString(request.getSession().getAttribute("userInfo"),"");
		UserInfo userInfo = mapper.readValue(userInfoObject, UserInfo.class);
		String resPassword = userInfo.getPassword();
		return userInfoService.validPassword(resPassword, oldPassword);
	}

	/**
	 * 重置密码
	 * @param userId
	 * @return
	 */
	@RequestMapping(value="/resetPassword", method = RequestMethod.POST)
	@ApiOperation(value="重置密码", notes = "重置密码")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "userId", value = "用户编号", paramType = "form")
	})
	public void resetPassword(Integer userId) {
		userInfoService.resetPassword(userId);
	}
	
	/**
	 * 根据编号查找用户信息
	 * @param userId 用户编号
	 * @return	UserInfo	用户信息
	 */
	@RequestMapping(value="/edit", method = RequestMethod.GET)
	@ApiOperation(value="编辑用户信息", notes = "编辑用户信息，用于新增或者修改")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "userId", value = "用户编号", paramType = "query")
	})
	public UserInfo edit(Integer userId) {
		UserInfo userInfo = new UserInfo();
		if (userId != null) {
			userInfo = userInfoService.findById(userId);
		}
		return userInfo;
	}

	/**
	 * 保存用户信息
	 * @param userInfoJson	用户信息json字符串
	 * @return
	 * @throws JsonParseException
	 * @throws JsonMappingException
	 * @throws IOException
	 */
	@RequestMapping(value="/save", method = RequestMethod.POST)
	@ApiOperation(value="保存用户", notes = "保存用户信息，新增或者修改")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "userInfoJson", value = "用户信息JSON串", paramType = "form"),
			@ApiImplicitParam(name = "userRoleIds", value = "用户角色编号，多个之间用逗号隔开", paramType = "form")
	})
	public Map<String, Object> save(String userInfoJson , String userRoleIds) throws JsonParseException, JsonMappingException, IOException {
		Map<String, Object> modelMap = new HashMap<>();
		// JSON字符串转对象
		ObjectMapper mapper = new ObjectMapper();
		UserInfo userInfo = (UserInfo)mapper.readValue(userInfoJson, UserInfo.class);
		UserRole userRole = new UserRole();
		 if (StringUtils.isNotBlank(userRoleIds) && !"[]".equals(userRoleIds)) {
			 userRoleIds = userRoleIds.substring(1, userRoleIds.length());
			 userRoleIds = userRoleIds.substring(0, userRoleIds.length() - 1);
	            String[] idArray = userRoleIds.split(",");
	           if(userInfo.getUserId() == null){
	        	   userInfoService.add(userInfo);
	        	  /* userRole.setUserId(userInfo.getUserId());
	        	   for (String id : idArray) {
		            	userRole.setRoleId(Integer.parseInt(id));
			            userRoleService.add(userRole);
		            }*/
	           }else{
	        	   userInfoService.update(userInfo);
	        	   userRoleService.deleteById(userInfo.getUserId());
	           }
	           userRole.setUserId(userInfo.getUserId());
	           for (String id : idArray) {
	        	   userRole.setRoleId(Integer.parseInt(id));
	        	   userRoleService.add(userRole);
	           }
	           
	          
		 }
		
		/*String[] split1 = userRoleIds.split("[");
		String[] split2 = split1[0].split("]");
		String[] userRoleids = split2[0].split(",");*/
		
		//int userRoleids = Integer.parseInt(userRoleIds);
		/*UserRole userRole = new UserRole();
		for (int i = 0; i < userRoleids.length; i++) {
			userRole.setRoleId(Integer.parseInt(userRoleids[i]));
		}*/
		modelMap.put("status", "success");
		return modelMap;
	}
	
	/**
	 * 根据编号删除用户编纂 
	 * @param userId	用户编号
	 * @return
	 */
	@RequestMapping(value="/remove", method = RequestMethod.POST)
	@ApiOperation(value="删除用户", notes = "按用户编号删除用户")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "userId", value = "用户名", paramType = "form")
	})
	public Map<String, Object> remove(Integer userId) {
		Map<String, Object> modelMap = new HashMap<>();
		if (userId != null) {
			userInfoService.remove(userId);
			userRoleService.deleteById(userId);//删除用户角色信息
		}
		modelMap.put("status", "success");
		return modelMap;
	}
	
	/**
	 * 修改密码
	 * @param request
	 * @param oldPassword	原密码
	 * @param newPassword	新密码
	 * @return
	 */
	@RequestMapping(value="/modifyPwd", method = RequestMethod.POST)
	@ApiOperation(value="修改密码", notes = "修改用户登录密码")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "oldPassword", value = "源密码", paramType = "form"),
			@ApiImplicitParam(name = "newPassword", value = "新密码", paramType = "form")
	})
	public UserInfo modifyPwd(HttpServletRequest request, String oldPassword, String newPassword) {
		try {
			ObjectMapper mapper = new ObjectMapper();
			String userInfoObject = ObjectUtils.toString(request.getSession().getAttribute("userInfo"),"");
			UserInfo userInfo = mapper.readValue(userInfoObject, UserInfo.class);
			if (userInfo.getPassword().equals(userInfoService.getMD5(oldPassword))) {
				userInfo.setPassword(userInfoService.getMD5(newPassword));
				userInfoService.update(userInfo);
				
				String userInfoObjectNew = mapper.writeValueAsString(userInfo);
				request.getSession().setAttribute("userInfo", userInfoObjectNew);
				return userInfo;
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 查询当前库下所有的用户
	 * @param request
	 * @return
	 */
	@RequestMapping(value="/findAllUserByOrgId", method = RequestMethod.GET)
	@ApiOperation(value="查询当前库下所有的用户", notes = "查询当前库下所有的用户")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "form")
	})
	public List<UserInfo> findAllUserByOrgId(Integer orgId){
		UserInfoExample example = new UserInfoExample();
		Criteria criteria = example.createCriteria();
		List<UserInfo> list = null;
		try {
			// 拼接查询条件单位
			if (null != orgId && !"".equals(orgId)) {
				criteria.andOrgIdEqualTo(orgId);
			}
			//criteria.andDelFlagEqualTo(1); // 查询正常数据（暂时不做限制）
			list = userInfoService.queryByExample(example);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return list;
	}
	
	/**
     * 查询所有的用户枚举，返回id与用户姓名的map.
     * @param request
     * @return
     */
    @RequestMapping(value="/findAllUserMapByOrgId", method = RequestMethod.GET)
    public Map<Integer, Object> findAllUserMapByOrgId(HttpServletRequest request) {
        Map<Integer, Object> map = new HashMap<>();
		
			//ObjectMapper mapper = new ObjectMapper();
			/*String depotInfoObject = ObjectUtils.toString(request.getSession().getAttribute("depotInfo"),"");
			OrgInfo orgInfo = mapper.readValue(depotInfoObject, OrgInfo.class);*/
			
			UserInfoExample keeperExample = new UserInfoExample();
			//Criteria criteria = keeperExample.createCriteria();
			List<UserInfo> list = null;
			try {
				//Integer orgId = null;
				/*if (orgInfo != null) {
					orgId=orgInfo.getOrgId();//所属库id
				}
				// 拼接查询条件单位
				if (null != orgId && !"".equals(orgId)) {
					criteria.andOrgIdEqualTo(orgId);
				}*/
				list = userInfoService.queryByExample(keeperExample);
				for (UserInfo userInfo : list) {
					map.put(userInfo.getUserId(), userInfo.getRealName());
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
        
        return map;
    }

    /**
	 * 生成验证码图片, 并将验证码图片上传到服务器，然后将生成的验证码串值保存到session 中
	 * 
	 * @param request
	 **/
	@RequestMapping(value="/getCode", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="生成验证码图片", notes = "生成验证码图片")
	public Map<String, Object> getCode(HttpServletRequest request, Integer fontSize, Integer width, Integer heigth) {

		// 图片流
		BufferedImage image;

		// 获取验证码数字字符串
		String yzm_code = SecurityCode.getSecurityCode(4, SecurityCodeLevel.Hard, false).toLowerCase();
		request.getSession().setAttribute("Service_Code", yzm_code);
		// 生成验证码图片
		image = SecurityImage.createImage(yzm_code, fontSize, width, heigth);

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try {
			ImageIO.write(image, "jpg", baos);
		} catch (IOException e1) {
			e1.printStackTrace();
		}

        //将图片转换为base64位二进制编码返回
        String imgBase64 = new Base64().encodeToString(baos.toByteArray());
  		Map<String, Object> map = new HashMap<String, Object>();
  		map.put("file", imgBase64);
  		return map;
	}
}
