package com.chinaitop.depot.business.controller.audit;


import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.act.model.HistoryList;
import com.chinaitop.depot.act.service.ActTaskService;
import com.chinaitop.depot.business.model.BusinessApproval;
import com.chinaitop.depot.business.model.BusinessContract;
import com.chinaitop.depot.business.model.BusinessFile;
import com.chinaitop.depot.business.model.BusinessStoreWareDetail;
import com.chinaitop.depot.business.model.enums.StoreWareType;
import com.chinaitop.depot.business.model.vo.BusinessContractAuditVO;
import com.chinaitop.depot.business.service.BusinessApprovalService;
import com.chinaitop.depot.business.service.BusinessContractService;
import com.chinaitop.depot.business.service.BusinessFileService;
import com.chinaitop.depot.business.service.BusinessStoreWareDetailService;
import com.chinaitop.depot.utils.activiti.StringUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 粮库业务管理-合同审批管理
* <p>Title: BusinessContractAuditController</p>
* <p>Description: </p>
* @author yaoyabin
* @date 2017年10月25日 下午5:45:39
 */
@RestController
@RequestMapping(value="/depot/business/contract/audit")
@Api(value= "BusinessContractAuditController", description = "合同审批管理")
public class BusinessContractAuditController {
    
    @Resource
    private BusinessContractService businessContractService;
    @Resource
    private BusinessStoreWareDetailService businessStoreWareDetailService;
    @Resource
    private BusinessApprovalService businessApprovalService;
    @Resource
    private ActTaskService actTaskService;
    @Autowired
    private BusinessFileService businessFileService;
    
    /**
     * 获取审批列表.
     * @param pageNum  页码
     * @param pageSize  每页显示条数
     * @return
     */
	 @RequestMapping(value="/getList", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="获取合同审批列表", notes = "查询数据列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "query")
	})
 
    public PageInfo<BusinessContractAuditVO> getList(Integer pageNum, Integer pageSize, 
            HttpServletRequest request, BusinessContractAuditVO businessContractAuditVO,Integer userId) {
    	/*UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
    	Integer userId = userInfo.getUserId();*/
        PageHelper.startPage(pageNum, pageSize);
        Map<String,Object> paramMap = new HashMap<String, Object>();
        paramMap.put("operator", userId);
        paramMap.put("changeRecordFlag", "0");
        
        if(StringUtils.isNotBlank(businessContractAuditVO.getContractNumber())) {
            paramMap.put("contractNumber", businessContractAuditVO.getContractNumber());
        }
        
        if(StringUtils.isNotBlank(businessContractAuditVO.getAuditState())) {
            paramMap.put("result", businessContractAuditVO.getAuditState());
        }
        paramMap.put("ctype", 6913);
        List<BusinessContractAuditVO> list = businessContractService.getApprovalList(paramMap);
        PageInfo<BusinessContractAuditVO> pageInfo = new PageInfo<BusinessContractAuditVO>(list);
        return pageInfo;
    }
    
    //中转
    /**
     * 获取审批列表.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
    @RequestMapping(value="/getListTransfer",produces = MediaType.APPLICATION_JSON_VALUE,  method=RequestMethod.GET)
	@ApiOperation(value="获取中转合同审批列表", notes = "查询数据列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "query")
	})
    public PageInfo<BusinessContractAuditVO> getListTransfer(Integer pageNum, Integer pageSize, 
    		HttpServletRequest request, BusinessContractAuditVO businessContractAuditVO,Integer userId) {
    	/*UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
    	Integer userId = userInfo.getUserId();*/
    	PageHelper.startPage(pageNum, pageSize);
    	Map<String,Object> paramMap = new HashMap<String, Object>();
    	paramMap.put("operator", userId);
    	paramMap.put("changeRecordFlag", "0");
    	
    	if(StringUtils.isNotBlank(businessContractAuditVO.getContractNumber())) {
    		paramMap.put("contractNumber", businessContractAuditVO.getContractNumber());
    	}
    	
    	if(StringUtils.isNotBlank(businessContractAuditVO.getAuditState())) {
    		paramMap.put("result", businessContractAuditVO.getAuditState());
    	}
        paramMap.put("contract_type", 6913);
    	List<BusinessContractAuditVO> list = businessContractService.getApprovalList(paramMap);
    	PageInfo<BusinessContractAuditVO> pageInfo = new PageInfo<BusinessContractAuditVO>(list);
    	return pageInfo;
    }
    
    /**
     * 获取审批页面详情.
     * @param id 业务主键
     * @param processInstanceId 流程实例id
     * @return   
     */
    @RequestMapping(value="/getAudit", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="获取合同审批页面详情", notes = "获取审批页面详情")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "合同id", paramType = "query"),
		@ApiImplicitParam(name = "processInstanceId", value = "流程实例id", paramType = "query"),
		@ApiImplicitParam(name = "userName", value = "当前登录人name", paramType = "query")
	})
    public Map<String, Object> getAudit(Integer id, String processInstanceId,String userName) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        BusinessContract contract = businessContractService.findById(id);
        modelMap.put("contract", contract);
       // 获取对应的文件.
        List<BusinessFile> fileList= businessFileService.getByBid(id,"contract", null, null);
        
        modelMap.put("fileList", fileList);
        // 审批历史.
        if(processInstanceId!=null){
        	HistoryList historyList= null;
        	List<HistoryList> auditList = actTaskService.histoicFlowList(processInstanceId, null, null);
        	if(auditList.size()>0 && auditList!=null){
        		for(int i = auditList.size() - 1; i >= 0; i--) { 
        			if(auditList.get(i).getComment().contains("驳回")) {
        				if(i<auditList.size()) {
        					if(auditList.get(i).getActivityName().contains("库领导审批")){
        						historyList = auditList.get(i-1);
        					}else{
        						historyList = auditList.get(i);
        					}
        					if(historyList!=null){
        						break;
        					}
        				} else {
        					historyList = null;
        				}
        			}
        		}
        	}
			modelMap.put("subAudit", historyList);
        	modelMap.put("auditList", auditList);
        }
        
        if(contract.getPlanBid()!=null){
        	// 查询该条合同引的计划的剩余数量
        	List<BusinessStoreWareDetail> storeWareDetailListPlan= businessStoreWareDetailService.getByZidAndType(contract.getPlanBid(), StoreWareType.plan.getValue());
        	//计算每条明细所对应的计划的剩余数量
        	Map<String, Object> param_map = new HashMap<String, Object>();
        	param_map.put("plan_id", contract.getPlanBid());
            List<Map<String, Object>> map = businessStoreWareDetailService.selectPlanUseCount(param_map);
            BusinessStoreWareDetail obj = null;
            double sysl = 0d;
            double sum_use_num = 0d; //已使用总量
            double plan_sum_num = 0d; //原计划数量
            for (int i = 0; i < storeWareDetailListPlan.size(); i++) {
            	obj = storeWareDetailListPlan.get(i);
            	for (Map<String, Object> m : map) {
            		if(null == m.get("warehouse_id")){ //油
                		if (m.get("house_id").equals(obj.getHouseId())) {
                			if (!"".equals(m.get("in_count")) && null != m.get("in_count") && !"".equals(obj.getInCount()) && null != obj.getInCount()) {
                				plan_sum_num = Double.parseDouble(obj.getInCount());
                				sum_use_num = Double.parseDouble(m.get("in_count")+"");
                				if (sum_use_num <= plan_sum_num) {
                					sysl = plan_sum_num - sum_use_num;
                					obj.setRemainingNumber(sysl+""); //页面上明细列表的数量
                					if (sysl == 0) {
                						obj.setRemainingNumber("0");
                					}
                				} else if (sum_use_num > plan_sum_num){
                					sysl = sum_use_num - plan_sum_num;
                					obj.setRemainingNumber("超出:" + sysl);
                				}
                			}
                			if (!"".equals(m.get("out_count")) && null != m.get("out_count") && !"".equals(obj.getOutCount()) && null != obj.getOutCount()) {
                				plan_sum_num = Double.parseDouble(obj.getOutCount());
                				sum_use_num = Double.parseDouble(m.get("out_count")+"");
                				if (sum_use_num <= plan_sum_num) {
                					sysl = plan_sum_num - sum_use_num;
                					obj.setOutRemainingNumber(""+sysl);
                				} else if (sum_use_num > plan_sum_num){
                					sysl = sum_use_num - plan_sum_num;
                					obj.setOutRemainingNumber("超出:" + sysl);
                				}
                			}
                		}
                	
            		}else{ //粮食

                		if (m.get("house_id").equals(obj.getHouseId()) && m.get("warehouse_id").equals(obj.getWarehouseId())) {
                			if (!"".equals(m.get("in_count")) && null != m.get("in_count") && !"".equals(obj.getInCount()) && null != obj.getInCount()) {
                				plan_sum_num = Double.parseDouble(obj.getInCount());
                				sum_use_num = Double.parseDouble(m.get("in_count")+"");
                				if (sum_use_num <= plan_sum_num) {
                					sysl = plan_sum_num - sum_use_num;
                					obj.setRemainingNumber(sysl+""); //页面上明细列表的数量
                					if (sysl == 0) {
                						obj.setRemainingNumber("0");
                					}
                				} else if (sum_use_num > plan_sum_num){
                					sysl = sum_use_num - plan_sum_num;
                					obj.setRemainingNumber("超出:" + sysl);
                				}
                			}
                			if (!"".equals(m.get("out_count")) && null != m.get("out_count") && !"".equals(obj.getOutCount()) && null != obj.getOutCount()) {
                				plan_sum_num = Double.parseDouble(obj.getOutCount());
                				sum_use_num = Double.parseDouble(m.get("out_count")+"");
                				if (sum_use_num <= plan_sum_num) {
                					sysl = plan_sum_num - sum_use_num;
                					obj.setOutRemainingNumber(""+sysl);
                				} else if (sum_use_num > plan_sum_num){
                					sysl = sum_use_num - plan_sum_num;
                					obj.setOutRemainingNumber("超出:" + sysl);
                				}
                			}
                		}
                	
            		}
            	}
            }

        	// 该条合同下的明细
	        List<BusinessStoreWareDetail> storeWareDetailList= businessStoreWareDetailService.getByZidAndType(id, StoreWareType.contract.getValue());
	        for (BusinessStoreWareDetail businessPlanDetail : storeWareDetailListPlan) {
	        	 for (BusinessStoreWareDetail businessStoreWareDetail : storeWareDetailList) {
	        		 if(null == businessPlanDetail.getWarehouseId()){ //油
	        			 if(businessPlanDetail.getHouseId().equals(businessStoreWareDetail.getHouseId())){
	      						if(businessPlanDetail.getRemainingNumber()!=null){
	      							businessStoreWareDetail.setRemainingNumber(businessPlanDetail.getRemainingNumber());
	      						}
	      						if(businessPlanDetail.getOutRemainingNumber()!=null){
	      							businessStoreWareDetail.setOutRemainingNumber(businessPlanDetail.getOutRemainingNumber());
	      						}
	      					}
	        		 }else{
	        			 if(businessPlanDetail.getHouseId().equals(businessStoreWareDetail.getHouseId()) &&
	      					   businessPlanDetail.getWarehouseId().equals(businessStoreWareDetail.getWarehouseId())){
	      					    
	      						if(businessPlanDetail.getRemainingNumber()!=null){
	      							businessStoreWareDetail.setRemainingNumber(businessPlanDetail.getRemainingNumber());
	      						}
	      						if(businessPlanDetail.getOutRemainingNumber()!=null){
	      							businessStoreWareDetail.setOutRemainingNumber(businessPlanDetail.getOutRemainingNumber());
	      						}
	      						
	      					}
	        		 }
					
				}
			}
	        modelMap.put("storeWareDetailList", storeWareDetailList);
        }else{
	        List<BusinessStoreWareDetail> storeWareDetailList= businessStoreWareDetailService.getByZidAndType(id, StoreWareType.contract.getValue());
	        modelMap.put("storeWareDetailList", storeWareDetailList);
        }
        return modelMap;
    }
    
    /**
     * 获取审批页面详情.
     * @param id 业务主键
     * @param processInstanceId 流程实例id
     * @return
     */
    @RequestMapping(value="/getAuditForChange",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
   	@ApiOperation(value="获取合同变更审批页面详情", notes = "查询数据列表，支持分页")
   	@ApiImplicitParams({
   		@ApiImplicitParam(name = "id", value = "合同id", paramType = "query"),
   		@ApiImplicitParam(name = "processInstanceId", value = "流程实例id", paramType = "query"),
   		@ApiImplicitParam(name = "userName", value = "当前登录人name", paramType = "query")
   	})
    public Map<String, Object> getAduitForChange(Integer id, String processInstanceId, String userName) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        // 主体.
        BusinessContract contract = businessContractService.findById(id);
        modelMap.put("contract", contract);
        // 获取对应的文件.
        List<BusinessFile> fileList= businessFileService.getByBid(id,"contract", null, null);
        
        modelMap.put("fileList", fileList);
        /*// 审批历史.
        List<HistoryList> auditList = actTaskService.histoicFlowList(processInstanceId, null, null);
        modelMap.put("auditList", auditList);*/
        
       // 审批历史.
        if(processInstanceId!=null){
        	HistoryList historyList= null;
        	List<HistoryList> auditList = actTaskService.histoicFlowList(processInstanceId, null, null);
        	if(auditList.size()>0 && auditList!=null){
        		for(int i = auditList.size() - 1; i >= 0; i--) { 
        			if(auditList.get(i).getComment().contains("驳回")) {
        				if(i<auditList.size()) {
        					historyList = auditList.get(i-1);
        					if(historyList!=null){
        						break;
        					}
        				} else {
        					historyList = null;
        				}
        			}
        		}
        	}
			modelMap.put("subAudit", historyList);
        	modelMap.put("auditList", auditList);
        }
        
        
        // 获取历史变更详情
        List<BusinessContract> changeDetailList = businessContractService.getChangeDetailList(id,"");
        modelMap.put("changeDetailList", changeDetailList);
        
        // 仓号货位号数量质量标准.
        List<BusinessStoreWareDetail> storeWareDetailList= businessStoreWareDetailService.getByZidAndType(id, StoreWareType.contract.getValue());
        if(contract.getPlanBid()!=null){
        	List<BusinessStoreWareDetail> planList= businessStoreWareDetailService.getByZidAndType(contract.getPlanBid(), StoreWareType.plan.getValue());
        	
        	/* 根据引用内容获取被引用合同明细的剩余数量 */
        	Map<String, Object> param_map = new HashMap<String, Object>();
        	param_map.put("plan_id", contract.getPlanBid());
            List<Map<String, Object>> map = businessStoreWareDetailService.selectPlanUseCount(param_map);
            BusinessStoreWareDetail obj = null;
            double sysl = 0d;
            double sum_use_num = 0d; //已使用总量
            double plan_sum_num = 0d; //原计划数量
            for (int i = 0; i < planList.size(); i++) {
            	obj = planList.get(i);
            	for (Map<String, Object> m : map) {
            		if(null == m.get("warehouse_id")){ //油
                		if (m.get("house_id").equals(obj.getHouseId())) {
                			if (!"".equals(m.get("in_count")) && null != m.get("in_count")) {
                				plan_sum_num = Double.parseDouble(obj.getInCount());
                				sum_use_num = Double.parseDouble(m.get("in_count")+"");
                				if (sum_use_num <= plan_sum_num) {
                					sysl = plan_sum_num - sum_use_num;
                					obj.setRemainingNumber(sysl+""); //页面上明细列表的数量
                					if (sysl == 0) {
                						obj.setRemainingNumber("0");
                					}
                				} else if (sum_use_num > plan_sum_num){
                					sysl = sum_use_num - plan_sum_num;
                					obj.setRemainingNumber("超出:" + sysl);
                				}
                			}
                			if (!"".equals(m.get("out_count")) && null != m.get("out_count")) {
                				plan_sum_num = Double.parseDouble(obj.getOutCount());
                				sum_use_num = Double.parseDouble(m.get("out_count")+"");
                				if (sum_use_num <= plan_sum_num) {
                					sysl = plan_sum_num - sum_use_num;
                					obj.setOutRemainingNumber(""+sysl);
                				} else if (sum_use_num > plan_sum_num){
                					sysl = sum_use_num - plan_sum_num;
                					obj.setOutRemainingNumber("超出:" + sysl);
                				}
                			}
                		}
                	
            		}else{ //粮

                		if (m.get("house_id").equals(obj.getHouseId()) && m.get("warehouse_id").equals(obj.getWarehouseId())) {
                			if (!"".equals(m.get("in_count")) && null != m.get("in_count")) {
                				plan_sum_num = Double.parseDouble(obj.getInCount());
                				sum_use_num = Double.parseDouble(m.get("in_count")+"");
                				if (sum_use_num <= plan_sum_num) {
                					sysl = plan_sum_num - sum_use_num;
                					obj.setRemainingNumber(sysl+""); //页面上明细列表的数量
                					if (sysl == 0) {
                						obj.setRemainingNumber("0");
                					}
                				} else if (sum_use_num > plan_sum_num){
                					sysl = sum_use_num - plan_sum_num;
                					obj.setRemainingNumber("超出:" + sysl);
                				}
                			}
                			if (!"".equals(m.get("out_count")) && null != m.get("out_count")) {
                				plan_sum_num = Double.parseDouble(obj.getOutCount());
                				sum_use_num = Double.parseDouble(m.get("out_count")+"");
                				if (sum_use_num <= plan_sum_num) {
                					sysl = plan_sum_num - sum_use_num;
                					obj.setOutRemainingNumber(""+sysl);
                				} else if (sum_use_num > plan_sum_num){
                					sysl = sum_use_num - plan_sum_num;
                					obj.setOutRemainingNumber("超出:" + sysl);
                				}
                			}
                		}
                	
            		}
            	}
            }
        	
        	/* 把获取的剩余数量给予当前的合同的明细剩余数量 */
        	for (BusinessStoreWareDetail contractDetail : storeWareDetailList) {
				for (BusinessStoreWareDetail planDetail : planList) {
					if(null == contractDetail.getWarehouseId()){ //油
						contractDetail.setRemainingNumber(planDetail.getRemainingNumber());
        				contractDetail.setOutRemainingNumber(planDetail.getOutRemainingNumber());
					}else{
						if(contractDetail.getHouseId().equals(planDetail.getHouseId())  && 
	        					contractDetail.getWarehouseId().equals(planDetail.getWarehouseId())){
	        				
//	        				if(contractDetail.getInCount()!=null){
//	        					contractDetail.setPlanRemainingNumber(planDetail.getRemainingNumber());
//	        				}
//	        				if(contractDetail.getOutCount()!=null){
//	        					contractDetail.setPlanOutRemainingNumber(planDetail.getOutRemainingNumber());
//	        				}
	        				contractDetail.setRemainingNumber(planDetail.getRemainingNumber());
	        				contractDetail.setOutRemainingNumber(planDetail.getOutRemainingNumber());
	        			}
					}
				}
				}
			}
        modelMap.put("storeWareDetailList", storeWareDetailList);
        return modelMap;
    }

    /**
     * 审批.
     * @param contractId 业务主键
     * @param businessApproval 中间表主体
     * @param assignee 下一个审批人
     * @param result 当前环节审批结果
     * @return
     */
	 @RequestMapping(value="/audit", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	 @ApiOperation(value="合同审批", notes = "合同审批")
	 @ApiImplicitParams({
	   		@ApiImplicitParam(name = "contractId", value = "业务主键", paramType = "query"),
	   		@ApiImplicitParam(name = "businessApproval", value = "中间表主体", paramType = "query"),
	   		@ApiImplicitParam(name = "assignee", value = "下一个审批人", paramType = "query"),
	   		@ApiImplicitParam(name = "result", value = "当前环节审批结果", paramType = "query"),
	   		@ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "query"),
	   		@ApiImplicitParam(name = "userName", value = "当前登录人name", paramType = "query")
	   	})
    public Map<String, Object> audit(Integer contractId, BusinessApproval businessApproval,String assignee,String result,Integer userId ,String realName,HttpServletRequest request) {
    	//UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
        Map<String, Object> modelMap = new HashMap<String, Object>();
        businessContractService.audit(contractId, businessApproval,assignee, result, userId , realName);
        modelMap.put("status", "success");
        return modelMap;
    }
    
    /**
     * 获取合同变更审批列表.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
	@RequestMapping(value="/getContractChangeList",produces = MediaType.APPLICATION_JSON_VALUE,  method=RequestMethod.GET)
    @ApiOperation(value="获取合同变更审批列表", notes = "获取合同变更审批列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "query")
	})
   
    public PageInfo<BusinessContractAuditVO> getContractChangeList(Integer pageNum, Integer pageSize,
            HttpServletRequest request, BusinessContractAuditVO businessContractAuditVO,Integer userId) {
    	/*UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
    	Integer userId = userInfo.getUserId();*/
        PageHelper.startPage(pageNum, pageSize);
        Map<String,Object> paramMap = new HashMap<String, Object>();
        // TODO 获取当前登录人
        paramMap.put("operator", userId);
        paramMap.put("changeRecordFlag", "1");
        
        if(StringUtils.isNotBlank(businessContractAuditVO.getContractNumber())) {
            paramMap.put("contractNumber", businessContractAuditVO.getContractNumber());
        }
        
       /* if(businessContractAuditVO.getGrainKind() != null) {
            paramMap.put("grainKind", businessContractAuditVO.getGrainKind());
        }*/
        
        if(StringUtils.isNotBlank(businessContractAuditVO.getAuditState())) {
            paramMap.put("result", businessContractAuditVO.getAuditState());
        }
        paramMap.put("ctype", 6913);
        List<BusinessContractAuditVO> list = businessContractService.getApprovalList(paramMap);
        PageInfo<BusinessContractAuditVO> pageInfo = new PageInfo<BusinessContractAuditVO>(list);
        return pageInfo;
    }
    
    //中转
    /**
     * 获取合同变更审批列表.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
	@RequestMapping(value="/getContractChangeListTransfer", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="获取中转合同变更审批列表", notes = "获取合同变更审批列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "query")
	})
   
    public PageInfo<BusinessContractAuditVO> getContractChangeListTransfer(Integer pageNum, Integer pageSize,
    		HttpServletRequest request, BusinessContractAuditVO businessContractAuditVO,Integer userId) {
    	/*UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
    	Integer userId = userInfo.getUserId();*/
    	PageHelper.startPage(pageNum, pageSize);
    	Map<String,Object> paramMap = new HashMap<String, Object>();
    	// TODO 获取当前登录人
    	paramMap.put("operator", userId);
    	paramMap.put("changeRecordFlag", "1");
    	
    	if(StringUtils.isNotBlank(businessContractAuditVO.getContractNumber())) {
    		paramMap.put("contractNumber", businessContractAuditVO.getContractNumber());
    	}
    	
    	/* if(businessContractAuditVO.getGrainKind() != null) {
            paramMap.put("grainKind", businessContractAuditVO.getGrainKind());
        }*/
    	
    	if(StringUtils.isNotBlank(businessContractAuditVO.getAuditState())) {
    		paramMap.put("result", businessContractAuditVO.getAuditState());
    	}
    	paramMap.put("contract_type", 6913);
    	List<BusinessContractAuditVO> list = businessContractService.getApprovalList(paramMap);
    	PageInfo<BusinessContractAuditVO> pageInfo = new PageInfo<BusinessContractAuditVO>(list);
    	return pageInfo;
    }
    
    
}
