package com.chinaitop.depot.business.controller.audit;


import com.chinaitop.depot.act.model.HistoryList;
import com.chinaitop.depot.act.service.ActTaskService;
import com.chinaitop.depot.business.model.BusinessApproval;
import com.chinaitop.depot.business.model.BusinessFile;
import com.chinaitop.depot.business.model.BusinessPlan;
import com.chinaitop.depot.business.model.BusinessStoreWareDetail;
import com.chinaitop.depot.business.model.enums.StoreWareType;
import com.chinaitop.depot.business.model.vo.BusinessPlanAuditVO;
import com.chinaitop.depot.business.service.BusinessApprovalService;
import com.chinaitop.depot.business.service.BusinessFileService;
import com.chinaitop.depot.business.service.BusinessPlanService;
import com.chinaitop.depot.business.service.BusinessStoreWareDetailService;
import com.chinaitop.depot.utils.activiti.StringUtils;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 粮库业务管理-计划审批管理
* <p>Title: BusinessPlanAuditController</p>
* <p>Description: </p>
* @author yaoyabin
* @date 2017年10月25日 下午5:50:37
 */
@RestController
@RequestMapping(value="/depot/business/plan/audit")
@Api(value= "BusinessPlanAuditController", description = "计划审批管理")
public class BusinessPlanAuditController {
    
    @Resource
    private BusinessPlanService businessPlanService;
    @Resource
    private BusinessStoreWareDetailService businessStoreWareDetailService;
    @Resource
    private BusinessApprovalService businessApprovalService;
    @Autowired
    private BusinessFileService businessFileService;
    @Resource
    private ActTaskService actTaskService;
    
    /**
     * 获取审批列表.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
    @RequestMapping(value="/getList",  method=RequestMethod.GET)
	@ApiOperation(value="获取计划审批列表", notes = "查询数据列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人的id", paramType = "query")
	})
    public PageInfo<BusinessPlanAuditVO> getList(Integer pageNum, Integer pageSize,HttpServletRequest request,BusinessPlanAuditVO businessPlanAuditVO,Integer userId) {
    	/*UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
    	Integer userId = userInfo.getUserId();*/
    	
        PageHelper.startPage(pageNum, pageSize);
        
        
        Map<String,Object> paramMap = new HashMap<String, Object>();
        paramMap.put("operator", userId);
        if(StringUtils.isNotEmpty(businessPlanAuditVO.getPlanNumber())&&businessPlanAuditVO.getPlanNumber()!=null){
        	String planNumber = businessPlanAuditVO.getPlanNumber();
        	paramMap.put("planNumber", planNumber);
		}
		if(StringUtils.isNotEmpty(businessPlanAuditVO.getAuditState())&&businessPlanAuditVO.getAuditState()!=null){
			String result = businessPlanAuditVO.getAuditState();
			paramMap.put("result", result);
		}
		//储备粮计划
		if(null!=businessPlanAuditVO.getAttributeType()){
			Integer attributeType = businessPlanAuditVO.getAttributeType();
			paramMap.put("attributeType", attributeType);
		}
        List<BusinessPlanAuditVO> list = businessPlanService.getApprovalList(paramMap);
        PageInfo<BusinessPlanAuditVO> pageInfo = new PageInfo<BusinessPlanAuditVO>(list);
        return pageInfo;
    }
    
    /**
     * 获取审批页面详情.
     * @param id  业务主键
     * @param processInstanceId 流程实例id
     * @return
     */
    @RequestMapping(value="/getAudit", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="获取计划审批页面详情", notes = "获取计划审批页面详情")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "计划id", paramType = "query"),
		@ApiImplicitParam(name = "processInstanceId", value = "流程实例id", paramType = "query")
	})
    public Map<String, Object> getAduit(Integer id, String processInstanceId,String userName,Integer userId) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        // 详情.
        BusinessPlan plan = businessPlanService.findById(id);
        modelMap.put("plan", plan);
        
        // 审批列表.
        if(processInstanceId!=null){
        	HistoryList historyList= null;
        	List<HistoryList> auditList = actTaskService.histoicFlowList(processInstanceId, null, null);
        	if(auditList.size()>0 && auditList!=null){
        		for(int i = auditList.size() - 1; i >= 0; i--) { 
        			if(auditList.get(i).getComment().contains("驳回")) {
        				if(i<auditList.size()) {
        					if(auditList.get(i).getActivityName().contains("库领导审批")){
        						historyList = auditList.get(i-1);
        					}else{
        						historyList = auditList.get(i);
        					}
        					if(historyList!=null){
        						break;
        					}
        				} else {
        					historyList = null;
        				}
        			}
        		}
        	}
			modelMap.put("subAudit", historyList);
        	modelMap.put("auditList", auditList);
        }
        
        // 获取对应的文件.
        List<BusinessFile> fileList= businessFileService.getByBid(id,"plan", null, null);
        
        modelMap.put("fileList", fileList);
        
        // 仓号货位号数量质量标准.
        List<BusinessStoreWareDetail> storeWareDetailList= businessStoreWareDetailService.getByZidAndType(id, StoreWareType.plan.getValue());
        modelMap.put("storeWareDetailList", storeWareDetailList);
        return modelMap;
    }

    /**
     * 审批.
     * @param planId 业务主键
     * @param businessApproval 中间表
     * @param assignee 下一个环节审批人
     * @param result 当前环节审批结果
     * @return
     */
    @RequestMapping(value="/audit", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="计划审批", notes = "计划审批")
    @ApiImplicitParams({
    	@ApiImplicitParam(name="planId", value="计划id", paramType="form"),
    	@ApiImplicitParam(name="businessApproval", value="中间表数据", paramType="form"),
    	@ApiImplicitParam(name="assignee", value="下一个环节审批人", paramType="form"),
    	@ApiImplicitParam(name="result", value="当前环节审批结果", paramType="form"),
    	@ApiImplicitParam(name="userId", value="当前登录人id", paramType="form"),
    	@ApiImplicitParam(name="realName", value="前登录人姓名", paramType="form")
    })
    public Map<String, Object> audit(HttpServletRequest request, Integer planId,
            BusinessApproval businessApproval, String assignee, String result,Integer userId, String realName) {
        //UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
        Map<String, Object> modelMap = new HashMap<String, Object>();
        businessPlanService.audit(planId, businessApproval,assignee,result,  userId,  realName);
        modelMap.put("status", "success");
        return modelMap;
    }

    /**
     * 获取计划验收列表.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
    @RequestMapping(value="/getAcceptanceList", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="计划验收", notes = "获取计划验收列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "userId", value = "当前登录人的id", paramType = "query"),
            @ApiImplicitParam(name = "businessPlanAuditVO", value = "查询条件", paramType = "query")
    })
    public PageInfo<BusinessPlanAuditVO> getAcceptanceList(Integer pageNum, Integer pageSize,String userId,BusinessPlanAuditVO businessPlanAuditVO) {
        PageHelper.startPage(pageNum, pageSize);
        String planNumber = businessPlanAuditVO.getPlanNumber();
        String planTitle = businessPlanAuditVO.getPlanTitle();
        Map<String,Object> paramMap = new HashMap<String, Object>();
        paramMap.put("operator", userId);
        if(StringUtils.isNotEmpty(businessPlanAuditVO.getPlanNumber())&&businessPlanAuditVO.getPlanNumber()!=null){
            paramMap.put("planNumber", planNumber);
        }
        if(StringUtils.isNotEmpty(businessPlanAuditVO.getPlanTitle())&&businessPlanAuditVO.getPlanTitle()!=null){
            paramMap.put("planTitle", planTitle);
        }
        List<BusinessPlanAuditVO> list = businessPlanService.getAcceptanceList(paramMap);
        PageInfo<BusinessPlanAuditVO> pageInfo = new PageInfo<BusinessPlanAuditVO>(list);
        return pageInfo;
    }

    /**
     * 保存
     * @param businessPlanId 计划json
     * @return
     * @throws JsonParseException
     * @throws JsonMappingException
     * @throws IOException
     */
    @RequestMapping(value="/save", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    @ApiOperation(value="保存计划验收", notes = "保存计划验收详细信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "businessPlanId", value = "计划id", paramType = "query"),
            @ApiImplicitParam(name = "businessPlancreateTime", value = "计划验收时间", paramType = "query"),
            @ApiImplicitParam(name = "businessStoreWareDetailJson", value = "验收详细信息", paramType = "query"),
            @ApiImplicitParam(name = "fileIds", value = "附件id", paramType = "query"),
            @ApiImplicitParam(name = "acceptanceNumber", value = "申请验收编号", paramType = "query"),
    })
    public Map<String, Object> save(String businessPlanId,String acceptanceNumber,String businessPlancreateTime, String qualityIds, String fileIds) throws JsonParseException, JsonMappingException, IOException {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        BusinessPlan plan = new BusinessPlan();
        try {
        	Date date = new Date();
        	plan.setAcceptTanceDate(date);
        	plan.setCustomerPlanState("60");
        	plan.setId(Integer.valueOf(businessPlanId));
        	businessPlanService.updatePlanStatus(plan);
            // JSON字符串转对象
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            Map<String, Object> returnMap = businessPlanService.addAcceptance(businessPlanId,acceptanceNumber, sdf.format(date), qualityIds, fileIds);
            if (returnMap.get("error") != null) {
                modelMap.put("status", "error");
                modelMap.put("msg", returnMap.get("error"));
                return modelMap;
            }
            modelMap.put("id", returnMap.get("id"));

            modelMap.put("status", "success");
        } catch (Exception e) {
            e.printStackTrace();
            modelMap.put("status", "error");
            modelMap.put("msg", "保存失败！");
            return modelMap;
        }
        return modelMap;
    }

    /**
     * 获取计划验收列表.
     * @param bid 验收id
     * @return
     */
    @RequestMapping(value="/getDataByZJId", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="计划验收详情", notes = "查看验收计划详情。")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bid", value = "主表id", paramType = "query"),
            @ApiImplicitParam(name = "fileType", value = "文件类型", paramType = "query")
    })
    public Map<String,Object> getDataByZJId(Integer bid,String fileType) {
        Map<String,Object> map = new HashMap<String, Object>();
        List<BusinessFile> fileList= businessFileService.getByBid(bid, fileType, null, null);
        map.put("fileList", fileList);
        return map;
    }
}
