package com.chinaitop.depot.business.controller.audit;


import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.act.model.HistoryList;
import com.chinaitop.depot.act.service.ActTaskService;
import com.chinaitop.depot.business.model.BusinessNoticeReceive;
import com.chinaitop.depot.business.service.BusinessApprovalService;
import com.chinaitop.depot.business.service.FeignSystemService;
import com.chinaitop.depot.business.service.ReceiveNoticeService;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 性质转变单审批
 * @author My Sunshine
 *
 */
@RestController
@RequestMapping(value="/depot/business/propSheet/audit")
@Api(value= "BusinessProSheetAuditController", description = "性质转变单审批管理")
public class BusinessProSheetAuditController {
    
	@Resource
	private ReceiveNoticeService receiveNoticeService;
	
    @Resource
    private BusinessApprovalService businessApprovalService;
    @Resource
    private ActTaskService actTaskService;
    @Resource
    private FeignSystemService feignSystemService;
    
    
    /**
     * 获取审批列表.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
    @RequestMapping(value="/getList",  method=RequestMethod.GET)
	@ApiOperation(value="获取性质转变单审批列表", notes = "查询数据列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人的id", paramType = "query")
	})
    public PageInfo<BusinessNoticeReceive> getList(Integer pageNum, Integer pageSize,HttpServletRequest request,BusinessNoticeReceive businessNoticeReceive,Integer userId) {
    	
    	PageHelper.startPage(pageNum, pageSize);
    	List<BusinessNoticeReceive> list = new ArrayList<BusinessNoticeReceive>();
        
    	
        List<Map<String,String>> userRoleList = feignSystemService.getUserRoleFeign(userId);

        //List<Map<String,String>> userRoleList = receiveNoticeService.getUserRole(userId);
        String roleNames = "";
        for (Map<String, String> map : userRoleList) {
        	String roleName =  map.get("role_name");
        	roleNames = roleNames + roleName+",";
		}
       // System.out.println(roleNames);
        List<BusinessNoticeReceive> listAll = receiveNoticeService.queryByExample(businessNoticeReceive);
        for (BusinessNoticeReceive businessNoticeReceive2 : listAll) {
        	//0仓储部经理审批 1质检部经理审批 2统计经理审批 3会计审批 4库领导审批
			if(businessNoticeReceive2.getAuditState().equals(0)){
				if(roleNames.contains("仓储部经理")){
					list.add(businessNoticeReceive2);
				}
			 }
			if(businessNoticeReceive2.getAuditState().equals(1)){
				if(roleNames.contains("质检部经理")){
					list.add(businessNoticeReceive2);
				}
			 }
			if(businessNoticeReceive2.getAuditState().equals(2)){
				if(roleNames.contains("统计部经理")){
					list.add(businessNoticeReceive2);
				}
			 }
			if(businessNoticeReceive2.getAuditState().equals(3)){
				if(roleNames.contains("会计")){
					list.add(businessNoticeReceive2);
				}
			 }
			if(businessNoticeReceive2.getAuditState().equals(4)){
				if(roleNames.contains("库领导")){
					list.add(businessNoticeReceive2);
				}
			 }
		}
        
        PageHelper.startPage(pageNum, pageSize);
        PageInfo<BusinessNoticeReceive> pageInfo = new PageInfo<BusinessNoticeReceive>(list);
        if(pageInfo.getTotal()>0){
        	pageInfo.setStartRow(pageInfo.getStartRow()+1);
        	pageInfo.setEndRow(pageInfo.getEndRow()+1);
        }
        return pageInfo;
    }
    
    /**
     * 获取审批页面详情.
     * @param id  业务主键
     * @param processInstanceId 流程实例id
     * @return
     */
    @RequestMapping(value="/getAudit", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="获取审批页面详情", notes = "获取审批页面详情")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "原数据的id", paramType = "query")
	})
    public Map<String, Object> getAduit(Integer id) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        // 详情.
        BusinessNoticeReceive businessNoticeReceive = receiveNoticeService.findById(id);
        modelMap.put("businessNoticeReceive", businessNoticeReceive);
        
        modelMap.put("xzzbdh", businessNoticeReceive.getLsxzzbdh());
        // 审批历史.
        List<HistoryList> auditList = businessApprovalService.getAudit(modelMap);//该id是元数据的id 是中间表的project_id
        modelMap.put("auditList", auditList);
        
        return modelMap;
    }

    /**
     * 审批.
     * @param planId 业务主键
     * @param businessApproval 中间表
     * @param assignee 下一个环节审批人
     * @param result 当前环节审批结果
     * @return
     */
    @RequestMapping(value="/audit", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="审批", notes = "审批")
    @ApiImplicitParams({
    	@ApiImplicitParam(name="receiveNoticeJson", value="原数据", paramType="form"),
    	@ApiImplicitParam(name="content", value="审批意见", paramType="form"),
    	@ApiImplicitParam(name="userId", value="当前登录人id", paramType="form"),
    	@ApiImplicitParam(name="realName", value="前登录人姓名", paramType="form")
    })
    public Map<String, Object> audit(HttpServletRequest request, 
    		String receiveNoticeJson, Integer userId, String realName,String orgName,String content) {
        //UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
        Map<String, Object> modelMap = new HashMap<String, Object>();
        ObjectMapper mapper = new ObjectMapper();
        BusinessNoticeReceive receiveNotice;
		try {
			receiveNotice = (BusinessNoticeReceive)mapper.readValue(receiveNoticeJson, BusinessNoticeReceive.class);
			receiveNoticeService.audit(receiveNotice, userId,  realName,orgName,content);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
       
        modelMap.put("status", "success");
        return modelMap;
    }

 

    
}
