package com.chinaitop.depot.business.controller.audit;


import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.act.model.HistoryList;
import com.chinaitop.depot.act.service.ActTaskService;
import com.chinaitop.depot.business.model.BusinessApproval;
import com.chinaitop.depot.business.model.BusinessDeliveryStorageNotice;
import com.chinaitop.depot.business.model.BusinessStoreWareDetail;
import com.chinaitop.depot.business.model.enums.StoreWareType;
import com.chinaitop.depot.business.model.vo.BusinessDeliveryStorageNoticeAuditVO;
import com.chinaitop.depot.business.service.BusinessApprovalService;
import com.chinaitop.depot.business.service.BusinessDeliveryStorageNoticeService;
import com.chinaitop.depot.business.service.BusinessStoreWareDetailService;
import com.chinaitop.depot.utils.activiti.StringUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 粮库业务管理-出入库通知单审批
* <p>Title: BusinessDeliveryStorageNoticeAuditController</p>
* <p>Description: </p>
* @author My   Sunshine
 */
@RestController
@RequestMapping(value="/depot/business/grainNotice/audit")
@Api(value= "BusinessDeliveryStorageNoticeAuditController", description = "出入库通知单审批")
public class GrainNoticeAuditController {
    
    @Resource
    private BusinessDeliveryStorageNoticeService businessDeliveryStorageNoticeService;
    @Resource
    private BusinessStoreWareDetailService businessStoreWareDetailService;
    @Resource
    private BusinessApprovalService businessApprovalService;
    @Resource
    private ActTaskService actTaskService;
    
    /**
     * 获取审批列表.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
    @RequestMapping(value="/getList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="获取审批列表", notes = "查询数据列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "query")
	})
    public PageInfo<BusinessDeliveryStorageNoticeAuditVO> getList(Integer pageNum, Integer pageSize, 
            BusinessDeliveryStorageNotice notice, HttpServletRequest request,Integer userId) {
    	
        PageHelper.startPage(pageNum, pageSize);
        Map<String,Object> paramMap = new HashMap<String, Object>();
        paramMap.put("operator", userId);
        paramMap.put("changeRecordFlag", "0");
        if (StringUtils.isNotEmpty(notice.getBillType())) {
            paramMap.put("billType", notice.getBillType());
        }
        if (StringUtils.isNotBlank(notice.getAuditState())) {
            paramMap.put("result", notice.getAuditState());
        }
        paramMap.put("createUnit", 1);//储备粮下的通知单
        List<BusinessDeliveryStorageNoticeAuditVO> list = businessDeliveryStorageNoticeService.getApprovalList(paramMap);
        PageInfo<BusinessDeliveryStorageNoticeAuditVO> pageInfo = new PageInfo<BusinessDeliveryStorageNoticeAuditVO>(list);
        return pageInfo;
    }
    
    /**
     * 获取审批页面详情.
     * @param id 业务主键
     * @param processInstanceId 流程实例id
     * @return
     */
    @RequestMapping(value="/getAudit", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
   	@ApiOperation(value="获取审批页面详情", notes = "获取审批页面详情")
   	@ApiImplicitParams({
   		@ApiImplicitParam(name = "id", value = "通知单id", paramType = "query"),
   		@ApiImplicitParam(name = "processInstanceId", value = "流程实例id", paramType = "query"),
   		@ApiImplicitParam(name = "userName", value = "当前登录人name", paramType = "query")
   	})
    public Map<String, Object> getAduit(Integer id, String processInstanceId,String userName) {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        // 详情.
        BusinessDeliveryStorageNotice deliveryStorageNotice = businessDeliveryStorageNoticeService.findById(id);
        modelMap.put("deliveryStorageNotice", deliveryStorageNotice);
        
        // 审批列表.
        //List<HistoryList> auditList = actTaskService.histoicFlowList(processInstanceId, null, null);
        //modelMap.put("auditList", auditList);
        
        // 审批列表.
        if(processInstanceId!=null){
        	HistoryList historyList= null;
        	List<HistoryList> auditList = actTaskService.histoicFlowList(processInstanceId, null, null);
        	if(auditList.size()>0 && auditList!=null){
        		for(int i = auditList.size() - 1; i >= 0; i--) { 
        			if(auditList.get(i).getComment().contains("驳回")) {
        				if(i<auditList.size()) {
        					/*if(auditList.get(i).getActivityName().contains("库领导审批")){
        						historyList = auditList.get(i-1);
        					}else{
        						historyList = auditList.get(i);
        					}*/
        					historyList = auditList.get(1);
        					if(historyList!=null){
        						break;
        					}
        				} else {
        					historyList = null;
        				}
        			}
        		}
        	}
			modelMap.put("subAudit", historyList);
        	modelMap.put("auditList", auditList);
        }
        // 仓号货位号数量质量标准.
        List<BusinessStoreWareDetail> storeWareDetailList= businessStoreWareDetailService.getByZidAndType(id, StoreWareType.notice.getValue());
        modelMap.put("storeWareDetailList", storeWareDetailList);
        return modelMap;
    }

    /**
     * 审批.
     * @param deliveryStorageNoticeId 业务主键
     * @param businessApproval 中间表
     * @param assignee 下一个环节审批人
     * @param result 当前环节审批结果
     * @return
     */
    @RequestMapping(value="/audit", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
   	@ApiOperation(value="通知单审批", notes = "审批")
    @ApiImplicitParams({
   		@ApiImplicitParam(name = "deliveryStorageNoticeId", value = "业务主键", paramType = "query"),
   		@ApiImplicitParam(name = "businessApproval", value = "中间表", paramType = "query"),
   		@ApiImplicitParam(name = "assignee", value = "下一个环节审批人", paramType = "query"),
   		@ApiImplicitParam(name = "result", value = "当前环节审批结果", paramType = "query"),
   		@ApiImplicitParam(name = "name", value = "当前登录人name", paramType = "query")
   	})
    public Map<String, Object> audit(Integer deliveryStorageNoticeId, BusinessApproval businessApproval,String assignee,String result,String name ,HttpServletRequest request) {
    	//UserInfo userInfo= (UserInfo) request.getSession().getAttribute("userInfo");
        Map<String, Object> modelMap = new HashMap<String, Object>();
        businessDeliveryStorageNoticeService.audit(deliveryStorageNoticeId, businessApproval,assignee,result, name);
        modelMap.put("status", "success");
        return modelMap;
    }
    
    /**
     * 获取通知单变更审批列表.
     * @param pageNum 页码
     * @param pageSize 每页显示条数
     * @return
     */
	@RequestMapping(value="/getNoticeChangeList",produces = MediaType.APPLICATION_JSON_VALUE,  method=RequestMethod.GET)
    @ApiOperation(value="获取通知单变更审批列表", notes = "获取通知单变更审批列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "当前登录人id", paramType = "query")
	})
   
    public PageInfo<BusinessDeliveryStorageNoticeAuditVO> getNoticeChangeList(Integer pageNum, Integer pageSize,
            HttpServletRequest request, BusinessDeliveryStorageNotice notice,Integer userId) {
        PageHelper.startPage(pageNum, pageSize);
        Map<String,Object> paramMap = new HashMap<String, Object>();
        // TODO 获取当前登录人
        paramMap.put("operator", userId);
        paramMap.put("changeRecordFlag", "1");
        
        if (StringUtils.isNotEmpty(notice.getBillType())) {
            paramMap.put("billType", notice.getBillType());
        }
        if (StringUtils.isNotBlank(notice.getAuditState())) {
            paramMap.put("result", notice.getAuditState());
        }
        paramMap.put("createUnit", 1);//储备粮下的通知单
        List<BusinessDeliveryStorageNoticeAuditVO> list = businessDeliveryStorageNoticeService.getApprovalList(paramMap);
        PageInfo<BusinessDeliveryStorageNoticeAuditVO> pageInfo = new PageInfo<BusinessDeliveryStorageNoticeAuditVO>(list);
        return pageInfo;
    }
}
