package com.chinaitop.depot.business.service;

import com.chinaitop.depot.business.model.BusinessApproval;
import com.chinaitop.depot.business.model.BusinessContract;
import com.chinaitop.depot.business.model.BusinessContractExample;
import com.chinaitop.depot.business.model.BusinessStoreWareDetail;
import com.chinaitop.depot.business.model.vo.BusinessContractAuditVO;
import com.github.pagehelper.PageInfo;

import java.util.List;
import java.util.Map;

public interface GrainContractService {
	
	List<BusinessContract> queryByExample(BusinessContractExample example);
	
	BusinessContract findById(Integer id);
	
	void update(BusinessContract contract, List<BusinessStoreWareDetail> storeWareDetailList, String fileIds, String deleteFileIds);
	
	void remove(Integer id,String processInstanceId,String deleteReason, Integer planBid);
	
	/**
	 * 提交.
	 * @param contract
	 * @param nextUserId 下一个审批人id.
	 * @param userInfo
	 * @return
	 */
	Map<String, Object> submit(BusinessContract contract, String nextUserId, Integer userId,String realName);

    /**
     * 获取审批列表.
     * @param paramMap
     * @return
     */
    List<BusinessContractAuditVO> getApprovalList(Map<String, Object> paramMap);

    /**
     * 
     * @param contractId
     * @param businessApproval
     * @param nextUserId 下一个审批人id.
     * @param result 审批结论
     * @param userInfo 当前用户信息.
     */
    void audit(Integer contractId, BusinessApproval businessApproval,String nextUserId, String result, Integer userId ,String realName);

    /**
     * 保存合同变更数据.
     * @param contract
     * @param storeWareDetailList 
     * @param fileIds 
     */
    Map<String, Object> saveChange(BusinessContract contract, List<BusinessStoreWareDetail> storeWareDetailList, Integer userId, String userName, String realName, Integer depotInfoOrgId,Integer id, String fileIds);
    /**
     * 修改合同变更数据.
     * @param contract
     * @param storeWareDetailList 
     */
    void updateChange(BusinessContract contract, List<BusinessStoreWareDetail> storeWareDetailList);

    /**
     * 删除合同变更数据.
     * @param id
     * @param originalContractBid 原合同id
     * @param deleteReason 
     * @param processInstanceId 
     * @param planBid 
     * @param grainQuantity 
     * @return 
     */
    Map<String, Object> changeContractRemove(Integer id, Integer originalContractBid, String processInstanceId, String deleteReason, Integer planBid);

    /**
     * 获取历次变更详情.
     * @param id
     * @param type 
     * @return
     */
    List<BusinessContract> getChangeDetailList(Integer id, String type);

    /**
     * @param pageNum
     * @param pageSize
     * @param contract
     * @return
     */
    PageInfo<BusinessContractAuditVO> getScheduleList(Integer pageNum,
            Integer pageSize, BusinessContract contract);

    /**
     * @param pageNum
     * @param pageSize
     * @param userId
     * @param contract
     * @param order 
     * @param contractType 合同类型，空就是正常的合同，change就是合同变更.
     * @return
     */
    List<BusinessContract> queryByExample(Integer pageNum, Integer pageSize,
            Integer userId, BusinessContract contract, String order, String contractType);

    /**
     * @param pageNum
     * @param pageSize
     * @param contract
     * @param billType 
     * @return
     */
    List<BusinessContract> getAuditPassList(Integer pageNum, Integer pageSize,
            BusinessContract contract, Integer billType,String lyflag);

    /**
     * 校验合同编号，是否可用.
     * @param id 主键.
     * @param contractNumber 合同编号.
     * @return true 为可用，false为不可用.
     */
    Boolean validContractNumber(Integer id, String contractNumber);

    /**
     * 启用
     * @param id
     */
    Map<String, String> application(Integer id);

    /**
     * 暂停
     * @param id
     * @param refuseReason 
     */
    void discontinue(Integer id, String refuseReason);
    
    /**
     * 终止.
     * @param id
     */
    void finish(Integer id,String refuseReason);

    /**
     * 通过组织ID查询出来的计划和合同数据
     * 查询追溯--业务信息
     */
    List<Map<String,Object>> getAllBusiness(String orgId);

    /**
     * @param contract
     * @param storeWareDetailList
     * @param fileIds 
     * @param userInfo
     * @param depotInfo
     * @return
     */
    Map<String, Object> add(BusinessContract contract,
            List<BusinessStoreWareDetail> storeWareDetailList, Integer userId,
            String realName,Integer depotInfoOrgId, String fileIds);

    /**
     * 根据计划id修改应用状态.
     * @param paramMap
     */
    void updateApplicationByPlanBid(Map<String, Object> paramMap);


	//审批通过列表(所有类型的合同，用于移动端).
	List<BusinessContract> getAuditPassListApp(Integer pageNum, Integer pageSize, BusinessContract contract,
			Integer billType);

	List<BusinessContract> queryContractChangeList(Integer userId, Integer pageNum, Integer pageSize);

	/*List<BusinessContract> getAllContractInfo();*/

}
