package com.chinaitop.depot.business.service;

import com.chinaitop.depot.business.model.UserInfo;
import com.chinaitop.depot.business.model.UserInfoExample;

import java.util.HashMap;
import java.util.List;

public interface UserInfoService {
	
	/**
	 * 验证用户名，密码是否正确
	 * @param username
	 * @param password
	 * @return
	 */
	UserInfo valid(String username, String password);
	
	/**
	 * 验证用户名是否可用
	 * @param userId
	 * @param username
	 * @return
	 */
	boolean validUsername(Integer userId, String username);

	/**
	 * 验证用户输入的原密码是否正确
	 * @param resPassword 数据库中保存的密码
	 * @param oldPassword	用户输入的原密码
	 * @return
	 */
	boolean validPassword(String resPassword, String oldPassword);

	/**
	 * 重置密码
	 * @param userId	用户编号
	 */
	void resetPassword(Integer userId);
	
	/**
	 * 根据条件查询用户列表
	 * @param example
	 * @return
	 */
	List<UserInfo> queryByExample(UserInfoExample example);
	
	/**
	 * 根据用户编号获取用户信息
	 * @param userId
	 * @return
	 */
	UserInfo findById(Integer userId);
	
	/**
	 * 新增用户
	 * @param userInfo
	 */
	void add(UserInfo userInfo);
	
	/**
	 * 更新用户信息
	 * @param userInfo
	 */
	void update(UserInfo userInfo);
	
	/**
	 * 根据用户编号删除用户信息
	 * @param userId
	 */
	void remove(Integer userId);

	//列表查询
	List<UserInfo> queryAll(HashMap<Object, Object> map);
	
	String getMD5(String str);

}
