package com.chinaitop.depot.business.service.impl;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.activiti.engine.runtime.ProcessInstance;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.act.service.ActTaskService;
import com.chinaitop.depot.agile.model.BusinessScedule;
import com.chinaitop.depot.business.mapper.BusinessContractMapper;
import com.chinaitop.depot.business.model.BusinessApproval;
import com.chinaitop.depot.business.model.BusinessContract;
import com.chinaitop.depot.business.model.BusinessContractExample;
import com.chinaitop.depot.business.model.BusinessContractExample.Criteria;
import com.chinaitop.depot.business.model.BusinessDeliveryStorageNotice;
import com.chinaitop.depot.business.model.BusinessDeliveryStorageNoticeExample;
import com.chinaitop.depot.business.model.BusinessPlan;
import com.chinaitop.depot.business.model.BusinessStoreWareDetail;
import com.chinaitop.depot.business.model.UserInfo;
import com.chinaitop.depot.business.model.enums.AuditType;
import com.chinaitop.depot.business.model.enums.CrkNoticeStatus;
import com.chinaitop.depot.business.model.enums.StoreWareType;
import com.chinaitop.depot.business.model.enums.TaskType;
import com.chinaitop.depot.business.model.vo.BusinessContractAuditVO;
import com.chinaitop.depot.business.service.BusinessApprovalService;
import com.chinaitop.depot.business.service.BusinessContractService;
import com.chinaitop.depot.business.service.BusinessDeliveryStorageNoticeService;
import com.chinaitop.depot.business.service.BusinessFileService;
import com.chinaitop.depot.business.service.BusinessPlanService;
import com.chinaitop.depot.business.service.BusinessStoreWareDetailService;
import com.chinaitop.depot.business.service.FeignAgileService;
import com.chinaitop.depot.business.service.UserInfoService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Maps;


@Service
public class BusinessContractServiceImpl implements BusinessContractService {
    @Resource
    private BusinessContractMapper contractMapper;
    
    @Resource
    private BusinessApprovalService businessApprovalService;
    @Resource
    private BusinessPlanService planService;
    @Resource
    private BusinessDeliveryStorageNoticeService deliveryStorageNoticeService;
    @Resource
    private BusinessStoreWareDetailService businessStoreWareDetailService;
    @Resource
    private BusinessContractService contractService;
    @Resource
	private FeignAgileService feignAgileService;
    /*@Resource
    private BusinessSceduleService businessSceduleService;*/
    @Resource
    private ActTaskService actTaskService;
    @Resource
    private UserInfoService userInfoService;
    @Resource
    private BusinessStoreWareDetailService detailService;
    @Resource
    private BusinessFileService businessFileService;
    

    public List<BusinessContract> queryByExample(BusinessContractExample example) {
        return contractMapper.selectByExample(example);
    }

    public BusinessContract findById(Integer id) {
        return contractMapper.selectByPrimaryKey(id);
    }

    /**
     * 新增数据，启动流程.
     */
    @Transactional(readOnly = false)
    public Map<String, Object> add(BusinessContract contract, List<BusinessStoreWareDetail> storeWareDetailList,
            Integer userId,String realName,Integer depotInfoOrgId,String fileIds) {
        Map<String, Object> returnMap = new HashMap<String, Object>();
        
        /*// 通过规则，获取合同编号.
        try {
            contract.setContractNumber(basicCodeRuleService.getCode(CodeRuleType.contract, depotInfo.getOrgId()));
        } catch (Exception e) {
            e.printStackTrace();
            returnMap.put("error", e.getMessage());
            return returnMap;
        }*/
        // 合同数量.
        String grainQuantity = contract.getGrainQuantity();
        BigDecimal grainQuantityB = new BigDecimal(grainQuantity);
        // 合同单价.
       // String grainPrice = contract.getGrainPrice();
        //BigDecimal grainPriceB = new BigDecimal(grainPrice);
        
        // 总金额 = 合同数量*合同单价. 两位小数，四舍五入.
       // contract.setMoneyQuantity(grainQuantityB.multiply(grainPriceB).setScale(2, BigDecimal.ROUND_HALF_UP).toString());
        
        // 设置审批状态为，审批待提交.
        contract.setAuditState(AuditType.UNSUBMIT.getAuditCode());
        // 创建时间.
        contract.setCreateTime(new Date());
        // 设置登录人信息.
        contract.setCreater(realName);
        contract.setCreaterId(userId);
        contract.setCreateMan(realName);
        contract.setOrgId(depotInfoOrgId);
        //contract.setCreateUnit("经营一部");
        
        // 没有被变更.
        contract.setChanging("0");
        // 不是变更数据.
        contract.setChangeRecordFlag("0");
        // 启用状态设定为执行中，启用
        contract.setInApplication(CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode());
        // 保存，返回id,id在流程启动时，需要使用.
        contractMapper.insert(contract);

        // 将id返回到页面，用于上传文件使用.
        returnMap.put("id", contract.getId());

        // 启动流程
        /*ProcessInstance processInstance = actTaskService.startProcess(TaskType.contract.getTaskType(), contract.getId().toString(), 
                userInfo.getRealName(), userInfo.getUserId().toString(), depotInfo.getOrgId().toString());*/
        String startProcess1 = actTaskService.startProcessNew(TaskType.contract.getTaskType(), contract.getId().toString(), 
                realName, userId.toString(), depotInfoOrgId.toString());
        JSONObject jsStr = JSONObject.parseObject(startProcess1);
        String processInstanceId = jsStr.getString("processInstanceId");
        String processDefinitionId = jsStr.getString("processDefinitionId");
        
        // 设置流程实例id和流程定义id到合同中.
        // 设置流程实例id.
        contract.setProcessInstanceId(processInstanceId);
        // 设置流程定义id.
        contract.setProcessDefinitionId(processDefinitionId);
        // 设置最原始id，为当前id,以备变更使用.
        contract.setRootContractBid(contract.getId());
        
        contractMapper.updateByPrimaryKey(contract);
        
        // 新增仓房货位数量质量标准子表.
        if (storeWareDetailList != null && storeWareDetailList.size() > 0) {
            for (BusinessStoreWareDetail storeWareDetail : storeWareDetailList) {
                storeWareDetail.setZid(contract.getId());
                storeWareDetail.setType(StoreWareType.contract.getValue());
                storeWareDetail.setCreateTime(new Date());
                businessStoreWareDetailService.add(storeWareDetail);
            }
        }
       // 修改fileIds对应附件的bid为当前计划的id.
        businessFileService.updateFileByFileIdsAndBid(fileIds, contract.getId(),"contract");
        return returnMap;
    }

    /**
     * 修改.
     */
    public void update(BusinessContract contract, List<BusinessStoreWareDetail> storeWareDetailList,String fileIds, String deleteFileIds) {
        contractMapper.updateByPrimaryKey(contract);
        // 修改fileIds对应附件的bid为当前计划的id.
        businessFileService.updateFileByFileIdsAndBid(fileIds, contract.getId(),"contract");
        // 删除附件.
        businessFileService.deleteFileByFileIdArrayString(deleteFileIds);
        // 修改仓房货位数量质量标准子表.
        if (storeWareDetailList != null && storeWareDetailList.size() > 0) {
            for (BusinessStoreWareDetail storeWareDetail : storeWareDetailList) {
                storeWareDetail.setZid(contract.getId());
                if (storeWareDetail.getId() == null) {
                    storeWareDetail.setType(StoreWareType.contract.getValue());
                    storeWareDetail.setCreateTime(new Date());
                    // 新增.
                    businessStoreWareDetailService.add(storeWareDetail);
                } else {
                    // 修改.
                    businessStoreWareDetailService.update(storeWareDetail);
                }
            }
        }
    }

    /**
     * 删除.
     */
    public void remove(Integer id,String processInstanceId,String deleteReason, Integer planBid) {

    	//如果引用了计划 则修改 计划的数量
//        if(planBid!=null){
//        	List<BusinessStoreWareDetail> list = detailService.getByZidAndType(planBid, "plan");
//        	List<BusinessStoreWareDetail> contractList = detailService.getByZidAndType(id, "contract");
//        	for (BusinessStoreWareDetail businessStoreWareDetail : list) {
//        		for (BusinessStoreWareDetail contractDetail : contractList) {
//        			if(contractDetail.getGrainAnnual().equals(businessStoreWareDetail.getGrainAnnual())  && 
//        					contractDetail.getGrainAttribute().equals(businessStoreWareDetail.getGrainAttribute())  &&	
//        					contractDetail.getGrainDetailKind().equals(businessStoreWareDetail.getGrainDetailKind())  &&
//        					contractDetail.getGrainKind().equals(businessStoreWareDetail.getGrainKind())  &&
//        					contractDetail.getGrainGrade().equals(businessStoreWareDetail.getGrainGrade())  &&
//        					contractDetail.getGrainProducingArea().equals(businessStoreWareDetail.getGrainProducingArea())){
//        				
//        				if(contractDetail.getInCount()!=null){
//        					Integer a = Integer.valueOf(contractDetail.getInCount())+Integer.valueOf(businessStoreWareDetail.getRemainingNumber());
//        					businessStoreWareDetail.setRemainingNumber(String.valueOf(a));
//        					System.out.println(businessStoreWareDetail.getRemainingNumber());
//        				}
//        				if(contractDetail.getOutCount()!=null){
//        					Integer b = Integer.valueOf(contractDetail.getOutCount())+Integer.valueOf(businessStoreWareDetail.getOutRemainingNumber());
//        					businessStoreWareDetail.setOutRemainingNumber(String.valueOf(b));
//        					System.out.println(businessStoreWareDetail.getOutRemainingNumber());
//        				}
//        				int z = detailService.updateById(businessStoreWareDetail);
//        			}
//        			
//        			
//				}
//        	}
//        }
        
        // 删除数据.
        contractMapper.deleteByPrimaryKey(id);
        
        // 删除明细列表数据.
        businessStoreWareDetailService.deleteByTypeAndZid("contract", id);
        
        // 删除工作流数据.
        try {
            if (deleteReason!=null) {
                actTaskService.delRunTask(processInstanceId, deleteReason);
            } else {
                deleteReason="删除原因无";
                actTaskService.delRunTask(processInstanceId, deleteReason);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // 提交
    @Transactional(readOnly = false)
    public Map<String, Object> submit(BusinessContract contract, String nextUserId,Integer userId,String realName) {
       /* // 当前登录人的id
        Integer userId=userInfo.getUserId();
        // 当前登录人的姓名
        String realName = userInfo.getRealName();
        */
        Map<String, Object> returnMap = new HashMap<String, Object>();
        returnMap.put("success", "success");
        
        BusinessContract contractB = contractMapper.selectByPrimaryKey(contract.getId());
        
        if (contractB == null) {
            throw new RuntimeException("没有找到相应的数据");
        }
        
        String auditState = contractB.getAuditState();
        if (StringUtils.isNotBlank(auditState) 
                && !auditState.equals(AuditType.UNSUBMIT.getAuditCode()) 
                && !auditState.equals(AuditType.REJECT.getAuditCode())) {
            returnMap.put("success", "error");
            returnMap.put("msg", "您已经提交该数据，无法再次提交！");
            return returnMap;
        }
        // 获取到当前进行到的任务节点.
//        Task singleResult = taskService.createTaskQuery().processInstanceId(contractB.getProcessInstanceId()).singleResult();
        String taskNode = actTaskService.taskNode(contractB.getProcessInstanceId());
        JSONObject taskNodeNew = JSONObject.parseObject(taskNode);
        String taskId = taskNodeNew.getString("taskId");
        String taskName = taskNodeNew.getString("taskName");
        // 修改流程.
        Map<String, Object> vars = Maps.newHashMap();
        vars.put("pass", "1");
        vars.put("inputUser", nextUserId);
        
        actTaskService.complete(taskId, contractB.getProcessInstanceId(), "", "", vars);
        
        // 上一次流程结束，下一流程开始，当前流程.
//        singleResult = taskService.createTaskQuery().processInstanceId(contractB.getProcessInstanceId()).singleResult();
        
        // 设置审批状态为，审批中.
        contractB.setAuditState(AuditType.SUBMIT.getAuditCode());
        // 修改.
        contractMapper.updateByPrimaryKey(contractB);
        
        // 设置流程中间表.
        // 下一环节名称
//        String taskName = singleResult.getName();
        // 任务id
//        String taskId = singleResult.getId();
        
        BusinessApproval approval = new BusinessApproval();
        
        
        
        approval.setProcessInstanceId(contractB.getProcessInstanceId());
        approval.setFromDepartment("经营一部");
        approval.setFromPeople(realName);
        approval.setFromUserName(realName);
        StringBuilder nameContract=new StringBuilder();
        
        if ("1".equals(contractB.getChangeRecordFlag())) {
            // 类型是合同变更
            approval.setTaskType(TaskType.contractChange.getTaskType());
            approval.setProjectName(TaskType.contractChange.getTaskName());
            nameContract.append(TaskType.contractChange.getTaskName());
            nameContract.append(" 编号: ");
            nameContract.append(contractB.getContractNumber());
            approval.setTaskTypeName(nameContract.toString());
        } else {
            // 类型是 合同
            approval.setTaskType(TaskType.contract.getTaskType());
            approval.setProjectName(TaskType.contract.getTaskName());
            nameContract.append(TaskType.contract.getTaskName());
            nameContract.append(" 编号: ");
            nameContract.append(contractB.getContractNumber());
            approval.setTaskTypeName(nameContract.toString());
            
        }
        
        //保存申请人的信息,时间.
        approval.setApplyName(realName);
        approval.setApplyNameId(userId.toString());
        approval.setApplyTime(new Date());
        
        approval.setResult("待审批");
        approval.setIsHide(0);
        approval.setProjectId(contractB.getId());
        // 创建时间.
        approval.setCreateTime(new Date());
        // 操作人
        approval.setOperator(nextUserId);
        // 通过id,获取user 对象.
        UserInfo userInfo2 = userInfoService.findById(Integer.valueOf(nextUserId));
        // 操作人名称.
        approval.setOperatorName(userInfo2.getRealName());
        approval.setTaskName(taskName);
        approval.setTaskId(taskId);
        
        businessApprovalService.add(approval);
        
        return returnMap;
    }
    
    
    // 获取审批列表.
    public List<BusinessContractAuditVO> getApprovalList(Map<String, Object> paramMap) {
        return contractMapper.getApprovalList(paramMap);
    }

    // 审批.
    @Transactional(readOnly = false)
    public void audit(Integer contractId, BusinessApproval businessApproval, String nextUserId, String result, Integer userId,String realName) {
    	//拒绝的时候修改相应计划的数量
    	/*if("3".equals(result)){ 
    		 //根据合同id查询这条数据 取得计划id
    		BusinessContract contract = contractService.findById(contractId);
        	if(contract.getPlanBid()!=null ){ 
        		//获取该条合同下的 明细
        		List<BusinessStoreWareDetail> contractList = detailService.getByZidAndType(contractId, "contract");
        		//获取该条合同引相应计划的明细
        		List<BusinessStoreWareDetail> planList = detailService.getByZidAndType(contract.getPlanBid(), "plan");
        		
        		//获取相应的合同、计划 明细 修改其数量
        		for (int i = 0; i < contractList.size(); i++) {
        			for (int j = 0; j < planList.size(); j++) {
						if(contractList.get(i).getGrainKind().equals(planList.get(j).getGrainKind()) && 
						   contractList.get(i).getGrainDetailKind().equals(planList.get(j).getGrainDetailKind()) &&
						   contractList.get(i).getGrainGrade().equals(planList.get(j).getGrainGrade()) &&
						   contractList.get(i).getGrainAnnual().equals(planList.get(j).getGrainAnnual()) &&
						   contractList.get(i).getGrainAttribute().equals(planList.get(j).getGrainAttribute())){
							
							if(contractList.get(i).getInCount()!=null){
								Integer a = Integer.valueOf(contractList.get(i).getInCount())+Integer.valueOf(planList.get(j).getRemainingNumber());
								planList.get(j).setRemainingNumber(String.valueOf(a));
		        			}else if(contractList.get(i).getOutCount()!=null){
		        				Integer b = Integer.valueOf(contractList.get(i).getOutCount())+Integer.valueOf(planList.get(j).getOutRemainingNumber());
		        				planList.get(j).setOutRemainingNumber(String.valueOf(b));
		        			}
						    //修改计划的数量
							int z = detailService.updateById(planList.get(j));
						}
					}
        			
				}
        	}
    	}*/
    	
    	
    	
    	BusinessApproval oldBusinessApproval = businessApprovalService.findById(businessApproval.getId());
        // 检测是否并发审核
        String oldResult = oldBusinessApproval.getResult();
        if ("同意".equals(oldResult) || "退回".equals(oldResult)) {
            throw new RuntimeException("该订单已经审核过，不能再次审核！");
        }
        String resultS = "";
        if ("0".equals(result)) {
            resultS = "退回";
        } else if ("1".equals(result)) {
            resultS = "同意";
        } else if ("2".equals(result)) {
            resultS = "驳回";
        } else if ("3".equals(result)) {
            resultS = "拒绝";
        }
        
        // 修改当前中间表.
        oldBusinessApproval.setResult(resultS);
        oldBusinessApproval.setContent(businessApproval.getContent());
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String nowF = sdf.format(new Date());
        oldBusinessApproval.setOperateTime(nowF);
        businessApprovalService.updateByPrimaryKeySelective(oldBusinessApproval);
        
        //得到当前的流程实例id
        String processInstanceId = businessApproval.getProcessInstanceId();
        //Task task = actTaskService.selectTask(processInstanceId);
        //String selectTaskId = actTaskService.selectTaskId(processInstanceId);

        String taskNode = actTaskService.taskNode(businessApproval.getProcessInstanceId());
        JSONObject taskNodeNew = JSONObject.parseObject(taskNode);
        String taskId = taskNodeNew.getString("taskId");
        String taskName = taskNodeNew.getString("taskName");

        // 修改流程.
        Map<String, Object> vars = Maps.newHashMap();
        vars.put("pass", result);
        if(result.equals("1")){
            vars.put("inputUser", nextUserId);
        }else if(result.equals("0")){
            //退回时，退给上一个人
            String findHistoryTaskName = actTaskService.findHistoryTaskName(processInstanceId);
            vars.put("inputUser", findHistoryTaskName);
        }else if (result.equals("2")) {
            String findHistoryFirstTaskName = actTaskService.findHistoryFirstTaskName(processInstanceId);
            //驳回环节(获取提交第一个人)
            vars.put("inputUser", findHistoryFirstTaskName);
        }else if (result.equals("3")) {
            //拒绝
        	String findHistoryTaskName = actTaskService.findHistoryTaskName(processInstanceId);
            vars.put("inputUser", findHistoryTaskName);
        }else {
            //位置环节
            return; 
        }
        actTaskService.complete(taskId, processInstanceId, businessApproval.getContent(), "", vars);
        
        ProcessInstance processEnd = actTaskService.isProcessEnd(processInstanceId);
        // 新增流程中间表.
//        Task singleResult = taskService.createTaskQuery().processInstanceId(businessApproval.getProcessInstanceId()).singleResult();
        BusinessContract contractB = contractMapper.selectByPrimaryKey(contractId);
        if (processEnd == null) {
            if ("1".equals(contractB.getChangeRecordFlag())) {
                // 类型是合同变更
                // 合同变更审批通过，需要将原合同的changing状态改为2，这样原合同就不能被再使用了.
                Integer originalContractBid = contractB.getOriginalContractBid();
                BusinessContract contractOriginal = contractMapper.selectByPrimaryKey(originalContractBid);
                contractOriginal.setChanging("2");
                contractMapper.updateByPrimaryKey(contractOriginal);
            }
            
            // 审批通过时间.
            contractB.setAgreeTime(new Date());
            // 修改合同的审批状态为2，审批结束.
            if("3".equals(result)){
                contractB.setAuditState(AuditType.REFUSE.getAuditCode());
                if ("1".equals(contractB.getChangeRecordFlag())) {
                    // 类型是合同变更
                    // 要修改原被变更的合同的状态为未被更改.
                    BusinessContract contractOld = contractMapper.selectByPrimaryKey(contractB.getOriginalContractBid());
                    contractOld.setChanging("0");
                    contractMapper.updateByPrimaryKey(contractOld);
                }
                
            } else if ("1".equals(result)) {
                contractB.setAuditState(AuditType.AGREE.getAuditCode());
            }
            contractMapper.updateByPrimaryKey(contractB);

            //同意修改审批人
            BusinessApproval approval = new BusinessApproval();
            approval.setId(oldBusinessApproval.getId());
            approval.setTaskId(taskId);
            approval.setTaskName(taskName);
            businessApprovalService.updateByPrimaryKeySelective(approval);
        } else {
            
            if ("0".equals(result)) {
                //resultS = "退回";
            } else if ("2".equals(result)) {
                //resultS = "驳回";
                contractB.setAuditState(AuditType.REJECT.getAuditCode());
                contractMapper.updateByPrimaryKey(contractB);
            }else if("1".equals(result)){
                // 下一环节名称.
//                String taskName = singleResult.getName();
                // 任务id.
//                String taskId = singleResult.getId();
            	

                
                BusinessApproval approval = new BusinessApproval();
                approval.setProcessInstanceId(businessApproval.getProcessInstanceId());
                // TODO 当前部门.
                approval.setFromDepartment("经营一部");
                approval.setFromPeople(userId.toString());
                approval.setFromUserName(realName);
                approval.setCreateTime(new Date());
                StringBuilder nameContract=new StringBuilder();
                if ("1".equals(contractB.getChangeRecordFlag())) {
                    // 类型是合同变更
                    approval.setTaskType(TaskType.contractChange.getTaskType());
                    approval.setProjectName(TaskType.contractChange.getTaskName());
                    nameContract.append(TaskType.contractChange.getTaskName());
                    nameContract.append(" 编号: ");
                    nameContract.append(contractB.getContractNumber());
                    approval.setTaskTypeName(nameContract.toString());
                } else {
                    // 类型是 合同
                    approval.setTaskType(TaskType.contract.getTaskType());
                    approval.setProjectName(TaskType.contract.getTaskName());
                    nameContract.append(TaskType.contract.getTaskName());
                    nameContract.append(" 编号: ");
                    nameContract.append(contractB.getContractNumber());
                    approval.setTaskTypeName(nameContract.toString());
                    
                }
                approval.setResult("待审批");
                approval.setIsHide(0);
                approval.setProjectId(contractId);
                
                // 申请人的信息,时间.
                approval.setApplyName(oldBusinessApproval.getApplyName());
                approval.setApplyNameId(oldBusinessApproval.getApplyNameId());
                approval.setApplyTime(oldBusinessApproval.getCreateTime());
                
                UserInfo userInfo2=userInfoService.findById(Integer.valueOf(nextUserId));
                // 审批人
                approval.setOperator(nextUserId);
                // 设置审批人的中文名称.
                approval.setOperatorName(userInfo2.getRealName());
                approval.setTaskName(taskName);
                approval.setTaskId(taskId);
                
                businessApprovalService.add(approval);
            }
            
        }
    }


    /**
     * 新增变更合同.
     */
    @Override
    @Transactional(readOnly = false)
    public Map<String, Object>  saveChange(BusinessContract contract, List<BusinessStoreWareDetail> storeWareDetailList,
            Integer userId,String userName, String realName,Integer depotInfoOrgId,Integer id,String fileIds,String deleteFileIds) {

        Map<String, Object> returnMap = new HashMap<String, Object>();
        /*//当前登录人的id
        Integer userId=userInfo.getUserId();
        //当前登录人的姓名
        String name=userInfo.getUsername();*/
        // 修改原合同变更标志  change_record_flag  设置为1
        BusinessContract contractOld = contractMapper.selectByPrimaryKey(id);
        //contractOld.setCreateTime(new Date());
        //获取当前登录人
        contractOld.setCreater(realName);
        contractOld.setCreaterId(userId);
        // 设置正在被变种中，不能再被变更.
        contractOld.setChanging("1");
        contractMapper.updateByPrimaryKey(contractOld);
        
        
        // 新增合同变更记录. 
        // 合同编号  合同变更，不修改合同编号.
        // contract.setContractNumber(generatorNumber(depotInfo.getOrgCode()));
        // 设置id为空.
        contract.setId(null);
        // 设置原合同id.
        contract.setOriginalContractBid(contractOld.getId());
        // 设置原合同编号.
        contract.setOriginalContract(contractOld.getContractNumber());
        // 设置最原始合同id.
        contract.setRootContractBid(contractOld.getRootContractBid());
        // 设置合同变更记录标志为，是原合同的变更记录.
        contract.setChangeRecordFlag("1");
        // 设置变更时间.
        contract.setChangeTime(new Date());
        // 设置创建时间.
        contract.setCreateTime(new Date());
        // 设置审批状态为，审批待审批.
        contract.setAuditState(AuditType.UNSUBMIT.getAuditCode());
        
        // 计算合同总金额.
        // 合同数量.
        //String grainQuantity = contract.getGrainQuantity();
        //合同总金额
        //BigDecimal grainQuantityB = new BigDecimal(grainQuantity);
        // 合同单价.
        //String grainPrice = contract.getGrainPrice();
        //BigDecimal grainPriceB = new BigDecimal(grainPrice);
        
        // 总金额 = 合同数量*合同单价. 两位小数，四舍五入.
        //contract.setMoneyQuantity(grainQuantityB.multiply(grainPriceB).setScale(2, BigDecimal.ROUND_HALF_UP).toString());
        
        
        // 新增合同变更记录
        contractMapper.insert(contract);

        returnMap.put("id",contract.getId());

        
//        ProcessInstance processInstance = actTaskService.startProcess(TaskType.contract.getTaskType(), contract.getId().toString(), name, userId.toString(), depotInfo.getOrgId().toString());
        String startProcess1 = actTaskService.startProcessNew(TaskType.contract.getTaskType(), contract.getId().toString(), userName, userId.toString(), depotInfoOrgId.toString());
        JSONObject jsStr = JSONObject.parseObject(startProcess1);
        String processInstanceId = jsStr.getString("processInstanceId");
        String processDefinitionId = jsStr.getString("processDefinitionId");
        // 设置流程实例id和流程定义id到合同中.
        // 设置流程实例id.
        contract.setProcessInstanceId(processInstanceId);
        // 设置流程定义id.
        contract.setProcessDefinitionId(processDefinitionId);
        //  修改合同变更记录.
        contractMapper.updateByPrimaryKey(contract);
        
        // 新增仓房货位数量质量标准子表.
        if (storeWareDetailList != null && storeWareDetailList.size() > 0) {
            for (BusinessStoreWareDetail storeWareDetail : storeWareDetailList) {
            	
                storeWareDetail.setZid(contract.getId());
                storeWareDetail.setType(StoreWareType.contract.getValue());
                storeWareDetail.setCreateTime(new Date());
                storeWareDetail.setId(null);
                businessStoreWareDetailService.add(storeWareDetail);
            }
        }
        if(deleteFileIds!=null){
        	// 删除附件.
        	businessFileService.deleteFileByFileIdArrayString(deleteFileIds);
        }
        
       // 修改fileIds对应附件的bid为当前合同的id.
        businessFileService.updateFileByFileIdsAndBid(fileIds, contract.getId(),"contract");
        return returnMap;
    }

    /**
     * 修改变更合同.
     */
    @Override
    public void updateChange(BusinessContract contract, List<BusinessStoreWareDetail> storeWareDetailList) {
        contractMapper.updateByPrimaryKey(contract);
    }


    /**
     * 修改变更合同.
     * @return 
     */
    @Override
    @Transactional(readOnly = false)
    public Map<String, Object> changeContractRemove(Integer id, Integer originalContractBid, String processInstanceId, String deleteReason,Integer planBid) {
        Map<String, Object> returnMap = new HashMap<String, Object>();
        try {
            // 修改原合同为，为未被更改.  originalContractBid(原合同id)
            BusinessContract contractOld = contractMapper.selectByPrimaryKey(originalContractBid);
            contractOld.setChanging("0");
            contractMapper.updateByPrimaryKey(contractOld);
            //如果引用了计划 则修改 计划的数量  contractOld.getPlanBid() 引计划的id
//            if(contractOld.getPlanBid()!=null){  //contractOld  原来的合同
//            	//计划的明细
//            	List<BusinessStoreWareDetail> list = detailService.getByZidAndType(contractOld.getPlanBid(), "plan"); 
//            	//现在的合同明细
//            	List<BusinessStoreWareDetail> contractList = detailService.getByZidAndType(id, "contract");
//            	//原合同明细
//            	List<BusinessStoreWareDetail> contractListStart = detailService.getByZidAndType(originalContractBid, "contract");
//
//            	for (BusinessStoreWareDetail businessStoreWareDetail : list) {
//            		for (BusinessStoreWareDetail contractDetail : contractList) {
//            			for (BusinessStoreWareDetail contractDetailStart : contractListStart) {
//							
//            				if(contractDetail.getGrainAnnual().equals(businessStoreWareDetail.getGrainAnnual()) && 
//            						contractDetail.getGrainAnnual().equals(contractDetailStart.getGrainAnnual()) &&
//            						contractDetailStart.getGrainAnnual().equals(businessStoreWareDetail.getGrainAnnual()) &&
//            						
//            						contractDetail.getGrainAttribute().equals(businessStoreWareDetail.getGrainAttribute())  &&	
//            						contractDetail.getGrainAttribute().equals(contractDetailStart.getGrainAttribute())  &&	
//            						contractDetailStart.getGrainAttribute().equals(businessStoreWareDetail.getGrainAttribute())  &&	
//            						
//            						contractDetail.getGrainDetailKind().equals(businessStoreWareDetail.getGrainDetailKind())  &&
//            						contractDetail.getGrainDetailKind().equals(contractDetailStart.getGrainDetailKind())  &&
//            						contractDetailStart.getGrainDetailKind().equals(businessStoreWareDetail.getGrainDetailKind())  &&
//            						
//            						contractDetail.getGrainKind().equals(businessStoreWareDetail.getGrainKind())  &&
//            						contractDetail.getGrainKind().equals(contractDetailStart.getGrainKind())  &&
//            						contractDetailStart.getGrainKind().equals(businessStoreWareDetail.getGrainKind())  &&
//            						
//            						contractDetail.getGrainGrade().equals(businessStoreWareDetail.getGrainGrade())  &&
//            						contractDetail.getGrainGrade().equals(contractDetailStart.getGrainGrade())  &&
//            						contractDetailStart.getGrainGrade().equals(businessStoreWareDetail.getGrainGrade())  &&
//            						
//            						contractDetail.getGrainProducingArea().equals(businessStoreWareDetail.getGrainProducingArea()) &&
//            						contractDetail.getGrainProducingArea().equals(contractDetailStart.getGrainProducingArea()) &&
//            						contractDetailStart.getGrainProducingArea().equals(businessStoreWareDetail.getGrainProducingArea())){
//            					
//            					
//            					//原合同
//            					if(contractDetailStart.getInCount()!=null){
//            						if(Integer.valueOf(contractDetailStart.getInCount())-Integer.valueOf(contractDetail.getInCount())>0){
//            							//原合同比 现合同数量多   计划的剩余数量要增加
//            							Integer a = Integer.valueOf(contractDetailStart.getInCount())-Integer.valueOf(contractDetail.getInCount());
//            							Integer b = Integer.valueOf(businessStoreWareDetail.getRemainingNumber())-a;
//            							businessStoreWareDetail.setRemainingNumber(String.valueOf(b));
//            						}else{
//            							Integer a= Integer.valueOf(contractDetail.getInCount())-Integer.valueOf(contractDetailStart.getInCount());
//            							Integer b = Integer.valueOf(businessStoreWareDetail.getRemainingNumber())+a;
//            							businessStoreWareDetail.setRemainingNumber(String.valueOf(b));
//            						}
//            						
//            					}
//            					if(contractDetailStart.getOutCount()!=null){
//            						if(Integer.valueOf(contractDetailStart.getOutCount())-Integer.valueOf(contractDetail.getOutCount())>0){
//            							//原合同比 现合同数量多   计划的剩余数量要增加
//            							Integer a = Integer.valueOf(contractDetailStart.getOutCount())-Integer.valueOf(contractDetail.getOutCount());
//            							Integer b = Integer.valueOf(businessStoreWareDetail.getOutRemainingNumber())-a;
//            							businessStoreWareDetail.setOutRemainingNumber(String.valueOf(b));
//            						}else{
//            							Integer a= Integer.valueOf(contractDetail.getOutCount())-Integer.valueOf(contractDetailStart.getOutCount());
//            							Integer b = Integer.valueOf(businessStoreWareDetail.getOutRemainingNumber())+a;
//            							businessStoreWareDetail.setOutRemainingNumber(String.valueOf(b));
//            						}
//            					}
//            					int z = detailService.updateById(businessStoreWareDetail);
//            				}
//						}
//            			
//            			
//    				}
//            	}
//            }
            
            // 删除变更合同.
            contractMapper.deleteByPrimaryKey(id);
            
            // 删除明细列表数据.
            businessStoreWareDetailService.deleteByTypeAndZid("contract", id);
            
            returnMap.put("status", "success");
        } catch (Exception e) {
            e.printStackTrace();
            returnMap.put("status", "error");
            returnMap.put("msg", "删除失败！");
        }
        return returnMap;
    }


    /**
     * 获取历次变更详情.
     * @param id
     * @return
     */
    public List<BusinessContract> getChangeDetailList(Integer id, String type) {
        BusinessContract contract = contractMapper.selectByPrimaryKey(id);
        // 获取初始合同id.
        Integer rootContractBid = contract.getRootContractBid();
        if (rootContractBid != null) {
            BusinessContractExample example = new BusinessContractExample();
            Criteria createCriteria = example.createCriteria();
            
            // 如果是进入新增 修改合同页面，就不显示当前id对应的数据.
            if (!"add".equals(type)) {
                createCriteria.andIdNotEqualTo(id);
            }
            createCriteria.andRootContractBidEqualTo(rootContractBid);
            // 只显示变更合同.
            createCriteria.andChangeRecordFlagEqualTo("1");
            // 只显示审批通过的.
            createCriteria.andAuditStateEqualTo("2");
            // 按照修改时间排序.
            example.setOrderByClause("change_time");
            return contractMapper.selectByExample(example);
        }
        return null;
    }
    
    /**
     * 启用.
     * @param id
     */
    @Override
    @Transactional(readOnly = false)
    public Map<String, String> application(Integer id) {
        Map<String, String> returnMap = new HashMap<String, String>();
        // 判断对应的计划是否暂停和终止.
        BusinessContract contractY = contractMapper.selectByPrimaryKey(id);
        // 获取计划id.
        Integer planBid = contractY.getPlanBid();
        // 起始合同id.
        Integer rootContractBid = contractY.getRootContractBid();
        
        if (planBid != null) {
            BusinessPlan plan = planService.findById(planBid);
            if (plan != null) {
                Integer inApplication = plan.getInApplication();
                if (inApplication != CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode()) {
                    // 计划的状态不是启用.
                    if (inApplication == CrkNoticeStatus.ZANTING.getCrkNoticeCode()) {
                        returnMap.put("error", "对应的计划已经暂停,无法启用该合同！");
                    } else if (inApplication == CrkNoticeStatus.ZHONGZHI.getCrkNoticeCode()) {
                        returnMap.put("error", "对应的计划已经终止,无法启用该合同！");
                    }
                    return returnMap;
                }
            }
        }
        
        // 启用合同.
        BusinessContract contract = new BusinessContract();
        contract.setId(id);
        contract.setInApplication(CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode());
        contract.setCreateDepot("");
        contractMapper.updateByPrimaryKeySelective(contract);
        
        // 启用合同对应的出入库通知单
        // 因为合同可能变更，使用起始合同id进行查询.
        Map<String,Object> paramMap = new HashMap<String, Object>();
        paramMap.put("inApplication", CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode());
        paramMap.put("rootContractBid", rootContractBid);
        deliveryStorageNoticeService.updateApplication(paramMap);
        
        // 修改出入库系统出入库通知单状态.
        // 通过起始合同id，查询所有对应的出入库通知单.
       /* BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
        example.createCriteria().andRootContractBidEqualTo(rootContractBid);
        List<BusinessDeliveryStorageNotice> noticeList = deliveryStorageNoticeService.queryByExample(example);
        if (noticeList != null && noticeList.size() > 0) {
            for (BusinessDeliveryStorageNotice notice : noticeList) {
                paramMap = new HashMap<String, Object>();
                paramMap.put("tzdh", notice.getId());
                paramMap.put("FLOWSTATUS", CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode());
                // 修改出入库系统的 通知单状态.
                feignAgileService.updateNoticeStatus(paramMap);
                //serverApi.updateNoticeStatus(paramMap);
            }
        }*/
        return returnMap;
    }

    /**
     * 暂停.
     * @param id
     */
    @Override
    @Transactional(readOnly = false)
    public void discontinue(Integer id,String refuseReason) {
        BusinessContract contractY = contractMapper.selectByPrimaryKey(id);
        Integer rootContractBid = contractY.getRootContractBid();
        // 暂停合同.
        BusinessContract contract = new BusinessContract();
        contract.setId(id);
        contract.setInApplication(CrkNoticeStatus.ZANTING.getCrkNoticeCode());
        contract.setCreateDepot(refuseReason);
        contractMapper.updateByPrimaryKeySelective(contract);
        
        // 暂停合同对应的出入库通知单.
        // 因为合同可能变更，使用起始合同id进行查询.
        Map<String,Object> paramMap = new HashMap<String, Object>();
        paramMap.put("inApplication", CrkNoticeStatus.ZANTING.getCrkNoticeCode());
        paramMap.put("rootContractBid", rootContractBid);
        deliveryStorageNoticeService.updateApplication(paramMap);
        
        // 修改出入库系统出入库通知单状态.
        // 通过起始合同id，查询所有对应的出入库通知单.
        /*BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
        example.createCriteria().andRootContractBidEqualTo(rootContractBid);
        List<BusinessDeliveryStorageNotice> noticeList = deliveryStorageNoticeService.queryByExample(example);
        if (noticeList != null && noticeList.size() > 0) {
            for (BusinessDeliveryStorageNotice notice : noticeList) {
                paramMap = new HashMap<String, Object>();
                paramMap.put("tzdh", notice.getId());
                paramMap.put("FLOWSTATUS", CrkNoticeStatus.ZANTING.getCrkNoticeCode());
                // 修改出入库系统的 通知单状态.
                feignAgileService.updateNoticeStatus(paramMap);
                //serverApi.updateNoticeStatus(paramMap);
            }
        }*/
    }

    /**
     * 终止.
     * @param id
     */
    @Override
    @Transactional(readOnly = false)
    public void finish(Integer id,String refuseReason) {
        BusinessContract contractY = contractMapper.selectByPrimaryKey(id);
        Integer rootContractBid = contractY.getRootContractBid();
        // 终止合同.
        BusinessContract contract = new BusinessContract();
        contract.setId(id);
        contract.setInApplication(CrkNoticeStatus.ZHONGZHI.getCrkNoticeCode());
        contract.setCreateDepot(refuseReason);
        contractMapper.updateByPrimaryKeySelective(contract);
        
        // 终止合同对应的出入库通知单.
        // 因为合同可能变更，使用起始合同id进行查询.
        Map<String,Object> paramMap = new HashMap<String, Object>();
        paramMap.put("inApplication", CrkNoticeStatus.ZHONGZHI.getCrkNoticeCode());
        paramMap.put("rootContractBid", rootContractBid);
        deliveryStorageNoticeService.updateApplication(paramMap);
        
        // 修改出入库系统出入库通知单状态.
        // 通过起始合同id，查询所有对应的出入库通知单.
        /*BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
        example.createCriteria().andRootContractBidEqualTo(rootContractBid);
        List<BusinessDeliveryStorageNotice> noticeList = deliveryStorageNoticeService.queryByExample(example);
        if (noticeList != null && noticeList.size() > 0) {
            for (BusinessDeliveryStorageNotice notice : noticeList) {
                paramMap = new HashMap<String, Object>();
                paramMap.put("tzdh", notice.getId());
                paramMap.put("FLOWSTATUS", CrkNoticeStatus.ZHONGZHI.getCrkNoticeCode());
                // 修改出入库系统的 通知单状态.
                feignAgileService.updateNoticeStatus(paramMap);
               // serverApi.updateNoticeStatus(paramMap);
            }
        }*/
    }

    /**
     * 合同执行进度.
     */
    @Override
    public PageInfo<BusinessContractAuditVO> getScheduleList(Integer pageNum,
            Integer pageSize, BusinessContract contract, Integer orgId) {
        // 可以查询所有人通过的合同,而且必须是没有被修改完成的，changing 不是2.
        BusinessContractExample example = new BusinessContractExample();
        Criteria createCriteria = example.createCriteria();
        createCriteria.andAuditStateEqualTo(AuditType.AGREE.getAuditCode());
        createCriteria.andChangingNotEqualTo("2");
        createCriteria.andContractTypeNotEqualTo(6913);//除去中转合同
        if (contract != null) {
            // 合同编号
            if (StringUtils.isNotBlank(contract.getContractNumber())) {
                createCriteria.andContractNumberLike("%" + contract.getContractNumber() + "%");
            }
            
            // 合同类型
            if (contract.getContractType() != null) {
                createCriteria.andContractTypeEqualTo(contract.getContractType());
            }
            
            // 计划id.
            if (contract.getPlanBid() != null) {
                createCriteria.andPlanBidEqualTo(contract.getPlanBid());
            }
            // 库id.
            if (contract.getOrgId() != null) {
                createCriteria.andOrgIdEqualTo(contract.getOrgId());
            }
        }
        if (null != orgId) {
        	createCriteria.andOrgIdEqualTo(orgId);
        }
        
        PageInfo<BusinessContractAuditVO> pageInfo = new PageInfo<BusinessContractAuditVO>();
        // 审批通过时间倒序.
        example.setOrderByClause("agree_time desc");
        // 分页.
        if(pageNum!=null && pageSize!=null){
        	PageHelper.startPage(pageNum, pageSize);
        }
        List<BusinessContractAuditVO> contractList = contractMapper.selectByExampleToVO(example);
        if (contractList != null && contractList.size() > 0) {
            // 拼接  起始合同id 这里不拼接id,是因为.
            StringBuffer sb = new StringBuffer();
            /*StringBuffer fyzz = new StringBuffer(); //发运中转
            StringBuffer dzzz = new StringBuffer(); //到站中转
*/            sb.append("(");
            /*fyzz.append("(");
            dzzz.append("(");*/
            for (BusinessContractAuditVO businessContractV : contractList) {
            	/*if(businessContractV.getCustomerId()!=null){
            		if(businessContractV.getCustomerId().equals("7027")){ //到站中转
                    	dzzz.append("\""+businessContractV.getRootContractBid()+ "\"").append(",");
                    }else if(businessContractV.getCustomerId().equals("7028")){ //发运中转
                    	fyzz.append("\"" + businessContractV.getRootContractBid() + "\"").append(",");
                    }
            	}else{
            		sb.append("\"" + businessContractV.getRootContractBid() + "\"").append(",");
            	}*/
            	sb.append("\"" + businessContractV.getRootContractBid() + "\"").append(",");
            }
            String substring = sb.substring(0, sb.length() - 1);
            /*String fyzzSubstring = fyzz.substring(0, fyzz.length() - 1);
            String dzzzSubstring = dzzz.substring(0, dzzz.length() - 1);*/
            sb = new StringBuffer(substring);
            /*fyzz = new StringBuffer(fyzzSubstring);
            dzzz = new StringBuffer(dzzzSubstring);*/
            sb.append(")");
            /*fyzz.append(")");
            dzzz.append(")");*/
            List<BusinessScedule> businessSceduleList = feignAgileService.getContractSceduleInContractId(sb.toString()); //合同
            //List<BusinessScedule> businessSceduleList = serverApi.getContractSceduleInContractId(sb.toString());
          /*  List<BusinessScedule> dzContractSceduleList = null ;
            List<BusinessScedule> fyContractSceduleList = null;
            if(dzzz.indexOf(",")!=-1){
            	 dzContractSceduleList = feignAgileService.getDzContractSceduleInContractId(dzzz.toString()); //到站中转的合同
            	//dzContractSceduleList = serverApi.getDzContractSceduleInContractId(dzzz.toString());
            }
            if(fyzz.indexOf(",")!=-1){
            	 fyContractSceduleList = feignAgileService.getFyContractSceduleInContractId(fyzz.toString()); //发运中转的合同
            	 //fyContractSceduleList = serverApi.getFyContractSceduleInContractId(fyzz.toString()); //发运中转的合同
            }
            if(dzContractSceduleList!=null && dzContractSceduleList.size()>0){
            	for (BusinessScedule dzScedule : dzContractSceduleList) {
            		businessSceduleList.add(dzScedule);
            	}
            }
            if(fyContractSceduleList!=null && fyContractSceduleList.size()>0){
            	for (BusinessScedule fyScedule : fyContractSceduleList) {
            		businessSceduleList.add(fyScedule);
            	}
            }*/
            // 通过合同id,去出入库查询.
            if (businessSceduleList != null && businessSceduleList.size() > 0) {
                // 循环合同list.
                for (BusinessContractAuditVO businessContractV : contractList) {
                    // 循环出入库，查询的list.
                    Boolean flag = false;
                    for (BusinessScedule businessScedule : businessSceduleList) {
                        if (businessContractV.getRootContractBid().toString().equals(businessScedule.getHtId())) {
                            flag = true;
                            // 在出入库系统中，存在该合同对应的出入库数据，进行计算，得出完成比率.
                            String crkLx = businessScedule.getCrkLx();
                            if ("1".equals(crkLx)) {
                                // 入库
                                // 入库数量.
                            	String sumRksl = businessScedule.getSumWcsl();
                            	BigDecimal s = new BigDecimal(1000);
                                BigDecimal sumCkslss = new BigDecimal(sumRksl);
                   			    BigDecimal sumRkslD = sumCkslss.divide(s);
                   			    
                                // 出入库中的出库数量单位是公斤，合同中是吨，所以出入库数量要除以1000.
                                //sumRkslD = sumRkslD.divide(new BigDecimal("1000"), 4, BigDecimal.ROUND_HALF_DOWN);
                                
                                // 赋值： 设置完成数量. stripTrailingZeros() 去除结尾的0
                                businessContractV.setCompleteAmount(sumRkslD.stripTrailingZeros().toPlainString());
                                
                                // 要求数量.
                                String demandAmount = businessContractV.getGrainQuantity();
                                BigDecimal demandAmountD = new BigDecimal(demandAmount.toString());
                                
                                // 除法. 已完成数量/要求数量
                                BigDecimal complete = sumRkslD.divide(demandAmountD, 4, BigDecimal.ROUND_HALF_DOWN);
                                // 乘以100，得到的是百分比.
                                BigDecimal completeRate = complete.multiply(new BigDecimal("100"));
                                // 赋值： 设置完成百分比.
                                businessContractV.setCompleteRate(completeRate.stripTrailingZeros().toPlainString());
                            } else { //出库的和中转的(中转的都是出库)
                                // 出库
                                // 出库数量.
                            	String sumCksl = businessScedule.getSumWcsl();
                            	BigDecimal s = new BigDecimal(1000);
                                BigDecimal sumCkslss = new BigDecimal(sumCksl);
                   			    BigDecimal sumCkslD = sumCkslss.divide(s);
                   			    
                   			    
                                // 出入库中的出库数量单位是公斤，合同中是吨，所以出入库数量要除以1000.
                                //sumCkslD = sumCkslD.divide(new BigDecimal("1000"), 4, BigDecimal.ROUND_HALF_DOWN);
                                
                                // 赋值：设置完成数量 . stripTrailingZeros() 去除结尾的0
                                businessContractV.setCompleteAmount(sumCkslD.stripTrailingZeros().toPlainString());
                                
                                // 要求数量.
                                String demandAmount = businessContractV.getGrainQuantity();
                                BigDecimal demandAmountD = new BigDecimal(demandAmount.toString());
                                
                                // 除法. 已完成数量/要求数量
                                BigDecimal complete = sumCkslD.divide(demandAmountD, 4, BigDecimal.ROUND_HALF_DOWN);
                                // 乘以100，得到的是百分比.
                                BigDecimal completeRate = complete.multiply(new BigDecimal("100"));
                                // 赋值： 设置完成百分比.
                                businessContractV.setCompleteRate(completeRate.stripTrailingZeros().toPlainString());
                            }
                            // 在出入库list中删除该数据，这样下次循环数量就会减少了.
                            businessSceduleList.remove(businessScedule);
                            // 跳出当前的出入库循环.
                            break;
                        }
                    }
                    if (!flag) {
                        // 进行到这里，说明出入库系统中，不存在该合同对应的出入库数据. 设置完成数量，和完成比例为0.
                        businessContractV.setCompleteAmount("0");
                        businessContractV.setCompleteRate("0");
                    }
                }
            }else {
                // 没有找到任何出入库数据. 设置完成数量，和完成比例为0.
                for (BusinessContractAuditVO businessContractV : contractList) {
                    businessContractV.setCompleteAmount("0");
                    businessContractV.setCompleteRate("0");
                }
            }
            
            pageInfo = new PageInfo<BusinessContractAuditVO>(contractList);
        }
        return pageInfo;
    }


    //合同
    @Override
    public List<BusinessContract> queryByExample(Integer pageNum,
            Integer pageSize, Integer userId, BusinessContract contract, String order, String contractType) {
        BusinessContractExample example = new BusinessContractExample();
        Criteria createCriteria = example.createCriteria();
        if (null != userId) {
        	createCriteria.andCreaterIdEqualTo(userId);
        }
        if (contract != null) {
            // 合同标题
            if (StringUtils.isNotBlank(contract.getContractTitle())) {
                createCriteria.andContractTitleLike("%" + contract.getContractTitle() + "%");
            }
            // 合同编号
            if (StringUtils.isNotBlank(contract.getContractNumber())) {
                createCriteria.andContractNumberLike("%" + contract.getContractNumber() + "%");
            }
            // 合同类型
            if (contract.getContractType() != null) {
                createCriteria.andContractTypeEqualTo(contract.getContractType());
            }
            // 审批状态.
            if (contract.getAuditState() != null && !"".equals(contract.getAuditState())) {
                createCriteria.andAuditStateEqualTo(contract.getAuditState());
            }
            createCriteria.andContractTypeNotEqualTo(6913); //除去中转合同
            // 判断是否是合同变更.
            if (StringUtils.isNotBlank(contractType) && "change".equals(contractType)) {
                // 合同变更标志 为  1，也就是合同变更数据.
                //contract.setChangeRecordFlag("1");
                createCriteria.andChangeRecordFlagEqualTo("1");
                // 变更状态为没有被变更成功. changing：0未变更； 1变更中； 2已变更；
                // 注：如果一条合同变更多次，只会显示最新一次的变更.
                createCriteria.andChangingNotEqualTo("2");
            }else{ //未变更的合同(合同申请)
            	createCriteria.andChangeRecordFlagEqualTo("0");
            }
            
        }
        if (StringUtils.isNotBlank(order)) {
            example.setOrderByClause(order);
        }
        PageHelper.startPage(pageNum, pageSize);
        createCriteria.andCreateUnitIsNull();//储备粮合同是1 普通合同为空
        List<BusinessContract> list = contractMapper.selectByExample(example);
        return list;
    }


    //中转合同
    @Override
    public List<BusinessContract> queryByExampleTransfer(Integer pageNum,
            Integer pageSize, Integer userId, BusinessContract contract, String order, String contractType) {
        BusinessContractExample example = new BusinessContractExample();
        Criteria createCriteria = example.createCriteria();
        createCriteria.andCreaterIdEqualTo(userId);
        createCriteria.andContractTypeEqualTo(6913); //中转合同
        if (contract != null) {
            // 合同标题
            if (StringUtils.isNotBlank(contract.getContractTitle())) {
                createCriteria.andContractTitleLike("%" + contract.getContractTitle() + "%");
            }
            // 合同编号
            if (StringUtils.isNotBlank(contract.getContractNumber())) {
                createCriteria.andContractNumberLike("%" + contract.getContractNumber() + "%");
            }
            // 合同类型
            if (contract.getContractType() != null) {
                createCriteria.andContractTypeEqualTo(contract.getContractType());
            }
            // 审批状态.
            /*if (contract.getAuditState() != null) {
                createCriteria.andAuditStateEqualTo(contract.getAuditState());
            }*/
            if (StringUtils.isNotBlank(contract.getAuditState())){
                createCriteria.andAuditStateEqualTo(contract.getAuditState());
            }
            // 判断是否是合同变更.
            if (StringUtils.isNotBlank(contractType) && "change".equals(contractType)) {
                // 合同变更标志 为  1，也就是合同变更数据.
                //contract.setChangeRecordFlag("1");
                createCriteria.andChangeRecordFlagEqualTo("1");
                // 变更状态为没有被变更成功. changing：0未变更； 1变更中； 2已变更；
                // 注：如果一条合同变更多次，只会显示最新一次的变更.
                createCriteria.andChangingNotEqualTo("2");
            }else{ //未变更的合同(合同申请)
            	createCriteria.andChangeRecordFlagEqualTo("0");
            }
            
        }
        if (StringUtils.isNotBlank(order)) {
            example.setOrderByClause(order);
        }
        PageHelper.startPage(pageNum, pageSize);
        List<BusinessContract> list = contractMapper.selectByExample(example);
        return list;
    }
    
    
    @Override
    public List<BusinessContract> getAuditPassList(Integer pageNum,
            Integer pageSize, BusinessContract contract, Integer billType) {
        // 审批通过能查询所有人，审批通过的
        BusinessContractExample example = new BusinessContractExample();
        Criteria createCriteria = example.createCriteria();
        // 审批通过.
        createCriteria.andAuditStateEqualTo(AuditType.AGREE.getAuditCode());
        // 变更状态不是，已变更的合同.
        createCriteria.andChangingNotEqualTo("2");
        // 计划id.
        if (contract.getPlanBid() != null) {
            createCriteria.andPlanBidEqualTo(contract.getPlanBid());
        }
        // 合同标题
        if (StringUtils.isNotBlank(contract.getContractTitle())) {
            createCriteria.andContractTitleLike("%" + contract.getContractTitle() + "%");
        }
        // 合同编号
        if (StringUtils.isNotBlank(contract.getContractNumber())) {
            createCriteria.andContractNumberLike("%" + contract.getContractNumber() + "%");
        }
        // 合同类型
        if (contract.getContractType() != null) {
            createCriteria.andContractTypeEqualTo(contract.getContractType());
        }
        
        // 是否启用.
        if (contract.getInApplication() != null) {
            createCriteria.andInApplicationEqualTo(contract.getInApplication());
        }
        // 按照审批通过时间排序.
        example.setOrderByClause(" agree_time desc");
        // 分页.
        PageHelper.startPage(pageNum, pageSize);
        //除去中转合同
        createCriteria.andContractTypeNotEqualTo(6913); 
        createCriteria.andCreateUnitIsNull();//储备粮合同是1 普通合同为空
        List<BusinessContract> list = contractMapper.selectByExample(example);
        return list;
    }


    //中转审批
    @Override
    public List<BusinessContract> getAuditPassListTransfer(Integer pageNum,
            Integer pageSize, BusinessContract contract, Integer billType) {
        // 审批通过能查询所有人，审批通过的
        BusinessContractExample example = new BusinessContractExample();
        Criteria createCriteria = example.createCriteria();
        // 审批通过.
        createCriteria.andAuditStateEqualTo(AuditType.AGREE.getAuditCode());
        // 变更状态不是，已变更的合同.
        createCriteria.andChangingNotEqualTo("2");
        // 计划id.
        if (contract.getPlanBid() != null) {
            createCriteria.andPlanBidEqualTo(contract.getPlanBid());
        }
        // 合同标题
        if (StringUtils.isNotBlank(contract.getContractTitle())) {
            createCriteria.andContractTitleLike("%" + contract.getContractTitle() + "%");
        }
        // 合同编号
        if (StringUtils.isNotBlank(contract.getContractNumber())) {
            createCriteria.andContractNumberLike("%" + contract.getContractNumber() + "%");
        }
        // 合同类型
        if (contract.getContractType() != null) {
            createCriteria.andContractTypeEqualTo(contract.getContractType());
        }
        if (billType != null) {
            if (billType == 1) {
                // 入.
                createCriteria.andContractTypeEqualTo(3147);
            } else if (billType == 3) {
                // 出.
                createCriteria.andContractTypeEqualTo(3148);
            }
        }
        
        // 是否启用.
        if (contract.getInApplication() != null) {
            createCriteria.andInApplicationEqualTo(contract.getInApplication());
        }
        // 发运中转
        if (contract.getCustomerId() != null) {
        	createCriteria.andCustomerIdEqualTo(contract.getCustomerId());
        }
        // 按照审批通过时间排序.
        example.setOrderByClause(" agree_time desc");
        // 分页.
        PageHelper.startPage(pageNum, pageSize);
        //除去中转合同
        createCriteria.andContractTypeEqualTo(6913); 
        List<BusinessContract> list = contractMapper.selectByExample(example);
        return list;
    }
    
    /**
     * 校验合同编号，是否可用.
     * @param id 主键.
     * @param contractNumber 合同编号.
     * @return true 为可用，false为不可用.
     */
    @Override
    public Boolean validContractNumber(Integer id, String contractNumber) {
        BusinessContractExample example = new BusinessContractExample();
        Criteria createCriteria = example.createCriteria();
        createCriteria.andContractNumberEqualTo(contractNumber);
        if (id != null) {
            createCriteria.andIdNotEqualTo(id);
        }
        int count = contractMapper.countByExample(example);
        return count > 0 ? false : true;
    }

    /**
     * 计划和合同的数据
     * @param orgId
     * @return
     */
    @Override
    public List<Map<String, Object>> getAllBusiness(String orgId) {
        return contractMapper.selectBusinessByOrgId(orgId);
    }

    /**
     * 获取合同编号.
     * @param orgCode
     * @return
     */
    public String generatorNumber(String orgCode) {
        String todayStr = new SimpleDateFormat("yyyyMMdd").format(new Date());
        BusinessContractExample example = new BusinessContractExample();
        example.createCriteria().andPlanNumberLike(orgCode + "-HT-" + todayStr + "%");
        example.setOrderByClause("id desc");
        List<BusinessContract> list = contractMapper.selectByExample(example);
        String orderStr = "01";
        if (list != null && list.size() > 0) {
            BusinessContract contract = list.get(0);
            String contractNumbe = contract.getContractNumber();
            Integer number = Integer.parseInt(contractNumbe.substring(contractNumbe.length() - 2));
            orderStr = String.valueOf(number + 1);
            if (number < 9) {
                orderStr = "0" + orderStr;
            }
        } 
        return orgCode + "-HT-" + todayStr + "-" + orderStr;
    }


    @Override
    public void updateApplicationByPlanBid(Map<String, Object> paramMap) {
        contractMapper.updateApplicationByPlanBid(paramMap);
    }


	@Override
	public List<BusinessContract> getAuditPassListApp(Integer pageNum, Integer pageSize, BusinessContract contract,
			Integer billType) {
		// TODO Auto-generated method stub
		// 审批通过能查询所有人，审批通过的
        BusinessContractExample example = new BusinessContractExample();
        Criteria createCriteria = example.createCriteria();
        // 审批通过.
        createCriteria.andAuditStateEqualTo(AuditType.AGREE.getAuditCode());
        // 变更状态不是，已变更的合同.
        createCriteria.andChangingNotEqualTo("2");
        // 计划id.
        if (contract.getPlanBid() != null) {
            createCriteria.andPlanBidEqualTo(contract.getPlanBid());
        }
        // 合同标题
        if (StringUtils.isNotBlank(contract.getContractTitle())) {
            createCriteria.andContractTitleLike("%" + contract.getContractTitle() + "%");
        }
        // 合同编号
        if (StringUtils.isNotBlank(contract.getContractNumber())) {
            createCriteria.andContractNumberLike("%" + contract.getContractNumber() + "%");
        }
        // 合同类型
        if (contract.getContractType() != null) {
            createCriteria.andContractTypeEqualTo(contract.getContractType());
        }
        if (billType != null) {
            if (billType == 1) {
                // 入.
                createCriteria.andContractTypeEqualTo(3147);
            } else if (billType == 3) {
                // 出.
                createCriteria.andContractTypeEqualTo(3148);
            }
        }
        
        // 品种
       /* if (contract.getGrainKind() != null) {
            createCriteria.andGrainKindEqualTo(contract.getGrainKind());
        }*/
        // 是否启用.
        if (contract.getInApplication() != null) {
            createCriteria.andInApplicationEqualTo(contract.getInApplication());
        }
        // 按照审批通过时间排序.
        example.setOrderByClause(" agree_time desc");
        // 分页.
        PageHelper.startPage(pageNum, pageSize);
        List<BusinessContract> list = contractMapper.selectByExample(example);
        return list;
	}

	@Override
	public List<BusinessContract> queryContractChangeList(Integer userId, Integer pageNum, Integer pageSize) {
		
		/*BusinessDeliveryStorageNoticeExample noticeExample = new BusinessDeliveryStorageNoticeExample();
		BusinessDeliveryStorageNoticeExample.Criteria noticeCriteria = noticeExample.createCriteria();
		noticeCriteria.andContractBidIsNotNull();
		List<BusinessDeliveryStorageNotice> noticeList = deliveryStorageNoticeService.queryByExample(noticeExample);
		List<Integer> contract_ids = null;
		if (null != noticeList && noticeList.size() > 0) {
			contract_ids = new ArrayList<Integer>();
			for (BusinessDeliveryStorageNotice notice : noticeList) {
				contract_ids.add(notice.getContractBid());
			}
		}*/
		
		BusinessContractExample example = new BusinessContractExample();
        Criteria createCriteria = example.createCriteria();
        // 审批通过的.
        createCriteria.andAuditStateEqualTo(AuditType.AGREE.getAuditCode());
        // 变更状态为没有被变更.
        createCriteria.andChangingEqualTo("0");
        // 只能变更自己申请的合同.
        createCriteria.andCreaterIdEqualTo(userId);
        /*// 没有被通知单引用的合同
        if (null != contract_ids) {
        	createCriteria.andIdNotIn(contract_ids);
        }*/
        // 倒序.
        example.setOrderByClause(" id desc");
       
        PageHelper.startPage(pageNum, pageSize);
        //除去中转合同
        createCriteria.andContractTypeNotEqualTo(6913);
        //除去储备粮合同
        createCriteria.andCreateUnitIsNull();
        List<BusinessContract> list = contractService.queryByExample(example);

		return list;
	}

	/*@Override
	public List<BusinessContract> getAllContractInfo() {
		// TODO Auto-generated method stub
		return contractMapper.getAllContractInfo();
	}*/

}
