/**
 * 
 */
package com.chinaitop.depot.business.service.impl;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Properties;
import java.util.UUID;

import com.chinaitop.depot.utils.ParameterUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import com.chinaitop.depot.business.mapper.BusinessFileMapper;
import com.chinaitop.depot.business.model.BusinessFile;
import com.chinaitop.depot.business.model.BusinessFileExample;
import com.chinaitop.depot.business.model.BusinessFileExample.Criteria;
import com.chinaitop.depot.business.service.BusinessFileService;
import com.chinaitop.depot.common.service.impl.FileUploadServiceImpl;

/**
 * <p>Title: BusinessFileServiceImpl</p>
 * <p>Description: </p>
 * 
 */
@Service
public class BusinessFileServiceImpl implements BusinessFileService {
    @Autowired
    private BusinessFileMapper fileMapper;
    
    @Override
    public int save(BusinessFile businessFile) {
        fileMapper.insert(businessFile);
        return businessFile.getId();
    }

    @Override
    public List<BusinessFile> getByBid(Integer bid,String fileType, Integer orgId, String order) {
        BusinessFileExample example = new BusinessFileExample();
        Criteria criteria = example.createCriteria();
        
        if("acceptance".equals(fileType) && orgId==null){  //验收的文件上传，用id查询
          //应用于省级平台展示文件
        	if (null != bid) {
    			criteria.andIdEqualTo(bid);
    			//criteria.andBidIsNotNull(); //验收保存数据成功 bid不为空
    		}
    	}else{
    		if (null != fileType) {
        		criteria.andTypeEqualTo(fileType);
        	}
        	if (null != bid) {
        		criteria.andBidEqualTo(bid);
        	}
        	if (null != orgId) {
        		criteria.andOrgIdEqualTo(orgId);
        	}
        	if (null != order && !"".equals(order)) {
        		example.setOrderByClause(order);
        	}
    	}
        
        return fileMapper.selectByExample(example);
    }

    /**
     *  根据id进行删除,删除数据库数据和文件数据.
     */
    @Override
    public void remove(Integer id, String rootPath) {
        BusinessFile file = fileMapper.selectByPrimaryKey(id);
        String filePath = file.getFilePath();
        // 删除文件.
        // path是指欲下载的文件的路径。
        File fileF = new File(rootPath + filePath);
        // 如果文件存在进行删除.
        if (fileF.exists()) {
            fileF.delete();
        }
        // 删除数据库数据.
        fileMapper.deleteByPrimaryKey(id);
    }

    /**
     * 根据bid进行删除,删除数据库数据和文件数据.
     */
    @Override
    public void removeByBid(Integer bid, String rootPath) {
        if (bid != null && StringUtils.isNotBlank(rootPath)) {
            // 根据bid进行查询.
            BusinessFileExample example = new BusinessFileExample();
            Criteria createCriteria = example.createCriteria();
            createCriteria.andBidEqualTo(bid);
            List<BusinessFile> list = fileMapper.selectByExample(example);
            if (list != null && list.size() > 0) {
                // 遍历删除.
                for (BusinessFile businessFile : list) {
                    this.remove(businessFile.getId(), rootPath);
                }
            }
        }
    }
    
    /**
     * 修改fileIds对应附件的bid.
     * @param fileIds
     * @param bid
     */
    public void updateFileByFileIdsAndBid(String fileIds, Integer bid,String fileType) {
    	//System.out.println(fileIds+"****"+bid+"////"+fileType);
        if (StringUtils.isNotBlank(fileIds) && !"[]".equals(fileIds)) {
            BusinessFile businessFile = new BusinessFile();
            businessFile.setBid(bid);
            
            BusinessFileExample example = new BusinessFileExample();
            Criteria createCriteria = example.createCriteria();
            // 去除开头结尾.
            fileIds = fileIds.substring(1, fileIds.length());
            fileIds = fileIds.substring(0, fileIds.length() - 1);
            String[] idArray = fileIds.split(",");
            List<Integer> idList = new ArrayList<Integer>();
            if(idArray.length>0 && idArray!=null){
            	 for (String id : idArray) {
            		 id = id.replaceAll("\\p{P}", "");
            		 if(null !=id  && "" != id && id.length()>0){
            			idList.add(Integer.parseInt(id));
            		 }
                 }
            	 if(idList!=null && idList.size()>0){
            		 createCriteria.andIdIn(idList);
                     createCriteria.andTypeEqualTo(fileType);
                     fileMapper.updateByExampleSelective(businessFile, example);
            	 }
            }
        }
    }

    /**
     * 根据fileId数组字符串删除文件.
     * fileIds是个数组字符串，如：[1,2,3]
     */
    @Override
    public void deleteFileByFileIdArrayString(String fileIds) {
        if (StringUtils.isNotBlank(fileIds) && !"[]".equals(fileIds)) {
            // 去除开头结尾.
            fileIds = fileIds.substring(1, fileIds.length());
            fileIds = fileIds.substring(0, fileIds.length() - 1);
            String[] idArray = fileIds.split(",");
            // 循环删除.
            for (String id : idArray) {
                fileMapper.deleteByPrimaryKey(Integer.parseInt(id));
            }
        }
    }

    
    
    /**
     * 通过原始文件名称，获取新的文件名称(用于保存，保证文件名不重复).
     */
    public String getNewFileName(String originalFilename) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddhhmmss");
        String suffix = "";
        if(originalFilename.lastIndexOf(".") != -1){
            suffix = originalFilename.substring(originalFilename.lastIndexOf("."));
        }
        String newFilename = sdf.format(new Date()) + UUID.randomUUID().toString() +suffix;
        return newFilename;
    }
    
    public Properties readProperties(String filePath) {
        Properties props = new Properties();
        try {
            InputStream in = new BufferedInputStream(FileUploadServiceImpl.class.getResourceAsStream(filePath));
            props.load(in);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return props;
    }

    @Value("${web.upload-path}")
    private String rootPath;

    @Override
    @Transactional(readOnly = false)
    public BusinessFile uploadFile(MultipartFile file, String type, Integer bid, Integer userId,Integer orgId) {
    	BusinessFile businessFile = new BusinessFile();
        String path = "/upload/" + type;
        // 文件相对路径
        String relativePath = rootPath + path;
        // 如路径不存在，则创建它
        File directory = new File(relativePath);
        if (!directory.exists()) {
            directory.mkdirs();
        }

        // 文件原名称.
        String originalFilename = file.getOriginalFilename();
        // 新文件名.
        String newFileName = this.getNewFileName(originalFilename);

        // 文件保存路径.
        String filePath = relativePath + "/" + newFileName;
        path += "/"+newFileName;
        // 复制 文件
        try {
            file.transferTo(new File(filePath));
        } catch (IllegalStateException | IOException e) {
            e.printStackTrace();
        }

        // 区分windows和linux
        int i = rootPath.indexOf(":");
        if (i != -1) {
            filePath = filePath.substring(i + 1,filePath.length());
        }

        // 保存到数据库.
        businessFile.setBid(bid);
        businessFile.setType(type);
        businessFile.setFileName(newFileName);
        businessFile.setOriginalFileName(originalFilename);
        businessFile.setFilePath(path);
        businessFile.setCreateTime(new Date());
        businessFile.setCreater(userId);
        businessFile.setOrgId(orgId);

        // 保存.
        fileMapper.insert(businessFile);
        return businessFile;
    }

	@Override
	public List<BusinessFile> selectByExample(BusinessFileExample example) {
		// TODO Auto-generated method stub
		return fileMapper.selectByExample(example);
	}
    
}
