package com.chinaitop.depot.business.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

import javax.annotation.Resource;

import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import com.chinaitop.depot.agile.model.BusinessScedule;
import com.chinaitop.depot.business.mapper.BusinessDeliveryStorageNoticeMapper;
import com.chinaitop.depot.business.mapper.BusinessMergeNoticeMapper;
import com.chinaitop.depot.business.model.BusinessDeliveryStorageNotice;
import com.chinaitop.depot.business.model.BusinessMergeNotice;
import com.chinaitop.depot.business.model.BusinessMergeNoticeDetail;
import com.chinaitop.depot.business.model.BusinessStoreWareDetail;
import com.chinaitop.depot.business.service.BusinessMergeNoticeService;
import com.chinaitop.depot.business.service.BusinessStoreWareDetailService;
import com.chinaitop.depot.business.service.FeignAgileService;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

@Service
public class BusinessMergeNoticeServiceImpl implements BusinessMergeNoticeService {

	@Resource
	private BusinessMergeNoticeMapper businessMergeNoticeMapper;
	@Resource
	private FeignAgileService feignAgileService;
	@Resource
    private BusinessDeliveryStorageNoticeMapper businessDeliveryStorageNoticeMapper;
	@Resource
    private BusinessStoreWareDetailService businessStoreWareDetailService;

	@Override
	public List<BusinessMergeNoticeDetail> findByHtbhCondition(String htbh, Integer orgId) throws Exception {
		BusinessMergeNoticeDetail obj = new BusinessMergeNoticeDetail();
		obj.setOrgId(orgId);
		obj.setHtbh(htbh);
		List<BusinessMergeNoticeDetail> tzd_list = businessMergeNoticeMapper.findByHtbhCondition(obj);
		if (!ObjectUtils.isEmpty(tzd_list)) {
			// 通过通知单id,去出入库查询.
			List<String> tzdid_list = tzd_list.stream().map(BusinessMergeNoticeDetail::getTzdid).collect(Collectors.toList());
			String tzdid_str = "('"+String.join("','", tzdid_list)+"')";
			List<BusinessScedule> businessSceduleList = feignAgileService.getNoticeSceduleInNoticeId(tzdid_str);
            if (businessSceduleList != null && businessSceduleList.size() > 0) {
            	for (BusinessMergeNoticeDetail d : tzd_list) {
            		// 循环出入库，查询的list.
            		d.setSysl(new BigDecimal("0"));//先设置为0， 避免没有查到数据显示为空的情况
                    for (BusinessScedule businessScedule : businessSceduleList) {
                    	if (d.getTzdid().equals(businessScedule.getTzdId())) {

                    		BigDecimal s = new BigDecimal(1000);
                    		BigDecimal sumSysl = new BigDecimal(businessScedule.getSumSysl()).divide(s).setScale(3, BigDecimal.ROUND_DOWN);
                    		d.setSysl(sumSysl);

                			// 在出入库list中删除该数据，这样下次循环数量就会减少了.
                			businessSceduleList.remove(businessScedule);
                			// 跳出当前的出入库循环.
                			break;
                    	}
                    }
            	}
            } else {
            	for (BusinessMergeNoticeDetail d : tzd_list) {
            		d.setSysl(new BigDecimal("0"));
            	}
            }
		}
		return tzd_list;
	}

	@Override
	public void save(String x_arr, String y_arr) throws Exception {

		ObjectMapper mapper = new ObjectMapper();

		/* 格式化主表数据(也是目标通知单数据) */
		BusinessMergeNotice businessMergeNotice = mapper.readValue(x_arr, BusinessMergeNotice.class);

		/* 格式化原通知单数据 */
		List<BusinessMergeNoticeDetail> detailList = mapper.readValue(y_arr, new TypeReference<List<BusinessMergeNoticeDetail>>(){});

		/* 初始化主表主键ID */
		String uuid = UUID.randomUUID().toString().replace("-", "");

		/* 初始化合并后数量 */
		BigDecimal hbhsl = businessMergeNotice.getJhsl();

		/* 子表数据集合 */
		List<BusinessMergeNoticeDetail> list = new ArrayList<>();

		/* 组装原通知单数据 */
		for (BusinessMergeNoticeDetail ytzd : detailList) {
			//把每条原通知单的剩余数量累计到目标通知单中
			hbhsl = hbhsl.add(ytzd.getSysl()).setScale(3, BigDecimal.ROUND_DOWN);

			ytzd.setId(UUID.randomUUID().toString().replace("-", ""));
			ytzd.setCjr(businessMergeNotice.getCjr());
			ytzd.setCjsj(new Date());
			ytzd.setUpdatetime(new Date());
			ytzd.setZid(uuid);
			ytzd.setDatatype("0");
			BigDecimal y_hbhsl = ytzd.getJhsl().subtract(ytzd.getSysl()).setScale(3, BigDecimal.ROUND_DOWN);
			ytzd.setHbhsl(y_hbhsl);
			list.add(ytzd);
		}

		/* 组装目标通知单数据 */
		BusinessMergeNoticeDetail newDetail = new BusinessMergeNoticeDetail();
		newDetail.setId(UUID.randomUUID().toString().replace("-", ""));
		newDetail.setCjr(businessMergeNotice.getCjr());
		newDetail.setCjsj(new Date());
		newDetail.setUpdatetime(new Date());
		newDetail.setOrgId(businessMergeNotice.getOrgId());
		newDetail.setZid(uuid);
		newDetail.setDatatype("1");
		newDetail.setTzdid(businessMergeNotice.getTzdid());
		newDetail.setTzdbh(businessMergeNotice.getTzdbh());
		newDetail.setHtbh(businessMergeNotice.getHtbh());
		newDetail.setKhmc(businessMergeNotice.getKhmc());
		newDetail.setLspz(businessMergeNotice.getLspz());
		newDetail.setCh(businessMergeNotice.getCh());
		newDetail.setHwh(businessMergeNotice.getHwh());
		newDetail.setJhsl(businessMergeNotice.getJhsl());
		newDetail.setSysl(businessMergeNotice.getSysl());
		newDetail.setHbhsl(hbhsl);
		list.add(newDetail);

		/* 组装合并通知单主表数据 */
		businessMergeNotice.setId(uuid);
		businessMergeNotice.setUpdatetime(new Date());
		businessMergeNotice.setHbhsl(hbhsl);
		businessMergeNoticeMapper.insertBusinessMergeNotice(businessMergeNotice);

		/* 保存子表数据 */
		for (BusinessMergeNoticeDetail bmnd : list) {
			businessMergeNoticeMapper.insertBusinessMergeNoticeDetail(bmnd);
		}

		/* 更新通知单原数据的计划数量 */
		BigDecimal jhsl = null;
		BigDecimal y_sysl_sum = new BigDecimal(0);//原通知单的剩余数量之和
		for (BusinessMergeNoticeDetail bmnd : list) {
			//原通知单
			if ("0".equals(bmnd.getDatatype())) {
				y_sysl_sum = bmnd.getSysl().add(y_sysl_sum).setScale(3, BigDecimal.ROUND_DOWN);
				//新的计划数量=原计划数量-剩余数量
				jhsl = bmnd.getJhsl().subtract(bmnd.getSysl()).setScale(3, BigDecimal.ROUND_DOWN);
			}
			//目标通知单
			if ("1".equals(bmnd.getDatatype())) {
				//新的计划数量=原计划数量+剩余数量
				jhsl = bmnd.getJhsl().add(y_sysl_sum).setScale(3, BigDecimal.ROUND_DOWN);
			}
			BusinessDeliveryStorageNotice businessDeliveryStorageNotice = businessDeliveryStorageNoticeMapper.selectByPrimaryKey(Integer.parseInt(bmnd.getTzdid()));
			List<BusinessStoreWareDetail> bswdList = businessStoreWareDetailService.getByZidAndType(Integer.parseInt(bmnd.getTzdid()), "notice");
			if (null != businessDeliveryStorageNotice && null != bswdList && bswdList.size() > 0) {
				businessDeliveryStorageNotice.setShipingCount(jhsl.toString());
				businessDeliveryStorageNotice.setIsMerge("0");//设置为合并过
				for (BusinessStoreWareDetail b : bswdList) {
					String zje = "";
					if ("1".equals(businessDeliveryStorageNotice.getBillType())) {
						b.setInCount(jhsl.toString());
						zje = new BigDecimal(b.getInCount()).multiply(new BigDecimal(b.getInPrice())).setScale(3, BigDecimal.ROUND_DOWN).toString();
						b.setInDetailTotalPrice(zje);
						businessDeliveryStorageNotice.setMoneyQuantity(b.getInDetailTotalPrice());
					}
					if ("3".equals(businessDeliveryStorageNotice.getBillType())) {
						b.setOutCount(jhsl.toString());
						zje = new BigDecimal(b.getOutCount()).multiply(new BigDecimal(b.getOutPrice())).setScale(3, BigDecimal.ROUND_DOWN).toString();
						b.setOutDetailTotalPrice(zje);
						businessDeliveryStorageNotice.setMoneyQuantity(b.getOutDetailTotalPrice());
					}
					businessStoreWareDetailService.updateById(b);
				}
				businessDeliveryStorageNoticeMapper.updateByPrimaryKey(businessDeliveryStorageNotice);
			}
		}
	}

	@Override
	public List<BusinessMergeNotice> findByList(Integer orgId, String htbh, String khmc, Integer ch) throws Exception {
		BusinessMergeNotice obj = new BusinessMergeNotice();
		if (StringUtils.isNotBlank(htbh)) {
			obj.setHtbh("%"+htbh+"%");
		}
		if (StringUtils.isNotBlank(khmc)) {
			obj.setKhmc("%"+khmc+"%");
		}
		if (null != orgId) {
			obj.setOrgId(orgId);
		}
		if (null != ch) {
			obj.setCh(ch);
		}
		List<BusinessMergeNotice> list = businessMergeNoticeMapper.findByPageList(obj);
		return list;
	}

	@Override
	public Map<String, Object> findByIdList(String id) throws Exception {
		BusinessMergeNoticeDetail obj = new BusinessMergeNoticeDetail();
		obj.setZid(id);
		BusinessMergeNotice businessMergeNotice = businessMergeNoticeMapper.findByIdMergeNotice(id);
		List<BusinessMergeNoticeDetail> list = businessMergeNoticeMapper.findByZidDetailList(obj);
		List<BusinessMergeNoticeDetail> y_list = list.stream().filter(BusinessMergeNoticeDetail -> BusinessMergeNoticeDetail.getDatatype().equals("0")).collect(Collectors.toList());
		List<BusinessMergeNoticeDetail> x_list = list.stream().filter(BusinessMergeNoticeDetail -> BusinessMergeNoticeDetail.getDatatype().equals("1")).collect(Collectors.toList());
		Map<String, Object> map = new HashMap<>();
		map.put("y_list", y_list);
		map.put("x_list", x_list);
		map.put("obj", businessMergeNotice);
		return map;
	}

	@Override
	public List<BusinessMergeNoticeDetail> findByTzdidQueryList(String tzdid) throws Exception {
		BusinessMergeNoticeDetail obj = new BusinessMergeNoticeDetail();
		obj.setTzdid(tzdid);
		List<BusinessMergeNoticeDetail> list = businessMergeNoticeMapper.findByZidDetailList(obj);
		return list;
	}

}
