package com.chinaitop.depot.business.service.impl;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.act.service.ActTaskService;
import com.chinaitop.depot.agile.model.BusinessScedule;
import com.chinaitop.depot.business.mapper.BusinessPlanMapper;
import com.chinaitop.depot.business.model.BusinessApproval;
import com.chinaitop.depot.business.model.BusinessContract;
import com.chinaitop.depot.business.model.BusinessContractExample;
import com.chinaitop.depot.business.model.BusinessDeliveryStorageNotice;
import com.chinaitop.depot.business.model.BusinessDeliveryStorageNoticeExample;
import com.chinaitop.depot.business.model.BusinessPlan;
import com.chinaitop.depot.business.model.BusinessPlanExample;
import com.chinaitop.depot.business.model.BusinessPlanExample.Criteria;
import com.chinaitop.depot.business.model.BusinessStoreWareDetail;
import com.chinaitop.depot.business.model.UserInfo;
import com.chinaitop.depot.business.model.enums.AuditType;
import com.chinaitop.depot.business.model.enums.BasicEnum;
import com.chinaitop.depot.business.model.enums.CrkNoticeStatus;
import com.chinaitop.depot.business.model.enums.StoreWareType;
import com.chinaitop.depot.business.model.enums.TaskType;
import com.chinaitop.depot.business.model.vo.BusinessPlanAuditVO;
import com.chinaitop.depot.business.service.BusinessApprovalService;
import com.chinaitop.depot.business.service.BusinessContractService;
import com.chinaitop.depot.business.service.BusinessDeliveryStorageNoticeService;
import com.chinaitop.depot.business.service.BusinessFileService;
import com.chinaitop.depot.business.service.BusinessPlanService;
import com.chinaitop.depot.business.service.BusinessStoreWareDetailService;
import com.chinaitop.depot.business.service.FeignAgileService;
import com.chinaitop.depot.business.service.UserInfoService;
import com.chinaitop.depot.utils.ParameterUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Maps;


@Service
public class BusinessPlanServiceImpl implements BusinessPlanService {
    
    @Resource
    private BusinessPlanMapper planMapper;
    
    @Resource
    private BusinessFileService businessFileService;
    @Resource
    private BusinessStoreWareDetailService businessStoreWareDetailService;
   // @Resource
    //private BasicCodeRuleService basicCodeRuleService;
    @Resource
    private BusinessContractService contractService;
    @Resource
    private BusinessDeliveryStorageNoticeService deliveryStorageNoticeService;
    @Resource
    private ActTaskService actTaskService;
    @Resource
    private BusinessApprovalService businessApprovalService;
    @Resource
	private FeignAgileService feignAgileService;
    /*@Resource
    private BusinessSceduleService businessSceduleService;*/
    @Resource
    private UserInfoService userInfoService;

    public List<BusinessPlan> queryByExample(BusinessPlanExample example) {
        return planMapper.selectByExample(example);
    }

    
    public BusinessPlan findById(Integer id) {
        return planMapper.selectByPrimaryKey(id);
    }

    /**
     * 新增数据，启动流程.
     * @throws Exception 
     */
    @Override
    @Transactional(readOnly = false)
    public Map<String, Object> add(BusinessPlan plan, List<BusinessStoreWareDetail> storeWareDetailList,
            String fileIds , Integer userId , Integer orgId ,String name ,Integer depotInfoOrgId) {
        Map<String, Object> returnMap = new HashMap<String, Object>();
        /*//当前登录人的id
        Integer userId = null;
        //当前登录人的姓名
        String name = null;
        try {
            userId = userInfo.getUserId();
            name = userInfo.getRealName();
        } catch (Exception e) {
            e.printStackTrace();
            returnMap.put("error", "获取不到登录人信息，请重新登录！");
            return returnMap;
        }*/
        
        /*
        // 通过规则，获取计划编号.
        try {
            plan.setPlanNumber(basicCodeRuleService.getCode(CodeRuleType.plan, depotInfo.getOrgId()));
        } catch (Exception e) {
            e.printStackTrace();
            returnMap.put("error", e.getMessage());
            return returnMap;
        }*/
        
        // 设置审批状态为，审批待审批.
        plan.setAuditState(AuditType.UNSUBMIT.getAuditCode());
        //plan.setCreateTime(new Date());
        // 当前审批节点.
        plan.setCurrentApprovePath("待提交");
        // 设置登录人信息.
        //plan.setCreater(name);
        plan.setCreaterId(userId);
        plan.setOrgId(orgId);
        // plan.setCreateUnit("经营一部");
        // 应用状态设定为执行中，应用
        plan.setInApplication(CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode());
        // 保存，返回id,id在流程启动时，需要使用.
        planMapper.insert(plan);
        
        // 将id返回到页面，用于上传文件使用.
        returnMap.put("id", plan.getId());
        
        // 启动流程
        /*ProcessInstance processInstance = actTaskService.startProcess(TaskType.plan.getTaskType(),
                plan.getId().toString(), name, userId.toString(), depotInfo.getOrgId().toString());*/
        String startProcess1 = actTaskService.startProcessNew(TaskType.plan.getTaskType(),
                plan.getId().toString(), name, userId.toString(), depotInfoOrgId.toString());
        JSONObject jsStr = JSONObject.parseObject(startProcess1);
        String processInstanceId = jsStr.getString("processInstanceId");
        String processDefinitionId = jsStr.getString("processDefinitionId");
        // 设置流程实例id和流程定义id到合同中.
        // 设置流程实例id.
        plan.setProcessInstanceId(processInstanceId);
        // 设置流程定义id.
        plan.setProcessDefinitionId(processDefinitionId);
        
        planMapper.updateByPrimaryKey(plan);
        
        // 新增仓房货位数量质量标准子表.
        if (storeWareDetailList != null && storeWareDetailList.size() > 0) {
            for (BusinessStoreWareDetail storeWareDetail : storeWareDetailList) {
                storeWareDetail.setZid(plan.getId());
                storeWareDetail.setType(StoreWareType.plan.getValue());
                storeWareDetail.setCreateTime(new Date());
                businessStoreWareDetailService.add(storeWareDetail);
            }
        }
        
        // 修改fileIds对应附件的bid为当前计划的id.
        businessFileService.updateFileByFileIdsAndBid(fileIds, plan.getId(),"plan");
        return returnMap;
    }

    /**
     * 修改.
     * @param plan
     * @param fileIds
     */
    public void update(BusinessPlan plan, List<BusinessStoreWareDetail> storeWareDetailList, String fileIds, String deleteFileIds) {
        planMapper.updateByPrimaryKey(plan);
        // 修改fileIds对应附件的bid为当前计划的id.
        businessFileService.updateFileByFileIdsAndBid(fileIds, plan.getId(),"plan");
        // 删除附件.
        businessFileService.deleteFileByFileIdArrayString(deleteFileIds);
        
        if (storeWareDetailList != null && storeWareDetailList.size() > 0) {
            for (BusinessStoreWareDetail storeWareDetail : storeWareDetailList) {
                storeWareDetail.setZid(plan.getId());
                if (storeWareDetail.getId() == null) {
                    // 新增.
                    businessStoreWareDetailService.add(storeWareDetail);
                } else {
                    // 修改.
                    businessStoreWareDetailService.update(storeWareDetail);
                }
            }
        }
    }

    /**
     * 删除.
     * @param id
     * @param processInstanceId
     * @param deleteReason
     * @param rootPath
     */
    public void remove(Integer id, String processInstanceId, String deleteReason, String rootPath) {
        // 删除数据.
        planMapper.deleteByPrimaryKey(id);
        
        // 删除明细列表数据.
        businessStoreWareDetailService.deleteByTypeAndZid(StoreWareType.plan.getValue(), id);
        
        // 删除附件.
        businessFileService.removeByBid(id, rootPath);
        try {
            // 删除工作流数据.
            if (deleteReason!=null) {
                actTaskService.delRunTask(processInstanceId, deleteReason);
            } else {
                deleteReason="删除原因无";
                actTaskService.delRunTask(processInstanceId, deleteReason);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        
    }

    // 提交
    @Transactional(readOnly = false)
    public Map<String, Object> submit(Integer id, String nextUserId, Integer userId , String realName) {
        Map<String, Object> returnMap = new HashMap<String, Object>();
        returnMap.put("success", "success");
        
        BusinessPlan planB = planMapper.selectByPrimaryKey(id);
        
        if (planB == null) {
            throw new RuntimeException("没有找到相应的数据");
        }
        
        String auditState = planB.getAuditState();
        if (StringUtils.isNotBlank(auditState) 
                && !auditState.equals(AuditType.UNSUBMIT.getAuditCode()) 
                && !auditState.equals(AuditType.REJECT.getAuditCode())) {
            returnMap.put("success", "error");
            returnMap.put("msg", "您已经提交该数据，无法再次提交！");
            return returnMap;
        }
        // 获取到当前进行到的任务节点.
//        Task singleResult = taskService.createTaskQuery().processInstanceId(planB.getProcessInstanceId()).singleResult();
        String taskNode = actTaskService.taskNode(planB.getProcessInstanceId());
        JSONObject taskNodeNew = JSONObject.parseObject(taskNode);
        String taskId = taskNodeNew.getString("taskId");
        String taskName = taskNodeNew.getString("taskName");
        // 修改流程.
        Map<String, Object> vars = Maps.newHashMap();
        //需要修改pass
        vars.put("pass", "1");
        vars.put("inputUser", nextUserId);
        actTaskService.complete(taskId, planB.getProcessInstanceId(), "", "", vars);
        
        // 获取到下一个的任务节点.
//        Task singleResultNext = taskService.createTaskQuery().processInstanceId(planB.getProcessInstanceId()).singleResult();
        
        // 下一环节名称
//        String taskName = singleResultNext.getName();
        // 任务id
//        String taskId = singleResultNext.getId();
        
        // 设置审批状态为，审批中.
        planB.setAuditState(AuditType.SUBMIT.getAuditCode());
        // 当前审批节点.
        planB.setCurrentApprovePath(taskName);
        // 修改主表.
        planMapper.updateByPrimaryKey(planB);
        
        StringBuilder namePlan=new StringBuilder();
        namePlan.append(TaskType.plan.getTaskName());
        namePlan.append(" 编号: ");
        namePlan.append(planB.getPlanNumber());
        // 设置流程中间表.
        BusinessApproval approval = new BusinessApproval();
        approval.setTaskTypeName(namePlan.toString());
        approval.setProjectId(planB.getId());
        approval.setProcessInstanceId(planB.getProcessInstanceId());
        approval.setFromDepartment("经营一部");
        approval.setFromPeople(userId.toString());
        approval.setFromUserName(realName);
        approval.setTaskType(TaskType.plan.getTaskType());
        approval.setProjectName(TaskType.plan.getTaskName());
        approval.setResult("待审批");
        approval.setIsHide(0);
        approval.setCreateTime(new Date());
        // 保存申请人的信息,时间.
        approval.setApplyName(realName);
        approval.setApplyNameId(userId.toString());
        approval.setApplyTime(new Date());
        
        // 下一个待审批人id.
        approval.setOperator(nextUserId);
        // 通过id,获取user 对象.
        UserInfo userInfo2=userInfoService.findById(Integer.valueOf(nextUserId));
        // 下一个审批人名称
        approval.setOperatorName(userInfo2.getRealName());
        approval.setTaskName(taskName);
        approval.setTaskId(taskId);
        
        businessApprovalService.add(approval);
        
        return returnMap;
    }
    
    public List<BusinessPlanAuditVO> getApprovalList(Map<String, Object> paramMap) {
        return planMapper.getApprovalList(paramMap);
    }


    // 审批.
    @Transactional(readOnly = false)
    public void audit(Integer planId, BusinessApproval businessApproval,
            String nextUserId, String result, Integer userId, String realName) {
        BusinessApproval oldBusinessApproval = businessApprovalService.findById(businessApproval.getId());
        // 检测是否并发审核
        String oldResult = oldBusinessApproval.getResult();
        if ("同意".equals(oldResult) || "退回".equals(oldResult)) {
            throw new RuntimeException("该订单已经审核过，不能再次审核！");
        }

        String resultS = "";
        if ("0".equals(result)) {
            resultS = "退回";
        } else if ("1".equals(result)) {
            resultS = "同意";
        } else if ("2".equals(result)) {
            resultS = "驳回";
        }else if ("3".equals(result)) {
            resultS = "拒绝";
        }

        // 修改当前中间表.
        // 审批结果.
        oldBusinessApproval.setResult(resultS);
        // 审批意见.
        oldBusinessApproval.setContent(businessApproval.getContent());
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String nowF = sdf.format(new Date());
        // 操作时间.
        oldBusinessApproval.setOperateTime(nowF);
        businessApprovalService.updateByPrimaryKeySelective(oldBusinessApproval);

        // 得到当前的流程实例id
        String processInstanceId = businessApproval.getProcessInstanceId();

        //获取任务节点
        String taskNode = actTaskService.taskNode(processInstanceId);
        JSONObject taskNodeNew = JSONObject.parseObject(taskNode);
        String taskId = taskNodeNew.getString("taskId");
        String taskName = taskNodeNew.getString("taskName");
        // 修改流程.
        Map<String, Object> vars = Maps.newHashMap();
        vars.put("pass", result);
        if (result.equals("1")) {
            vars.put("inputUser", nextUserId);
        } else if(result.equals("0")) {
            //退回时，退给上一个人
            String findHistoryTaskName = actTaskService.findHistoryTaskName(processInstanceId);
            vars.put("inputUser", findHistoryTaskName);
        } else if (result.equals("2")) {
            //驳回环节(获取提交第一个人)
            String findHistoryFirstTaskName = actTaskService.findHistoryFirstTaskName(processInstanceId);
            vars.put("inputUser", findHistoryFirstTaskName);
        } else if (result.equals("3")) {
            //拒绝
        	/*String findHistoryTaskName = actTaskService.findHistoryTaskName(processInstanceId);
            vars.put("inputUser", findHistoryTaskName);*/
        } else {
            //位置环节
            return; 
        }
        // 退回，驳回 需要添加名字
        actTaskService.complete(taskId, processInstanceId, businessApproval.getContent(), "", vars);

        // 查询流程状态（判断流程正在执行，还是结束）
        boolean processEnd = actTaskService.isProcessEndNew(processInstanceId);
        BusinessPlan plan = planMapper.selectByPrimaryKey(planId);
        if (processEnd) {
            // 审批结束.
            // 审批通过时间.
            plan.setAgreeTime(new Date());
            // 当前审批节点.
            plan.setCurrentApprovePath("结束");
            if ("3".equals(result)) {
                plan.setAuditState(AuditType.REFUSE.getAuditCode());
            } else if ("1".equals(result)) {
                plan.setAuditState(AuditType.AGREE.getAuditCode());
            }
            plan.setCurrentApprovePath(taskName);
            planMapper.updateByPrimaryKey(plan);

            //同意修改审批人
            BusinessApproval approval = new BusinessApproval();
            approval.setId(oldBusinessApproval.getId());
            approval.setTaskId(taskId);
            approval.setTaskName(taskName);
            businessApprovalService.updateByPrimaryKeySelective(approval);
        } else {
            if ("0".equals(result)) {
                //resultS = "退回";
            } else if ("2".equals(result)) {
                //resultS = "驳回";
                // 审批状态.
                plan.setAuditState(AuditType.REJECT.getAuditCode());
                // 当前审批节点.
                plan.setCurrentApprovePath(taskName);
                planMapper.updateByPrimaryKey(plan);
            } else if ("3".equals(result)) {
                //resultS = "拒绝";
                // 审批状态.
                plan.setAuditState(AuditType.REFUSE.getAuditCode());
                // 当前审批节点.
                plan.setCurrentApprovePath(taskName);
                planMapper.updateByPrimaryKey(plan);
            } else if ("1".equals(result)) {
                // 同意.
                // 新增流程中间表.
                
                // 当前审批节点.
                plan.setCurrentApprovePath(taskName);
                // 修改主表.
                planMapper.updateByPrimaryKey(plan);
                
                BusinessApproval approval = new BusinessApproval();
                
                StringBuilder namePlan=new StringBuilder();
                namePlan.append(TaskType.plan.getTaskName());
                namePlan.append(" 编号: ");
                namePlan.append(plan.getPlanNumber());
                approval.setTaskTypeName(namePlan.toString());
                
                // 业务id.
                approval.setProjectId(planId);
                // 流程实例id.
                approval.setProcessInstanceId(businessApproval.getProcessInstanceId());
                approval.setFromDepartment("经营一部");
                approval.setFromPeople(userId.toString());
                approval.setFromUserName(realName);
                approval.setCreateTime(new Date());
                approval.setTaskType(TaskType.plan.getTaskType());
                approval.setProjectName(TaskType.plan.getTaskName());
                approval.setResult("待审批");
                approval.setIsHide(0);
                
                // 保存申请人的信息,时间.
                approval.setApplyName(oldBusinessApproval.getApplyName());
                approval.setApplyNameId(oldBusinessApproval.getApplyNameId());
                approval.setApplyTime(oldBusinessApproval.getApplyTime());
                // 下一个审批人id.
                approval.setOperator(nextUserId);
                // 通过id,获取user 对象.
                UserInfo userInfo2=userInfoService.findById(Integer.valueOf(nextUserId));
                // 下一个审批人名称
                approval.setOperatorName(userInfo2.getRealName());
                approval.setTaskName(taskName);
                approval.setTaskId(taskId);
                // 中间表，新增.
                businessApprovalService.add(approval);
            }
        }
        
    }

    /**
     * 启用.
     * @param id
     */
    @Override
    @Transactional(readOnly = false)
    public void application(Integer id) {
        // 启用计划.
        BusinessPlan plan = new BusinessPlan();
        plan.setId(id);
        plan.setInApplication(CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode());
        plan.setRefuseReason("");
        planMapper.updateByPrimaryKeySelective(plan);
        
        // 计划下的合同，启用.
        Map<String,Object> paramMap = new HashMap<String, Object>();
        paramMap.put("inApplication", CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode());
        paramMap.put("planBid", id);
        contractService.updateApplicationByPlanBid(paramMap);
        // 计划对应的出入库通知单启用.
        deliveryStorageNoticeService.updateApplication(paramMap);
        
        // 修改出入库系统通知单状态.
        // 根据计划id，查询所有通知单.
       /* BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
        example.createCriteria().andPlanBidEqualTo(id);
        List<BusinessDeliveryStorageNotice> noticeList = deliveryStorageNoticeService.queryByExample(example);
        if (noticeList != null && noticeList.size() > 0) {
            for (BusinessDeliveryStorageNotice notice : noticeList) {
                paramMap = new HashMap<String, Object>();
                paramMap.put("tzdh", notice.getId());
                paramMap.put("FLOWSTATUS", CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode());
                // 修改出入库系统的 通知单状态.
                feignAgileService.updateNoticeStatus(paramMap);
                //serverApi.updateNoticeStatus(paramMap);
            }
        }*/
    }

    /**
     * 暂停.
     * @param id
     */
    @Override
    @Transactional(readOnly = false)
    public void discontinue(Integer id ,String refuseReason) {
        // 暂停计划.
        BusinessPlan plan = planMapper.selectByPrimaryKey(id);
        plan.setInApplication(CrkNoticeStatus.ZANTING.getCrkNoticeCode());
        plan.setRefuseReason(refuseReason);
        planMapper.updateByPrimaryKeySelective(plan);
        
        // 计划下的合同，暂停.
        Map<String,Object> paramMap = new HashMap<String, Object>();
        paramMap.put("inApplication", CrkNoticeStatus.ZANTING.getCrkNoticeCode());
        paramMap.put("planBid", id);
        contractService.updateApplicationByPlanBid(paramMap);
        // 计划对应的出入库通知单暂停.
        deliveryStorageNoticeService.updateApplication(paramMap);
        
        // 修改出入库系统通知单状态.
        // 根据计划id，查询所有通知单.
        /*BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
        example.createCriteria().andPlanBidEqualTo(id);
        List<BusinessDeliveryStorageNotice> noticeList = deliveryStorageNoticeService.queryByExample(example);
        if (noticeList != null && noticeList.size() > 0) {
            for (BusinessDeliveryStorageNotice notice : noticeList) {
                paramMap = new HashMap<String, Object>();
                paramMap.put("tzdh", notice.getId());
                paramMap.put("FLOWSTATUS", CrkNoticeStatus.ZANTING.getCrkNoticeCode());
                // 修改出入库系统的 通知单状态.
                feignAgileService.updateNoticeStatus(paramMap);
                //serverApi.updateNoticeStatus(paramMap);
            }
        }*/
    }

    /**
     * 终止.
     */
    @Override
    @Transactional(readOnly = false)
    public void finish(Integer id,String refuseReason) {
        // 终止计划.
        BusinessPlan plan = planMapper.selectByPrimaryKey(id);
        plan.setInApplication(CrkNoticeStatus.ZHONGZHI.getCrkNoticeCode());
        plan.setRefuseReason(refuseReason);
        planMapper.updateByPrimaryKeySelective(plan);
        
        // 计划下的合同，终止.
        Map<String,Object> paramMap = new HashMap<String, Object>();
        paramMap.put("inApplication", CrkNoticeStatus.ZHONGZHI.getCrkNoticeCode());
        paramMap.put("planBid", id);
        contractService.updateApplicationByPlanBid(paramMap);
        // 计划对应的出入库通知单终止.
        deliveryStorageNoticeService.updateApplication(paramMap);
        
        // 修改出入库系统通知单状态.
        // 根据计划id，查询所有通知单.
        /*BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
        example.createCriteria().andPlanBidEqualTo(id);
        List<BusinessDeliveryStorageNotice> noticeList = deliveryStorageNoticeService.queryByExample(example);
        if (noticeList != null && noticeList.size() > 0) {
            for (BusinessDeliveryStorageNotice notice : noticeList) {
                paramMap = new HashMap<String, Object>();
                paramMap.put("tzdh", notice.getId());
                paramMap.put("FLOWSTATUS", CrkNoticeStatus.ZHONGZHI.getCrkNoticeCode());
                // 修改出入库系统的 通知单状态.
                feignAgileService.updateNoticeStatus(paramMap);
                //serverApi.updateNoticeStatus(paramMap);
            }
        }*/
    }
    
    /**
     * 查询计划进度列表.
     */
    @Override
    @Transactional(readOnly = false)
    public PageInfo<BusinessPlanAuditVO> getScheduleList(Integer pageNum,
            Integer pageSize, BusinessPlan businessPlan, Integer orgId) {
        // 可以查询所有人通过的计划
        BusinessPlanExample example = new BusinessPlanExample();
        Criteria createCriteria = example.createCriteria();
        createCriteria.andAuditStateEqualTo(AuditType.AGREE.getAuditCode());
        // 计划编号.
        if (businessPlan != null) {
            // 计划编号
            if (StringUtils.isNotBlank(businessPlan.getPlanNumber())) {
                createCriteria.andPlanNumberLike("%" + businessPlan.getPlanNumber() + "%");
            }
            //库ID
            if (null != businessPlan.getOrgId()) {
                createCriteria.andOrgIdEqualTo(businessPlan.getOrgId());
            }
            // 粮食品种
            /*if (businessPlan.getGrainKind() != null) {
                createCriteria.andGrainKindEqualTo(businessPlan.getGrainKind());
            }*/
            
        }
        if (null != orgId) {
            createCriteria.andOrgIdEqualTo(orgId);
        }
        PageInfo<BusinessPlanAuditVO> pageInfo = new PageInfo<BusinessPlanAuditVO>();
        // 审批通过时间倒序.
        example.setOrderByClause("agree_time desc");
        // 分页
        PageHelper.startPage(pageNum, pageSize);
        // 查询.
        List<BusinessPlanAuditVO> planList = planMapper.selectByExampleToVO(example);
        
        if (planList != null && planList.size() > 0) {
            // 拼接计划id.
            StringBuffer sb = new StringBuffer();
            sb.append("(");
            for (BusinessPlanAuditVO businessPlanV : planList) {
                sb.append("\"" + businessPlanV.getId() + "\"").append(",");
            }
            String substring = sb.substring(0, sb.length() - 1);
            sb = new StringBuffer(substring);
            sb.append(")");
            List<BusinessScedule> businessSceduleList = feignAgileService.getPlanSceduleInPlanId(sb.toString());
            //List<BusinessScedule> businessSceduleList = serverApi.getPlanSceduleInPlanId(sb.toString());
            // 通过计划id,去出入库查询.
            if (businessSceduleList != null && businessSceduleList.size() > 0) {
                // 循环计划list.
                for (BusinessPlanAuditVO businessPlanV : planList) {
                    // 循环出入库，查询的list.
                    Boolean flag = false;
                    for (BusinessScedule businessScedule : businessSceduleList) {
                        if (businessPlanV.getId().toString().equals(businessScedule.getJhId())) {
                            flag = true;
                            // 在出入库系统中，存在该计划对应的出入库数据，进行计算，得出完成比率.
                            String crkLx = businessScedule.getCrkLx();
                            if ("1".equals(crkLx)) {
                                // 入库
                                // 入库数量.
                            	String sumRksl = businessScedule.getSumWcsl();
                                BigDecimal s = new BigDecimal(1000);
                                BigDecimal sumRkslss = new BigDecimal(sumRksl);
                   			    BigDecimal sumRkslD = sumRkslss.divide(s);
                   			 
                                // 赋值：设置完成数量. stripTrailingZeros() 去除结尾的0
                                businessPlanV.setCompleteAmount(sumRkslD.stripTrailingZeros().toPlainString());
                                // 计划要求数量.
                                //String demandAmount = businessPlanV.getTotalAmount();
                                String demandAmount = businessPlanV.getInAmount();
                                BigDecimal demandAmountD = new BigDecimal(demandAmount);
                                
                                // 除法. 已完成数量/要求数量
                                BigDecimal complete = sumRkslD.divide(demandAmountD, 4, BigDecimal.ROUND_HALF_DOWN);
                                // 乘以100，得到的是百分比.
                                BigDecimal completeRate = complete.multiply(new BigDecimal("100"));
                                // 赋值：设置完成百分比.
                                businessPlanV.setCompleteRate(completeRate.stripTrailingZeros().toPlainString());
                            } else if ("3".equals(crkLx)) {
                                // 出库
                                // 出库数量.
                            	String sumCksl = businessScedule.getSumWcsl();
                            	BigDecimal s = new BigDecimal(1000);
                                BigDecimal sumCkslss = new BigDecimal(sumCksl);
                   			    BigDecimal sumCkslD = sumCkslss.divide(s);
                   			    
                                // 赋值：设置完成数量. stripTrailingZeros() 去除结尾的0
                                businessPlanV.setCompleteAmount(sumCkslD.stripTrailingZeros().toPlainString());
                                
                                // 计划要求数量.
                               // String demandAmount = businessPlanV.getTotalAmount();
                                 String demandAmount = businessPlanV.getOutAmount();
                                BigDecimal demandAmountD = new BigDecimal(demandAmount);
                                
                                // 除法. 已完成数量/要求数量
                                BigDecimal complete = sumCkslD.divide(demandAmountD, 4, BigDecimal.ROUND_HALF_DOWN);
                                // 乘以100，得到的是百分比.
                                BigDecimal completeRate = complete.multiply(new BigDecimal("100"));
                                // 赋值：设置完成百分比.
                                businessPlanV.setCompleteRate(completeRate.stripTrailingZeros().toPlainString());
                            }
                            // 在出入库list中删除该数据，这样下次循环数量就会减少了.
                            businessSceduleList.remove(businessScedule);
                            // 跳出当前的出入库循环.
                            break;
                        }
                    }
                    if (!flag) {
                        // 进行到这里，说明出入库系统中，不存在该计划对应的出入库数据. 设置完成数量，和完成比例为0.
                        businessPlanV.setCompleteAmount("0");
                        businessPlanV.setCompleteRate("0");
                    }
                }
            } else {
                // 没有找到任何出入库数据. 设置完成数量，和完成比例为0.
                for (BusinessPlanAuditVO businessPlanV : planList) {
                    businessPlanV.setCompleteAmount("0");
                    businessPlanV.setCompleteRate("0");
                }
            }
            pageInfo = new PageInfo<BusinessPlanAuditVO>(planList);
        }
        return pageInfo;
    }


    @Override
    public List<BusinessPlan> queryByExample(Integer pageNum, Integer pageSize,
            Integer userId, BusinessPlan businessPlan, String order, Integer billType,String type,Integer orgId) {
        BusinessPlanExample example = new BusinessPlanExample();
        Criteria createCriteria = example.createCriteria();
        if (userId != null) {
            createCriteria.andCreaterIdEqualTo(userId);
        }
        
        // 计划编号.
        if (StringUtils.isNotBlank(businessPlan.getPlanNumber())){
            createCriteria.andPlanNumberLike("%"+businessPlan.getPlanNumber()+"%");
        }
        
        // 是否应用.
        if (businessPlan.getInApplication() != null) {
            createCriteria.andInApplicationEqualTo(businessPlan.getInApplication());
        }
        
        // 计划审批状态.
        if (StringUtils.isNotBlank(businessPlan.getAuditState())){
            createCriteria.andAuditStateEqualTo(businessPlan.getAuditState());
        }
        // 计划名称
        if (StringUtils.isNotBlank(businessPlan.getPlanTitle())) {
            createCriteria.andPlanTitleLike("%"+businessPlan.getPlanTitle()+"%");
        }
        
        // 计划类型
        if (businessPlan.getExecuteType() != null) {
            //createCriteria.andExecuteTypeEqualTo(businessPlan.getExecuteType());
            createCriteria.andExecuteTypeNotEqualTo(businessPlan.getExecuteType());

        }
        
        //储备粮计划
        if(businessPlan.getAttributeType() != null ){
        	createCriteria.andAttributeTypeEqualTo(businessPlan.getAttributeType());
        }
        
        //验收数据
        if(businessPlan.getCustomerPlanState() != null){
        	if(businessPlan.getCustomerPlanState().equals("100")){
        		createCriteria.andCustomerPlanStateEqualTo(businessPlan.getCustomerPlanState());
        	}else if(businessPlan.getCustomerPlanState().equals("50")){
        		createCriteria.andCustomerPlanStateEqualTo(businessPlan.getCustomerPlanState());
        	}else{
        		createCriteria.andCustomerPlanStateNotEqualTo("100");
        	}
        }
        
        /*if (billType != null) {
            List<Integer> list = new ArrayList<Integer>();
            if (billType == 1) {
                list.add(BasicEnum.planType_shouchu.getValue());
                list.add(BasicEnum.planType_ylshouchu.getValue());
                createCriteria.andExecuteTypeIn(list);
            } else if (billType == 3) {
                list.add(BasicEnum.planType_xiaoshou.getValue());
                list.add(BasicEnum.planType_ylxiaoshou.getValue());
                createCriteria.andExecuteTypeIn(list);
            }
        }*/
        
        if (billType != null) {
            List<Integer> list = new ArrayList<Integer>();
            if (billType == 1) {//入
            	if(type.equals("ls")){
            		list.add(BasicEnum.planType_shouchu.getValue());
            	}else if(type.equals("yl")){
            		list.add(BasicEnum.planType_yshouchu.getValue());
            	}
                createCriteria.andExecuteTypeIn(list);
            } else if (billType == 3) {
            	if(type.equals("ls")){
            		list.add(BasicEnum.planType_xiaoshou.getValue());
            	}else if(type.equals("yl")){
            		list.add(BasicEnum.planType_yxiaoshou.getValue());
            	}
                createCriteria.andExecuteTypeIn(list);
            }
        }
        
        
        if (StringUtils.isNotBlank(order)) {
            example.setOrderByClause(order);
        }
      
        PageHelper.startPage(pageNum, pageSize);
        
        List<BusinessPlan> selectByExample = planMapper.selectByExample(example);
        return selectByExample;
    }


    @Override
    public Boolean validPlanNumber(Integer id, String planNumber) {
        BusinessPlanExample example = new BusinessPlanExample();
        Criteria createCriteria = example.createCriteria();
        createCriteria.andPlanNumberEqualTo(planNumber);
        if (id != null) {
            createCriteria.andIdNotEqualTo(id);
        }
        int count = planMapper.countByExample(example);
        return count > 0 ? false : true;
    }


    /**
     * 根据id查询追溯数据.
     * @param id 主键
     * @return
     */
    @Override
    public Map<String, Object> getSynthInfo(Integer id) {
        Map<String, Object> returnMap = new HashMap<String, Object>();
        BusinessPlan plan = planMapper.selectByPrimaryKey(id);
        returnMap.put("plan", plan);
        
        // 获取第一个通过的合同  审批通过时间.
        BusinessContractExample contractExample = new BusinessContractExample();
        com.chinaitop.depot.business.model.BusinessContractExample.Criteria contractCriteria = contractExample.createCriteria();
        contractCriteria.andPlanBidEqualTo(id);
        // 审批通过.
        contractCriteria.andAuditStateEqualTo(AuditType.AGREE.getAuditCode());
        // 审批时间正序.
        contractExample.setOrderByClause("agree_time asc");
        // 获取一条. 获取的就是最先审批通过的合同.
        PageHelper.startPage(1, 1);
        List<BusinessContract> contractList = contractService.queryByExample(contractExample);
        if (contractList != null && contractList.size() > 0) {
            returnMap.put("contractFirstDate", contractList.get(0).getAgreeTime());
        }
        
        // 获取第一个通过的通知单  审批通过 时间.
        BusinessDeliveryStorageNoticeExample noticeExample = new BusinessDeliveryStorageNoticeExample();
        com.chinaitop.depot.business.model.BusinessDeliveryStorageNoticeExample.Criteria noticeCriteria = noticeExample.createCriteria();
        noticeCriteria.andPlanBidEqualTo(id);
        noticeCriteria.andAuditStateEqualTo(AuditType.AGREE.getAuditCode());
        noticeExample.setOrderByClause("agree_time asc");
        PageHelper.startPage(1, 1);
        List<BusinessDeliveryStorageNotice> noticeList = deliveryStorageNoticeService.queryByExample(noticeExample);
        if (noticeList != null && noticeList.size() > 0) {
            returnMap.put("noticeFirstDate", noticeList.get(0).getAgreeTime());
        }
        return returnMap;
    }


    /**
     * 已上报的轮换计划
     */
	@Override
	public List<BusinessPlan> queryReportByExample(Integer pageNum, Integer pageSize, Integer userId,
			BusinessPlan businessPlan) {
		// TODO Auto-generated method stub
		BusinessPlanExample example = new BusinessPlanExample();
        Criteria createCriteria = example.createCriteria();
        if (userId != null) {
            createCriteria.andCreaterIdEqualTo(userId);
        }
        createCriteria.andAuditStateEqualTo(String.valueOf(2));//审批结束的
        createCriteria.andExecuteTypeEqualTo(3156);//轮换计划
        //createCriteria.andInApplicationEqualTo(1);//启动中的
        //createCriteria.andCustomerPlanStateEqualTo(String.valueOf(10)); 
        //createCriteria.andCustomerPlanStateIsNotNull();//已上报
        // 计划编号.
        if (StringUtils.isNotBlank(businessPlan.getPlanNumber())){
            createCriteria.andPlanNumberLike("%"+businessPlan.getPlanNumber()+"%");
        }
        if(businessPlan.getInApplication()!=null){
        	createCriteria.andInApplicationEqualTo(businessPlan.getInApplication());
        }
        if(businessPlan.getCustomerPlanState()!=null){
        	List<String> list = new ArrayList<String>();
        	String[] status = businessPlan.getCustomerPlanState().split(",");
        	for (int i = 0; i < status.length; i++) {
				String string = status[i];
				list.add(string);
			}
        	createCriteria.andCustomerPlanStateIn(list);
        }
        
        PageHelper.startPage(pageNum, pageSize);
        
        List<BusinessPlan> selectByExample = planMapper.selectByExample(example);
        return selectByExample;
	}

    /**
     * 新增验收数据
     * @throws Exception
     */
    @Override
    @Transactional(readOnly = false)
    public Map<String, Object> addAcceptance(String businessPlanId,String acceptanceNumber, String businessPlancreateTime, String qualityIds, String fileIds) {
        Map<String, Object> returnMap = new HashMap<String, Object>();
        Map<String,String> map = new HashMap<String, String>();
        StringBuilder sql = new StringBuilder();
        StringBuilder sqlDetail = new StringBuilder();
        sql.append("(\""+ businessPlanId +"\",");
        sql.append("\""+ businessPlancreateTime +"\",");
        sql.append("\""+ acceptanceNumber +"\",");
        sql.append("\" 1 \")");
        map.put("sql",sql.toString());
        // 保存，返回id,id在流程启动时，需要使用.
        planMapper.addAcceptance(map);

        Integer acceptanceId = Integer.parseInt(ObjectUtils.toString(map.get("id")));
        if(ParameterUtil.isnotnull(acceptanceId)){
            for (String qualityId : qualityIds.split(",")) {
                sqlDetail.append("(\""+ acceptanceId +"\",");
                sqlDetail.append("\""+ qualityId +"\"),");
            }
            map.put("sqlDetail",sqlDetail.toString().substring(0,sqlDetail.toString().length()-1));
            planMapper.addAcceptanceDetail(map);
        }
        // 修改fileIds对应附件的bid为当前计划的id.
        businessFileService.updateFileByFileIdsAndBid(fileIds, acceptanceId,"acceptance");
        return returnMap;
    }

    public List<BusinessPlanAuditVO> getAcceptanceList(Map<String, Object> paramMap) {
        return planMapper.getAcceptanceList(paramMap);
    }
	

	//根据id修改计划状态
	@Override
	public void updatePlanStatus(BusinessPlan plan) {
		// TODO Auto-generated method stub
		planMapper.updateByPrimaryKeySelective(plan);
	}

	/**
	 * 验收列表
	 */
	@Override
	public List<Map<String, Object>> queryAcceptanceList(Map<String, Object> map) {
		// TODO Auto-generated method stub
		return planMapper.queryAcceptanceList(map);
	}


	@Override
	public List<Map<String, Object>> getjkqPlanList(Map<String, Object> map) {
		return planMapper.getjkqPlanList(map);
	}


	@Override
	public List<Map<String, Object>> getjkqPlanDayList(Map<String, Object> map) {
		return planMapper.getjkqDayList(map);
	}


	@Override
	public Integer getjybjl(Integer orgId) {
		// TODO Auto-generated method stub
		return planMapper.getjybjl(orgId);
	}


	@Override
	public Integer getzzbjl(Integer orgId) {
		// TODO Auto-generated method stub
		return planMapper.getzzbjl(orgId);
	}

}
