package com.chinaitop.depot.business.service.impl;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.act.service.ActTaskService;
import com.chinaitop.depot.agile.model.BusinessScedule;
import com.chinaitop.depot.business.controller.GrainNoticeController;
import com.chinaitop.depot.business.mapper.BusinessDeliveryStorageNoticeMapper;
import com.chinaitop.depot.business.model.BusinessApproval;
import com.chinaitop.depot.business.model.BusinessContract;
import com.chinaitop.depot.business.model.BusinessContractExample;
import com.chinaitop.depot.business.model.BusinessDeliveryStorageNotice;
import com.chinaitop.depot.business.model.BusinessDeliveryStorageNoticeExample;
import com.chinaitop.depot.business.model.BusinessDeliveryStorageNoticeExample.Criteria;
import com.chinaitop.depot.business.model.BusinessPlan;
import com.chinaitop.depot.business.model.BusinessStoreWareDetail;
import com.chinaitop.depot.business.model.UserInfo;
import com.chinaitop.depot.business.model.enums.AuditType;
import com.chinaitop.depot.business.model.enums.CrkNoticeStatus;
import com.chinaitop.depot.business.model.enums.StoreWareType;
import com.chinaitop.depot.business.model.enums.TaskType;
import com.chinaitop.depot.business.model.vo.BusinessDeliveryStorageNoticeAuditVO;
import com.chinaitop.depot.business.service.BusinessApprovalService;
import com.chinaitop.depot.business.service.BusinessContractService;
import com.chinaitop.depot.business.service.BusinessDeliveryStorageNoticeService;
import com.chinaitop.depot.business.service.BusinessPlanService;
import com.chinaitop.depot.business.service.BusinessStoreWareDetailService;
import com.chinaitop.depot.business.service.FeignAgileService;
import com.chinaitop.depot.business.service.GrainContractService;
import com.chinaitop.depot.business.service.GrainNoticeService;
import com.chinaitop.depot.business.service.UserInfoService;
import com.chinaitop.depot.system.service.SysCodeService;
import com.chinaitop.depot.utils.HTTPUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Maps;


@Service
public class GrainNoticeServiceImpl implements GrainNoticeService {
	public static final Logger logger = LoggerFactory.getLogger(GrainNoticeServiceImpl.class);
	@Resource
    private BusinessDeliveryStorageNoticeMapper businessDeliveryStorageNoticeMapper;
    @Resource
    private BusinessStoreWareDetailService businessStoreWareDetailService;
    
    @Resource
    private BusinessPlanService planService;
    @Resource
    private GrainContractService contractService;
    /*@Resource
    private BusinessSceduleService businessSceduleService;*/
    @Resource
    private ActTaskService actTaskService;
    @Resource
    private BusinessApprovalService businessApprovalService;
    @Resource
    private UserInfoService userInfoService;
    @Resource
	private FeignAgileService feignAgileService;
    @Resource
	private SysCodeService sysCodeService;
    
    // 入库通知单枚举id,出入库系统使用.
    private static Integer STORAGENOTICEENUMID = 2816;
    // 出库通知单枚举id,出入库系统使用.
    private static Integer DELIVERYNOTICEENUMID = 2818;

    public List<BusinessDeliveryStorageNotice> queryByExample(BusinessDeliveryStorageNoticeExample example) {
        return businessDeliveryStorageNoticeMapper.selectByExample(example);
    }

    
    public BusinessDeliveryStorageNotice findById(Integer id) {
        return businessDeliveryStorageNoticeMapper.selectByPrimaryKey(id);
    }

    /**
     * 新增，如果是出库，需要修改子表.
     */
    @Transactional(readOnly = false)
    public Map<String, Object> add(BusinessDeliveryStorageNotice businessDeliveryStorageNotice, 
            List<BusinessStoreWareDetail> storeWareDetailList, Integer userId, String name ,Integer orgId,Integer depotInfoOrgId) {
        Map<String, Object> returnMap = new HashMap<String, Object>();
     // 没有被变更.
        businessDeliveryStorageNotice.setChanging("0");
        // 不是变更数据.
        businessDeliveryStorageNotice.setChangeRecordFlag("0");
        // 设置审批状态为，审批待审批.
        businessDeliveryStorageNotice.setAuditState(AuditType.UNSUBMIT.getAuditCode());
        // 设置登录人信息.
        businessDeliveryStorageNotice.setBillDate(new Date());
        businessDeliveryStorageNotice.setCreateTime(new Date());
        businessDeliveryStorageNotice.setBillMan(name);
        businessDeliveryStorageNotice.setCreater(name);
        businessDeliveryStorageNotice.setCreaterId(userId);
        businessDeliveryStorageNotice.setOrgId(orgId);
        // businessDeliveryStorageNotice.setBillUnit("经营一部");
        
        // 启用状态设定为执行中，启用
        businessDeliveryStorageNotice.setInApplication(CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode());
        
        // 流程任务类型.
        TaskType taskType = null;
        try {
            if ("1".equals(businessDeliveryStorageNotice.getBillType())) {
                // 入库
                taskType = TaskType.storageNotice;
                // 入库通知单枚举id,出入库系统使用.
                businessDeliveryStorageNotice.setBillTypeId(STORAGENOTICEENUMID);
            } else if ("3".equals(businessDeliveryStorageNotice.getBillType())) {
                // 出库
                taskType = TaskType.deliveryNotice;
                // 出库通知单枚举id,出入库系统使用.
                businessDeliveryStorageNotice.setBillTypeId(DELIVERYNOTICEENUMID);
            } else {
                throw new RuntimeException("通知单类型有误，保存失败！");
            }
        } catch (Exception e) {
            e.printStackTrace();
            returnMap.put("error", e.getMessage());
            return returnMap;
        }
        
        // 保存.
        businessDeliveryStorageNoticeMapper.insert(businessDeliveryStorageNotice);

        returnMap.put("id",businessDeliveryStorageNotice.getId());
        
        // 启动流程
        String startProcess1 = actTaskService.startProcessNew(taskType.getTaskType(),
                businessDeliveryStorageNotice.getId().toString(), name, userId.toString(), depotInfoOrgId.toString());
        JSONObject jsStr = JSONObject.parseObject(startProcess1);
        String processInstanceId = jsStr.getString("processInstanceId");
        String processDefinitionId = jsStr.getString("processDefinitionId");
        // 设置流程实例id和流程定义id到合同中.
        // 设置流程实例id.
        businessDeliveryStorageNotice.setProcessInstanceId(processInstanceId);
        // 设置流程定义id.
        businessDeliveryStorageNotice.setProcessDefinitionId(processDefinitionId);
        // 设置最原始id，为当前id,以备变更使用.
        businessDeliveryStorageNotice.setRootNoticeBid(businessDeliveryStorageNotice.getId());
        // 修改.
        businessDeliveryStorageNoticeMapper.updateByPrimaryKey(businessDeliveryStorageNotice);
        
        // 新增仓房货位数量质量标准子表.
        if (storeWareDetailList != null && storeWareDetailList.size() > 0) {
            for (BusinessStoreWareDetail storeWareDetail : storeWareDetailList) {
                storeWareDetail.setZid(businessDeliveryStorageNotice.getId());
                storeWareDetail.setType(StoreWareType.notice.getValue());
                storeWareDetail.setCreateTime(new Date());
                if("1".equals(businessDeliveryStorageNotice.getBillType())){ //入库
                	storeWareDetail.setOutCount("");
                	storeWareDetail.setOutPrice("");
                	storeWareDetail.setOutRemainingNumber("");
                	storeWareDetail.setJstzdId(storeWareDetail.getId()+"");
                	storeWareDetail.setId(null);
                }
                if("3".equals(businessDeliveryStorageNotice.getBillType())){ //出库
                	storeWareDetail.setInCount("");
                	storeWareDetail.setInPrice("");
                	storeWareDetail.setRemainingNumber("");
                	if(null == storeWareDetail.getJstzdId()){//(选择合同的时候不做处理)
                		//创建出库通知单的时候选择的是接收通知单
                		storeWareDetail.setJstzdId(storeWareDetail.getId()+"");
                    	storeWareDetail.setId(null);
                	}
                }
                businessStoreWareDetailService.add(storeWareDetail);
            }
        }
        
        return returnMap;
    }

    /**
     * 修改，如果是出库，需要修改子表.
     */
    @Transactional(readOnly = false)
    public void update(BusinessDeliveryStorageNotice businessDeliveryStorageNotice,
            List<BusinessStoreWareDetail> storeWareDetailList) {
        businessDeliveryStorageNoticeMapper.updateByPrimaryKey(businessDeliveryStorageNotice);
        
        if (storeWareDetailList != null && storeWareDetailList.size() > 0) {
            for (BusinessStoreWareDetail storeWareDetail : storeWareDetailList) {
                storeWareDetail.setZid(businessDeliveryStorageNotice.getId());
                if (storeWareDetail.getId() == null) {
                    // 新增.
                    businessStoreWareDetailService.add(storeWareDetail);
                } else {
                    // 修改.
                    businessStoreWareDetailService.update(storeWareDetail);
                }
            }
        }
    }

    /**
     * 删除，主表和子表数据(出库).
     */
    public void remove(Integer id, String processInstanceId, String deleteReason,Integer planBid, Integer contractBid) {
//        if(planBid!=null && contractBid==null){
//        	//只引计划
//        	List<BusinessStoreWareDetail> list = businessStoreWareDetailService.getByZidAndType(planBid, "plan");
//        	List<BusinessStoreWareDetail> noticeList = businessStoreWareDetailService.getByZidAndType(id, "notice");
//        	for (BusinessStoreWareDetail businessStoreWareDetail : list) {
//        		for (BusinessStoreWareDetail noticeDetail : noticeList) {
//        			if(noticeDetail.getGrainAnnual().equals(businessStoreWareDetail.getGrainAnnual())  && 
//        					noticeDetail.getGrainAttribute().equals(businessStoreWareDetail.getGrainAttribute())  &&	
//        					noticeDetail.getGrainDetailKind().equals(businessStoreWareDetail.getGrainDetailKind())  &&
//        					noticeDetail.getGrainKind().equals(businessStoreWareDetail.getGrainKind())  &&
//        					noticeDetail.getGrainGrade().equals(businessStoreWareDetail.getGrainGrade())  &&
//        					noticeDetail.getGrainProducingArea().equals(businessStoreWareDetail.getGrainProducingArea())){
//        				
//        				if(noticeDetail.getInCount()!=null){
//        					Integer a = Integer.valueOf(noticeDetail.getInCount())+Integer.valueOf(businessStoreWareDetail.getRemainingNumber());
//        					businessStoreWareDetail.setRemainingNumber(String.valueOf(a));
//        				}
//        				if(noticeDetail.getOutCount()!=null){
//        					Integer b = Integer.valueOf(noticeDetail.getOutCount())+Integer.valueOf(businessStoreWareDetail.getOutRemainingNumber());
//        					businessStoreWareDetail.setOutRemainingNumber(String.valueOf(b));
//        				}
//        				int z = businessStoreWareDetailService.updateById(businessStoreWareDetail);
//        			}
//        			
//				}
//        	}
//        	
//        }else{
//        	//只引合同 或者 引合同有对应的计划
//        	List<BusinessStoreWareDetail> contractList = businessStoreWareDetailService.getByZidAndType(contractBid, "contract");
//        	List<BusinessStoreWareDetail> noticeList = businessStoreWareDetailService.getByZidAndType(id, "notice");
//
//        	for (BusinessStoreWareDetail businessStoreWareDetail : contractList) {
//        		for (BusinessStoreWareDetail noticeDetail : noticeList) {
//        			if(noticeDetail.getGrainAnnual().equals(businessStoreWareDetail.getGrainAnnual())  && 
//        					noticeDetail.getGrainAttribute().equals(businessStoreWareDetail.getGrainAttribute())  &&	
//        					noticeDetail.getGrainDetailKind().equals(businessStoreWareDetail.getGrainDetailKind())  &&
//        					noticeDetail.getGrainKind().equals(businessStoreWareDetail.getGrainKind())  &&
//        					noticeDetail.getGrainGrade().equals(businessStoreWareDetail.getGrainGrade())  &&
//        					noticeDetail.getGrainProducingArea().equals(businessStoreWareDetail.getGrainProducingArea())){
//        				
//        				if(noticeDetail.getInCount()!=null){
//        					Integer a = Integer.valueOf(noticeDetail.getInCount())+Integer.valueOf(businessStoreWareDetail.getRemainingNumber());
//        					businessStoreWareDetail.setRemainingNumber(String.valueOf(a));
//        				}
//        				if(noticeDetail.getOutCount()!=null){
//        					Integer b = Integer.valueOf(noticeDetail.getOutCount())+Integer.valueOf(businessStoreWareDetail.getOutRemainingNumber());
//        					businessStoreWareDetail.setOutRemainingNumber(String.valueOf(b));
//        				}
//        				int z = businessStoreWareDetailService.updateById(businessStoreWareDetail);
//        			}
//        			
//				}
//        	}
//
//        }
    	
    	// 删除主数据.
        businessDeliveryStorageNoticeMapper.deleteByPrimaryKey(id);
        
        // 删除明细列表数据.
        businessStoreWareDetailService.deleteByTypeAndZid(StoreWareType.notice.getValue(), id);
        
        // 删除工作流数据.
        try {
            if (deleteReason!=null) {
                actTaskService.delRunTask(processInstanceId, deleteReason);
            } else {
                deleteReason="删除原因无";
                actTaskService.delRunTask(processInstanceId, deleteReason);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // 提交
    @Transactional(readOnly = false)
    public Map<String, Object> submit(BusinessDeliveryStorageNotice businessDeliveryStorageNotice, String nextUserId, Integer userId,String realName) {
        /*// 当前登录人的id
        Integer userId = userInfo.getUserId();
        // 当前登录人的姓名
        String realName = userInfo.getRealName();*/
        Map<String, Object> returnMap = new HashMap<String, Object>();
        returnMap.put("success", "success");
        // 获取业务数据.
        BusinessDeliveryStorageNotice businessDeliveryStorageNoticeB = businessDeliveryStorageNoticeMapper.selectByPrimaryKey(businessDeliveryStorageNotice.getId());
        
        if (businessDeliveryStorageNoticeB == null) {
            throw new RuntimeException("没有找到相应的数据");
        }
        
        // 获取当前的审批状态，查看是否已经被提交.
        String auditState = businessDeliveryStorageNoticeB.getAuditState();
        if (StringUtils.isNotBlank(auditState) 
                && !auditState.equals(AuditType.UNSUBMIT.getAuditCode()) 
                && !auditState.equals(AuditType.REJECT.getAuditCode())) {
            returnMap.put("success", "error");
            returnMap.put("msg", "您已经提交该数据，无法再次提交！");
            return returnMap;
        }
        
        // 获取到当前进行到的任务节点.
//        Task singleResult = taskService.createTaskQuery().processInstanceId(businessDeliveryStorageNoticeB.getProcessInstanceId()).singleResult();
        String taskNode = actTaskService.taskNode(businessDeliveryStorageNoticeB.getProcessInstanceId());
        JSONObject taskNodeNew = JSONObject.parseObject(taskNode);
        String taskId = taskNodeNew.getString("taskId");
        String taskName = taskNodeNew.getString("taskName");
        
        // 修改流程.
        Map<String, Object> vars = Maps.newHashMap();
        vars.put("pass", "1");
        vars.put("inputUser", nextUserId);
        
        actTaskService.complete(taskId, businessDeliveryStorageNoticeB.getProcessInstanceId(), "", "", vars);

        // 设置审批状态为，审批中.
        businessDeliveryStorageNoticeB.setAuditState(AuditType.SUBMIT.getAuditCode());
        // 修改.
        businessDeliveryStorageNoticeMapper.updateByPrimaryKey(businessDeliveryStorageNoticeB);
        
        // 上一次流程结束，下一流程开始，当前流程.
//        singleResult = taskService.createTaskQuery().processInstanceId(businessDeliveryStorageNoticeB.getProcessInstanceId()).singleResult();
        
        // 流程任务类型.
        TaskType taskType = null;
        StringBuilder nameDeliveryStorageNotice=new StringBuilder();
        BusinessApproval approval = new BusinessApproval();
        
        if ("1".equals(businessDeliveryStorageNoticeB.getBillType())) {
            // 入库
            taskType = TaskType.storageNotice;
            
            nameDeliveryStorageNotice.append(TaskType.storageNotice.getTaskName());
            nameDeliveryStorageNotice.append(" 编号: ");
            nameDeliveryStorageNotice.append(businessDeliveryStorageNoticeB.getBillNumber());
            approval.setTaskTypeName(nameDeliveryStorageNotice.toString());
        } else if ("3".equals(businessDeliveryStorageNoticeB.getBillType())) {
            // 出库
            taskType = TaskType.deliveryNotice;
            nameDeliveryStorageNotice.append(TaskType.deliveryNotice.getTaskName());
            nameDeliveryStorageNotice.append(" 编号: ");
            nameDeliveryStorageNotice.append(businessDeliveryStorageNoticeB.getBillNumber());
            approval.setTaskTypeName(nameDeliveryStorageNotice.toString());
            
        }
        
        // 设置流程中间表.
        //下一环节名称
//        String taskName = singleResult.getName();
        //任务id
//        String taskId = singleResult.getId();
        
        // 设置业务id.
        approval.setProjectId(businessDeliveryStorageNoticeB.getId());
        approval.setProcessInstanceId(businessDeliveryStorageNoticeB.getProcessInstanceId());
        approval.setFromDepartment("经营一部");
        approval.setFromPeople(realName);
        approval.setFromUserName(realName);
        approval.setCreateTime(new Date());
        approval.setTaskType(taskType.getTaskType());
        approval.setProjectName(taskType.getTaskName());
        approval.setResult("待审批");
        approval.setIsHide(0);
        
        //保存申请人的信息,时间.
        approval.setApplyName(realName);
        approval.setApplyNameId(userId.toString());
        approval.setApplyTime(new Date());
        
        UserInfo userInfo2=userInfoService.findById(Integer.valueOf(nextUserId));
        // 审批人
        approval.setOperator(nextUserId);
        // 设置审批人的中文名称.
        approval.setOperatorName(userInfo2.getRealName());
        // 工作流,环节名称.
        approval.setTaskName(taskName);
        // 工作流,taskId.
        approval.setTaskId(taskId);
        
        businessApprovalService.add(approval);
        
        return returnMap;
    }
    
    /**
     * 获取审批列表.
     */
    public List<BusinessDeliveryStorageNoticeAuditVO> getApprovalList(Map<String, Object> paramMap) {
        return businessDeliveryStorageNoticeMapper.getApprovalList(paramMap);
    }


    /**
     * 审批
     * @param deliveryStorageNoticeId 出入库通知单id.
     * @param businessApproval 中间表.
     * @param nextUserId 下一个审批人.
     * @param result 审批结果.
     */
    @Transactional(readOnly = false)
    public void audit(Integer deliveryStorageNoticeId, BusinessApproval businessApproval, String nextUserId, String result ,String name) {
        /*//当前登录人的姓名
        String name=userInfo.getUsername();*/
        // 获取中间表数据.
        BusinessApproval oldBusinessApproval = businessApprovalService.findById(businessApproval.getId());
        // 检测是否并发审核.
        String oldResult = oldBusinessApproval.getResult();
        if ("同意".equals(oldResult) || "退回".equals(oldResult)) {
            throw new RuntimeException("该订单已经审核过，不能再次审核！");
        }
        
        String resultS = "";
        if ("0".equals(result)) {
            resultS = "退回";
        } else if ("1".equals(result)) {
            resultS = "同意";
        } else if ("2".equals(result)) {
            resultS = "驳回";
        } else if ("3".equals(result)) {
            resultS = "拒绝";
        }
        
        // 修改当前中间表.
        oldBusinessApproval.setResult(resultS);
        oldBusinessApproval.setContent(businessApproval.getContent());
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String nowF = sdf.format(new Date());
        oldBusinessApproval.setOperateTime(nowF);
        businessApprovalService.updateByPrimaryKeySelective(oldBusinessApproval);
        
        //得到当前的流程实例id
        String processInstanceId = businessApproval.getProcessInstanceId();
        // Task task = actTaskService.selectTask(processInstanceId);
        //String selectTaskId = actTaskService.selectTaskId(processInstanceId);

        String taskNode = actTaskService.taskNode(businessApproval.getProcessInstanceId());
        JSONObject taskNodeNew = JSONObject.parseObject(taskNode);
        String taskId = taskNodeNew.getString("taskId");
        String taskName = taskNodeNew.getString("taskName");
        // 修改流程.
        Map<String, Object> vars = Maps.newHashMap();
        vars.put("pass", result);
        if (result.equals("1")) {
            // 同意.
            vars.put("inputUser", nextUserId);
        } else if(result.equals("0")) {
            // 退回时，退给上一个人.
            String findHistoryTaskName = actTaskService.findHistoryTaskName(processInstanceId);
            vars.put("inputUser", findHistoryTaskName);
        } else if (result.equals("2")) {
            String findHistoryFirstTaskName = actTaskService.findHistoryFirstTaskName(processInstanceId);
            // 驳回环节(获取提交第一个人).
            vars.put("inputUser", findHistoryFirstTaskName);
        } else if (result.equals("3")) {
            // 拒绝.
        	/*String findHistoryTaskName = actTaskService.findHistoryTaskName(processInstanceId);
            vars.put("inputUser", findHistoryTaskName);*/
        } else {
            // 位置环节
            return; 
        }
        actTaskService.complete(taskId, processInstanceId, businessApproval.getContent(), "", vars);
        
        // 判断流程是否结束，processEnd为null，说明已经没有流程可以执行，说明流程结束.
//        ProcessInstance processEnd = actTaskService.isProcessEnd(processInstanceId);
        boolean processEnd = actTaskService.isProcessEndNew(processInstanceId);
        // 新增流程中间表.
//        Task singleResult = taskService.createTaskQuery().processInstanceId(businessApproval.getProcessInstanceId()).singleResult();
        
        BusinessDeliveryStorageNotice businessDeliveryStorageNoticeB = businessDeliveryStorageNoticeMapper.selectByPrimaryKey(deliveryStorageNoticeId);
        if (processEnd) {
        	
        	if ("1".equals(businessDeliveryStorageNoticeB.getChangeRecordFlag())) {
                // 类型是合同变更
                // 合同变更审批通过，需要将原合同的changing状态改为2，这样原合同就不能被再使用了.
                Integer originalNoticeBid = businessDeliveryStorageNoticeB.getOriginalNoticeBid();
                BusinessDeliveryStorageNotice noticeOriginal = businessDeliveryStorageNoticeMapper.selectByPrimaryKey(originalNoticeBid);
                noticeOriginal.setChanging("2");
                businessDeliveryStorageNoticeMapper.updateByPrimaryKey(noticeOriginal);
            }
        	
            // 审批结束.
            // 审批通过时间.
            businessDeliveryStorageNoticeB.setAgreeTime(new Date());
            if("3".equals(result)){
                businessDeliveryStorageNoticeB.setAuditState(AuditType.REFUSE.getAuditCode());
                if ("1".equals(businessDeliveryStorageNoticeB.getChangeRecordFlag())) {
                    // 类型是通知单变更
                    // 要修改原被变更的合同的状态为未被更改.
                	BusinessDeliveryStorageNotice noticeOld = businessDeliveryStorageNoticeMapper.selectByPrimaryKey(businessDeliveryStorageNoticeB.getOriginalNoticeBid());
                	noticeOld.setChanging("0");
                	businessDeliveryStorageNoticeMapper.updateByPrimaryKey(noticeOld);
                }
            }else if ("1".equals(result)) {
                businessDeliveryStorageNoticeB.setAuditState(AuditType.AGREE.getAuditCode());
            }
            // 修改业务数据的的审批状态为2.
            businessDeliveryStorageNoticeMapper.updateByPrimaryKey(businessDeliveryStorageNoticeB);


            BusinessApproval approval = new BusinessApproval();
            approval.setId(oldBusinessApproval.getId());
            approval.setTaskId(taskId);
            approval.setTaskName(taskName);
            businessApprovalService.updateByPrimaryKeySelective(approval);

        } else {
            // 流程未结束.
            if ("0".equals(result)) {
                //resultS = "退回";
            } else if ("2".equals(result)) {
                //resultS = "驳回";
                businessDeliveryStorageNoticeB.setAuditState(AuditType.REJECT.getAuditCode());
                businessDeliveryStorageNoticeMapper.updateByPrimaryKey(businessDeliveryStorageNoticeB);
            }else if("1".equals(result)){

            	
                // 下一环节名称.
//                String taskName = singleResult.getName();
                // 任务id.
//                String taskId = singleResult.getId();
                
                BusinessApproval approval = new BusinessApproval();
                // 设置业务id.
                approval.setProjectId(deliveryStorageNoticeId);
                // 工作流，流程实例id.
                approval.setProcessInstanceId(businessApproval.getProcessInstanceId());
                approval.setFromDepartment("经营一部");
                approval.setFromPeople(name);
                approval.setFromUserName(name);
                approval.setCreateTime(new Date());
                
                // 申请人的信息,时间.
                approval.setApplyName(oldBusinessApproval.getApplyName());
                approval.setApplyNameId(oldBusinessApproval.getApplyNameId());
                approval.setApplyTime(oldBusinessApproval.getCreateTime());
                StringBuilder nameDeliveryStorageNotice=new StringBuilder();
                // 流程任务类型.
                TaskType taskType = null;
                if ("1".equals(businessDeliveryStorageNoticeB.getBillType())) {
                    // 入库
                	if ("1".equals(businessDeliveryStorageNoticeB.getChangeRecordFlag())) {
                        // 类型是通知单变更
                        approval.setTaskType(TaskType.storageNoticeChange.getTaskType());// 任务类型
                        approval.setProjectName(TaskType.storageNoticeChange.getTaskName());// 任务名称.
                        nameDeliveryStorageNotice.append(TaskType.storageNoticeChange.getTaskName());
                        nameDeliveryStorageNotice.append(" 编号: ");
                        nameDeliveryStorageNotice.append(businessDeliveryStorageNoticeB.getBillNumber());
                        approval.setTaskTypeName(businessDeliveryStorageNoticeB.toString());
                    } else {
                    	// 入库
                        approval.setTaskType(TaskType.storageNotice.getTaskType());
                        approval.setProjectName(TaskType.storageNotice.getTaskName());
                        nameDeliveryStorageNotice.append(TaskType.storageNotice.getTaskName());
                        nameDeliveryStorageNotice.append(" 编号: ");
                        nameDeliveryStorageNotice.append(businessDeliveryStorageNoticeB.getBillNumber());
                        approval.setTaskTypeName(nameDeliveryStorageNotice.toString());
                        
                    }
                } else if ("3".equals(businessDeliveryStorageNoticeB.getBillType())) {
                    // 出库
                	if ("1".equals(businessDeliveryStorageNoticeB.getChangeRecordFlag())) {
                        // 类型是通知单变更
                        approval.setTaskType(TaskType.deliveryNoticeChange.getTaskType());
                        approval.setProjectName(TaskType.deliveryNoticeChange.getTaskName());
                        nameDeliveryStorageNotice.append(TaskType.deliveryNoticeChange.getTaskName());
                        nameDeliveryStorageNotice.append(" 编号: ");
                        nameDeliveryStorageNotice.append(businessDeliveryStorageNoticeB.getBillNumber());
                        approval.setTaskTypeName(businessDeliveryStorageNoticeB.toString());
                    } else {
                    	// 出库
                        approval.setTaskType(TaskType.deliveryNotice.getTaskType());
                        approval.setProjectName(TaskType.deliveryNotice.getTaskName());
                        nameDeliveryStorageNotice.append(TaskType.deliveryNotice.getTaskName());
                        nameDeliveryStorageNotice.append(" 编号: ");
                        nameDeliveryStorageNotice.append(businessDeliveryStorageNoticeB.getBillNumber());
                        approval.setTaskTypeName(nameDeliveryStorageNotice.toString());
                    }
                }
                
                // 审批状态.
                approval.setResult("待审批");
                // 是否隐藏.
                approval.setIsHide(0);
                // 审批人
                approval.setOperator(nextUserId);
                UserInfo userInfo2=userInfoService.findById(Integer.valueOf(nextUserId));
                // 设置审批人的中文名称.
                approval.setOperatorName(userInfo2.getRealName());
                // 工作流,环节名称.
                approval.setTaskName(taskName);
                // 工作流,taskId.
                approval.setTaskId(taskId);
                
                businessApprovalService.add(approval);
                
            }
            
        }
    }
    
    /**
     * 启用.
     * @param id
     */
    @Override
    @Transactional(readOnly = false)
    public Map<String, String> application(Integer id) {
        Map<String, String> returnMap = new HashMap<String, String>();
        BusinessDeliveryStorageNotice deliveryStorageNoticeY = businessDeliveryStorageNoticeMapper.selectByPrimaryKey(id);
        
        // 先进行判断，对应的计划，合同是否是启用的，只有启用的才可以修改.
        if (deliveryStorageNoticeY != null) {
            // 先判断对应的计划是否已经暂停或者终止.
            Integer planBid = deliveryStorageNoticeY.getPlanBid();
            if (planBid != null) {
                BusinessPlan plan = planService.findById(planBid);
                if (plan != null) {
                    Integer inApplication = plan.getInApplication();
                    // 计划的状态是不是启用.
                    if (inApplication != CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode()) {
                        // 不是启用
                        if (inApplication == CrkNoticeStatus.ZANTING.getCrkNoticeCode()) {
                            returnMap.put("error", "对应的计划已经暂停,无法启用该通知单！");
                        } else if (inApplication == CrkNoticeStatus.ZHONGZHI.getCrkNoticeCode()) {
                            returnMap.put("error", "对应的计划已经终止,无法启用该通知单！");
                        }
                        return returnMap;
                    }
                }
            }
            
            // 判断完计划，再判断相应的合同是否已经暂停或者终止.
            Integer rootContractBid = deliveryStorageNoticeY.getRootContractBid();
            if (rootContractBid != null) {
                // 判断合同，因为考虑到合同变更，所以以root_contract_bid 为查询条件，判断合同中root_contract_bid等于root_contract_bid值的最后一条数据.
                BusinessContractExample example = new BusinessContractExample();
                com.chinaitop.depot.business.model.BusinessContractExample.Criteria createCriteria = example.createCriteria();
                
                createCriteria.andRootContractBidEqualTo(rootContractBid);
                // 审批通过.
                createCriteria.andAuditStateEqualTo(AuditType.AGREE.getAuditCode());
                // 变更状态不是，已变更的合同.
                createCriteria.andChangingNotEqualTo("2");
                // 按照审批通过时间排序.
                example.setOrderByClause(" agree_time desc");
                // 分页，正常来说，只有一条.
                PageHelper.startPage(1, 1);
                List<BusinessContract> contractList = contractService.queryByExample(example);
                if (contractList != null && contractList.size() > 0) {
                    BusinessContract contract = contractList.get(0);
                    Integer inApplication = contract.getInApplication();
                    if (inApplication != CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode()) {
                        // 计划的状态不是启用.
                        if (inApplication == CrkNoticeStatus.ZANTING.getCrkNoticeCode()) {
                            returnMap.put("error", "对应的合同已经暂停,无法启用该通知单！");
                        } else if (inApplication == CrkNoticeStatus.ZHONGZHI.getCrkNoticeCode()) {
                            returnMap.put("error", "对应的合同已经终止,无法启用该通知单！");
                        }
                        return returnMap;
                    }
                }
            }
        }
        
        // 启用出入库通知单.
        BusinessDeliveryStorageNotice deliveryStorageNotice = new BusinessDeliveryStorageNotice();
        
        // 修改的内容.
        deliveryStorageNotice.setInApplication(CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode());
        deliveryStorageNotice.setCreateDepot("");
        // 查询条件.
        BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
        Criteria createCriteria = example.createCriteria();
        // id.
        createCriteria.andIdEqualTo(id);
        // 不能修改已经执行完毕的通知单的状态.
        createCriteria.andInApplicationNotEqualTo(CrkNoticeStatus.ZHIXINGWANBI.getCrkNoticeCode());
        
        businessDeliveryStorageNoticeMapper.updateByExampleSelective(deliveryStorageNotice, example);
        
        
        // 修改出入库系统通知单状态.
        /*Map<String,Object> paramMap = new HashMap<String, Object>();
        paramMap.put("tzdh", id);
        paramMap.put("FLOWSTATUS", CrkNoticeStatus.ZHIXINGZHONG.getCrkNoticeCode());
        // 修改出入库系统的 通知单状态.
        feignAgileService.updateNoticeStatus(paramMap);*/
         //serverApi.updateNoticeStatus(paramMap);
        return returnMap;
    }

    /**
     * 暂停.
     * @param id
     */
    @Override
    @Transactional(readOnly = false)
    public void discontinue(Integer id,String refuseReason) {
        // 暂停出入库通知单.
        BusinessDeliveryStorageNotice deliveryStorageNotice = new BusinessDeliveryStorageNotice();
        
        // 修改的内容.
        deliveryStorageNotice.setId(id);
        deliveryStorageNotice.setInApplication(CrkNoticeStatus.ZANTING.getCrkNoticeCode());
        deliveryStorageNotice.setCreateDepot(refuseReason);
        // 查询条件.
        BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
        Criteria createCriteria = example.createCriteria();
        // id.
        createCriteria.andIdEqualTo(id);
        // 不能修改已经执行完毕的通知单的状态.
        createCriteria.andInApplicationNotEqualTo(CrkNoticeStatus.ZHIXINGWANBI.getCrkNoticeCode());
        
        businessDeliveryStorageNoticeMapper.updateByExampleSelective(deliveryStorageNotice, example);
        
        // 修改出入库系统通知单状态.
        /*Map<String,Object> paramMap = new HashMap<String, Object>();
        paramMap.put("tzdh", id);
        paramMap.put("FLOWSTATUS", CrkNoticeStatus.ZANTING.getCrkNoticeCode());
        // 修改出入库系统的 通知单状态.
        feignAgileService.updateNoticeStatus(paramMap);*/
        //serverApi.updateNoticeStatus(paramMap);
    }
    
    /**
     * 终止.
     * @param id
     */
    @Override
    @Transactional(readOnly = false)
    public void finish(Integer id,String refuseReason) {
        // 终止出入库通知单.
        BusinessDeliveryStorageNotice deliveryStorageNotice = new BusinessDeliveryStorageNotice();
        
        // 修改的内容.
        deliveryStorageNotice.setId(id);
        deliveryStorageNotice.setInApplication(CrkNoticeStatus.ZHONGZHI.getCrkNoticeCode());
        deliveryStorageNotice.setCreateDepot(refuseReason);
        // 查询条件.
        BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
        Criteria createCriteria = example.createCriteria();
        // id.
        createCriteria.andIdEqualTo(id);
        // 不能修改已经执行完毕的通知单的状态.
        createCriteria.andInApplicationNotEqualTo(CrkNoticeStatus.ZHIXINGWANBI.getCrkNoticeCode());
        
        businessDeliveryStorageNoticeMapper.updateByExampleSelective(deliveryStorageNotice, example);
        
        
        // 修改出入库系统通知单状态.
        /*Map<String,Object> paramMap = new HashMap<String, Object>();
        paramMap.put("tzdh", id);
        paramMap.put("FLOWSTATUS", CrkNoticeStatus.ZHONGZHI.getCrkNoticeCode());
        // 修改出入库系统的 通知单状态.
        feignAgileService.updateNoticeStatus(paramMap);*/
        //serverApi.updateNoticeStatus(paramMap);
    }

    /**
     * 查看通知单执行进度.
     */
    @Override
    public PageInfo<BusinessDeliveryStorageNoticeAuditVO> getScheduleList(
            Integer pageNum, Integer pageSize,
            BusinessDeliveryStorageNotice deliveryStorageNotice) {
        // 可以查询所有人通过的通知单
        BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
        Criteria createCriteria = example.createCriteria();
        createCriteria.andAuditStateEqualTo(AuditType.AGREE.getAuditCode());
        if (deliveryStorageNotice != null) {
            // 凭证类型.
            if (StringUtils.isNotBlank(deliveryStorageNotice.getBillType())) {
                createCriteria.andBillTypeEqualTo(deliveryStorageNotice.getBillType());
            }
            
            // 通知单编号.
            if (StringUtils.isNotBlank(deliveryStorageNotice.getBillNumber())) {
                createCriteria.andBillNumberLike("%" + deliveryStorageNotice.getBillNumber() + "%");
            }
            
            // 计划id.
            if (deliveryStorageNotice.getPlanBid() != null) {
                createCriteria.andPlanBidEqualTo(deliveryStorageNotice.getPlanBid());
            }
            
            // 原始合同id.
            if (deliveryStorageNotice.getRootContractBid() != null) {
                createCriteria.andRootContractBidEqualTo(deliveryStorageNotice.getRootContractBid());
            }
        }
        
        PageInfo<BusinessDeliveryStorageNoticeAuditVO> pageInfo = new PageInfo<BusinessDeliveryStorageNoticeAuditVO>();
        // 审批通过时间倒序.
        example.setOrderByClause("agree_time desc");
        // 分页.
        PageHelper.startPage(pageNum, pageSize);
        List<BusinessDeliveryStorageNoticeAuditVO> noticeList = businessDeliveryStorageNoticeMapper.selectByExampleToVO(example);
        if (noticeList != null && noticeList.size() > 0) {
            // 拼接通知单id.
            StringBuffer sb = new StringBuffer();
            sb.append("(");
            for (BusinessDeliveryStorageNoticeAuditVO businessDeliveryStorageNoticeV : noticeList) {
                sb.append("\"" + businessDeliveryStorageNoticeV.getId() + "\"").append(",");
            }
            String substring = sb.substring(0, sb.length() - 1);
            sb = new StringBuffer(substring);
            sb.append(")");
            List<BusinessScedule> businessSceduleList = feignAgileService.getNoticeSceduleInNoticeId(sb.toString());
            //List<BusinessScedule> businessSceduleList = serverApi.getNoticeSceduleInNoticeId(sb.toString());
            // 通过通知单id,去出入库查询.
            if (businessSceduleList != null && businessSceduleList.size() > 0) {
                // 循环通知单list.
                for (BusinessDeliveryStorageNoticeAuditVO businessDeliveryStorageNoticeV : noticeList) {
                    // 循环出入库，查询的list.
                    Boolean flag = false;
                    for (BusinessScedule businessScedule : businessSceduleList) {
                        if (businessDeliveryStorageNoticeV.getId().toString().equals(businessScedule.getTzdId())) {
                            flag = true;
                            // 在出入库系统中，存在该通知单对应的出入库数据，进行计算，得出完成比率.
                            String crkLx = businessScedule.getCrkLx();
                            if ("1".equals(crkLx)) {
                                // 入库
                                // 入库数量.
                                String sumRksl = businessScedule.getSumRksl();
                                BigDecimal sumRkslD = new BigDecimal(sumRksl);
                                // 赋值：设置完成数量. stripTrailingZeros() 去除结尾的0
                                businessDeliveryStorageNoticeV.setCompleteAmount(sumRkslD.stripTrailingZeros().toPlainString());
                                // 通知单要求数量.
                                String demandAmount = businessDeliveryStorageNoticeV.getShipingCount();
                                BigDecimal demandAmountD = new BigDecimal(demandAmount);
                                
                                // 除法. 已完成数量/要求数量
                                BigDecimal complete = sumRkslD.divide(demandAmountD, 4, BigDecimal.ROUND_HALF_DOWN);
                                // 乘以100，得到的是百分比.
                                BigDecimal completeRate = complete.multiply(new BigDecimal("100"));
                                // 赋值：设置完成百分比.
                                businessDeliveryStorageNoticeV.setCompleteRate(completeRate.stripTrailingZeros().toPlainString());
                            } else if ("3".equals(crkLx)) {
                                // 出库
                                // 出库数量.
                                String sumCksl = businessScedule.getSumCksl();
                                BigDecimal sumCkslD = new BigDecimal(sumCksl);
                                
                                // 赋值：设置完成数量. stripTrailingZeros() 去除结尾的0
                                businessDeliveryStorageNoticeV.setCompleteAmount(sumCkslD.stripTrailingZeros().toPlainString());
                                
                                // 通知单要求数量.
                                String demandAmount = businessDeliveryStorageNoticeV.getShipingCount();
                                BigDecimal demandAmountD = new BigDecimal(demandAmount);
                                
                                // 除法. 已完成数量/要求数量
                                BigDecimal complete = sumCkslD.divide(demandAmountD, 4, BigDecimal.ROUND_HALF_DOWN);
                                // 乘以100，得到的是百分比.
                                BigDecimal completeRate = complete.multiply(new BigDecimal("100"));
                                // 赋值：设置完成百分比.
                                businessDeliveryStorageNoticeV.setCompleteRate(completeRate.stripTrailingZeros().toPlainString());
                            }
                            // 在出入库list中删除该数据，这样下次循环数量就会减少了.
                            businessSceduleList.remove(businessScedule);
                            // 跳出当前的出入库循环.
                            break;
                        }
                    }
                    if (!flag) {
                        // 进行到这里，说明出入库系统中，不存在该通知单对应的出入库数据. 设置完成数量，和完成比例为0.
                        businessDeliveryStorageNoticeV.setCompleteAmount("0");
                        businessDeliveryStorageNoticeV.setCompleteRate("0");
                    }
                }
            } else {
                // 没有找到任何出入库数据. 设置完成数量，和完成比例为0.
                for (BusinessDeliveryStorageNoticeAuditVO businessDeliveryStorageNoticeV : noticeList) {
                    businessDeliveryStorageNoticeV.setCompleteAmount("0");
                    businessDeliveryStorageNoticeV.setCompleteRate("0");
                }
            }
            pageInfo = new PageInfo<BusinessDeliveryStorageNoticeAuditVO>(noticeList);
        }
        return pageInfo;
    }


    @Override
    public List<BusinessDeliveryStorageNotice> queryByExample(Integer pageNum,
            Integer pageSize, Integer userId,BusinessDeliveryStorageNotice notice, String order,String noticeType) {
        BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
        Criteria createCriteria = example.createCriteria();
        Criteria createCriteria1 = example.createCriteria();
         //储备粮管理(create_unit=1)
        createCriteria.andCreateUnitEqualTo(1+"");
        createCriteria1.andCreateUnitEqualTo(1+"");
        // 用户id.
        if (userId != null) {
            createCriteria.andCreaterIdEqualTo(userId);
            createCriteria1.andCreaterIdEqualTo(userId);
        }
        if (notice != null) {
            // 凭证类型.
            if (StringUtils.isNotBlank(notice.getBillType())) {
                createCriteria.andBillTypeEqualTo(notice.getBillType());
                createCriteria1.andBillTypeEqualTo(notice.getBillType());
            }
            
            // 通知单编号.
            if (StringUtils.isNotBlank(notice.getBillNumber())) {
                createCriteria.andBillNumberLike("%" + notice.getBillNumber() + "%");
                createCriteria1.andBillNumberLike("%" + notice.getBillNumber() + "%");
            }
            
            // 计划id.
            if (notice.getPlanBid() != null) {
                createCriteria.andPlanBidEqualTo(notice.getPlanBid());
                createCriteria1.andPlanBidEqualTo(notice.getPlanBid());
            }
            // 合同id.
            if (notice.getContractBid() != null) {
                createCriteria.andContractBidEqualTo(notice.getContractBid());
                createCriteria1.andContractBidEqualTo(notice.getContractBid());
            }
            // 客户名称.
            if (StringUtils.isNotBlank(notice.getDeliveryCustomer())) {
                createCriteria.andDeliveryCustomerLike("%" + notice.getDeliveryCustomer() + "%");
                createCriteria1.andDeliveryCustomerLike("%" + notice.getDeliveryCustomer() + "%");
            }
            
            // 计划审批状态.
            if (StringUtils.isNotBlank(notice.getAuditState())){
                createCriteria.andAuditStateEqualTo(notice.getAuditState());
                createCriteria1.andAuditStateEqualTo(notice.getAuditState());
            }
            
            //主库还是代储库的通知单
            if(notice.getOrgId()!=null){
            		createCriteria.andOrgIdEqualTo(notice.getOrgId());
            		createCriteria1.andHouseIdEqualTo(notice.getOrgId());
            }
            
         // 判断是否是通知单变更.
            if (StringUtils.isNotBlank(noticeType) && "change".equals(noticeType)) {
                // 合同变更标志 为  1，也就是合同变更数据.
                //contract.setChangeRecordFlag("1");
                createCriteria.andChangeRecordFlagEqualTo("1");
                createCriteria1.andChangeRecordFlagEqualTo("1");
                // 变更状态为没有被变更成功. changing：0未变更； 1变更中； 2已变更；
                // 注：如果一条合同变更多次，只会显示最新一次的变更.
                createCriteria.andChangingNotEqualTo("2");
                createCriteria1.andChangingNotEqualTo("2");
            }else{ //未变更的合同(合同申请)
                createCriteria.andChangeRecordFlagEqualTo("0");
                createCriteria1.andChangeRecordFlagEqualTo("0");
            }
        }
        createCriteria.andBillTypeNotEqualTo("2");
        createCriteria1.andBillTypeNotEqualTo("2");
        example.or(createCriteria1);
        if (StringUtils.isNotBlank(order)) {
            example.setOrderByClause(order);
        }
        if(pageNum!=null && pageSize!=null){
            PageHelper.startPage(pageNum, pageSize);
        }
        List<BusinessDeliveryStorageNotice> list = businessDeliveryStorageNoticeMapper.selectByExample(example);
        return list;
    }
    
    

    @Override
    public void updateApplication(Map<String, Object> paramMap) {
        businessDeliveryStorageNoticeMapper.updateApplication(paramMap);
    }

    
    

	@Override
	public boolean isQuoteContract(Integer contract_id) {
		boolean flag = false;
		BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
		Criteria criteria = example.createCriteria();
		criteria.andContractBidEqualTo(contract_id);
		List<BusinessDeliveryStorageNotice> list = businessDeliveryStorageNoticeMapper.selectByExample(example);
		if (null != list && list.size() > 0) {
			flag = true;
		}
		return flag;
	}


	//储备粮通知单变更
		@Override
		public List<BusinessDeliveryStorageNotice> queryGrainNoticeChangeList(Integer userId, Integer pageNum,
				Integer pageSize) {
			        // TODO Auto-generated method stub
					BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
			        Criteria createCriteria = example.createCriteria();
			        // 审批通过的.
			        createCriteria.andAuditStateEqualTo(AuditType.AGREE.getAuditCode());
			        // 变更状态为没有被变更.
			        createCriteria.andChangingEqualTo("0");
			        // 只能变更自己申请的合同.
			        createCriteria.andCreaterIdEqualTo(userId);
			        /*// 没有被通知单引用的合同
			        if (null != contract_ids) {
			        	createCriteria.andIdNotIn(contract_ids);
			        }*/
			        // 倒序.
			        example.setOrderByClause(" id desc");
			       
			        if(pageNum!=null && pageSize!=null){
			        	PageHelper.startPage(pageNum, pageSize);
			        }
			        
			        //储备粮
			        //createCriteria.andCreateUnitIsNotNull();
			        createCriteria.andCreateUnitEqualTo(1+"");
			        List<BusinessDeliveryStorageNotice> list = businessDeliveryStorageNoticeMapper.selectByExample(example);

					return list;
		}
		
		
		@Override
		public Map<String, Object> saveChange(BusinessDeliveryStorageNotice businessDeliveryStorageNotice,
				List<BusinessStoreWareDetail> storeWareDetailList, Integer userId, String name, Integer depotInfoOrgId,
				Integer id) {


	        Map<String, Object> returnMap = new HashMap<String, Object>();
	        // 修改原合同变更标志  change_record_flag  设置为1
	        BusinessDeliveryStorageNotice noticeOld = businessDeliveryStorageNoticeMapper.selectByPrimaryKey(id);
	        //获取当前登录人
	        noticeOld.setCreater(name);
	        noticeOld.setCreaterId(userId);
	        // 设置正在被变种中，不能再被变更.
	        noticeOld.setChanging("1");
	        businessDeliveryStorageNoticeMapper.updateByPrimaryKey(noticeOld);
	        
	        // 设置id为空.
	        businessDeliveryStorageNotice.setId(null);
	        // 设置原合同id.
	        businessDeliveryStorageNotice.setOriginalNoticeBid(noticeOld.getId());
	        // 设置原合同编号.
	        businessDeliveryStorageNotice.setOriginalNotice(noticeOld.getBillNumber());
	        // 设置最原始合同id.
	        businessDeliveryStorageNotice.setRootContractBid(noticeOld.getRootNoticeBid());
	        // 设置合同变更记录标志为，是原合同的变更记录.
	        businessDeliveryStorageNotice.setChangeRecordFlag("1");
	        // 设置变更时间.
	        businessDeliveryStorageNotice.setChangeTime(new Date());
	        // 设置创建时间.
	        businessDeliveryStorageNotice.setCreateTime(new Date());
	        // 设置审批状态为，审批待审批.
	        businessDeliveryStorageNotice.setAuditState(AuditType.UNSUBMIT.getAuditCode());
	        
	        // 流程任务类型.
	        TaskType taskType = null;
	        try {
	            if ("1".equals(businessDeliveryStorageNotice.getBillType())) {
	                // 入库
	                taskType = TaskType.storageNotice;
	                // 入库通知单枚举id,出入库系统使用.
	                businessDeliveryStorageNotice.setBillTypeId(STORAGENOTICEENUMID);
	            } else if ("3".equals(businessDeliveryStorageNotice.getBillType())) {
	                // 出库
	                taskType = TaskType.deliveryNotice;
	                // 出库通知单枚举id,出入库系统使用.
	                businessDeliveryStorageNotice.setBillTypeId(DELIVERYNOTICEENUMID);
	            } else {
	                throw new RuntimeException("通知单类型有误，保存失败！");
	            }
	        } catch (Exception e) {
	            e.printStackTrace();
	            returnMap.put("error", e.getMessage());
	            return returnMap;
	        }
	        
	        // 新增合同变更记录
	        businessDeliveryStorageNoticeMapper.insert(businessDeliveryStorageNotice);

	        returnMap.put("id",businessDeliveryStorageNotice.getId());

	        
//	        ProcessInstance processInstance = actTaskService.startProcess(TaskType.contract.getTaskType(), contract.getId().toString(), name, userId.toString(), depotInfo.getOrgId().toString());
	        String startProcess1 = actTaskService.startProcessNew(taskType.getTaskType(), businessDeliveryStorageNotice.getId().toString(), name, userId.toString(), depotInfoOrgId.toString());
	        JSONObject jsStr = JSONObject.parseObject(startProcess1);
	        String processInstanceId = jsStr.getString("processInstanceId");
	        String processDefinitionId = jsStr.getString("processDefinitionId");
	        // 设置流程实例id和流程定义id到合同中.
	        // 设置流程实例id.
	        businessDeliveryStorageNotice.setProcessInstanceId(processInstanceId);
	        // 设置流程定义id.
	        businessDeliveryStorageNotice.setProcessDefinitionId(processDefinitionId);
	        //  修改合同变更记录.
	        businessDeliveryStorageNoticeMapper.updateByPrimaryKey(businessDeliveryStorageNotice);
	        
	        // 新增仓房货位数量质量标准子表.
	        if (storeWareDetailList != null && storeWareDetailList.size() > 0) {
	            for (BusinessStoreWareDetail storeWareDetail : storeWareDetailList) {
	            	storeWareDetail.setId(null);
	                storeWareDetail.setZid(businessDeliveryStorageNotice.getId());
	                storeWareDetail.setType(StoreWareType.notice.getValue());
	                storeWareDetail.setCreateTime(new Date());
	                if("1".equals(businessDeliveryStorageNotice.getBillType())){ //入库
	                	storeWareDetail.setOutCount("");
	                	storeWareDetail.setOutPrice("");
	                	storeWareDetail.setOutRemainingNumber("");
	                }
	                if("3".equals(businessDeliveryStorageNotice.getBillType())){ //出库
	                	storeWareDetail.setInCount("");
	                	storeWareDetail.setInPrice("");
	                	storeWareDetail.setRemainingNumber("");
	                }
	                businessStoreWareDetailService.add(storeWareDetail);
	            }
	        }
	        
	        return returnMap;
	    
		}
		
		@Override
		public void changeNoticeRemove(Integer id, Integer originalNoticeBid, String processInstanceId,String deleteReason) {
				
			// 修改原合同为，为未被更改.  originalContractBid(原合同id)
			BusinessDeliveryStorageNotice noticeOld = businessDeliveryStorageNoticeMapper.selectByPrimaryKey(originalNoticeBid);
			noticeOld.setChanging("0");
			businessDeliveryStorageNoticeMapper.updateByPrimaryKey(noticeOld);
	        
	        
			// TODO Auto-generated method stub
			// 删除主数据.
	        businessDeliveryStorageNoticeMapper.deleteByPrimaryKey(id);
	        
	        // 删除明细列表数据.
	        businessStoreWareDetailService.deleteByTypeAndZid(StoreWareType.notice.getValue(), id);
	        
	        // 删除工作流数据.
	        try {
	            if (deleteReason!=null) {
	                actTaskService.delRunTask(processInstanceId, deleteReason);
	            } else {
	                deleteReason="删除原因无";
	                actTaskService.delRunTask(processInstanceId, deleteReason);
	            }
	        } catch (Exception e) {
	            e.printStackTrace();
	        }
		}


		@Override
		public List<BusinessDeliveryStorageNotice> getChangeDetailList(Integer id, String type) {
			// TODO Auto-generated method stub
			BusinessDeliveryStorageNotice notice = businessDeliveryStorageNoticeMapper.selectByPrimaryKey(id);
	        // 获取初始合同id.
	        Integer rootNoticetBid = notice.getRootNoticeBid();
	        if (rootNoticetBid != null) {
	        	BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
	            Criteria createCriteria = example.createCriteria();
	            
	            // 如果是进入新增 修改合同页面，就不显示当前id对应的数据.
	            if (!"add".equals(type)) {
	                createCriteria.andIdNotEqualTo(id);
	            }
	            createCriteria.andRootNoticeBidEqualTo(rootNoticetBid);
	            // 只显示变更合同.
	            createCriteria.andChangeRecordFlagEqualTo("1");
	            // 只显示审批通过的.
	            createCriteria.andAuditStateEqualTo("2");
	            // 按照修改时间排序.
	            example.setOrderByClause("change_time");
	            return businessDeliveryStorageNoticeMapper.selectByExample(example);
	        }
	        return null;
		}


		@Override
		public List<BusinessDeliveryStorageNotice> getAuditPassList(Integer pageNum, Integer pageSize,
				BusinessDeliveryStorageNotice notice) {
			// TODO Auto-generated method stub
			BusinessDeliveryStorageNoticeExample example = new BusinessDeliveryStorageNoticeExample();
            Criteria createCriteria = example.createCriteria();
            // 审批通过.
            createCriteria.andAuditStateEqualTo(AuditType.AGREE.getAuditCode());
            // 变更状态不是，已变更的通知单.
            createCriteria.andChangingNotEqualTo("2");
            if (notice != null) {
                // 凭证类型.
                if (StringUtils.isNotBlank(notice.getBillType())) {
                    createCriteria.andBillTypeEqualTo(notice.getBillType());
                }
                
                // 通知单编号.
                if (StringUtils.isNotBlank(notice.getBillNumber())) {
                    createCriteria.andBillNumberLike("%" + notice.getBillNumber() + "%");
                }
                
                // 计划id.
                if (notice.getPlanBid() != null) {
                    createCriteria.andPlanBidEqualTo(notice.getPlanBid());
                }
                
                // 原始合同id.
                if (notice.getRootContractBid() != null) {
                    createCriteria.andRootContractBidEqualTo(notice.getRootContractBid());
                }
                
                //单位id
                if(notice.getOrgId() != null){
                	createCriteria.andOrgIdEqualTo(notice.getOrgId());
                }
            }
            // 按照审批通过时间排序.
            example.setOrderByClause(" agree_time desc");
            // 分页.
            if(pageNum!=null && pageSize!=null){
            	PageHelper.startPage(pageNum, pageSize);
            }
            
            createCriteria.andCreateUnitEqualTo("1");  //储备粮合同
            List<BusinessDeliveryStorageNotice> list = businessDeliveryStorageNoticeMapper.selectByExample(example);
            return list;
		}


		@Override
		public String pushCktzdts(String fsr, String fsdw) throws Exception {
			String path = sysCodeService.getSVal(10, "cktzdts");
			logger.info("查到的配置接口数据是："+path);
			String result = null;
			if (StringUtils.isNotBlank(path)) {
				//组装请求参数
				Map<String, String> params = new HashMap<>();
				params.put("fsr", fsr);
				params.put("fsdw", fsdw);

				logger.info("给给市平台发送出库通知单提示信息参数是："+params.toString());
				logger.info("给给市平台发送出库通知单提示信息地址是："+path);

				//给市平台发送请求
				result = HTTPUtils.doPost(path, params);

				//保存推送合同引用状态的消息和返回结果
				//systemOutsideDataService.addlogger("cktzdts", params.toString(), result);
			}
			return result;
		}
    
}
