package com.chinaitop.depot.business.service.impl;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.business.mapper.BusinessMovelibraryMapper;
import com.chinaitop.depot.business.mapper.BusinessMovelibrarySonMapper;
import com.chinaitop.depot.business.model.BusinessMovelibrary;
import com.chinaitop.depot.business.model.BusinessMovelibrarySon;
import com.chinaitop.depot.business.service.FeignAgileService;
import com.chinaitop.depot.business.service.FeignBasicService;
import com.chinaitop.depot.business.service.MoveLibraryNoticeService;
import com.chinaitop.depot.business.service.SystemOutsideDataService;
import com.github.pagehelper.PageHelper;

@Service
public class MoveLibraryNoticeServiceImpl implements MoveLibraryNoticeService {

	@Resource
    private BusinessMovelibraryMapper businessReceiveMovelibraryMapper;
	@Resource
	private BusinessMovelibrarySonMapper businessMovelibrarySonMapper;
	@Resource
	private FeignBasicService feignBasicService;
	@Resource
	private FeignAgileService feignAgileService;
	
	@Resource
	private SystemOutsideDataService systemOutsideDataService;
	
	@Override
	public List<BusinessMovelibrary> queryByExample(Integer pageNum, Integer pageSize,
			BusinessMovelibrary businessReceiveMovelibrary) {
		// TODO Auto-generated method stub
		
		 Map<String, Object> map = new HashMap<String, Object>();
		 if(businessReceiveMovelibrary.getNoticeNumber()!=null && businessReceiveMovelibrary.getNoticeNumber()!=""){
			 map.put("noticeNumber", businessReceiveMovelibrary.getNoticeNumber());
		 }
		 if(businessReceiveMovelibrary.getNoticeType()!=null){
			 map.put("noticeType", businessReceiveMovelibrary.getNoticeType());
		 }
		 if(businessReceiveMovelibrary.getOrgId()!=null){
			 map.put("orgId", businessReceiveMovelibrary.getOrgId());
		 }
		 if(businessReceiveMovelibrary.getIsUse()!=null){
			 map.put("isUse", businessReceiveMovelibrary.getIsUse());
		 }
        //分页
        if(pageNum!=null && pageSize!=null){
        	PageHelper.startPage(pageNum, pageSize);
        }
        
        List<BusinessMovelibrary> selectByExample = businessReceiveMovelibraryMapper.selectList(map);
        
        return selectByExample;
	}
	
	
	@Override
	public BusinessMovelibrary findById(Integer id) {
		// TODO Auto-generated method stub
		return businessReceiveMovelibraryMapper.selectByPrimaryKey(id);
	}


	/**
	 * 接收传过来的数据，转化格式，主表新增子表新增
	 * 子表新增时默认该仓房的通知单未使用
	 */
	@Override
	public Map<String, Object> add(String notice) {
		
        Map<String, Object> modelMap = new HashMap<String, Object>();
        try {
            
        	//接收数据转化格式
        	JSONObject jsonObject = JSON.parseObject(notice);
        	String noticeNumber = jsonObject.getString("yktzdbm");//通知单编号
        	String endDate = jsonObject.getString("ckqx");//出库期限
        	String jhsl = jsonObject.getString("jhsl");//计划数量（吨）
        	String fhdwcfbm = jsonObject.getString("fhdwcfbm");//发货单位仓房编码
        	String shdwcfbm = jsonObject.getString("shdwcfbm");//收货单位仓房编码
        	
        	if(null != fhdwcfbm && null != shdwcfbm){
            	//主表
            	BusinessMovelibrary businessReceiveMovelibrary = new BusinessMovelibrary();
            	businessReceiveMovelibrary.setNoticeNumber(noticeNumber); //接收通知单编号
            	businessReceiveMovelibrary.setIsQuality(0);//不需要质检
            	businessReceiveMovelibrary.setPlanCount(jhsl);//计划数量
            	businessReceiveMovelibrary.setNoticeType(5);//出库(主表是出库的数据，从表是入库)
            	businessReceiveMovelibrary.setIsUse(0); //未使用
            	//businessReceiveMovelibrary.setPlanNumber(planNumber);
            	//businessReceiveMovelibrary.setContractNumber(contractNumber);
            	SimpleDateFormat ft = new SimpleDateFormat("yyyy-MM-dd");
                Date enddate = ft.parse(endDate);
                businessReceiveMovelibrary.setEndDate(enddate);; //出库期限(日期格式)
                Date date = new Date();
                businessReceiveMovelibrary.setReceiveDate(date);//receive_date 接收日期

                //仓房houseId 货位warehouseId 粮食品种grainKind 明细品种 grain_detail_kind 等级grain_grade
                 //根据仓房编码查询仓房、货位、组织机构数据(发货单位) 
                  Map<String, Object> wareDataMap = feignBasicService.getWareDataByHwbm(fhdwcfbm);
                  Integer houseId = (Integer) wareDataMap.get("houseId");
                  String storehouseName = (String) wareDataMap.get("storehouseName");
                  Integer wareHouseId = (Integer) wareDataMap.get("wareHouseId");
                  String warehouseName =  (String) wareDataMap.get("warehouseName");
                  Integer orgId = (Integer) wareDataMap.get("orgId");
                  String  transUnitName = (String) wareDataMap.get("transUnitName");
                  businessReceiveMovelibrary.setHouseId(houseId); //仓房
                  businessReceiveMovelibrary.setWarehouseId(wareHouseId); //货位
                  businessReceiveMovelibrary.setHouseName(storehouseName);//仓房名称
                  businessReceiveMovelibrary.setWarehouseName(warehouseName);; //货位名称
                  businessReceiveMovelibrary.setOrgId(orgId);  //组织id
                  businessReceiveMovelibrary.setUnitId(orgId); //发货单位id
                  businessReceiveMovelibrary.setUnitName(transUnitName);//发货单位name
                  
                  //根据仓房、货位、orgId获取粮食品种等信息(库存获取)
                  Map<String, Object> map = feignAgileService.getDateByChHwh(orgId, houseId, wareHouseId);
                  Integer grainAttribute = (Integer) map.get("hwxz"); //粮油性质
                  Integer grainKind = (Integer) map.get("pz"); //品种
                  Integer grainDetailKind = (Integer) map.get("mxpz"); //明细品种
                  Integer grainGrade = (Integer) map.get("dj"); //等级
                  businessReceiveMovelibrary.setGrainKind(grainKind); //品种
                  businessReceiveMovelibrary.setGrainDetailKind(grainDetailKind); //明细品种
                  businessReceiveMovelibrary.setGrainGrade(grainGrade); //等级
                  businessReceiveMovelibrary.setGrainAttribute(grainAttribute); //粮油性质
                  
                  //新增到主表
                  int a = businessReceiveMovelibraryMapper.insert(businessReceiveMovelibrary);
                  if(a>0){
                	//收货单位(子表)
                      if(shdwcfbm.contains(";")){
                  		String[] strArr = shdwcfbm.split(";");
                  		for (int i = 0; i < strArr.length; ++i){
                  			BusinessMovelibrarySon businessMovelibrarySon = new BusinessMovelibrarySon();
                  			Map<String, Object> sonMap = feignBasicService.getWareDataByHwbm(strArr[i]);
                              Integer houseIds = (Integer) sonMap.get("houseId");
                              Integer wareHouseIds = (Integer) sonMap.get("wareHouseId");
                              String storehouseNames = (String) sonMap.get("storehouseName");
                              String warehouseNames =  (String) sonMap.get("warehouseName");
                              Integer unitId = (Integer) sonMap.get("orgId");
                              String  unitName = (String) sonMap.get("transUnitName");
                              
                              businessMovelibrarySon.setHouseId(houseIds);
                              businessMovelibrarySon.setIsUse(0);//未使用
                              businessMovelibrarySon.setUnitId(unitId);
                              businessMovelibrarySon.setUnitName(unitName);
                              businessMovelibrarySon.setWarehouseId(wareHouseIds);
                              businessMovelibrarySon.setHouseName(storehouseNames);
                              businessMovelibrarySon.setWarehouseName(warehouseNames);
                              businessMovelibrarySon.setZid(businessReceiveMovelibrary.getId());
                              businessMovelibrarySon.setNoticeNumber(noticeNumber);
                              //businessMovelibrarySon.setContractNumber(contractNumber);
                             // businessMovelibrarySon.setPlanNumber(planNumber);
                              businessMovelibrarySon.setGrainKind(grainKind); //品种
                              businessMovelibrarySon.setGrainDetailKind(grainDetailKind); //明细品种
                              businessMovelibrarySon.setGrainGrade(grainGrade); //等级
                              businessMovelibrarySon.setGrainAttribute(grainAttribute); //粮油性质
                              businessMovelibrarySon.setIsQuality(0);//不需要质检
                              businessMovelibrarySon.setPlanCount(jhsl);//计划数量
                              businessMovelibrarySon.setNoticeType(4);//入库(主表是出库的数据，从表是入库)
                              businessMovelibrarySon.setEndDate(enddate);; //出库期限(日期格式)
                              businessMovelibrarySon.setReceiveDate(date);//receive_date 接收日期
                              businessMovelibrarySon.setOrgId(unitId);
                              businessMovelibrarySonMapper.insert(businessMovelibrarySon);
                  		}
                  	}else{
                  		BusinessMovelibrarySon businessMovelibrarySon = new BusinessMovelibrarySon();
              			Map<String, Object> sonMap = feignBasicService.getWareDataByHwbm(shdwcfbm);
              			Integer houseIds = (Integer) sonMap.get("houseId");
                         Integer wareHouseIds = (Integer) sonMap.get("wareHouseId");
                         String storehouseNames = (String) sonMap.get("storehouseName");
                         String warehouseNames =  (String) sonMap.get("warehouseName");
                         Integer unitId = (Integer) sonMap.get("orgId");
                         String  unitName = (String) sonMap.get("transUnitName");
                          businessMovelibrarySon.setHouseId(houseIds);
                          businessMovelibrarySon.setHouseName(storehouseNames);
                          businessMovelibrarySon.setIsUse(0);//未使用
                          businessMovelibrarySon.setUnitId(unitId);
                          businessMovelibrarySon.setUnitName(unitName);
                          businessMovelibrarySon.setWarehouseId(wareHouseIds);
                          businessMovelibrarySon.setWarehouseName(warehouseNames);
                          businessMovelibrarySon.setZid(businessReceiveMovelibrary.getId());
                          businessMovelibrarySon.setNoticeNumber(noticeNumber);
                         // businessMovelibrarySon.setContractNumber(contractNumber);
                          //businessMovelibrarySon.setPlanNumber(planNumber);
                          businessMovelibrarySon.setPlanCount(jhsl);
                          businessMovelibrarySon.setGrainKind(grainKind); //品种
                          businessMovelibrarySon.setGrainDetailKind(grainDetailKind); //明细品种
                          businessMovelibrarySon.setGrainGrade(grainGrade); //等级
                          businessMovelibrarySon.setGrainAttribute(grainAttribute); //粮油性质
                          businessMovelibrarySon.setIsQuality(0);//不需要质检
                          businessMovelibrarySon.setPlanCount(jhsl);//计划数量
                          businessMovelibrarySon.setNoticeType(4);//入库(主表是出库的数据，从表是入库)
                          businessMovelibrarySon.setEndDate(enddate);; //出库期限(日期格式)
                          businessMovelibrarySon.setReceiveDate(date);//receive_date 接收日期
                          businessMovelibrarySon.setOrgId(unitId);
                          businessMovelibrarySonMapper.insert(businessMovelibrarySon);
                  	}
                      modelMap.put("status", "success");
                  }else{
                	  modelMap.put("msg", "主表保存失败");
                  }
                  
        	}else{
        		modelMap.put("msg", "无发货单位或收货单位仓房编码");
        	}
        	//String noticeNumber = jsonObject.get("yktzdbm").toString(); //通知单编号
        	//String endDate = jsonObject.get("ckqx").toString(); //出库期限
        	//String jhsl = jsonObject.get("jhsl").toString(); //计划数量（吨）
        	/*BigDecimal s = new BigDecimal(1000);
			BigDecimal sumjhsl = new BigDecimal(jhsl);
			BigDecimal sumCkslss = sumjhsl.multiply(s);
			String planCount = sumCkslss.toString(); //吨转化为公斤
*/        	
        	//String contractNumber = jsonObject.get("htbh").toString(); //合同编号
        	//String planNumber = jsonObject.get("jhh").toString(); //计划号
        	//String isTest = jsonObject.get("isTest").toString(); //是否质检
        	//String fhdwcfbm = jsonObject.get("fhdwcfbm").toString(); //发货单位仓房编码
        	//String shdwcfbm = jsonObject.get("shdwcfbm").toString(); //收货单位仓房编码
        	systemOutsideDataService.addlogger("yktzd", notice, modelMap.toString());
        } catch (Exception e) {
            e.printStackTrace();
            modelMap.put("status", "error");
            modelMap.put("msg", "保存失败！");
            systemOutsideDataService.addlogger("yktzd", notice, modelMap.toString());
        } 
		
		return modelMap;
	}


	@Override
	public BusinessMovelibrary getSumPlanCount(Integer id) {
		// TODO Auto-generated method stub
		return businessReceiveMovelibraryMapper.getSumPlanCount(id);
	}


	@Override
	public String useCount(Integer id) {
		// TODO Auto-generated method stub
		return businessReceiveMovelibraryMapper.useCount(id);
	}


	@Override
	public void updatePlanCount(Map<String, Object> map) {
		// TODO Auto-generated method stub
		businessReceiveMovelibraryMapper.updatePlanCount(map);
	}


	@Override
	public Integer getIsnullCount(Integer id) {
		// TODO Auto-generated method stub
		return businessReceiveMovelibraryMapper.getIsnullCount(id);
	}


	@Override
	public void updateLastCount(Map<String, Object> map) {
		// TODO Auto-generated method stub
		businessReceiveMovelibraryMapper.updateLastCount(map);
	}


	@Override
	public void updateUseState(Integer moveLibraryId) {
		// TODO Auto-generated method stub
		businessReceiveMovelibraryMapper.updateUseState(moveLibraryId);
	}

	
	
	
	

}
