package com.chinaitop.depot.business.service.impl;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.business.mapper.BusinessNoticeReceiveMapper;
import com.chinaitop.depot.business.model.BusinessApproval;
import com.chinaitop.depot.business.model.BusinessContractReceive;
import com.chinaitop.depot.business.model.BusinessNoticeReceive;
import com.chinaitop.depot.business.model.BusinessNoticeReceiveExample;
import com.chinaitop.depot.business.model.BusinessNoticeReceiveExample.Criteria;
import com.chinaitop.depot.business.service.BusinessApprovalService;
import com.chinaitop.depot.business.service.BusinessContractReceiveService;
import com.chinaitop.depot.business.service.FeignAgileService;
import com.chinaitop.depot.business.service.FeignBasicService;
import com.chinaitop.depot.business.service.ReceiveNoticeService;
import com.chinaitop.depot.business.service.SystemOutsideDataService;
import com.chinaitop.depot.feignService.DepotSharedFeignService;
import com.chinaitop.depot.feignService.DepotSystemFeignService;
import com.chinaitop.depot.system.service.SysCodeService;
import com.chinaitop.depot.utils.HTTPUtils;
import com.chinaitop.depot.utils.ParameterUtil;
import com.chinaitop.depot.utils.createKC;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;

@Service
public class ReceiveNoticeServiceImpl implements ReceiveNoticeService {

	@Resource
    private BusinessNoticeReceiveMapper businessNoticeReceiveMapper;
	@Resource
	private FeignBasicService feignBasicService;
	@Resource
	private FeignAgileService feignAgileService;
	@Resource
	private DepotSystemFeignService depotSystemFeignService;
	@Resource
	private DepotSharedFeignService depotSharedFeignService;
	@Resource
    private BusinessApprovalService businessApprovalService;
	@Resource
	private BusinessContractReceiveService businessContractReceiveService;
	@Resource
	private SysCodeService sysCodeService;
	@Resource
	private SystemOutsideDataService systemOutsideDataService;

	public static final Logger logger = LoggerFactory.getLogger(ReceiveNoticeServiceImpl.class);
	
	// 获取ip地址
    @Value("${crk-webservice}")
    private String url;
    
	@Override
	public List<BusinessNoticeReceive> queryByExample(Integer pageNum, Integer pageSize,
			BusinessNoticeReceive businessNoticeReceive,String cktzd,String lyflag) {
		
		BusinessNoticeReceiveExample example = new BusinessNoticeReceiveExample();
        Criteria createCriteria = example.createCriteria();
        
        if (lyflag != null) {
        	if (lyflag.equals("0") ){//油
        		createCriteria.andHouseIdIsNotNull();
        		createCriteria.andWareHouseIdIsNull();
        	} else if (lyflag.equals("1")) {//粮
        		createCriteria.andHouseIdIsNotNull();
        		createCriteria.andWareHouseIdIsNotNull();
        	}
        } else {
        	//仓房
            if (businessNoticeReceive.getHouseId() != null) {
            	createCriteria.andHouseIdEqualTo(businessNoticeReceive.getHouseId());
            }
            //货位
            if (businessNoticeReceive.getWareHouseId() != null) {
            	createCriteria.andWareHouseIdEqualTo(businessNoticeReceive.getWareHouseId());
            }
        }
        
        //出入库类型
        if(businessNoticeReceive.getCrktype()!=null && businessNoticeReceive.getCrktype()!=""){
        	createCriteria.andCrktypeEqualTo(businessNoticeReceive.getCrktype());
        }
		//业务类型  竞买 销售 轮换
        if (cktzd!=null && cktzd.equals("0")) {
        	if (StringUtils.isNotBlank(businessNoticeReceive.getYwtype())) {
            	createCriteria.andYwtypeNotEqualTo(businessNoticeReceive.getYwtype());//查ywtype!=1的数据
            }
        } else {
        	if (businessNoticeReceive.getYwtype() != null && businessNoticeReceive.getYwtype()!=""){
            	createCriteria.andYwtypeEqualTo(businessNoticeReceive.getYwtype());
            }
        }
        
        //组织id
        if(businessNoticeReceive.getOrgId()!=null){
        	createCriteria.andOrgIdEqualTo(businessNoticeReceive.getOrgId());
        }
        //通知单编号
        if(businessNoticeReceive.getNoticeNumber()!=null && businessNoticeReceive.getNoticeNumber()!=""){
        	createCriteria.andNoticeNumberEqualTo(businessNoticeReceive.getNoticeNumber());
        }
        //品种
        if(businessNoticeReceive.getGrainKind()!=null){
        	createCriteria.andGrainKindEqualTo(businessNoticeReceive.getGrainKind());
        }
        //性质
        if(businessNoticeReceive.getGrainAttribute()!=null){
        	createCriteria.andGrainAttributeEqualTo(businessNoticeReceive.getGrainAttribute());
        }
        //等级
        if(businessNoticeReceive.getGrainGrade()!=null){
        	createCriteria.andGrainGradeEqualTo(businessNoticeReceive.getGrainGrade());
        }
        //审批通过的
        if(businessNoticeReceive.getAuditState() != null){
        	if(businessNoticeReceive.getAuditState().equals(14)){ //未审批结束的
        		createCriteria.andAuditStateNotEqualTo(6);//6是入库性质转变单审批结束的
        	}else{
        		createCriteria.andAuditStateEqualTo(businessNoticeReceive.getAuditState());
        	}
        }
        //是否是可以引用的通知单
        if(businessNoticeReceive.getIsNotice()!=null){
        	createCriteria.andIsNoticeEqualTo(businessNoticeReceive.getIsNotice());
        }
        if(businessNoticeReceive.getIsSheet()!=null){
        	createCriteria.andIsSheetEqualTo(businessNoticeReceive.getIsSheet());
        }

        if(pageNum!=null && pageSize!=null){
        	PageHelper.startPage(pageNum, pageSize);
        }
        example.setOrderByClause("receive_time desc");
        List<BusinessNoticeReceive> selectByExample = businessNoticeReceiveMapper.selectByExample(example);
        return selectByExample;
	}


	
	@Override
	public BusinessNoticeReceive findById(Integer id) {
		// TODO Auto-generated method stub
		return businessNoticeReceiveMapper.selectByPrimaryKey(id);
	}



	@Override
	public void update(BusinessNoticeReceive businessNoticeReceive) {
		// TODO Auto-generated method stub
		businessNoticeReceiveMapper.updateByPrimaryKeySelective(businessNoticeReceive);
	}



	@Override
	public Map<String, Object> receiveNotice(String notice) throws Exception {

        Map<String, Object> modelMap = new HashMap<String, Object>();

        if (notice != null) {
            // JSON字符串转对象
            JSONObject jsonObject = JSONObject.parseObject(notice);
            BusinessNoticeReceive businessNoticeReceive = new BusinessNoticeReceive();
            businessNoticeReceive.setNoticeNumber(jsonObject.getString("tzdid")); //接收通知单编号
            
            /**
             * 根据通知单编号查询是否存在 存在则更改出库期限时间
             * 否则 新增数据
             */

            BusinessNoticeReceiveExample example = new BusinessNoticeReceiveExample();
            Criteria createCriteria = example.createCriteria();
            createCriteria.andNoticeNumberEqualTo(jsonObject.getString("tzdid"));
            List<BusinessNoticeReceive> selectByExample = businessNoticeReceiveMapper.selectByExample(example);
            
            if(selectByExample.size()>0){//存在修改(只修改出库期限)
                BusinessNoticeReceive record = new BusinessNoticeReceive();
                record.setId(selectByExample.get(0).getId());
                String  lsyqsl = jsonObject.getString("lsyqsl");
                record.setCount(lsyqsl); //计划数量（吨）
                record.setOutgoingPeriod(jsonObject.getString("ckqx"));
            	int k = businessNoticeReceiveMapper.updateByPrimaryKeySelective(record);
            	if(k<0){
            		modelMap.put("msg", "没有通知单编号为"+jsonObject.getString("tzdid")+"的数据");
                    modelMap.put("status", "error");
            	}
            }else{
            	/**
            	 * 新增
            	 */
            	if(jsonObject.getString("tzdlx")!=null && jsonObject.getString("ywlx")!=null){
                    if(ObjectUtils.toString(jsonObject.getString("tzdlx")).equals("01")){ //出入库类型(01：出库 02:入库 03:出入库)
                        businessNoticeReceive.setCrktype("3"); //1入 3出
                        businessNoticeReceive.setForwardingUnit(jsonObject.getString("khssyqmc")); //发货单位
                    }else if (ObjectUtils.toString(jsonObject.getString("tzdlx")).equals("02")){
                        businessNoticeReceive.setCrktype("1"); //1入 3出
                        businessNoticeReceive.setReceiveUnit(jsonObject.getString("ssyqmc")); //收货单位
                    }else if(ObjectUtils.toString(jsonObject.getString("tzdlx")).equals("03")){
                        businessNoticeReceive.setCrktype("13"); //1入 3出
                        businessNoticeReceive.setForwardingUnit(jsonObject.getString("khssyqmc")); //发货单位
                    } else {
                    	modelMap.put("msg", "未识别的通知单类型");
                        modelMap.put("status", "error");
                        return modelMap;
                    }

                    //业务类型 (3:兜底,4:轮换=出库包干,6:竞买=入库竞价,7:销售=出库竞价  )
                    if (ObjectUtils.toString(jsonObject.getString("ywlx")).equals("6")) {  //业务类型 (6:竞买=入库竞价；7：销售=出库竞价  4：轮换=出库包干)
                        businessNoticeReceive.setYwtype("1"); //1收储=竞买=竞价 2 销售=竞价 3轮换=包干
                    } else if (ObjectUtils.toString(jsonObject.getString("ywlx")).equals("4")){//4：轮换=出库包干
                        businessNoticeReceive.setYwtype("3");
                    }else if (ObjectUtils.toString(jsonObject.getString("ywlx")).equals("7")){//7：销售=出库竞价
                        businessNoticeReceive.setYwtype("2");
                    } else if (ObjectUtils.toString(jsonObject.getString("ywlx")).equals("3")){//兜底
                        businessNoticeReceive.setYwtype("4");
                    } else {
                    	modelMap.put("msg", "未识别的业务类型");
                        modelMap.put("status", "error");
                        return modelMap;
                    }

                    businessNoticeReceive.setContractNumber(jsonObject.getString("sshtbm")); //协议编号/合同编号

                    String hwbm = jsonObject.getString("cfbm");
                    if(hwbm!=null){
                        Integer grainAttribute = 0 ;
                        Integer grainKind = 0;
                        //根据仓房编码查询仓房、货位、组织机构数据
                        Map<String, Object> wareDataMap = feignBasicService.getWareDataByHwbm(hwbm);
                        logger.info("根据值"+hwbm+"获取到的货位数据如下：");
            			logger.info(wareDataMap+"");
                        if (null == wareDataMap || 
                        		"0".equals(wareDataMap.get("houseId").toString()) || 
                        		"0".equals(wareDataMap.get("wareHouseId").toString()) || 
                        		"0".equals(wareDataMap.get("orgId").toString())) {

                            modelMap.put("msg", "根据值"+hwbm+"未获取到对应仓房货位信息，请核对数据");
                            modelMap.put("status", "error");
                            return modelMap;
                        } else {
                            Integer houseId = (Integer) wareDataMap.get("houseId");
                            Integer wareHouseId = (Integer) wareDataMap.get("wareHouseId");
                            Integer orgId = (Integer) wareDataMap.get("orgId");
                            businessNoticeReceive.setHouseId(houseId); //仓房
                            if (!wareHouseId.toString().equals(houseId.toString())) {//只有粮仓才保存货位，油罐不保存货位
                            	businessNoticeReceive.setWareHouseId(wareHouseId); //货位
                            }
                            businessNoticeReceive.setOrgId(orgId);  //组织id
                            if (ObjectUtils.toString(jsonObject.getString("tzdlx")).equals("02")) {//入库类型
                            	String pzbm = jsonObject.getString("pzbm");//品种编码
                                if (StringUtils.isNotBlank(pzbm)) {
                                    String lypz = null;
                                    if ("2130000".equals(pzbm) || "1410000".equals(pzbm)) {//原油
                        				logger.info("根据品种编码"+pzbm+"获取到的品种数据如下：");
                        				logger.info("原油对应的字典ID是：7144");
                        				lypz = "7144";
                        			} else {
                        				Map<String, Object> basicMap = feignBasicService.getIdByNameAndParentId(1061, null, pzbm);
                        				lypz = basicMap.get("enumId")+"";
                        				logger.info("根据"+pzbm+"获取到的品种数据如下：");
                        				logger.info(basicMap.toString());
                        			}
                        			if (StringUtils.isNotBlank(lypz)) {
                        				businessNoticeReceive.setGrainKind(Integer.parseInt(lypz));
                        			} else {
                        				modelMap.put("msg", "pzbm--"+pzbm+"--有误或不合理");
                                        modelMap.put("status", "error");
                                        return modelMap;
                        			}
                                } else {
                                	modelMap.put("msg", "pzbm不能为空");
                                    modelMap.put("status", "error");
                                    return modelMap;
                                }

                                String mxbm = jsonObject.getString("mxbm");//明细品种编码
                                if (StringUtils.isNotBlank(mxbm)) {
                                    String mxpz = null;
                                    if ("2130000".equals(mxbm) || "1410000".equals(mxbm)) {//原油下的明细下只用大豆原油
                        				logger.info("根据明细品种编码"+mxbm+"获取到的明细品种数据如下：");
                        				logger.info("原油明细品种对应的字典ID是：7145");
                        				mxpz = "7145";
                        			} else {
                        				Integer enumid = getEnumId(businessNoticeReceive.getGrainKind(), mxbm);
                        				mxpz = enumid+"";
                        				logger.info("根据"+mxbm+"获取到的明细品种数据是："+enumid);
                        			}
                        			if (StringUtils.isNotBlank(mxpz)) {
                        				businessNoticeReceive.setGrainDetailKind(Integer.parseInt(mxpz));
                        			} else {
                        				modelMap.put("msg", "mxbm--"+mxbm+"--有误或不合理");
                                        modelMap.put("status", "error");
                                        return modelMap;
                        			}
                                } else {
                                	modelMap.put("msg", "mxbm不能为空");
                                    modelMap.put("status", "error");
                                    return modelMap;
                                }
                                
                                String scndbm = jsonObject.getString("scnf");//生产年度
                                if (StringUtils.isNotBlank(mxbm)) {
                                	String scnf = null;
                                	Map<String, Object> basicMap = feignBasicService.getIdByNameAndParentId(1004, scndbm, null);
                                	scnf = basicMap.get("enumId")+"";
                    				logger.info("根据"+scndbm+"获取到的生产年份数据如下：");
                    				logger.info(basicMap.toString());
                    				if (StringUtils.isNotBlank(scnf)) {
                    					businessNoticeReceive.setProductiveYear(Integer.parseInt(scnf));//生产年份
                    					businessNoticeReceive.setGrainAnnual(Integer.parseInt(scnf));//收获年度
                    				}
                                }
                                businessNoticeReceive.setIsSheet(1);//入库类型的只有在市级企业端提交了入库验收申请，才会生成入库性质转变单
                                //入库兜底类型的不需要审批性质转变单
                                /*if (ObjectUtils.toString(jsonObject.getString("ywlx")).equals("3")) {
                                	businessNoticeReceive.setIsSheet(1);
                                } else {
                                	businessNoticeReceive.setIsSheet(0);
                                }*/
                                //businessNoticeReceive.setHzqlsxzdm(3052);//商品粮 转后,划转后粮食性质
                				businessNoticeReceive.setLsxzzbdh(jsonObject.getString("tzdid"));//粮食性质转变单单号
                				//SimpleDateFormat dateFormat= new SimpleDateFormat("yyyy-MM-dd :hh:mm:ss");
                				//businessNoticeReceive.setHzrq(new Date());//划转时间
                				//businessNoticeReceive.setIsSheet(0);//是性质转变单

                				String  lsyqsl = jsonObject.getString("lsyqsl");
                				businessNoticeReceive.setCount(lsyqsl); //计划数量（吨）
                				//businessNoticeReceive.setOutgoingPeriod(jsonObject.getString("ckqx")); //出库期限(日期格式)

                				businessNoticeReceive.setInApplication(1); //启用
                				businessNoticeReceive.setIsNotice(0);//0是可以引用的通知单
                				businessNoticeReceive.setAuditState(5);//下达未审批
                				businessNoticeReceive.setReceiveTime(new Date());//接收数据的时间
                				//businessNoticeReceive.setSfzswtzd("0"); //接收通知单菜单不展示该数据
                				String ytzdid = jsonObject.getString("ytzdid");
                				if (StringUtils.isNotBlank(ytzdid)) {//有值的话就代表是变更的通知单，不需要生成性质转变单
                					//businessNoticeReceive.setIsSheet(1);//这是属于通知单变更，不需要重新生成性质转变单
                					businessNoticeReceive.setYtzdid(ytzdid);
                				}
                				businessNoticeReceiveMapper.insert(businessNoticeReceive);
                            } else {//出库类型
                                //根据仓房、货位、orgId获取粮食品种等信息
                                Map<String, Object> map = feignAgileService.getDateByChHwh(orgId, houseId, wareHouseId);
                                logger.info("根据orgId="+orgId+"，houseId="+houseId+",warehouseId="+wareHouseId+"  获取到的库存数据如下：");
                    			logger.info(map+"");
                    			if (null != map && !"0".equals(map.get("hwxz").toString())) {
                    				grainAttribute = (Integer) map.get("hwxz"); //粮油性质
                    				grainKind = (Integer) map.get("pz"); //品种
                    				Integer grainDetailKind = (Integer) map.get("mxpz"); //明细品种
                    				Integer grainGrade = (Integer) map.get("dj"); //等级
                    				String inputTime = map.get("rq")+""; //入库时间
                    				Integer scnf = Integer.parseInt(map.get("scnf")+"");
                    				Double kcsl = (Double) map.get("kcsl");//千克
                    				Date date = null;
                    				if(inputTime!=null){
                    					date = ParameterUtil.string2datetime(inputTime);
                    					businessNoticeReceive.setInputTime(date);//入库时间
                    				}
                    				businessNoticeReceive.setProductiveYear(scnf);//生产年份
                    				businessNoticeReceive.setGrainAnnual(scnf);//收获年度
                    				
                    				Integer grainProducingArea = (Integer) map.get("gb"); //产地
                    				
                    				businessNoticeReceive.setGrainKind(grainKind); //品种
                    				businessNoticeReceive.setGrainDetailKind(grainDetailKind); //明细品种
                    				businessNoticeReceive.setGrainGrade(grainGrade); //等级
                    				businessNoticeReceive.setGrainAttribute(grainAttribute); //粮油性质
                    				businessNoticeReceive.setGrainProducingArea(grainProducingArea);//产地
                    				
                    				/**
                    				 * 库存数量转化为吨
                    				 */
                    				BigDecimal s = new BigDecimal(1000);
                    				BigDecimal kcslBig = new BigDecimal(kcsl);
                    				BigDecimal kcslKg = kcslBig.divide(s);
                    				
                    				businessNoticeReceive.setLssl(kcslKg.toString());//库存数量

                    				businessNoticeReceive.setHzqlsxzdm(3052);//商品粮 转后,划转后粮食性质
                    				businessNoticeReceive.setLsxzzbdh(jsonObject.getString("tzdid"));//粮食性质转变单单号
                    				//SimpleDateFormat dateFormat= new SimpleDateFormat("yyyy-MM-dd :hh:mm:ss");
                    				businessNoticeReceive.setHzrq(new Date());//划转时间
                    				businessNoticeReceive.setIsSheet(0);//是性质转变单

                    				String  lsyqsl = jsonObject.getString("lsyqsl");
                    				businessNoticeReceive.setCount(lsyqsl); //计划数量（吨）
                    				businessNoticeReceive.setOutgoingPeriod(jsonObject.getString("ckqx")); //出库期限(日期格式)

                    				businessNoticeReceive.setInApplication(1); //启用
                    				businessNoticeReceive.setIsNotice(0);//0是可以引用的通知单
                    				businessNoticeReceive.setAuditState(0);//下达未审批
                    				businessNoticeReceive.setReceiveTime(new Date());//接收数据的时间
                    				//businessNoticeReceive.setSfzswtzd("0"); //接收通知单菜单不展示该数据
                    				String ytzdid = jsonObject.getString("ytzdid");
                    				if (StringUtils.isNotBlank(ytzdid)) {//有值的话就代表是变更的通知单，不需要生成性质转变单
                    					businessNoticeReceive.setIsSheet(1);//这是属于通知单变更，不需要重新生成性质转变单
                    					businessNoticeReceive.setYtzdid(ytzdid);
                    					businessNoticeReceive.setAuditState(5);//状态设置为审批通过
                    				}
                    				businessNoticeReceiveMapper.insert(businessNoticeReceive);
                    			} else {
                    				modelMap.put("msg", "根据所传信息没有查到对应库存信息");
                    				modelMap.put("status", "error");
                    			}
                            }
                        }
                    } else {
                        modelMap.put("msg", "cfbm编码为空");
                        modelMap.put("status", "error");
                    }

                }else{
                    modelMap.put("msg", "通知单或者业务类型为空");
                    modelMap.put("status", "error");
                }
            }
            

        } else {
            modelMap.put("msg", "所传数据不能为空");
            modelMap.put("status", "error");
        }

		return modelMap;
	}

	/**
	 * 根据字典父级ID和国标编码获取下面的一个字典ID
	 * @param parentId
	 * @param gbCode
	 * @return
	 */
	private Integer getEnumId(int parentId, String gbCode) {
		Integer enumId = null;
		Map<String, Object> map = feignBasicService.findConditionEnum(parentId, null, null);
		if (null != map) {
			String json_str = JSON.toJSONString(map);
			JSONObject json_obj = JSONObject.parseObject(json_str);
			JSONArray array = JSONArray.parseArray(json_obj.getString("enumList"));
			for (int i = 0; i < array.size(); i++) {
				JSONObject object = JSONObject.parseObject(array.get(i).toString());
				String code = object.getString("gbcode");
				if (code.equals(gbCode)) {
					enumId = Integer.parseInt(object.getString("id"));
					break;
				}
			}
		}
		return enumId;
	}

	@Override
	public List<BusinessNoticeReceive> queryByExample(BusinessNoticeReceive businessNoticeReceive) {
		// TODO Auto-generated method stub
		
		
		BusinessNoticeReceiveExample example = new BusinessNoticeReceiveExample();
        Criteria createCriteria = example.createCriteria();
        
        //仓房
        if(businessNoticeReceive.getHouseId()!=null){
        	createCriteria.andHouseIdEqualTo(businessNoticeReceive.getHouseId());
        }
        //货位
        if(businessNoticeReceive.getWareHouseId()!=null){
        	createCriteria.andWareHouseIdEqualTo(businessNoticeReceive.getWareHouseId());
        }
        
        //货位
        if(businessNoticeReceive.getOrgId()!=null){
        	createCriteria.andOrgIdEqualTo(businessNoticeReceive.getOrgId());
        }
        if(businessNoticeReceive.getIsSheet()!=null){
        	createCriteria.andIsSheetEqualTo(businessNoticeReceive.getIsSheet());
        }
        example.setOrderByClause("receive_time desc");
        List<BusinessNoticeReceive> selectByExample = businessNoticeReceiveMapper.selectByExample(example);
        return selectByExample;
        
	}



	@Override
	public List<Map<String, String>> getUserRole(Integer userId) {
		// TODO Auto-generated method stub
		return businessNoticeReceiveMapper.getUserRole(userId);
	}



	@Override
	public void audit(BusinessNoticeReceive receiveNotice, Integer userId, String realName,String orgName,String content) {
		
        //businessApproval.setContent(jsonUserInfo.getString("realName") + "提交"); //提交人
        
		BusinessApproval businessApproval = new BusinessApproval();
		businessApproval.setOperateTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));//操作时间
        businessApproval.setOperator(userId+"");//已选择的审批人id
        businessApproval.setOperatorName(realName);//已选择的审批人姓名
        businessApproval.setCreateTime(new Date()); //创建时间
        businessApproval.setFromDepartment(orgName); //部门 orgName
    	businessApproval.setProjectName("性质转变单审批");//名称
        businessApproval.setTaskType("propertyTS");//任务类型
    	businessApproval.setResult("同意");//审批结果
    	businessApproval.setApplyTime(receiveNotice.getReceiveTime());//申请时间
        businessApproval.setTaskTypeName(receiveNotice.getLsxzzbdh());//任务类型名
        businessApproval.setIsHide(0);//
        businessApproval.setContent(content);//审批意见
        businessApproval.setProjectId(receiveNotice.getId());
        if(receiveNotice.getAuditState().equals(0)){//仓储部审批
        	//更改该条数据
        	receiveNotice.setCcshr(userId+"");
        	receiveNotice.setAuditState(1);
        	businessNoticeReceiveMapper.updateByPrimaryKeySelective(receiveNotice);
        	
        	//审批表数据的新增
            businessApproval.setTaskName("仓储部经理审批");//任务名称

        }else{
        	
        	if(receiveNotice.getAuditState().equals(1)){//质检审批
        		receiveNotice.setAuditState(2);
        		receiveNotice.setZjshr(userId+"");
        		businessApproval.setTaskName("质检部经理审批");//任务名称
        		
        	}else if(receiveNotice.getAuditState().equals(2)){//统计审批
        		receiveNotice.setAuditState(3);
        		receiveNotice.setTjshr(userId+"");
        		businessApproval.setTaskName("统计部经理审批");//任务名称
        		
        	}else if(receiveNotice.getAuditState().equals(3)){//会计审批
        		receiveNotice.setAuditState(4);
        		receiveNotice.setKjshr(userId+"");
        		businessApproval.setTaskName("会计部经理审批");//任务名称
        	}else if(receiveNotice.getAuditState().equals(4)){//库领导审批
        		receiveNotice.setAuditState(5);//审批完毕
        		receiveNotice.setLdshr(userId+"");
        		businessApproval.setTaskName("库领导审批");//任务名称
        	}

        	//修改本条数据
        	businessNoticeReceiveMapper.updateByPrimaryKeySelective(receiveNotice);

        }
        //增加审批表数据
    	businessApprovalService.add(businessApproval);

    	//必须是入库，库级和市级都审批完成，并且不是通知单，只是性质转变单
    	if ("5".equals(receiveNotice.getAuditState().toString()) 
    			&& "1".equals(receiveNotice.getIsNotice().toString()) 
    			&& "0".equals(receiveNotice.getIsSheet().toString()) 
    			&& "1".equals(receiveNotice.getCrktype())) {

    		retSptMeagess(receiveNotice);
    	}
	}

	@Override
	public void updateQualitycheckHwxz(BusinessNoticeReceive receiveNotice) {
		depotSharedFeignService.restQualitycheckHwxz(receiveNotice.getHouseId(), receiveNotice.getWareHouseId(), receiveNotice.getHzqlsxzdm().toString());
	}



	/**
	 * 给给市平台发送性质转变单审批完成的通知
	 * @param receiveNotice
	 */
	@Override
	public void retSptMeagess(BusinessNoticeReceive receiveNotice) {
		String result = "";

		//获取请求的URL地址
		try {
			String path = sysCodeService.getSVal(10, "xzzzxxts");
			System.out.println("查到的配置接口数据是："+path);
			if (null != receiveNotice.getCblx() && null != receiveNotice.getYsid()) {

				//组装请求参数
				Map<String, String> params = new HashMap<>();
				params.put("crklx", receiveNotice.getCblx());
				params.put("ysid", receiveNotice.getYsid());

				logger.info("给给市平台发送性质转变成功通知信息："+params.toString());
				logger.info("给给市平台发送性质转变成功通知信息地址是："+path);

				//给市平台发送请求
				result = HTTPUtils.doPost(path, params);

				//保存推送合同引用状态的消息和返回结果
				systemOutsideDataService.addlogger("xzzzxx_result", params.toString(), result);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * 调用出入库接口改变粮食性质
	 * @param id 性质转变单数据ID
	 * @return
	 */
	public boolean restLsxz(Integer id) {
		
		boolean flag = false;

		BusinessNoticeReceive receiveNotice = businessNoticeReceiveMapper.selectByPrimaryKey(id);
		Object[] parameters = new Object[11];
		parameters[0] = receiveNotice.getOrgId() + "";
		parameters[1] = receiveNotice.getGrainKind() + "";
		parameters[2] = receiveNotice.getGrainDetailKind()+"";//明细品种
		parameters[3] = receiveNotice.getHouseId() + "";
		if (receiveNotice.getWareHouseId() != null) {//粮仓
			parameters[4] = receiveNotice.getWareHouseId() + "";
		} else {
			parameters[4] = receiveNotice.getHouseId() + "";//油罐
		}
		parameters[5] = receiveNotice.getHzqlsxzdm() + "";//变更后粮油性质
		parameters[6] = receiveNotice.getGrainAttribute() + "";//变更前粮油性质
		/**
		 * 库存数量转化为公斤
		 */
		BigDecimal s = new BigDecimal(1000);
		BigDecimal kcslBig = new BigDecimal(receiveNotice.getLssl());
		BigDecimal kcslKg = kcslBig.multiply(s);
		parameters[7] = kcslKg.toString();//数量
		parameters[8] = receiveNotice.getLsxzzbdh(); //文件号(性质转变单号)
		parameters[9] = "入库性质转变单";//损益原因
		parameters[10] = 1 + ""; //1:新增；2：修改；3：删除

		String crkurl = depotSystemFeignService.getUrl(receiveNotice.getOrgId().toString(), "2");
	    if (StringUtils.isNotBlank(crkurl)) {
	    	String url_id = crkurl.substring(0, crkurl.lastIndexOf("Base")-1);
	    	StringBuffer sbf = new StringBuffer();
	    	sbf.append(url_id).append(url);
	    	String str = createKC.createKC("ChangeGrainAttribute", sbf.toString(), parameters);
	    	systemOutsideDataService.addlogger("rkxzzb", JSON.toJSONString(parameters), str);
	    	if ("true".equals(str)) {
	    		flag = true;
			}
	    }
	    return flag;
	}

	@Override
	public int queryIsOrAudit(String xybh, String sptDataId,Integer flag) {
		// TODO Auto-generated method stub
		String houseId = "0";
		if(flag.equals(0)){//油罐
			 houseId =feignBasicService.getTankIdBySptDataID(sptDataId);
		}else if(flag.equals(1)){//仓房
			 houseId = feignBasicService.getHouseIdBySptDataID(sptDataId);
		}
		
		//根据仓房或者油罐id 查询是否有为完成审批的性质转变单
		BusinessNoticeReceiveExample example = new BusinessNoticeReceiveExample();
        Criteria createCriteria = example.createCriteria();
        createCriteria.andHouseIdEqualTo(Integer.valueOf(houseId));
        createCriteria.andContractNumberEqualTo(xybh);
        createCriteria.andIsSheetEqualTo(0);
        example.setOrderByClause("receive_time desc");
        List<BusinessNoticeReceive> list = businessNoticeReceiveMapper.selectByExample(example);
		if(list.size()>0 && !list.get(0).getAuditState().equals(5)){
			//查询到有未审批的数据
			return 1;
		}else{
			//没有查询到数据 或者都审批通过 就直接保存
			return 0;
		}
		
	}

	@Override
	public Map<String, Object> updateReceiveNotice(String notice) throws Exception {
		Map<String, Object> map = new HashMap<>();

		BusinessNoticeReceive businessNoticeReceive = formatObject(notice);
		businessNoticeReceiveMapper.updateByPrimaryKeySelective(businessNoticeReceive);
		map.put("status", 200);
		map.put("msg", "操作成功");

		return map;
	}

	@Override
	public void pushHtStatus(String notice) throws Exception {

		String result = "";

		//获取请求的URL地址
		String path = sysCodeService.getSVal(10, "tshtyyzt");

		BusinessNoticeReceive businessNoticeReceive = formatObject(notice);
		List<BusinessContractReceive> list = businessContractReceiveService.queryByExample(businessNoticeReceive.getHtbh(), businessNoticeReceive.getOrgId(), null, null, null, null);
		if (null != list && list.size() > 0) {
			String htbm = list.get(0).getSptHtid();

			//组装请求参数
			JSONObject object = new JSONObject();
			object.put("htbm", htbm);
			object.put("isChange", "0");
			logger.info("给给市平台发送合同被引用通知数据串："+object.toString());;

			//给市平台发送请求
			result = HTTPUtils.doPost(path, object.toString());

			//保存推送合同引用状态的消息和返回结果
			systemOutsideDataService.addlogger("tshtyyzt", object.toString(), result);
		}
	}

	private BusinessNoticeReceive formatObject(String data) throws Exception {
		ObjectMapper objectMapper = new ObjectMapper();
		BusinessNoticeReceive businessNoticeReceive = objectMapper.readValue(data, BusinessNoticeReceive.class);
		return businessNoticeReceive;
	}



	@SuppressWarnings("unused")
	@Override
	public Map<String, Object> scRkxzzbd(String sptId, String cblx, String ysid) {
		Map<String, Object> result_map = new HashMap<>();
		
		if (sptId != null) {

            BusinessNoticeReceive businessNoticeReceive = new BusinessNoticeReceive();

            businessNoticeReceive.setCrktype("1"); //默认值：入库
        	businessNoticeReceive.setYwtype("1");//默认值：竞价入库
        	businessNoticeReceive.setCblx(cblx);//储备类型(1市储，2区储)
        	businessNoticeReceive.setYsid(ysid);//市平台验收数据ID

            if (sptId != null) {
            	//查询仓房ID、货位ID、orgId
                Map<String, Object> basicMap = feignBasicService.getWareOrTankObj(sptId);
                logger.info("根据值"+sptId+"获取到的货位数据如下：");
                logger.info(basicMap+"");
                if ("0".equals(basicMap.get("houseId").toString()) 
                		|| "0".equals(basicMap.get("warehouseId").toString()) 
                		|| "0".equals(basicMap.get("orgId").toString())) {

                	result_map.put("msg", "根据值"+sptId+"未获取到对应仓房货位信息，请核对数据");
                	result_map.put("status", "error");
                    return result_map;
                } else {
                	Integer houseId = (Integer) basicMap.get("houseId");
                    Integer wareHouseId = (Integer) basicMap.get("warehouseId");
                    Integer orgId = (Integer) basicMap.get("orgId");
                    businessNoticeReceive.setHouseId(houseId); //仓房
                    if (!wareHouseId.toString().equals(houseId.toString())) {//只有粮仓才保存货位，油罐不保存货位
                    	businessNoticeReceive.setWareHouseId(wareHouseId); //货位
                    }
                    businessNoticeReceive.setOrgId(orgId);  //组织id
                }
            } else {
            	result_map.put("msg", "hwbm不能为空");
            	result_map.put("status", "error");
                return result_map;
            }

            Map<String, Object> kcmap = null;
            if (null == businessNoticeReceive.getWareHouseId()) {
            	kcmap = feignAgileService.getDateByChHwh(businessNoticeReceive.getOrgId(), businessNoticeReceive.getHouseId(), businessNoticeReceive.getHouseId());
            	logger.info("根据orgId="+businessNoticeReceive.getOrgId()+"，houseId="+businessNoticeReceive.getHouseId()+",warehouseId="+businessNoticeReceive.getHouseId()+"  获取到的库存数据如下：");
            } else {
            	kcmap = feignAgileService.getDateByChHwh(businessNoticeReceive.getOrgId(), businessNoticeReceive.getHouseId(), businessNoticeReceive.getWareHouseId());
            	logger.info("根据orgId="+businessNoticeReceive.getOrgId()+"，houseId="+businessNoticeReceive.getHouseId()+",warehouseId="+businessNoticeReceive.getWareHouseId()+"  获取到的库存数据如下：");
            }
			logger.info(kcmap+"");
			if (null != kcmap && !"0".equals(kcmap.get("hwxz"))) {
				String inputTime = kcmap.get("rq")+""; //入库时间
	            Double kcsl = (Double) kcmap.get("kcsl");//千克
	            Date date = null;
	            if(StringUtils.isNotBlank(inputTime)){
	                date = ParameterUtil.string2datetime(inputTime);
	                businessNoticeReceive.setInputTime(date);//入库时间
	            }
	            businessNoticeReceive.setProductiveYear(Integer.parseInt(kcmap.get("scnf")+""));//生产年份
	            businessNoticeReceive.setGrainAnnual(Integer.parseInt(kcmap.get("scnf")+""));//收获年度
                businessNoticeReceive.setGrainKind((Integer) kcmap.get("pz")); //品种
	            businessNoticeReceive.setGrainDetailKind((Integer) kcmap.get("mxpz")); //明细品种
	            businessNoticeReceive.setGrainGrade((Integer) kcmap.get("dj")); //等级
	            businessNoticeReceive.setGrainAttribute((Integer) kcmap.get("hwxz")); //粮油性质
	            businessNoticeReceive.setGrainProducingArea((Integer) kcmap.get("gb"));//产地
	            
	            /**
	             * 库存数量转化为吨
	             */
	            BigDecimal s = new BigDecimal(1000);
	            BigDecimal kcslBig = new BigDecimal(kcsl);
	            BigDecimal kcslKg = kcslBig.divide(s);
	            businessNoticeReceive.setLssl(kcslKg.toString());//库存数量
	            
	            if ("1".equals(cblx)) {//市储
	            	businessNoticeReceive.setHzqlsxzdm(6877);
	            }
	            if ("2".equals(cblx)) {//区储
	            	businessNoticeReceive.setHzqlsxzdm(6878);
	            }
	            //businessNoticeReceive.setHzqlsxzdm(3052);//商品粮 转后,划转后粮食性质
	            StringBuffer sbf = new StringBuffer(50);
	            Date thisdate = new Date();
	            sbf.append("RKXZZBD").append(String.format("%03d", businessNoticeReceive.getOrgId())).append(ParameterUtil.getDateYMDHMS(thisdate));
	            businessNoticeReceive.setLsxzzbdh(sbf.toString());//粮食性质转变单单号
	            //businessNoticeReceive.setHzrq(thisdate);//划转时间
	            businessNoticeReceive.setCount(businessNoticeReceive.getLssl()); //计划数量（吨）
	            businessNoticeReceive.setInApplication(1); //启用
	            businessNoticeReceive.setAuditState(0);//下达未审批
	            businessNoticeReceive.setReceiveTime(thisdate);//接收数据的时间
	            businessNoticeReceive.setIsNotice(1);//0是可以引用的通知单,1不是
	            businessNoticeReceive.setIsSheet(0);//0代表是性质转变单
	            businessNoticeReceiveMapper.insert(businessNoticeReceive);
	            
	            result_map.put("msg", "操作成功");
            	result_map.put("status", "success");
			} else {
				result_map.put("msg", "根据所传信息没有查到对应库存信息");
            	result_map.put("status", "error");
                return result_map;
			}
        } else {
        	result_map.put("msg", "所传数据不能为空");
        	result_map.put("status", "error");
        }
		
		return result_map;
	}

	@Override
	public void updateNoticeReceiveAuditState(String cblx, String ysid, String hzsj) {
		BusinessNoticeReceiveExample example = new BusinessNoticeReceiveExample();
        Criteria createCriteria = example.createCriteria();
        createCriteria.andCblxEqualTo(cblx);
        createCriteria.andYsidEqualTo(ysid);
        List<BusinessNoticeReceive> list = businessNoticeReceiveMapper.selectByExample(example);
        if (null != list && list.size() > 0) {
        	BusinessNoticeReceive obj = list.get(0);
        	obj.setAuditState(6);//入库的审批通过
        	obj.setHzrq(ParameterUtil.string2datetime(hzsj));//划转时间
        	businessNoticeReceiveMapper.updateByPrimaryKey(obj);

        	//把对应的粮情卡所挂载的粮食性质改为储备粮
			updateQualitycheckHwxz(obj);

			Map<String, Object> kcmap = null;
			if (null == obj.getWareHouseId()) {
            	kcmap = feignAgileService.getDateByChHwh(obj.getOrgId(), obj.getHouseId(), obj.getHouseId());
            	logger.info("根据orgId="+obj.getOrgId()+"，houseId="+obj.getHouseId()+",warehouseId="+obj.getHouseId()+"  获取到的库存数据如下：");
            } else {
            	kcmap = feignAgileService.getDateByChHwh(obj.getOrgId(), obj.getHouseId(), obj.getWareHouseId());
            	logger.info("根据orgId="+obj.getOrgId()+"，houseId="+obj.getHouseId()+",warehouseId="+obj.getWareHouseId()+"  获取到的库存数据如下：");
            }
			logger.info(kcmap+"");
			//如果转前性质跟当前库存的性质一致，那就转储，否则不发送转储指令
			if (null != kcmap && !obj.getHzqlsxzdm().toString().equals(kcmap.get("hwxz").toString())) {
				boolean flag = restLsxz(obj.getId());
				System.out.println("性质转变单调用出入库接口结果:"+flag);
			}
        }
	}

}
