package com.chinaitop.depot.business.service.impl;

import com.chinaitop.depot.business.mapper.UserInfoMapper;
import com.chinaitop.depot.business.mapper.UserRoleMapper;
import com.chinaitop.depot.business.model.UserInfo;
import com.chinaitop.depot.business.model.UserInfoExample;
import com.chinaitop.depot.business.model.UserInfoExample.Criteria;
import com.chinaitop.depot.business.model.UserRole;
import com.chinaitop.depot.business.model.UserRoleExample;
import com.chinaitop.depot.business.service.UserInfoService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

import java.math.BigInteger;
import java.security.MessageDigest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@Service
public class UserInfoServiceImpl implements UserInfoService {
	
	@Resource
	private UserInfoMapper UserInfoMapper;
	
	@Resource
	private UserRoleMapper userRoleMapper;

	@Override
	public UserInfo valid(String UserInfoName, String password) {
		// 获取MD5加密后的字符串
		password = this.getMD5(password);
		UserInfoExample example = new UserInfoExample();
		example.createCriteria().andUsernameEqualTo(UserInfoName).andPasswordEqualTo(password);
		List<UserInfo> list = UserInfoMapper.selectByExample(example);
		if (list.size() > 0) {
			UserInfo userInfo = list.get(0);
			UserRoleExample urExample = new UserRoleExample();
			urExample.createCriteria().andUserIdEqualTo(userInfo.getUserId());
			List<UserRole> urList = userRoleMapper.selectByExample(urExample);
			List<String> roleIds = new ArrayList<>();
			for (UserRole userRole : urList) {
				roleIds.add(String.valueOf(userRole.getRoleId()));
			}
			userInfo.setRoleIds(roleIds);
			return userInfo;
		}
		return null;
	}

	@Override
	public List<UserInfo> queryByExample(UserInfoExample example) {
		// TODO Auto-generated method stub
		return UserInfoMapper.selectByExample(example);
	}

	@Override
	public UserInfo findById(Integer userId) {
		// TODO Auto-generated method stub
		UserInfo userInfo =  UserInfoMapper.selectByPrimaryKey(userId);
		if (null != userInfo) {
			UserRoleExample example = new UserRoleExample();
			example.createCriteria().andUserIdEqualTo(userInfo.getUserId());
			List<UserRole> urList = userRoleMapper.selectByExample(example);
			List<String> roleIds = new ArrayList<>();
			for (UserRole userRole : urList) {
				roleIds.add(String.valueOf(userRole.getRoleId()));
			}
			userInfo.setRoleIds(roleIds);
		}
		return userInfo;
	}

	@Override
	public void add(UserInfo userInfo) {
		// TODO Auto-generated method stub
		userInfo.setPassword(this.getMD5("123456"));
		UserInfoMapper.insert(userInfo);
		// 添加用户角色信息
		for (String roleId : userInfo.getRoleIds()) {
			UserRole userRole = new UserRole();
			userRole.setUserId(userInfo.getUserId());
			userRole.setRoleId(Integer.parseInt(roleId));
			userRoleMapper.insert(userRole);
		}
	}

	@Override
	public void update(UserInfo userInfo) {
		// TODO Auto-generated method stub
		UserInfoMapper.updateByPrimaryKey(userInfo);
		// 更新用户角色信息,先删除，再增加
		UserRoleExample userRoleExample = new UserRoleExample();
		userRoleExample.createCriteria().andUserIdEqualTo(userInfo.getUserId());
		userRoleMapper.deleteByExample(userRoleExample);
		for (String roleId : userInfo.getRoleIds()) {
			UserRole userRole = new UserRole();
			userRole.setUserId(userInfo.getUserId());
			userRole.setRoleId(Integer.parseInt(roleId));
			userRoleMapper.insert(userRole);
		}
	}

	@Override
	public void remove(Integer userId) {
		// TODO Auto-generated method stub
		UserInfoMapper.deleteByPrimaryKey(userId);
	}

	@Override
	public boolean validUsername(Integer userId, String username) {
		// TODO Auto-generated method stub
		UserInfoExample example = new UserInfoExample();
		Criteria criteria = example.createCriteria();
		if (userId != null) {
			criteria.andUserIdNotEqualTo(userId);
		}
		criteria.andUsernameEqualTo(username);
		List<UserInfo> list = UserInfoMapper.selectByExample(example);
		if (list.size() > 0) {
			return false;
		}
		return true;
	}

	@Override
	public boolean validPassword(String resPassword, String oldPassword) {
		oldPassword = this.getMD5(oldPassword);
		return resPassword.equals(oldPassword);
	}

    @Override
    public void resetPassword(Integer userId) {
	    UserInfo userInfo = UserInfoMapper.selectByPrimaryKey(userId);
	    userInfo.setPassword(this.getMD5("123456"));
	    UserInfoMapper.updateByPrimaryKey(userInfo);
    }

	@Override
	public List<UserInfo> queryAll(HashMap<Object, Object> map) {
		// TODO Auto-generated method stub
		return UserInfoMapper.queryAll(map);
	}

	/**
	 * 对字符串md5加密(小写+字母)
	 *
	 * @param str 传入要加密的字符串
	 * @return  MD5加密后的字符串
	 */
	@Override
	public String getMD5(String str) {
		try {
			// 生成一个MD5加密计算摘要
			MessageDigest md = MessageDigest.getInstance("MD5");
			// 计算md5函数
			md.update(str.getBytes());
			// digest()最后确定返回md5 hash值，返回值为8为字符串。因为md5 hash值是16位的hex值，实际上就是8位的字符
			// BigInteger函数则将8位的字符串转换成16位hex值，用字符串来表示；得到字符串形式的hash值
			return new BigInteger(1, md.digest()).toString(16);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

}
