package com.chinaitop.depot.common.controller;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Properties;
import java.util.zip.ZipOutputStream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.common.service.FileUploadService;
import com.chinaitop.depot.utils.ZipUtils;

@RestController
@RequestMapping("")
public class DownloadController {

    @Autowired
    private FileUploadService fileUploadService;

	@Value("${web.upload-path}")
	private String uploadPath;
	/**
	 * 文件下载
	 * 
	 * @param filePath
	 * @param response
	 * @throws IOException
	 */
	@RequestMapping("/download")
	public void download(String filePath, HttpServletRequest request, String type, String originalFileName, HttpServletResponse response) throws IOException {
		// path是指欲下载的文件的路径。
		File file = new File(uploadPath + filePath);
	    //File file = new File(filePath);
		// 取得文件名。
		String fileName = file.getName();
		if ("business".equals(type)) {
		    fileName = originalFileName;
        } 
		String fileType = fileName.substring(fileName.lastIndexOf(".")+1);

		// 以流的形式下载文件。
		response.setHeader("content-type", "application/octet-stream");
        if("pdf".equals(fileType)){
           response.setContentType("application/pdf;charset=GBK");
        }else if("xls".equals(fileType) || "xlsx".equals(fileType)){
           response.setContentType("application/msexcel;charset=GBK");
        }else if("doc".equals(fileType) || "docx".equals(fileType)){
           response.setContentType("application/msword;charset=GBK");
        }else if("txt".equals(fileType)){
        	response.setContentType("text/plain;charset=GBK");
        }else {
        	response.setContentType("application/octet-stream");
	    }
		response.setHeader("Content-Disposition", "attachment;filename=" + 
				new String(fileName.getBytes("GB2312"), "ISO8859-1"));
		byte[] buff = new byte[1024];
		BufferedInputStream bis = null;
		OutputStream os = null;
		try {
			os = response.getOutputStream();
			bis = new BufferedInputStream(new FileInputStream(file));
			int i = bis.read(buff);
			while (i != -1) {
				os.write(buff, 0, buff.length);
				os.flush();
				i = bis.read(buff);
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (bis != null) {
				try {
					bis.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	/**
     * 打包压缩下载文件
     */
    @RequestMapping(value = "/downLoadZipFile")
    public void downLoadZipFile(HttpServletRequest request, HttpServletResponse response,
    		String fileName, String filePaths) throws IOException{
        
    	// 服务器路径
		String rootPath = request.getSession().getServletContext().getRealPath("");
		response.setContentType("APPLICATION/OCTET-STREAM");  
        response.setHeader("Content-Disposition","attachment; filename="+ 
        		new String(fileName.getBytes("GB2312"), "ISO8859-1") + ".zip");
        ZipOutputStream out = new ZipOutputStream(response.getOutputStream());
        try {
            for(String filePath : filePaths.split(";")){
                ZipUtils.doCompress(rootPath+filePath, out);
            }
            response.flushBuffer();
        } catch (Exception e) {
            e.printStackTrace();
        }finally{
            out.close();
        }
    }

}
