package com.chinaitop.depot.pushs.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang.ArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSON;
import com.chinaitop.depot.business.mapper.BusinessContractMapper;
import com.chinaitop.depot.business.mapper.BusinessDeliveryStorageNoticeMapper;
import com.chinaitop.depot.business.mapper.BusinessLadingMapper;
import com.chinaitop.depot.business.mapper.BusinessMergeNoticeMapper;
import com.chinaitop.depot.business.mapper.BusinessNoticeReceiveMapper;
import com.chinaitop.depot.business.mapper.BusinessStoreWareDetailMapper;
import com.chinaitop.depot.business.model.BusinessContract;
import com.chinaitop.depot.business.model.BusinessContractExample;
import com.chinaitop.depot.business.model.BusinessDeliveryStorageNotice;
import com.chinaitop.depot.business.model.BusinessDeliveryStorageNoticeExample;
import com.chinaitop.depot.business.model.BusinessMergeNoticeDetail;
import com.chinaitop.depot.business.model.BusinessNoticeReceive;
import com.chinaitop.depot.business.model.BusinessStoreWareDetail;
import com.chinaitop.depot.business.model.BusinessStoreWareDetailExample;
import com.chinaitop.depot.business.service.ReceiveNoticeService;
import com.chinaitop.depot.pushs.service.PushsService;
import com.chinaitop.depot.unissoft.model.ResponseEntity;
import com.chinaitop.depot.utils.ParameterUtil;

@Service
public class PushServiceImpl implements PushsService{
	
    final static Logger logger  =  LoggerFactory.getLogger(PushServiceImpl.class );


	@Resource
    private BusinessContractMapper businessContractMapper;
	@Resource
	private BusinessDeliveryStorageNoticeMapper businessDeliveryStorageNoticeMapper;
	@Resource
	private BusinessStoreWareDetailMapper businessStoreWareDetailMapper;
	@Resource
    private BusinessLadingMapper ladingMapper;
	@Resource
	private ReceiveNoticeService receiveNoticeService;
	@Resource
	private BusinessNoticeReceiveMapper receiveNoticeMapper;
	@Resource
	private BusinessMergeNoticeMapper businessMergeNoticeMapper;



	@Transactional(rollbackFor = Exception.class)
	@Override
	public String updateContractStatus(String inApplication, String ids) {
		Map<String,Object> mapData = new HashMap<String, Object>();
		//添加查询合同id方法
		BusinessContractExample businessContractExample = new BusinessContractExample();
		BusinessContractExample.Criteria criteria = businessContractExample.createCriteria();
		//传输过来的ids转换成list<Integer>类型的。
		int[] intIds = Arrays.stream(ids.split(",")).mapToInt(Integer::parseInt).toArray();
		Integer[] IntegerIds = ArrayUtils.toObject(intIds);
		criteria.andIdIn(Arrays.asList(IntegerIds));
//		businessContractExample.setOrderByClause("create_time DESC");
		List<BusinessContract> businessContracts = businessContractMapper.selectByExample(businessContractExample);
		if(businessContracts.size() == 0){
			mapData.put("error", "id is error");
			return JSON.toJSONString(ResponseEntity.ok(mapData));
		}
//		criteria.andCreateTimeEqualTo(businessContracts.get(0).getChangeTime());
		//添加要修改的状态值
		BusinessContract businessContract = new BusinessContract();
		businessContract.setInApplication(Integer.parseInt(inApplication));
		businessContractMapper.updateByExampleSelective(businessContract,businessContractExample);

		mapData.put("status", "success");
		return JSON.toJSONString(ResponseEntity.ok(mapData));
	}



	@Transactional(rollbackFor = Exception.class)
	@Override
	public String getNoticeDatas(String orgId, String inApplication, String ids ) {
		List<String> listType=new ArrayList<String>();
		listType.add("notice");
		listType.add("transferNotice");
		listType.add("ykNotice");
		//先查出主数据
		Map<String,Object> mapCondition = new HashMap<String, Object>();
		mapCondition.put("orgId",orgId);
		mapCondition.put("inApplication",inApplication);
		if(ParameterUtil.isnotnull(ids)) {
			List<String> idsList = Arrays.asList(ids.split(","));
			mapCondition.put("ids", idsList);
		}
		List<Map<String,Object>> mainList =businessDeliveryStorageNoticeMapper.uploadNoticeData(mapCondition);
		//再根据主数据id查出相关的从数据插入
		for (Map<String,Object> mains : mainList) {
			BusinessStoreWareDetailExample businessStoreWareDetailExample = new BusinessStoreWareDetailExample();
			BusinessStoreWareDetailExample.Criteria criteria = businessStoreWareDetailExample.createCriteria();
			criteria.andZidEqualTo(Integer.valueOf(mains.get("id").toString()));
			criteria.andTypeIn(listType);//默认查询通知单类型数据
			List<BusinessStoreWareDetail> json = businessStoreWareDetailMapper.selectByExample(businessStoreWareDetailExample);
			//单位转化为公斤
			BigDecimal bignum = new BigDecimal(1000);
			if(json.size()>0 && json!=null){
				if(mains.get("bill_type").toString().equals("2")){
					for (int i = 0; i < json.size(); i++) {
						if(null != json.get(i).getInCount() && !json.get(i).getInCount().equals("")){
							BigDecimal bignum1 = new BigDecimal(json.get(i).getInCount()); 
							BigDecimal inCount = bignum1.multiply(bignum);
							json.get(i).setInCount(inCount.toString());
						}
						if(null != json.get(i).getOutCount() && !json.get(i).getOutCount().equals("")){
							BigDecimal bignum2 = new BigDecimal(json.get(i).getOutCount()); 
							BigDecimal outCount = bignum2.multiply(bignum);
							json.get(i).setOutCount(outCount.toString());
						}
					}
				}else{
					for (int i = 0; i < json.size(); i++) {
						if(null != json.get(i).getInCount() && !json.get(i).getInCount().equals("") && !json.get(i).getInPrice().equals("") && null != json.get(i).getInPrice()){
							BigDecimal bignum1 = new BigDecimal(json.get(i).getInCount()); 
							BigDecimal biginprice1 = new BigDecimal(json.get(i).getInPrice()); 
							BigDecimal inCount = bignum1.multiply(bignum);
							//BigDecimal inDetailTotalPrice = inCount.multiply(biginprice1);
							BigDecimal inPrice = biginprice1.divide(bignum);
							json.get(i).setInCount(inCount.toString());
							json.get(i).setInPrice(inPrice.toString());
						}
						if(null != json.get(i).getOutCount() && !json.get(i).getOutCount().equals("") && null != json.get(i).getOutPrice() &&  !json.get(i).getOutPrice().equals("")){
							BigDecimal bignum2 = new BigDecimal(json.get(i).getOutCount()); 
							BigDecimal biginprice2 = new BigDecimal(json.get(i).getOutPrice()); 
							BigDecimal outCount = bignum2.multiply(bignum);
							//BigDecimal outDetailTotalPrice = outCount.multiply(biginprice2);
							BigDecimal outPrice = biginprice2.divide(bignum);
							json.get(i).setOutCount(outCount.toString());
							json.get(i).setOutPrice(outPrice.toString());
						}
					}
				}
				
				
				
				//判断出库期限 储备粮通知单
				if(mains.containsKey("create_unit") && mains.get("bill_type").toString().equals("3")){
					if(mains.get("create_unit").toString().equals("1")){ //取储备粮的通知单
						for (int j = 0; j < json.size(); j++) {
							if(json.get(j).getJstzdId().length()>0 && json.get(j).getJstzdId()!=null){
								BusinessNoticeReceive noticeReceive = receiveNoticeMapper.selectByPrimaryKey(Integer.valueOf(json.get(j).getJstzdId()));
								json.get(j).setOutboundExpirationDate(noticeReceive.getOutgoingPeriod());
								/*try {
									SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
									String nowTime = sdf.format(new Date());
									//1是0否
									if(sdf.parse(nowTime).getTime()<sdf.parse(noticeReceive.getOutgoingPeriod()).getTime()){
										json.get(j).setExpiredOrNot(0);
									}else{
										json.get(j).setExpiredOrNot(1);
									}
									json.get(j).setOutgoingPeriod(sdf.parse(noticeReceive.getOutgoingPeriod()));
								} catch (ParseException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}*/
							}
						}
					}
				}
				
			}
			
			mains.put("fromData",json);//从表数据以json形式插入主表数据
			
			//数量转化为公斤
			if(mains.get("shiping_count")!=null && !mains.get("shiping_count").equals("")){
				BigDecimal count = new BigDecimal(mains.get("shiping_count").toString());
				mains.put("shiping_count", count.multiply(bignum).toString());
			}
			/*if(mains.get("money_quantity")!=null && !mains.get("money_quantity").equals("")){
				BigDecimal money = new BigDecimal(mains.get("money_quantity").toString());
				mains.put("money_quantity", money.multiply(bignum).toString());
			}*/
			
			//处理储备粮中的通知单 
			if(null != mains.get("create_unit")){
				if(mains.get("create_unit").equals(1)){
					//传字段 计划文件号 和摘要 用于分仓保管账
					for (BusinessStoreWareDetail businessStoreWareDetail : json) {
						BusinessNoticeReceive businessNoticeReceive =new BusinessNoticeReceive();
						businessNoticeReceive.setHouseId(businessStoreWareDetail.getHouseId());
						businessNoticeReceive.setWareHouseId(businessStoreWareDetail.getWareHouseId());
						businessNoticeReceive.setGrainKind(businessStoreWareDetail.getGrainKind());
						businessNoticeReceive.setGrainAttribute(businessStoreWareDetail.getGrainAttribute());
						businessNoticeReceive.setOrgId(businessStoreWareDetail.getOrgId());
						List<BusinessNoticeReceive> noticeRecevieList = receiveNoticeService.queryByExample(null, null, businessNoticeReceive, null,null);
						if(null != noticeRecevieList && noticeRecevieList.size()>0){
							String jhwjh = noticeRecevieList.get(0).getNoticeNumber();
							String zy = noticeRecevieList.get(0).getYwtype();
							if(("1").equals(zy)){ //1收储 2 销售 3轮换
								zy = "收储";
							}else if(("2").equals(zy)){
								zy = "销售";
							}else if(("3").equals(zy)){
								zy = "轮换";
							}
							mains.put("jhwjh", jhwjh);
							mains.put("zy", zy);
						}
					}
				}
				
			}
		}

		Map<String,Object> mapCondition1 = new HashMap<String, Object>();
		mapCondition1.put("orgId",orgId);
		if(ParameterUtil.isnotnull(ids)) {
			List<String> idsList = Arrays.asList(ids.split(","));
			mapCondition1.put("ids", idsList);
		}
		
		List<Map<String,Object>> mainList1 =businessDeliveryStorageNoticeMapper.getNoticeData(mapCondition1);
		//再根据主数据id查出相关的从数据插入
		for (Map<String,Object> mains : mainList1) {
			//单位转化为公斤
			BigDecimal bignum = new BigDecimal(1000);
			//数量转化为公斤
			if(mains.get("shiping_count")!=null && !mains.get("shiping_count").equals("")){
				BigDecimal count = new BigDecimal(mains.get("shiping_count").toString());
				mains.put("shiping_count", count.multiply(bignum).toString());
			}
		}

		//查所有合并过的数据
		BusinessMergeNoticeDetail detailObj = new BusinessMergeNoticeDetail();
		detailObj.setOrgId(Integer.parseInt(orgId));

		List<Map<String,Object>> mainList2 = new ArrayList<>();
		List<BusinessMergeNoticeDetail> hbtzdList = businessMergeNoticeMapper.findByZidDetailList(detailObj);
		if (null != hbtzdList && hbtzdList.size() > 0) {
			List<Integer> tzdids = new ArrayList<>();
			for (BusinessMergeNoticeDetail detail : hbtzdList) {
				tzdids.add(Integer.parseInt(detail.getTzdid()));
			}
			Map<String,Object> mapCondition2 = new HashMap<String, Object>();
			mapCondition2.put("ids", tzdids);
			
			mainList2 = businessDeliveryStorageNoticeMapper.getNoticeData(mapCondition2);
			//再根据主数据id查出相关的从数据插入
			for (Map<String,Object> mains : mainList2) {
				BusinessStoreWareDetailExample businessStoreWareDetailExample = new BusinessStoreWareDetailExample();
				BusinessStoreWareDetailExample.Criteria criteria = businessStoreWareDetailExample.createCriteria();
				criteria.andZidEqualTo(Integer.valueOf(mains.get("id").toString()));
				criteria.andTypeEqualTo("notice");//默认查询通知单类型数据
				List<BusinessStoreWareDetail> json = businessStoreWareDetailMapper.selectByExample(businessStoreWareDetailExample);
				//单位转化为公斤
				BigDecimal bignum = new BigDecimal(1000);
				if(json.size()>0 && json!=null){
					if(mains.get("bill_type").toString().equals("2")){
						for (int i = 0; i < json.size(); i++) {
							if(null != json.get(i).getInCount() && !json.get(i).getInCount().equals("")){
								BigDecimal bignum1 = new BigDecimal(json.get(i).getInCount()); 
								BigDecimal inCount = bignum1.multiply(bignum);
								json.get(i).setInCount(inCount.toString());
							}
							if(null != json.get(i).getOutCount() && !json.get(i).getOutCount().equals("")){
								BigDecimal bignum2 = new BigDecimal(json.get(i).getOutCount()); 
								BigDecimal outCount = bignum2.multiply(bignum);
								json.get(i).setOutCount(outCount.toString());
							}
						}
					}else{
						for (int i = 0; i < json.size(); i++) {
							if(null != json.get(i).getInCount() && !json.get(i).getInCount().equals("") && !json.get(i).getInPrice().equals("") && null != json.get(i).getInPrice()){
								BigDecimal bignum1 = new BigDecimal(json.get(i).getInCount()); 
								BigDecimal biginprice1 = new BigDecimal(json.get(i).getInPrice()); 
								BigDecimal inCount = bignum1.multiply(bignum);
								//BigDecimal inDetailTotalPrice = inCount.multiply(biginprice1);
								BigDecimal inPrice = biginprice1.divide(bignum);
								json.get(i).setInCount(inCount.toString());
								json.get(i).setInPrice(inPrice.toString());
							}
							if(null != json.get(i).getOutCount() && !json.get(i).getOutCount().equals("") && null != json.get(i).getOutPrice() &&  !json.get(i).getOutPrice().equals("")){
								BigDecimal bignum2 = new BigDecimal(json.get(i).getOutCount()); 
								BigDecimal biginprice2 = new BigDecimal(json.get(i).getOutPrice()); 
								BigDecimal outCount = bignum2.multiply(bignum);
								//BigDecimal outDetailTotalPrice = outCount.multiply(biginprice2);
								BigDecimal outPrice = biginprice2.divide(bignum);
								json.get(i).setOutCount(outCount.toString());
								json.get(i).setOutPrice(outPrice.toString());
							}
						}
					}

					//判断出库期限 储备粮通知单
					if(mains.containsKey("create_unit") && mains.get("bill_type").toString().equals("3")){
						if(mains.get("create_unit").toString().equals("1")){ //取储备粮的通知单
							for (int j = 0; j < json.size(); j++) {
								if(json.get(j).getJstzdId().length()>0 && json.get(j).getJstzdId()!=null){
									BusinessNoticeReceive noticeReceive = receiveNoticeMapper.selectByPrimaryKey(Integer.valueOf(json.get(j).getJstzdId()));
									json.get(j).setOutboundExpirationDate(noticeReceive.getOutgoingPeriod());
								}
							}
						}
					}
					
				}

				mains.put("fromData",json);//从表数据以json形式插入主表数据
				
				//数量转化为公斤
				if(mains.get("shiping_count")!=null && !mains.get("shiping_count").equals("")){
					BigDecimal count = new BigDecimal(mains.get("shiping_count").toString());
					mains.put("shiping_count", count.multiply(bignum).toString());
				}

				//处理储备粮中的通知单 
				if(null != mains.get("create_unit")){
					if(mains.get("create_unit").equals(1)){
						//传字段 计划文件号 和摘要 用于分仓保管账
						for (BusinessStoreWareDetail businessStoreWareDetail : json) {
							BusinessNoticeReceive businessNoticeReceive =new BusinessNoticeReceive();
							businessNoticeReceive.setHouseId(businessStoreWareDetail.getHouseId());
							businessNoticeReceive.setWareHouseId(businessStoreWareDetail.getWareHouseId());
							businessNoticeReceive.setGrainKind(businessStoreWareDetail.getGrainKind());
							businessNoticeReceive.setGrainAttribute(businessStoreWareDetail.getGrainAttribute());
							businessNoticeReceive.setOrgId(businessStoreWareDetail.getOrgId());
							List<BusinessNoticeReceive> noticeRecevieList = receiveNoticeService.queryByExample(null, null, businessNoticeReceive, null,null);
							if(null != noticeRecevieList && noticeRecevieList.size()>0){
								String jhwjh = noticeRecevieList.get(0).getNoticeNumber();
								String zy = noticeRecevieList.get(0).getYwtype();
								if(("1").equals(zy)){ //1收储 2 销售 3轮换
									zy = "收储";
								}else if(("2").equals(zy)){
									zy = "销售";
								}else if(("3").equals(zy)){
									zy = "轮换";
								}
								mains.put("jhwjh", jhwjh);
								mains.put("zy", zy);
							}
						}
					}
					
				}
			}
			
		}

        Map<String,Object> mapData = new HashMap<String, Object>();
		mapData.put("mainData", mainList);
		mapData.put("mainData1", mainList1);
		mapData.put("mainData2", mainList2);
		mapData.put("status", "success");
		return JSON.toJSONString(ResponseEntity.ok(mapData));
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public String updateNoticeState(String inApplication, String ids) {
		//添加查询通知单id方法
		BusinessDeliveryStorageNoticeExample noticeExample = new BusinessDeliveryStorageNoticeExample();
		com.chinaitop.depot.business.model.BusinessDeliveryStorageNoticeExample.Criteria noticeCriteria = noticeExample.createCriteria();
		//传输过来的ids转换成list<Integer>类型的。
		int[] intIds = Arrays.stream(ids.split(",")).mapToInt(Integer::parseInt).toArray();
		Integer[] IntegerIds = ArrayUtils.toObject(intIds);
		noticeCriteria.andIdIn(Arrays.asList(IntegerIds));
		//添加要修改的状态值
		BusinessDeliveryStorageNotice businessDeliveryStorageNotice = new BusinessDeliveryStorageNotice();
		businessDeliveryStorageNotice.setInApplication(Integer.parseInt(inApplication));
		businessDeliveryStorageNoticeMapper.updateByExampleSelective(businessDeliveryStorageNotice,noticeExample);

		Map<String,Object> mapData = new HashMap<String, Object>();
		mapData.put("status", "success");
		return JSON.toJSONString(ResponseEntity.ok(mapData));
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public String getContractDatas(String orgId, String inApplication, String ids) {
		//查出数据
		Map<String,Object> mapCondition = new HashMap<String, Object>();
		mapCondition.put("inApplication",inApplication);
		if(ParameterUtil.isnotnull(ids)){
			List<String> conList = Arrays.asList(ids.split(","));
			mapCondition.put("ids",conList);
		}
		if(ParameterUtil.isnotnull(orgId)){
			mapCondition.put("orgId",orgId);
		}
		mapCondition.put("contractType",6913);//只查询中转合同
		mapCondition.put("auditState",2);//审批通过的中转合同
		List<Map<String,Object>> mainList =businessContractMapper.contractData(mapCondition);

		//再根据主数据id查出相关的从数据插入
		for (Map<String,Object> mains : mainList) {
			BusinessStoreWareDetailExample businessStoreWareDetailExample = new BusinessStoreWareDetailExample();
			BusinessStoreWareDetailExample.Criteria criteria = businessStoreWareDetailExample.createCriteria();
			criteria.andZidEqualTo(Integer.valueOf(mains.get("id").toString()));
			criteria.andTypeEqualTo("contract");//默认查询通知单类型数据
			criteria.andOrgIdEqualTo(Integer.parseInt(orgId));//查询组织机构id
			List<BusinessStoreWareDetail> json = businessStoreWareDetailMapper.selectByExample(businessStoreWareDetailExample);
			//单位转化为公斤
			BigDecimal bignum = new BigDecimal(1000);
			if(json.size()>0 && json!=null){
				for (int i = 0; i < json.size(); i++) {
					if(null != json.get(i).getInCount() && !json.get(i).getInCount().equals("")){
						BigDecimal bignum1 = new BigDecimal(json.get(i).getInCount()); 
						BigDecimal inCount = bignum1.multiply(bignum);
						json.get(i).setInCount(inCount.toString());
					}
					if(null != json.get(i).getOutCount() && !json.get(i).getOutCount().equals("")){
						BigDecimal bignum2 = new BigDecimal(json.get(i).getOutCount()); 
						BigDecimal outCount = bignum2.multiply(bignum);
						json.get(i).setOutCount(outCount.toString());
					}
				}
			}
			
			mains.put("fromData",json);//从表数据以json形式插入主表数据
			
			//数量转化为公斤
			if(mains.get("grain_quantity")!=null && !mains.get("grain_quantity").equals("")){
				BigDecimal count = new BigDecimal(mains.get("grain_quantity").toString());
				mains.put("grain_quantity", count.multiply(bignum).toString());
			}
		}

		Map<String,Object> mapData = new HashMap<String, Object>();
		mapData.put("status", "success");
		mapData.put("mainData", mainList);
		return JSON.toJSONString(ResponseEntity.ok(mapData));
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public String isHaveTzd(String orgId, String inApplication) {
		//先查出数据
		BusinessDeliveryStorageNoticeExample businessDeliveryStorageNoticeExample = new BusinessDeliveryStorageNoticeExample();
		BusinessDeliveryStorageNoticeExample.Criteria criteria = businessDeliveryStorageNoticeExample.createCriteria();
		//criteria.andOrgIdEqualTo(Integer.parseInt(orgId));
		criteria.andHouseIdEqualTo(Integer.parseInt(orgId)); //若是主库orgId和houseId是一样的  否则houseId是代储库的   orgId是主库
		criteria.andInApplicationEqualTo(Integer.parseInt(inApplication));
		List<BusinessDeliveryStorageNotice> mainList =businessDeliveryStorageNoticeMapper.selectByExample(businessDeliveryStorageNoticeExample);
		//把查出来的所有数据定义一个数组
		List<String> ids = new ArrayList<>();
		for (int i = 0; i < mainList.size(); i++) {
			ids.add(mainList.get(i).getId().toString());
		}
		Map<String,Object> mapData = new HashMap<String, Object>();
		mapData.put("status", "success");
		mapData.put("ids", ids);
		return JSON.toJSONString(ResponseEntity.ok(mapData));
	}



	@Override
	public String getLadingDatas(String orgId, String thdhs) {
		
		Map<String,Object> mapCondition = new HashMap<String, Object>();
		mapCondition.put("orgId",orgId);
		if(ParameterUtil.isnotnull(thdhs)) {
			List<String> thdhsList = Arrays.asList(thdhs.split(","));
			mapCondition.put("thdhs", thdhsList);
		}
		
		List<Map<String,Object>> mainList =ladingMapper.getLadingDatas(mapCondition);
		//吨转化为公斤
		BigDecimal basicnum = new BigDecimal(1000);
		if(mainList!=null && mainList.size()>0){
			for (Map<String, Object> map : mainList) {
				BigDecimal bignum1 = new BigDecimal(map.get("djzl").toString()); 
				map.put("djzl", bignum1.multiply(basicnum).toString());
			}
		}

        Map<String,Object> mapData = new HashMap<String, Object>();
		mapData.put("status", "success");
		mapData.put("mainData", mainList);
		return JSON.toJSONString(ResponseEntity.ok(mapData));
	}
}
